@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Default device bitmap copying implementation */
@@ -28,21 +28,6 @@
 #undef mdev
 #include "gxcpath.h"
 
-/* By default, implement tile_rectangle using strip_tile_rectangle. */
-int
-gx_default_tile_rectangle(gx_device * dev, const gx_tile_bitmap * tile,
-   int x, int y, int w, int h, gx_color_index color0, gx_color_index color1,
-                          int px, int py)
-{
-    gx_strip_bitmap tiles;
-
-    *(gx_tile_bitmap *) & tiles = *tile;
-    tiles.shift = tiles.rep_shift = 0;
-    tiles.num_planes = 1;
-    return (*dev_proc(dev, strip_tile_rectangle))
-        (dev, &tiles, x, y, w, h, color0, color1, px, py);
-}
-
 /* Implement copy_mono by filling lots of small rectangles. */
 /* This is very inefficient, but it works as a default. */
 int
@@ -169,16 +154,16 @@ gx_no_copy_alpha(gx_device * dev, const byte * data, int data_x,
 
 /* Currently we really should only be here if the target device is planar
    AND it supports devn colors AND is 8 or 16 bit.  For example tiffsep
-   and psdcmyk may make use of this if AA is enabled.  It is basically 
-   designed for devices that need more than 64 bits for color support 
+   and psdcmyk may make use of this if AA is enabled.  It is basically
+   designed for devices that need more than 64 bits for color support
 
-   So that I can follow things and  make it readable for future generations, 
+   So that I can follow things and  make it readable for future generations,
    I am not using the macro nightmare that default_copy_alpha uses. */
 int
 gx_default_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
            int raster, gx_bitmap_id id, int x, int y, int width, int height,
                       const gx_drawing_color *pdcolor, int depth)
-{ 
+{
     const byte *row_alpha;
     gs_memory_t *mem = dev->memory;
     int bpp = dev->color_info.depth;
@@ -194,14 +179,12 @@ gx_default_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
     byte *src_planes[GS_CLIENT_COLOR_MAX_COMPONENTS];
     gs_int_rect gb_rect;
     int byte_depth;
-    gx_color_index mask;
     int shift, word_width;
     gx_color_value *composite;
     byte *gb_buff;
     int x_curr, w_curr, gb_buff_start;
 
     byte_depth = bpp / ncomps;
-    mask = ((gx_color_index)1 << byte_depth) - 1;
     shift = 16 - byte_depth;
     word_width = byte_depth >> 3;
 
@@ -225,7 +208,7 @@ gx_default_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
                        | GB_ALIGN_STANDARD
                        | GB_OFFSET_0
                        | GB_RASTER_STANDARD
-                       | GB_SELECT_PLANES;    
+                       | GB_SELECT_PLANES;
     gb_rect.p.x = x;
     gb_rect.q.x = x + width;
     for (ry = y; ry < y + height; row_alpha += raster, ++ry) {
@@ -236,21 +219,21 @@ gx_default_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
         for (k = 0; k < ncomps; k++) {
             /* First set the params to zero for all planes except the one we want */
             /* I am not sure why get_bits_rectangle for the planar device can
-               not hand back the data in a proper planar form.  To get the 
+               not hand back the data in a proper planar form.  To get the
                individual planes seems that I need to jump through some hoops
                here */
-            for (j = 0; j < ncomps; j++) 
+            for (j = 0; j < ncomps; j++)
                 gb_params.data[j] = 0;
             gb_params.data[k] = gb_buff + k * out_raster;
-            code = dev_proc(dev, get_bits_rectangle) (dev, &gb_rect, 
-                                                      &gb_params, 0);
+            code = dev_proc(dev, get_bits_rectangle) (dev, &gb_rect,
+                                                      &gb_params);
             src_planes[k] = gb_params.data[k];
             if (code < 0) {
                 gs_free_object(mem, gb_buff, "copy_alpha_hl_color");
                 return code;
             }
         }
-        /* At this point we have to carry around some additional variables 
+        /* At this point we have to carry around some additional variables
            so that we can handle any buffer flushes due to alpha == 0 values.
            See below why this is needed */
         x_curr = x;
@@ -278,21 +261,21 @@ gx_default_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
 
             if (alpha == 0) {
                 /* With alpha 0 we want to avoid writing out this value.
-                 * While it is true that writting it out leaves the color 
-                 * unchanged,  any device that's watching what pixels are 
-                 * written (such as the pattern tile devices) may have problems. 
-                 * As in gx_default_copy_alpha the right thing to do is to write 
-                 * out what we have so far and then continue to collect when we 
+                 * While it is true that writting it out leaves the color
+                 * unchanged,  any device that's watching what pixels are
+                 * written (such as the pattern tile devices) may have problems.
+                 * As in gx_default_copy_alpha the right thing to do is to write
+                 * out what we have so far and then continue to collect when we
                  * get back to non zero alpha.  */
-                code = dev_proc(dev, copy_planes)(dev, &(gb_buff[gb_buff_start]), 
-                                                  0, out_raster, gs_no_bitmap_id, 
+                code = dev_proc(dev, copy_planes)(dev, &(gb_buff[gb_buff_start]),
+                                                  0, out_raster, gs_no_bitmap_id,
                                                   x_curr, ry, w_curr-1, 1, 1);
                 if (code < 0) {
                     gs_free_object(mem, gb_buff, "copy_alpha_hl_color");
                     return code;
                 }
                 /* reset ourselves */
-                gb_buff_start = gb_buff_start + w_curr;
+                gb_buff_start = gb_buff_start + w_curr * word_width;
                 w_curr = 0;
                 x_curr = rx + 1;
             } else {
@@ -305,7 +288,7 @@ gx_default_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
                     /* First get the old color */
                     for (k = 0; k < ncomps; k++) {
                         /* We only have 8 and 16 bit depth to worry about.
-                           However, this stuff should really be done with 
+                           However, this stuff should really be done with
                            the device encode/decode procedure. */
                         byte *ptr = ((src_planes[k]) + (sx - data_x) * word_width);
                         curr_cv[k] = 0;
@@ -318,29 +301,29 @@ gx_default_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
                                 curr_cv[k] += *ptr;
                                 curr_cv[k] += curr_cv[k] << 8;
                         }
-                        /* Now compute the new color which is a blend of 
+                        /* Now compute the new color which is a blend of
                            the old and the new */
                         blend_cv[k] =  ((curr_cv[k]<<8) +
                                         (((long) src_cv[k] - (long) curr_cv[k]) * alpha))>>8;
                         composite = &(blend_cv[0]);
                     }
-                } 
+                }
                 /* Update our plane data buffers.  Just reuse the current one */
                 for (k = 0; k < ncomps; k++) {
                     byte *ptr = ((src_planes[k]) + (sx - data_x) * word_width);
                     switch (word_width) {
                         case 2:
-                            *ptr++ = composite[k] & mask;
+                            *ptr++ = composite[k] >> 8;
                         case 1:
-                            *ptr++ = (composite[k] >> shift) & mask;
+                            *ptr++ = composite[k] >> shift;
                     }
                 }
             } /* else on alpha != 0 */
         } /* loop on x */
-        /* Flush what ever we have left.  We may only have a partial due to 
+        /* Flush what ever we have left.  We may only have a partial due to
            the presence of alpha = 0 values */
-        code = dev_proc(dev, copy_planes)(dev, &(gb_buff[gb_buff_start]), 
-                                          0, out_raster, gs_no_bitmap_id, 
+        code = dev_proc(dev, copy_planes)(dev, &(gb_buff[gb_buff_start]),
+                                          0, out_raster, gs_no_bitmap_id,
                                           x_curr, ry, w_curr, 1, 1);
     } /* loop on y */
     gs_free_object(mem, gb_buff, "copy_alpha_hl_color");
@@ -365,13 +348,14 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
         gs_memory_t *mem = dev->memory;
         int bpp = dev->color_info.depth;
         uchar ncomps = dev->color_info.num_components;
-        uint in_size = gx_device_raster(dev, false);
+        uint in_size = gx_device_raster_chunky(dev, false);
         byte *lin;
         uint out_size;
         byte *lout;
         int code = 0;
         gx_color_value color_cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
         int ry, lx;
+        gs_int_rect rect;
 
         fit_copy(dev, data, data_x, raster, id, x, y, width, height);
         row = data;
@@ -383,6 +367,8 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
             goto out;
         }
         (*dev_proc(dev, decode_color)) (dev, color, color_cv);
+        rect.p.x = 0;
+        rect.q.x = dev->width;
         for (ry = y; ry < y + height; row += raster, ++ry) {
             byte *line;
             int sx, rx;
@@ -391,10 +377,23 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
             int l_dbit = 0;
             byte l_dbyte = ((l_dbit) ? (byte)(*(l_dptr) & (0xff00 >> (l_dbit))) : 0);
             int l_xprev = x;
+            gs_get_bits_params_t params;
 
-            code = (*dev_proc(dev, get_bits)) (dev, ry, lin, &line);
+            params.options = (GB_ALIGN_STANDARD |
+                              (GB_RETURN_COPY | GB_RETURN_POINTER) |
+                              GB_OFFSET_0 |
+                              GB_RASTER_STANDARD | GB_PACKING_CHUNKY |
+                              GB_COLORS_NATIVE | GB_ALPHA_NONE);
+            params.x_offset = 0;
+            params.raster = bitmap_raster(dev->width * dev->color_info.depth);
+            params.data[0] = lin;
+            rect.p.y = ry;
+            rect.q.y = ry+1;
+            code = (*dev_proc(dev, get_bits_rectangle))(dev, &rect,
+                                                        &params);
             if (code < 0)
                 break;
+            line = params.data[0];
             lx = x;
             for (sx = data_x, rx = x; sx < data_x + width; ++sx, ++rx) {
                 gx_color_index previous = gx_no_color_index;
@@ -531,17 +530,6 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
     }
 }
 
-int
-gx_no_copy_rop(gx_device * dev,
-             const byte * sdata, int sourcex, uint sraster, gx_bitmap_id id,
-               const gx_color_index * scolors,
-             const gx_tile_bitmap * texture, const gx_color_index * tcolors,
-               int x, int y, int width, int height,
-               int phase_x, int phase_y, gs_logical_operation_t lop)
-{
-    return_error(gs_error_unknownerror);	/* not implemented */
-}
-
 int
 gx_default_fill_mask(gx_device * orig_dev,
                      const byte * data, int dx, int raster, gx_bitmap_id id,
@@ -598,15 +586,120 @@ gx_default_fill_mask(gx_device * orig_dev,
 }
 
 /* Default implementation of strip_tile_rect_devn.  With the current design
-   only devices that support devn color will be making use of this 
+   only devices that support devn color will be making use of this
    procedure and those are planar devices.  So we have an implemenation
    for planar devices and not a default implemenetation at this time. */
 int
 gx_default_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tiles,
-   int x, int y, int w, int h, const gx_drawing_color * pdcolor0, 
+   int x, int y, int w, int h, const gx_drawing_color * pdcolor0,
    const gx_drawing_color * pdcolor1, int px, int py)
 {
-    return_error(gs_error_unregistered); 
+    int width = tiles->size.x;
+    int height = tiles->size.y;
+    int raster = tiles->raster;
+    int rwidth = tiles->rep_width;
+    int rheight = tiles->rep_height;
+    int shift = tiles->shift;
+    gs_id tile_id = tiles->id;
+    int code;
+
+    if (rwidth == 0 || rheight == 0)
+        return_error(gs_error_unregistered); /* Must not happen. */
+    fit_fill_xy(dev, x, y, w, h);
+    if (w == 0 || h == 0)
+        return 0;
+
+    /* Loop over as many tiles as we need vertically */
+    while (1) {
+        int xoff = (shift == 0 ? px :
+                                 px + (y + py) / rheight * tiles->rep_shift);
+        /* irx = x offset within the tile to start copying from */
+        int irx = ((rwidth & (rwidth - 1)) == 0 ?	/* power of 2 */
+                   (x + xoff) & (rwidth - 1) :
+                   (x + xoff) % rwidth);
+        /* ry = y offset within the tile to start copying from */
+        int ry = ((rheight & (rheight - 1)) == 0 ?	/* power of 2 */
+                  (y + py) & (rheight - 1) :
+                  (y + py) % rheight);
+        /* icw = how many bits we can copy before we run out of tile and need to loop */
+        int icw = width - irx;
+        /* ch = how many rows we can copy before we run out of tile and need to loop */
+        int ch = height - ry;
+        byte *row0 = tiles->data + ry * raster;
+
+        /* Loop per line */
+        do {
+            byte *row = row0;
+            int w2 = w;
+            int left_in_tile = icw;
+            int runlen = 0;
+            int bit = 1<<((7-irx) & 7);
+            int x2 = x;
+            if (left_in_tile > w2)
+                left_in_tile = w2;
+
+            /* Loop per row */
+            while (w2) {
+                while (left_in_tile && ((bit & *row) == 0)) {
+                    bit >>= 1;
+                    if (bit == 0)
+                        bit = 128, row++;
+                    left_in_tile--;
+                    runlen++;
+                }
+                if (runlen) {
+                    gs_fixed_rect rect;
+                    rect.p.x = int2fixed(x2);
+                    rect.p.y = int2fixed(y);
+                    rect.q.x = rect.p.x + int2fixed(runlen);
+                    rect.q.y = rect.p.y + int2fixed(1);
+                    /* Pure colours are only used to signal transparency */
+                    if (pdcolor0->type != gx_dc_type_pure) {
+                        code = dev_proc(dev, fill_rectangle_hl_color)(dev, &rect, NULL, pdcolor0, NULL);
+                        if (code < 0)
+                            return code;
+                    }
+                    x2 += runlen;
+                    w2 -= runlen;
+                    runlen = 0;
+                }
+                while (left_in_tile && ((bit & *row) != 0)) {
+                    bit >>= 1;
+                    if (bit == 0)
+                        bit = 128, row++;
+                    left_in_tile--;
+                    runlen++;
+                }
+                if (runlen) {
+                    gs_fixed_rect rect;
+                    rect.p.x = int2fixed(x2);
+                    rect.p.y = int2fixed(y);
+                    rect.q.x = rect.p.x + int2fixed(runlen);
+                    rect.q.y = rect.p.y + int2fixed(1);
+                    /* Pure colours are only used to signal transparency */
+                    if (pdcolor1->type != gx_dc_type_pure) {
+                        code = dev_proc(dev, fill_rectangle_hl_color)(dev, &rect, NULL, pdcolor1, NULL);
+                        if (code < 0)
+                            return code;
+                    }
+                    x2 += runlen;
+                    w2 -= runlen;
+                    runlen = 0;
+                }
+                if (left_in_tile == 0) {
+                    left_in_tile = rwidth;
+                    if (left_in_tile > w2)
+                        left_in_tile = w2;
+                }
+            }
+
+            /* That's a line complete. */
+            y++;
+            if (--h == 0)
+                return 0;
+            row0 += raster;
+        } while (--ch != 0); /* Until we finish a tile vertically */
+    }
 }
 
 /* Default implementation of strip_tile_rectangle */
@@ -646,27 +739,7 @@ gx_default_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
     }
 #endif
 
-    if (dev_proc(dev, tile_rectangle) != gx_default_tile_rectangle) {
-        if (shift == 0) {	/*
-                                 * Temporarily patch the tile_rectangle procedure in the
-                                 * device so we don't get into a recursion loop if the
-                                 * device has a tile_rectangle procedure that conditionally
-                                 * calls the strip_tile_rectangle procedure.
-                                 */
-            dev_proc_tile_rectangle((*tile_proc)) =
-                dev_proc(dev, tile_rectangle);
-            int code = 0;
-
-            set_dev_proc(dev, tile_rectangle, gx_default_tile_rectangle);
-            code = (*tile_proc)
-                (dev, (const gx_tile_bitmap *)tiles, x, y, w, h,
-                 color0, color1, px, py);
-            set_dev_proc(dev, tile_rectangle, tile_proc);
-            return code;
-        }
-        /* We should probably optimize this case too, for the benefit */
-        /* of window systems, but we don't yet. */
-    } {				/*
+    {				/*
                                  * Note: we can't do the following computations until after
                                  * the fit_fill_xy.
                                  */
@@ -796,12 +869,13 @@ gx_default_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 }
 
 int
-gx_no_strip_copy_rop(gx_device * dev,
+gx_no_strip_copy_rop2(gx_device * dev,
              const byte * sdata, int sourcex, uint sraster, gx_bitmap_id id,
                      const gx_color_index * scolors,
            const gx_strip_bitmap * textures, const gx_color_index * tcolors,
                      int x, int y, int width, int height,
-                     int phase_x, int phase_y, gs_logical_operation_t lop)
+                     int phase_x, int phase_y, gs_logical_operation_t lop,
+                     uint planar_height)
 {
     return_error(gs_error_unknownerror);	/* not implemented */
 }
