@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -478,12 +478,16 @@ gdev_vector_prepare_stroke(gx_device_vector * vdev,
             int i, code;
 
             pattern = (float *)gs_alloc_bytes(vdev->memory->stable_memory, pattern_size * sizeof(float), "vector allocate dash pattern");
+            if (pattern == NULL)
+                return_error(gs_error_VMerror);
             for (i = 0; i < pattern_size; ++i)
                 pattern[i] = pgs->line_params.dash.pattern[i] * scale;
             code = (*vdev_proc(vdev, setdash))
                 (vdev, pattern, pattern_size, dash_offset);
-            if (code < 0)
+            if (code < 0) {
+                gs_free_object(vdev->memory->stable_memory, pattern, "vector free new dash pattern on error");
                 return code;
+            }
             if (vdev->dash_pattern)
                 gs_free_object(vdev->memory->stable_memory, vdev->dash_pattern, "vector free old dash pattern");
             vdev->dash_pattern = pattern;
@@ -630,6 +634,11 @@ gdev_vector_dopath_init(gdev_vector_dopath_state_t *state,
         gs_make_scaling(vdev->scale.x, vdev->scale.y, &state->scale_mat);
     }
     state->first = true;
+
+    /* This is purely to prevent Coverity from thinking gdev_vector_dopath()
+    could use uninitialised state->start.x. */
+    state->start.x = 0;
+    state->start.y = 0;
 }
 
 /*
@@ -638,7 +647,7 @@ gdev_vector_dopath_init(gdev_vector_dopath_state_t *state,
  */
 int
 gdev_vector_dopath_segment(gdev_vector_dopath_state_t *state, int pe_op,
-                           gs_fixed_point vs[3])
+                           gs_fixed_point *vs)
 {
     gx_device_vector *vdev = state->vdev;
     const gs_matrix *const pmat = &state->scale_mat;
@@ -781,8 +790,11 @@ gdev_vector_write_clip_path(gx_device_vector * vdev,
         const gx_clip_list *list = gx_cpath_list(pcpath);
 
         prect = list->head;
-        if (prect == 0)
+        if (prect == 0) {
             prect = &list->single;
+            if (prect->xmax < prect->xmin || prect->ymax < prect->ymin)
+                return 0;
+        }
     }
     /* Write out the rectangles. */
     code = (*vdev_proc(vdev, beginpath)) (vdev, gx_path_type_clip);
@@ -896,9 +908,10 @@ gdev_vector_begin_image(gx_device_vector * vdev,
           (pim->CombineWithColor && rop3_uses_T(pgs->log_op))) &&
          (code = gdev_vector_update_fill_color(vdev, pgs, pdcolor)) < 0) ||
         (vdev->bbox_device &&
-         (code = (*dev_proc(vdev->bbox_device, begin_image))
-          ((gx_device *) vdev->bbox_device, pgs, pim, format, prect,
-           pdcolor, pcpath, mem, &pie->bbox_info)) < 0)
+         (code = (*dev_proc(vdev->bbox_device, begin_typed_image))
+                           ((gx_device *) vdev->bbox_device, pgs, NULL,
+                            (gs_image_common_t *)pim, prect,
+                            pdcolor, pcpath, mem, &pie->bbox_info)) < 0)
         )
         return code;
     pie->memory = mem;
@@ -921,8 +934,7 @@ gdev_vector_end_image(gx_device_vector * vdev,
     int code;
 
     if (pie->default_info) {
-        code = gx_default_end_image((gx_device *) vdev, pie->default_info,
-                                    draw_last);
+        code = gx_image_end(pie->default_info, draw_last);
         if (code >= 0)
             code = 0;
     } else {			/* Fill out to the full image height. */
@@ -962,7 +974,7 @@ int gdev_vector_get_param(gx_device *dev, char *Param, void *list)
 {
     gs_param_list * plist = (gs_param_list *)list;
     gs_param_string ofns;
-    bool bool_true = 1;
+    bool bool_true = 1, bool_false = 0;
 
     ofns.data = (const byte *)vdev->fname,
         ofns.size = strlen(vdev->fname),
@@ -973,6 +985,9 @@ int gdev_vector_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "HighLevelDevice") == 0) {
         return param_write_bool(plist, "HighLevelDevice", &bool_true);
     }
+    if (strcmp(Param, "SupportsRasterOPs") == 0) {
+        return param_write_bool(plist, "SupportsRasterOPs", &bool_false);
+    }
     if (strcmp(Param, "NoInterpolateImagemasks") == 0) {
         return param_write_bool(plist, "NoInterpolateImagemasks", &bool_true);
     }
