@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -36,10 +36,6 @@ typedef struct gs_pattern_instance_s gs_pattern_instance_t;
 #  define GS_CLIENT_COLOR_MAX_COMPONENTS (64)
 #endif
 
-#ifndef MAX_COMPONENTS_IN_DEVN		
-#  define MAX_COMPONENTS_IN_DEVN (64)
-#endif
-
 /* There is a speed penalty for supporting lots of spot colors, so certain
  * devices (tiffsep, tiffsep1, psdcmyk, etc) offer -dMaxSpots. This allows
  * us to compile in a high 'hard' limit on the number of components
