@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /*  Data type definitions when using the gscms  */
@@ -39,6 +39,8 @@
 #define DEV_NEUTRAL_8 5
 #define DEV_NEUTRAL_16 5
 
+#define ARTIFEX_sRGB_HASH 0xfbea006420fca6be
+
 /* Define the preferred size of the output by the CMS */
 /* This can be different than the size of gx_color_value
    which can range between 8 and 16.  Here we can only
@@ -211,8 +213,8 @@ typedef enum {
     GS_TEXT_TAG = 0x1,
     GS_IMAGE_TAG = 0x2,
     GS_VECTOR_TAG = 0x4,
-    GS_UNKNOWN_TAG = 0x40,
-    GS_DEVICE_ENCODES_TAGS = 0x80
+    GS_UNKNOWN_TAG = 0x40000000,
+    GS_DEVICE_ENCODES_TAGS = (int)0x80000000
 } gs_graphics_type_tag_t;
 
 /* Source profile graphic tag rendering conditions */
