@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -326,9 +326,11 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
         case PtCr_idiv:
             if (vsp->value.i == 0)
                 return_error(gs_error_undefinedresult);
-            if ((vsp[-1].value.i /= vsp->value.i) == min_int &&
+            if (vsp[-1].value.i == min_int &&
                 vsp->value.i == -1)  /* anomalous boundary case, fail */
                 return_error(gs_error_rangecheck);
+            else
+                vsp[-1].value.i /= vsp->value.i;
             --vsp; continue;
         case PtCr_ln:
             vsp->value.f = log(vsp->value.f);
