@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* %ram% file device implementation */
@@ -86,8 +86,8 @@ typedef struct gsram_enum_s {
     gs_memory_t *memory;
 } gsram_enum;
 
-gs_private_st_ptrs3(st_gsram_enum, struct gsram_enum_s, "gsram_enum",
-    gsram_enum_enum_ptrs, gsram_enum_reloc_ptrs, pattern, e, memory);
+gs_private_st_ptrs2(st_gsram_enum, struct gsram_enum_s, "gsram_enum",
+    gsram_enum_enum_ptrs, gsram_enum_reloc_ptrs, pattern, memory);
 
 /* could make this runtime configurable later.  It doesn't allocate
    all the blocks in one go so it's not critical */
@@ -113,14 +113,16 @@ static int
 s_ram_switch(stream *, bool);
 
 static int
-ramfs_errno_to_code(int errno) {
-    switch (errno) {
+ramfs_errno_to_code(int error_number) {
+    switch (error_number) {
     case RAMFS_NOTFOUND:
         return_error(gs_error_undefinedfilename);
     case RAMFS_NOACCESS:
         return_error(gs_error_invalidfileaccess);
     case RAMFS_NOMEM:
         return_error(gs_error_VMerror);
+    case RAMFS_BADRANGE:
+        return_error(gs_error_rangecheck);
     /* just in case */
     default:
         return_error(gs_error_ioerror);
@@ -150,7 +152,8 @@ ram_open_file(gx_io_device * iodev, const char *fname, uint len,
     strncpy(namestr,fname,len);
     namestr[len] = 0;
 
-    if (!iodev) {/*iodev = iodev_default;*/
+    /* Make sure iodev is valid, and we have initialised the RAM file system (state != NULL) */
+    if (iodev == NULL || iodev->state == NULL) {/*iodev = iodev_default;*/
         gs_free_object(mem, namestr, "free temporary filename string");
         return gs_note_error(gs_error_invalidaccess);
     }
@@ -250,10 +253,11 @@ s_ram_read_seek(register stream * s, gs_offset_t pos)
         s->cursor.r.ptr = s->cbuf + offset - 1;
         return 0;
     }
-    if (pos < 0 || pos > s->file_limit ||
-        ramfile_seek((ramhandle*)s->file, s->file_offset + pos, RAMFS_SEEK_SET) != 0
-    )
-    return ERRC;
+
+    if (pos < 0 || pos > s->file_limit || s->file == NULL ||
+        ramfile_seek((ramhandle*)s->file, s->file_offset + pos, RAMFS_SEEK_SET) != 0)
+        return ERRC;
+
     s->cursor.r.ptr = s->cursor.r.limit = s->cbuf - 1;
     s->end_status = 0;
     s->position = pos;
@@ -337,8 +341,8 @@ s_ram_write_seek(stream * s, gs_offset_t pos)
     int code = sflush(s);
 
     if (code < 0) return code;
-    if (ramfile_seek((ramhandle*)s->file, pos, RAMFS_SEEK_SET) != 0)
-    return ERRC;
+    if (pos < 0 || ramfile_seek((ramhandle*)s->file, pos, RAMFS_SEEK_SET) != 0)
+        return ERRC;
     s->position = pos;
     return 0;
 }
@@ -449,7 +453,12 @@ ram_finalize(const gs_memory_t *memory, void * vptr)
 static int
 ram_delete(gx_io_device * iodev, const char *fname)
 {
-    ramfs* fs = GETRAMFS(iodev->state);
+    ramfs* fs;
+
+    if (iodev->state == NULL)
+        return_error(gs_error_ioerror);
+    else
+        fs = GETRAMFS(iodev->state);
 
     if(ramfs_unlink(fs,fname)!=0) {
     return_error(ramfs_errno_to_code(ramfs_error(fs)));
@@ -460,7 +469,12 @@ ram_delete(gx_io_device * iodev, const char *fname)
 static int
 ram_rename(gx_io_device * iodev, const char *from, const char *to)
 {
-    ramfs* fs = GETRAMFS(iodev->state);
+    ramfs* fs;
+
+    if (iodev->state == NULL)
+        return_error(gs_error_ioerror);
+    else
+        fs = GETRAMFS(iodev->state);
 
     if(ramfs_rename(fs,from,to)!=0) {
     return_error(ramfs_errno_to_code(ramfs_error(fs)));
@@ -472,7 +486,12 @@ static int
 ram_status(gx_io_device * iodev, const char *fname, struct stat *pstat)
 {
     ramhandle * f;
-    ramfs* fs = GETRAMFS(iodev->state);
+    ramfs* fs;
+
+    if (iodev->state == NULL)
+        return_error(gs_error_ioerror);
+    else
+        fs = GETRAMFS(iodev->state);
 
     f = ramfs_open(((ramfs_state *)iodev->state)->memory, fs,fname,RAMFS_READ);
     if(!f) return_error(ramfs_errno_to_code(ramfs_error(fs)));
@@ -498,7 +517,13 @@ ram_enumerate_init(gs_memory_t * mem, gx_io_device *iodev, const char *pat,
     mem, patlen+1, "ram_enumerate_file_init(pattern)"
     );
 
-    ramfs_enum * e = ramfs_enum_new(GETRAMFS(iodev->state));
+    ramfs_enum * e;
+
+    if (iodev->state == NULL)
+        return NULL;
+    else
+        e = ramfs_enum_new(GETRAMFS(iodev->state));
+
     if(penum && pattern && e) {
     memcpy(pattern, pat, patlen);
     pattern[patlen]=0;
@@ -553,10 +578,16 @@ ram_get_params(gx_io_device * iodev, gs_param_list * plist)
     int code;
     int i0 = 0, so = 1;
     bool btrue = true, bfalse = false;
-    ramfs* fs = GETRAMFS(iodev->state);
+    ramfs* fs;
+
     int BlockSize;
     long Free, LogicalSize;
 
+    if (iodev->state == NULL)
+        return_error(gs_error_ioerror);
+    else
+        fs = GETRAMFS(iodev->state);
+
     BlockSize = ramfs_blocksize(fs);
     LogicalSize = MAXBLOCKS;
     Free = ramfs_blocksfree(fs);
