@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -44,8 +44,10 @@
 /* Forward references */
 static gs_gstate *gstate_alloc(gs_memory_t *, client_name_t,
                                const gs_gstate *);
-static gs_gstate *gstate_clone(gs_gstate *, gs_memory_t *, client_name_t,
-                               gs_gstate_copy_reason_t);
+static gs_gstate *gstate_clone_for_gsave(gs_gstate *,
+                                         client_name_t);
+static gs_gstate *gstate_clone_for_gstate(const gs_gstate *, gs_memory_t *,
+                                          client_name_t);
 static void gstate_free_contents(gs_gstate *);
 static int gstate_copy(gs_gstate *, const gs_gstate *,
                         gs_gstate_copy_reason_t, client_name_t);
@@ -74,7 +76,7 @@ static void clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t c
  *
  *   (3a) Objects that are logically connected to individual gstates.
  *      We use reference counting to manage these.  Currently these are:
- *              halftone, dev_ht, cie_render, black_generation,
+ *              halftone, dev_ht(4), cie_render, black_generation,
  *              undercolor_removal, set_transfer.*, cie_joint_caches,
  *              clip_stack, {opacity,shape}.mask
  *      effective_transfer.* may point to some of the same objects as
@@ -161,7 +163,7 @@ extern_st(st_gs_gstate); /* for gstate_alloc() */
 /* Copy client data, using the copy_for procedure if available, */
 /* the copy procedure otherwise. */
 static int
-gstate_copy_client_data(gs_gstate * pgs, void *dto, void *dfrom,
+gstate_copy_client_data(const gs_gstate * pgs, void *dto, void *dfrom,
                         gs_gstate_copy_reason_t reason)
 {
     return (pgs->client_procs.copy_for != 0 ?
@@ -275,7 +277,7 @@ gs_gstate_client_data(const gs_gstate * pgs)
 }
 
 /* Free the chain of gstates.*/
-int
+void
 gs_gstate_free_chain(gs_gstate * pgs)
 {
    gs_gstate *saved = pgs, *tmp;
@@ -285,26 +287,25 @@ gs_gstate_free_chain(gs_gstate * pgs)
        gs_gstate_free(saved);
        saved = tmp;
    }
-   return 0;
 }
 
 /* Free a graphics state. */
-int
+void
 gs_gstate_free(gs_gstate * pgs)
 {
+    if (pgs == NULL)
+        return;
     gstate_free_contents(pgs);
     gs_free_object(pgs->memory, pgs, "gs_gstate_free");
-    return 0;
 }
 
 /* Save the graphics state. */
 int
 gs_gsave(gs_gstate * pgs)
 {
-    gs_gstate *pnew = gstate_clone(pgs, pgs->memory, "gs_gsave",
-                                  copy_for_gsave);
+    gs_gstate *pnew = gstate_clone_for_gsave(pgs, "gs_gsave");
 
-    if (pnew == 0)
+    if (pnew == NULL)
         return_error(gs_error_VMerror);
     /* As of PLRM3, the interaction between gsave and the clip stack is
      * now clear. gsave stores the clip stack into the saved graphics
@@ -314,10 +315,11 @@ gs_gsave(gs_gstate * pgs)
      * on pgs->clip_stack, but gstate_clone() has an exception for
      * the clip_stack field.
      */
-    pgs->clip_stack = 0;
+    pgs->clip_stack = NULL;
     pgs->saved = pnew;
     if (pgs->show_gstate == pgs)
         pgs->show_gstate = pnew->show_gstate = pnew;
+    pgs->trans_flags.xstate_change = false;
     pgs->level++;
     if_debug2m('g', pgs->memory, "[g]gsave -> "PRI_INTPTR", level = %d\n",
               (intptr_t)pnew, pgs->level);
@@ -461,27 +463,23 @@ gs_grestoreall(gs_gstate * pgs)
 
 /* Allocate and return a new graphics state. */
 gs_gstate *
-gs_gstate_copy(gs_gstate * pgs, gs_memory_t * mem)
+gs_gstate_copy(const gs_gstate * pgs, gs_memory_t * mem)
 {
     gs_gstate *pnew;
-    /* Prevent 'capturing' the view clip path. */
-    gx_clip_path *view_clip = pgs->view_clip;
 
-    pgs->view_clip = 0;
-    pnew = gstate_clone(pgs, mem, "gs_gstate", copy_for_gstate);
-    if (pnew == 0)
-        return 0;
+    pnew = gstate_clone_for_gstate(pgs, mem, "gs_gstate");
+    if (pnew == NULL)
+        return NULL;
     clip_stack_rc_adjust(pnew->clip_stack, 1, "gs_gstate_copy");
-    pgs->view_clip = view_clip;
-    pnew->saved = 0;
+    pnew->saved = NULL;
     /*
      * Prevent dangling references from the show_gstate pointer.  If
      * this context is its own show_gstate, set the pointer in the clone
      * to point to the clone; otherwise, set the pointer in the clone to
-     * 0, and let gs_setgstate fix it up.
+     * NULL, and let gs_setgstate fix it up.
      */
     pnew->show_gstate =
-        (pgs->show_gstate == pgs ? pnew : 0);
+        (pgs->show_gstate == pgs ? pnew : NULL);
     return pnew;
 }
 
@@ -573,7 +571,7 @@ gs_gstate_swap_memory(gs_gstate * pgs, gs_memory_t * mem)
 /*
  * Push an overprint compositor onto the current device. Note that if
  * the current device already is an overprint compositor, the
- * create_compositor will update its parameters but not create a new
+ * composite will update its parameters but not create a new
  * compositor device.
  */
 int
@@ -586,14 +584,14 @@ gs_gstate_update_overprint(gs_gstate * pgs, const gs_overprint_params_t * pparam
 
     code = gs_create_overprint(&pct, pparams, pgs->memory);
     if (code >= 0) {
-        code = dev_proc(dev, create_compositor)( dev,
+        code = dev_proc(dev, composite)( dev,
                                                    &ovptdev,
                                                    pct,
                                                    pgs,
                                                    pgs->memory,
                                                    NULL);
         if (code >= 0 || code == gs_error_handled){
-            if (ovptdev != dev) {
+            if (code == 1) {
                 gx_set_device_only(pgs, ovptdev);
                 /* Get rid of extra reference */
                 rc_decrement(ovptdev, "gs_gstate_update_overprint(ovptdev)");
@@ -641,7 +639,7 @@ gs_do_set_overprint(gs_gstate * pgs)
         gs_color_space_index pcs_index = gs_color_space_get_index(pcs);
 
         dev_proc(dev, get_profile)(dev, &dev_profile);
-        if (!dev_profile->sim_overprint)
+        if (dev_profile->overprint_control == gs_overprint_control_disable)
             return code;
 
         /* Transparency device that supports spots and where we have
@@ -649,25 +647,25 @@ gs_do_set_overprint(gs_gstate * pgs)
            is in a additive blend mode.  This could
            be written more compactly, but it would be unreadable. */
         if (dev_proc(dev, dev_spec_op)(dev, gxdso_pdf14_sep_device, NULL, 0) &&
-            (dev->color_info.polarity != GX_CINFO_POLARITY_SUBTRACTIVE) &&
-            (pcs_index == gs_color_space_index_DeviceN ||
-             pcs_index == gs_color_space_index_Separation)) {
+            (dev->color_info.polarity != GX_CINFO_POLARITY_SUBTRACTIVE)) {
             if (pcs_index == gs_color_space_index_Separation) {
                 if (!(pcs->params.separation.color_type == SEP_MIX ||
                       pcs->params.separation.color_type == SEP_ENUM)) {
                     /* Sep color is not a spot color.  We can't do OP and trans */
                     return code;
                 }
-            }
-            if (pcs_index == gs_color_space_index_DeviceN) {
+            } else if (pcs_index == gs_color_space_index_DeviceN) {
                 if (pcs->params.device_n.color_type != SEP_PURE_SPOT) {
                     /* DeviceN has process colors  We can't do OP and trans. */
                     return code;
                 }
             }
-
         }
 
+        /* If we have a CIE-based space, use the ICC equivalent space */
+        if (gs_color_space_is_PSCIE(pcs) && pcs->icc_equivalent != NULL)
+            pcs = pcs->icc_equivalent;
+
         /* The spaces that do not allow opm (e.g. ones that are not ICC or DeviceCMYK)
            will blow away any true setting later. But we have to be prepared
            in case this is a CMYK ICC space for example. Hence we set effective mode
@@ -847,6 +845,8 @@ gs_initgraphics(gs_gstate * pgs)
     const gs_gstate gstate_initial = {
             gs_gstate_initial(1.0)
         };
+    gs_matrix m;
+    gs_make_identity(&m);
 
     gs_initmatrix(pgs);
     if ((code = gs_newpath(pgs)) < 0 ||
@@ -980,7 +980,13 @@ gs_initgraphics(gs_gstate * pgs)
     if (code < 0)
         goto exit;
 
-    return 0;
+    code = gs_settextmatrix(pgs, &m);
+    if (code < 0)
+        goto exit;
+
+    code = gs_settextlinematrix(pgs, &m);
+    if (code < 0)
+        goto exit;
 exit:
     return code;
 }
@@ -1210,6 +1216,16 @@ gstate_free_parts(gs_gstate * parts, gs_memory_t * mem, client_name_t cname)
     }
 }
 
+static inline void
+gstate_parts_init_dev_color(gx_device_color *dc)
+{
+    gx_device_color_type dct = dc->type;
+    gs_graphics_type_tag_t gtt = dc->tag;
+    memset(dc, 0x00, sizeof(gx_device_color));
+    dc->type = dct;
+    dc->tag = gtt;
+}
+
 /* Allocate the privately allocated parts of a gstate. */
 static int
 gstate_alloc_parts(gs_gstate * parts, const gs_gstate * shared,
@@ -1254,6 +1270,8 @@ gstate_alloc_parts(gs_gstate * parts, const gs_gstate * shared,
         gstate_free_parts(parts, mem, cname);
         return_error(gs_error_VMerror);
     }
+    gstate_parts_init_dev_color(parts->color[0].dev_color);
+    gstate_parts_init_dev_color(parts->color[1].dev_color);
     return 0;
 }
 
@@ -1269,12 +1287,12 @@ gstate_alloc(gs_memory_t * mem, client_name_t cname, const gs_gstate * pfrom)
     gs_gstate *pgs =
         gs_alloc_struct(mem, gs_gstate, &st_gs_gstate, cname);
 
-    if (pgs == 0)
-        return 0;
+    if (pgs == NULL)
+        return NULL;
     memset(pgs, 0x00, sizeof(gs_gstate));
     if (gstate_alloc_parts(pgs, pfrom, mem, cname) < 0) {
         gs_free_object(mem, pgs, cname);
-        return 0;
+        return NULL;
     }
     pgs->memory = mem;
     return pgs;
@@ -1289,77 +1307,115 @@ gstate_copy_dash(gs_memory_t *mem, gx_dash_params *dash , const gs_gstate * pfro
                       pfrom->line_params.dash.offset, mem);
 }
 
-/* Clone an existing graphics state. */
-/* Return 0 if the allocation fails. */
-/* If reason is for_gsave, the clone refers to the old contents, */
-/* and we switch the old state to refer to the new contents. */
+typedef struct {
+    gs_gstate_parts  parts;
+    gx_dash_params   dash;
+} gs_gstate_clone_data;
+
 static gs_gstate *
-gstate_clone(gs_gstate * pfrom, gs_memory_t * mem, client_name_t cname,
-             gs_gstate_copy_reason_t reason)
+gstate_clone_core(const gs_gstate               *pfrom,
+                        gs_memory_t             *mem,
+                        client_name_t            cname,
+                        gs_gstate_clone_data    *clone_data,
+                        gs_gstate_copy_reason_t  reason)
 {
     gs_gstate *pgs = gstate_alloc(mem, cname, pfrom);
-    gs_gstate_parts parts;
     void *pdata = NULL;
-    gx_dash_params dash;
 
     if (pgs == NULL)
-        return 0;
-    GSTATE_ASSIGN_PARTS(&parts, pgs);
+        return NULL;
     if (pfrom->client_data != NULL) {
         pdata = (*pfrom->client_procs.alloc) (mem);
 
         if (pdata == NULL ||
-         gstate_copy_client_data(pfrom, pdata, pfrom->client_data, reason) < 0
-            )
-            goto fail;
+            gstate_copy_client_data(pfrom, pdata, pfrom->client_data,
+                                    reason) < 0)
+            goto failEarly;
     }
     /* Copy the dash and dash pattern if necessary. */
-    dash = gs_currentlineparams_inline(pfrom)->dash;
-    if (pfrom->line_params.dash.pattern) {
+    clone_data->dash = gs_currentlineparams_inline(pfrom)->dash;
+    if (clone_data->dash.pattern) {
         int code;
 
-        dash.pattern = NULL; /* Ensures a fresh allocation */
-        code = gstate_copy_dash(mem, &dash, pfrom);
+        clone_data->dash.pattern = NULL; /* Ensures a fresh allocation */
+        code = gstate_copy_dash(mem, &clone_data->dash, pfrom);
         if (code < 0)
             goto fail;
     }
+    /* Some records within pgs are allocated. We copy pfrom into pgs
+     * wholesale (to avoid problems with the structure being updated and
+     * us having to keep it in sync), so we copy those allocated regions
+     * out first. The caller of this routine will then put them back
+     * into either pgs or pfrom as appropriate. */
+    GSTATE_ASSIGN_PARTS(&clone_data->parts, pgs);
     *pgs = *pfrom;
     pgs->client_data = pdata;
-    gs_currentlineparams_inline(pgs)->dash = dash;
-    pgs->memory = mem;
 
     gs_gstate_copied(pgs);
     /* Don't do anything to clip_stack. */
 
     rc_increment(pgs->device);
-    *parts.color[0].ccolor    = *pfrom->color[0].ccolor;
-    *parts.color[0].dev_color = *pfrom->color[0].dev_color;
-    *parts.color[1].ccolor    = *pfrom->color[1].ccolor;
-    *parts.color[1].dev_color = *pfrom->color[1].dev_color;
-    if (reason == copy_for_gsave) {
-        float *dfrom = pfrom->line_params.dash.pattern;
-        float *dto = pgs->line_params.dash.pattern;
+    *clone_data->parts.color[0].ccolor    = *pgs->color[0].ccolor;
+    *clone_data->parts.color[0].dev_color = *pgs->color[0].dev_color;
+    *clone_data->parts.color[1].ccolor    = *pgs->color[1].ccolor;
+    *clone_data->parts.color[1].dev_color = *pgs->color[1].dev_color;
+    cs_adjust_counts_icc(pgs, 1);
+    cs_adjust_swappedcounts_icc(pgs, 1);
 
-        GSTATE_ASSIGN_PARTS(pfrom, &parts);
-        pgs->line_params.dash.pattern = dfrom;
-        pfrom->line_params.dash.pattern = dto;
-    } else {
-        GSTATE_ASSIGN_PARTS(pgs, &parts);
-    }
-    gs_swapcolors_quick(pgs);
-    cs_adjust_counts_icc(pgs, 1);
-    gs_swapcolors_quick(pgs);
-    cs_adjust_counts_icc(pgs, 1);
     return pgs;
+
   fail:
+    gs_free_object(mem, clone_data->dash.pattern, cname);
     if (pdata != NULL)
         (*pfrom->client_procs.free) (pdata, mem, pgs);
-    memset(pgs->color, 0, 2*sizeof(gs_gstate_color));
-    gs_free_object(mem, pgs->line_params.dash.pattern, cname);
-    GSTATE_ASSIGN_PARTS(pgs, &parts);
+  failEarly:
     gstate_free_parts(pgs, mem, cname);
     gs_free_object(mem, pgs, cname);
-    return 0;
+
+    return NULL;
+}
+
+
+/* Clone an existing graphics state for use in gsave. The clone refers
+ * to the old contents, and the old state refers to the new contents. */
+/* Return NULL if the allocation fails. */
+static gs_gstate *
+gstate_clone_for_gsave(gs_gstate     *pfrom,
+                       client_name_t  cname)
+{
+    gs_gstate_clone_data clone_data;
+    gs_gstate *pgs = gstate_clone_core(pfrom, pfrom->memory, cname,
+                                       &clone_data, copy_for_gsave);
+
+    if (pgs == NULL)
+        return NULL;
+
+    /* Newly allocated parts go back into pfrom, not pgs! */
+    GSTATE_ASSIGN_PARTS(pfrom, &clone_data.parts);
+    gs_currentlineparams_inline(pfrom)->dash = clone_data.dash;
+
+    return pgs;
+}
+
+/* Clone an existing graphics state. The view_clip is not copied. */
+/* Return NULL if the allocation fails. */
+static gs_gstate *
+gstate_clone_for_gstate(const gs_gstate     *pfrom,
+                              gs_memory_t   *mem,
+                              client_name_t  cname)
+{
+    gs_gstate_clone_data clone_data;
+    gs_gstate *pgs = gstate_clone_core(pfrom, mem, cname, &clone_data,
+                                       copy_for_gstate);
+
+    if (pgs == NULL)
+        return NULL;
+    GSTATE_ASSIGN_PARTS(pgs, &clone_data.parts);
+    pgs->view_clip = NULL;
+    gs_currentlineparams_inline(pgs)->dash = clone_data.dash;
+    pgs->memory = mem;
+
+    return pgs;
 }
 
 /* Adjust reference counters for the whole clip stack */
@@ -1410,10 +1466,8 @@ gstate_free_contents(gs_gstate * pgs)
     if (pgs->client_data != 0)
         (*pgs->client_procs.free) (pgs->client_data, mem, pgs);
     pgs->client_data = 0;
-    gs_swapcolors_quick(pgs);
-    cs_adjust_counts_icc(pgs, -1);
-    gs_swapcolors_quick(pgs);
     cs_adjust_counts_icc(pgs, -1);
+    cs_adjust_swappedcounts_icc(pgs, -1);
     pgs->color[0].color_space = 0;
     pgs->color[1].color_space = 0;
     gs_free_object(mem, pgs->line_params.dash.pattern, cname);
@@ -1445,9 +1499,7 @@ gstate_copy(gs_gstate * pto, const gs_gstate * pfrom,
      * Handle references from contents.
      */
     cs_adjust_counts_icc(pto, -1);
-    gs_swapcolors_quick(pto);
-    cs_adjust_counts_icc(pto, -1);
-    gs_swapcolors_quick(pto);
+    cs_adjust_swappedcounts_icc(pto, -1);
     gx_path_assign_preserve(pto->path, pfrom->path);
     gx_cpath_assign_preserve(pto->clip_path, pfrom->clip_path);
     /*
@@ -1498,9 +1550,7 @@ gstate_copy(gs_gstate * pto, const gs_gstate * pfrom,
     }
     GSTATE_ASSIGN_PARTS(pto, &parts);
     cs_adjust_counts_icc(pto, 1);
-    gs_swapcolors_quick(pto);
-    cs_adjust_counts_icc(pto, 1);
-    gs_swapcolors_quick(pto);
+    cs_adjust_swappedcounts_icc(pto, 1);
     pto->show_gstate =
         (pfrom->show_gstate == pfrom ? pto : 0);
     return 0;
