@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -193,8 +193,16 @@ gs_text_enum_init(gs_text_enum_t *pte, const gs_text_enum_procs_t *procs,
     pte->pgs = pgs;
     pte->orig_font = font;
     pte->pcpath = pcpath;
+    pte->pcpath = gx_cpath_alloc_shared(pcpath, mem, "gs_text_enum_init");
+    /* Assign the following before potentially returning an error
+       so there's enough there to all the cleanup
+     */
     pte->memory = mem;
     pte->procs = procs;
+    if (pte->pcpath == NULL) {
+        code = gs_note_error(gs_error_VMerror);
+        goto done;
+    }
 #ifdef DEBUG
     pte->text_enum_id = gs_next_text_enum_id(font);
 #else
@@ -210,6 +218,7 @@ gs_text_enum_init(gs_text_enum_t *pte, const gs_text_enum_procs_t *procs,
     pte->k_text_release = 0;
     if (code >= 0)
         rc_increment(dev);
+done:
     return code;
 }
 
@@ -358,9 +367,12 @@ gs_text_begin(gs_gstate * pgs, const gs_text_params_t * text,
         cmm_dev_profile_t* dev_profile;
 
         dev_proc(dev, get_profile)(dev, &dev_profile);
-        if ((dev_profile->overprint_control != gs_overprint_control_disable) &&
-            (dev_profile->device_profile[GS_DEFAULT_DEVICE_PROFILE]->data_cs == gsCMYK ||
-             dev_profile->device_profile[GS_DEFAULT_DEVICE_PROFILE]->data_cs == gsNCHANNEL)) {
+        /* Previously, we used to only update overprint here if the devices default
+         * colorspace was CMYK or NCHANNEL. This is at odds with above, where we
+         * always gs_do_set_overprint. This meant that for RGB+Spots devices we
+         * could arrive in overprint_fill_rectangle_hl_color without having set the
+         * op_state to anything, causing an assert. Let's just always update_overprint. */
+        if (dev_profile->overprint_control != gs_overprint_control_disable) {
             if (pgs->text_rendering_mode == 0) {
                 op_params.op_state = OP_STATE_FILL;
                 gs_gstate_update_overprint(pgs, &op_params);
@@ -781,6 +793,8 @@ gs_text_retry(gs_text_enum_t * pte)
 void
 gx_default_text_release(gs_text_enum_t *pte, client_name_t cname)
 {
+    gx_cpath_free((gx_clip_path *)pte->pcpath, "gx_default_text_release");
+    pte->pcpath = NULL;
     rc_decrement_only(pte->dev, cname);
     rc_decrement_only(pte->imaging_dev, cname);
 }
