@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -41,7 +41,6 @@ static dev_proc_copy_alpha_hl_color(clip_copy_alpha_hl_color);
 static dev_proc_fill_mask(clip_fill_mask);
 static dev_proc_strip_tile_rectangle(clip_strip_tile_rectangle);
 static dev_proc_strip_tile_rect_devn(clip_strip_tile_rect_devn);
-static dev_proc_strip_copy_rop(clip_strip_copy_rop);
 static dev_proc_strip_copy_rop2(clip_strip_copy_rop2);
 static dev_proc_get_clipping_box(clip_get_clipping_box);
 static dev_proc_get_bits_rectangle(clip_get_bits_rectangle);
@@ -50,85 +49,79 @@ static dev_proc_transform_pixel_region(clip_transform_pixel_region);
 static dev_proc_fill_stroke_path(clip_fill_stroke_path);
 
 /* The device descriptor. */
+static void
+clipper_initialize_device_procs(gx_device *dev)
+{
+    set_dev_proc(dev, open_device, clip_open);
+    set_dev_proc(dev, get_initial_matrix, gx_forward_get_initial_matrix);
+    set_dev_proc(dev, map_rgb_color, gx_forward_map_rgb_color);
+    set_dev_proc(dev, map_color_rgb, gx_forward_map_color_rgb);
+    set_dev_proc(dev, fill_rectangle, clip_fill_rectangle);
+    set_dev_proc(dev, copy_mono, clip_copy_mono);
+    set_dev_proc(dev, copy_color, clip_copy_color);
+    set_dev_proc(dev, get_params, gx_forward_get_params);
+    set_dev_proc(dev, put_params, gx_forward_put_params);
+    set_dev_proc(dev, map_cmyk_color, gx_forward_map_cmyk_color);
+    set_dev_proc(dev, get_page_device, gx_forward_get_page_device);
+    set_dev_proc(dev, get_alpha_bits, gx_forward_get_alpha_bits);
+    set_dev_proc(dev, copy_alpha, clip_copy_alpha);
+    set_dev_proc(dev, fill_path, clip_fill_path);
+    set_dev_proc(dev, fill_mask, clip_fill_mask);
+    set_dev_proc(dev, strip_tile_rectangle, clip_strip_tile_rectangle);
+    set_dev_proc(dev, get_clipping_box, clip_get_clipping_box);
+    set_dev_proc(dev, get_bits_rectangle, clip_get_bits_rectangle);
+    set_dev_proc(dev, composite, gx_forward_composite);
+    set_dev_proc(dev, get_hardware_params, gx_forward_get_hardware_params);
+    set_dev_proc(dev, get_color_mapping_procs, gx_forward_get_color_mapping_procs);
+    set_dev_proc(dev, get_color_comp_index, gx_forward_get_color_comp_index);
+    set_dev_proc(dev, encode_color, gx_forward_encode_color);
+    set_dev_proc(dev, decode_color, gx_forward_decode_color);
+    set_dev_proc(dev, fill_rectangle_hl_color, clip_fill_rectangle_hl_color);
+    set_dev_proc(dev, include_color_space, gx_forward_include_color_space);
+    set_dev_proc(dev, update_spot_equivalent_colors, gx_forward_update_spot_equivalent_colors);
+    set_dev_proc(dev, ret_devn_params, gx_forward_ret_devn_params);
+    set_dev_proc(dev, fillpage, gx_forward_fillpage);
+    set_dev_proc(dev, dev_spec_op, gx_forward_dev_spec_op);
+    set_dev_proc(dev, copy_planes, clip_copy_planes);
+    set_dev_proc(dev, get_profile, gx_forward_get_profile);
+    set_dev_proc(dev, set_graphics_type_tag, gx_forward_set_graphics_type_tag);
+    set_dev_proc(dev, strip_copy_rop2, clip_strip_copy_rop2);
+    set_dev_proc(dev, strip_tile_rect_devn, clip_strip_tile_rect_devn);
+    set_dev_proc(dev, copy_alpha_hl_color, clip_copy_alpha_hl_color);
+    set_dev_proc(dev, transform_pixel_region, clip_transform_pixel_region);
+    set_dev_proc(dev, fill_stroke_path, clip_fill_stroke_path);
+    /* Ideally the following defaults would be filled in for us, but that
+     * doesn't work at the moment. */
+    set_dev_proc(dev, sync_output, gx_default_sync_output);
+    set_dev_proc(dev, output_page, gx_default_output_page);
+    set_dev_proc(dev, close_device, gx_default_close_device);
+    set_dev_proc(dev, draw_thin_line, gx_default_draw_thin_line);
+    set_dev_proc(dev, stroke_path, gx_default_stroke_path);
+    set_dev_proc(dev, fill_trapezoid, gx_default_fill_trapezoid);
+    set_dev_proc(dev, fill_parallelogram, gx_default_fill_parallelogram);
+    set_dev_proc(dev, fill_triangle, gx_default_fill_triangle);
+    set_dev_proc(dev, draw_thin_line, gx_default_draw_thin_line);
+    set_dev_proc(dev, begin_typed_image, gx_default_begin_typed_image);
+    set_dev_proc(dev, text_begin, gx_default_text_begin);
+    set_dev_proc(dev, fill_linear_color_scanline, gx_default_fill_linear_color_scanline);
+    set_dev_proc(dev, fill_linear_color_trapezoid, gx_default_fill_linear_color_trapezoid);
+    set_dev_proc(dev, fill_linear_color_triangle, gx_default_fill_linear_color_triangle);
+}
+
+void
+gx_device_clip_finalize(const gs_memory_t *cmem, void *vpdev)
+{
+    gx_device_clip *dev = (gx_device_clip *)vpdev;
+    if (dev->rect_list != NULL) {
+        rc_decrement(dev->rect_list, "finalizing clipper device");
+        dev->rect_list = NULL;
+    }
+}
+
 static const gx_device_clip gs_clip_device =
-{std_device_std_body(gx_device_clip, 0, "clipper",
-                     0, 0, 1, 1),
- {clip_open,
-  gx_forward_get_initial_matrix,
-  gx_default_sync_output,
-  gx_default_output_page,
-  gx_default_close_device,
-  gx_forward_map_rgb_color,
-  gx_forward_map_color_rgb,
-  clip_fill_rectangle,
-  gx_default_tile_rectangle,
-  clip_copy_mono,
-  clip_copy_color,
-  gx_default_draw_line,
-  gx_default_get_bits,
-  gx_forward_get_params,
-  gx_forward_put_params,
-  gx_forward_map_cmyk_color,
-  gx_forward_get_xfont_procs,
-  gx_forward_get_xfont_device,
-  gx_forward_map_rgb_alpha_color,
-  gx_forward_get_page_device,
-  gx_forward_get_alpha_bits,
-  clip_copy_alpha,
-  gx_forward_get_band,
-  gx_default_copy_rop,
-  clip_fill_path,
-  gx_default_stroke_path,
-  clip_fill_mask,
-  gx_default_fill_trapezoid,
-  gx_default_fill_parallelogram,
-  gx_default_fill_triangle,
-  gx_default_draw_thin_line,
-  gx_default_begin_image,
-  gx_default_image_data,
-  gx_default_end_image,
-  clip_strip_tile_rectangle,
-  clip_strip_copy_rop,
-  clip_get_clipping_box,
-  gx_default_begin_typed_image,
-  clip_get_bits_rectangle,
-  gx_forward_map_color_rgb_alpha,
-  gx_forward_create_compositor,
-  gx_forward_get_hardware_params,
-  gx_default_text_begin,
-  gx_default_finish_copydevice,
-  NULL,			/* begin_transparency_group */
-  NULL,			/* end_transparency_group */
-  NULL,			/* begin_transparency_mask */
-  NULL,			/* end_transparency_mask */
-  NULL,			/* discard_transparency_layer */
-  gx_forward_get_color_mapping_procs,
-  gx_forward_get_color_comp_index,
-  gx_forward_encode_color,
-  gx_forward_decode_color,
-  NULL,
-  clip_fill_rectangle_hl_color,
-  gx_forward_include_color_space,
-  gx_default_fill_linear_color_scanline,
-  gx_default_fill_linear_color_trapezoid,
-  gx_default_fill_linear_color_triangle,
-  gx_forward_update_spot_equivalent_colors,
-  gx_forward_ret_devn_params,
-  gx_forward_fillpage,
-  NULL,                      /* push_transparency_state */
-  NULL,                      /* pop_transparency_state */
-  NULL,                      /* put_image */
-  gx_forward_dev_spec_op,
-  clip_copy_planes,          /* copy planes */
-  gx_forward_get_profile,
-  gx_forward_set_graphics_type_tag,
-  clip_strip_copy_rop2,
-  clip_strip_tile_rect_devn,
-  clip_copy_alpha_hl_color,
-  NULL,						/* process_page */
-  clip_transform_pixel_region,
-  clip_fill_stroke_path,
- }
+{std_device_std_body(gx_device_clip,
+                     clipper_initialize_device_procs, "clipper",
+                     0, 0, 1, 1)
 };
 
 /* Make a clipping device. */
@@ -138,6 +131,12 @@ gx_make_clip_device_on_stack(gx_device_clip * dev, const gx_clip_path *pcpath, g
     gx_device_init_on_stack((gx_device *)dev, (const gx_device *)&gs_clip_device, target->memory);
     dev->cpath = pcpath;
     dev->list = *gx_cpath_list(pcpath);
+    /* NOTE we do not count up the rect list even though we've taken a reference to it.
+     * this is because we would then need to count it down in gx_destroy_clip_device_on_stack
+     * and I have found at least one place where we do not call that function (!)
+     * We should be safe though, the clip rectangle list should not disappear before we
+     * exit the calling function at which point this device will disappear too.
+     */
     dev->translation.x = 0;
     dev->translation.y = 0;
     dev->HWResolution[0] = target->HWResolution[0];
@@ -146,8 +145,9 @@ gx_make_clip_device_on_stack(gx_device_clip * dev, const gx_clip_path *pcpath, g
     dev->target = target;
     dev->pad = target->pad;
     dev->log2_align_mod = target->log2_align_mod;
-    dev->is_planar = target->is_planar;
+    dev->num_planar_planes = target->num_planar_planes;
     dev->graphics_type_tag = target->graphics_type_tag;	/* initialize to same as target */
+    dev->non_strict_bounds = target->non_strict_bounds;
     /* There is no finalization for device on stack so no rc increment */
     (*dev_proc(dev, open_device)) ((gx_device *)dev);
 }
@@ -190,18 +190,30 @@ gx_make_clip_device_on_stack_if_needed(gx_device_clip * dev, const gx_clip_path
     dev->target = target;
     dev->pad = target->pad;
     dev->log2_align_mod = target->log2_align_mod;
-    dev->is_planar = target->is_planar;
+    dev->num_planar_planes = target->num_planar_planes;
     dev->graphics_type_tag = target->graphics_type_tag;	/* initialize to same as target */
+    dev->non_strict_bounds = target->non_strict_bounds;
     /* There is no finalization for device on stack so no rc increment */
     (*dev_proc(dev, open_device)) ((gx_device *)dev);
     return (gx_device *)dev;
 }
 void
-gx_make_clip_device_in_heap(gx_device_clip * dev, const gx_clip_path *pcpath, gx_device *target,
-                              gs_memory_t *mem)
+gx_make_clip_device_in_heap(gx_device_clip *dev,
+                      const gx_clip_path   *pcpath,
+                            gx_device      *target,
+                            gs_memory_t    *mem)
 {
-    gx_device_init((gx_device *)dev, (const gx_device *)&gs_clip_device, mem, true);
+    /* Can never fail */
+    (void)gx_device_init((gx_device *)dev,
+                         (const gx_device *)&gs_clip_device, mem, true);
     dev->list = *gx_cpath_list(pcpath);
+    dev->rect_list = pcpath->rect_list;
+    /* Bug #706771 we must make sure that the clip rectangle list does not
+     * vanish while we still have a pointer to it. Do that by increasing the
+     * reference count (obviously). We will decrement it in the device's finalize
+     * routine.
+     */
+    rc_increment(dev->rect_list);
     dev->translation.x = 0;
     dev->translation.y = 0;
     dev->HWResolution[0] = target->HWResolution[0];
@@ -209,13 +221,17 @@ gx_make_clip_device_in_heap(gx_device_clip * dev, const gx_clip_path *pcpath, gx
     dev->sgr = target->sgr;
     dev->pad = target->pad;
     dev->log2_align_mod = target->log2_align_mod;
-    dev->is_planar = target->is_planar;
+    dev->num_planar_planes = target->num_planar_planes;
+    dev->non_strict_bounds = target->non_strict_bounds;
     gx_device_set_target((gx_device_forward *)dev, target);
     gx_device_retain((gx_device *)dev, true); /* will free explicitly */
-    (*dev_proc(dev, open_device)) ((gx_device *)dev);
+    /* Can never fail */
+    (void)(*dev_proc(dev, open_device)) ((gx_device *)dev);
 }
 /* Define debugging statistics for the clipping loops. */
-#if defined(DEBUG) && !defined(GS_THREADSAFE)
+/* #define COLLECT_STATS_CLIP */
+
+#ifdef COLLECT_STATS_CLIP
 struct stats_clip_s {
     long
          loops, out, in_y, in, in1, down, up, x, no_x;
@@ -247,7 +263,7 @@ clip_enumerate_rest(gx_device_clip * rdev,
     int yc;
     int code;
 
-#if defined(DEBUG) && !defined(GS_THREADSAFE)
+#ifdef COLLECT_STATS_CLIP
     if (INCR(loops) % clip_interval == 0 && gs_debug_c('q')) {
         dmprintf5(rdev->memory,
                   "[q]loops=%ld out=%ld in_y=%ld in=%ld in1=%ld\n",
@@ -272,6 +288,11 @@ clip_enumerate_rest(gx_device_clip * rdev,
      * the list.
      */
     if (y >= rptr->ymax) {
+        /* Bug 706875: The 'stopper' here is a rectangle from (max_int, max_int) to
+         * (max_int, max_int). Hence it doesn't 'stop' cases when y == max_int.
+         * These shouldn't really happen, but let's be sure. */
+        if (y == max_int)
+            return 0;
         if ((rptr = rptr->next) != 0)
             while (INCR_THEN(up, y >= rptr->ymax))
                 rptr = rptr->next;
@@ -590,7 +611,7 @@ clip_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 }
 
 int
-clip_call_fill_rectangle_hl_color(clip_callback_data_t * pccd, int xc, int yc, 
+clip_call_fill_rectangle_hl_color(clip_callback_data_t * pccd, int xc, int yc,
                                   int xec, int yec)
 {
     gs_fixed_rect rect;
@@ -925,13 +946,15 @@ clip_copy_mono_t1(gx_device * dev,
         INCR(in_y);
         if (x >= rptr->xmin && xe <= rptr->xmax) {
             INCR(in);
+            /* Untranspose coords here. */
             return dev_proc(tdev, copy_mono)
-                    (tdev, data, sourcex, raster, id, y, x, h, w, color0, color1);
+                    (tdev, data, sourcex, raster, id, y, x, w, h, color0, color1);
         }
     }
     ccdata.tdev = tdev;
     ccdata.data = data, ccdata.sourcex = sourcex, ccdata.raster = raster;
     ccdata.color[0] = color0, ccdata.color[1] = color1;
+    /* Coords are passed in transposed here, but will appear untransposed at the end. */
     return clip_enumerate_rest(rdev, x, y, xe, ye,
                                clip_call_copy_mono, &ccdata);
 }
@@ -1245,7 +1268,7 @@ clip_copy_alpha(gx_device * dev,
 }
 
 int
-clip_call_copy_alpha_hl_color(clip_callback_data_t * pccd, int xc, int yc, 
+clip_call_copy_alpha_hl_color(clip_callback_data_t * pccd, int xc, int yc,
                               int xec, int yec)
 {
     return (*dev_proc(pccd->tdev, copy_alpha_hl_color))
@@ -1308,8 +1331,8 @@ clip_call_strip_tile_rect_devn(clip_callback_data_t * pccd, int xc, int yc, int
 static int
 clip_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tiles,
                                 int x, int y, int w, int h,
-                                const gx_drawing_color *pdcolor0, 
-                                const gx_drawing_color *pdcolor1, int phase_x, 
+                                const gx_drawing_color *pdcolor0,
+                                const gx_drawing_color *pdcolor1, int phase_x,
                                 int phase_y)
 {
     gx_device_clip *rdev = (gx_device_clip *) dev;
@@ -1344,35 +1367,6 @@ clip_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
     return clip_enumerate(rdev, x, y, w, h, clip_call_strip_tile_rectangle, &ccdata);
 }
 
-/* Copy a rectangle with RasterOp and strip texture. */
-int
-clip_call_strip_copy_rop(clip_callback_data_t * pccd, int xc, int yc, int xec, int yec)
-{
-    return (*dev_proc(pccd->tdev, strip_copy_rop))
-        (pccd->tdev, pccd->data + (yc - pccd->y) * pccd->raster,
-         pccd->sourcex + xc - pccd->x, pccd->raster, gx_no_bitmap_id,
-         pccd->scolors, pccd->textures, pccd->tcolors,
-         xc, yc, xec - xc, yec - yc, pccd->phase.x, pccd->phase.y,
-         pccd->lop);
-}
-static int
-clip_strip_copy_rop(gx_device * dev,
-              const byte * sdata, int sourcex, uint raster, gx_bitmap_id id,
-                    const gx_color_index * scolors,
-           const gx_strip_bitmap * textures, const gx_color_index * tcolors,
-                    int x, int y, int w, int h,
-                    int phase_x, int phase_y, gs_logical_operation_t lop)
-{
-    gx_device_clip *rdev = (gx_device_clip *) dev;
-    clip_callback_data_t ccdata;
-
-    ccdata.data = sdata, ccdata.sourcex = sourcex, ccdata.raster = raster;
-    ccdata.scolors = scolors, ccdata.textures = textures,
-        ccdata.tcolors = tcolors;
-    ccdata.phase.x = phase_x, ccdata.phase.y = phase_y, ccdata.lop = lop;
-    return clip_enumerate(rdev, x, y, w, h, clip_call_strip_copy_rop, &ccdata);
-}
-
 /* Copy a rectangle with RasterOp and strip texture. */
 int
 clip_call_strip_copy_rop2(clip_callback_data_t * pccd, int xc, int yc, int xec, int yec)
@@ -1427,6 +1421,8 @@ clip_get_clipping_box(gx_device * dev, gs_fixed_rect * pbox)
                 /* The head and tail elements are dummies.... */
                 gx_clip_rect *curr = rdev->list.head->next;
 
+                /* Our initial scan is done as ints, but we'll convert to
+                 * fixed later. */
                 cbox.p.x = cbox.p.y = max_int;
                 cbox.q.x = cbox.q.y = min_int;
                 /* scan the list for the outer bbox */
@@ -1441,10 +1437,13 @@ clip_get_clipping_box(gx_device * dev, gs_fixed_rect * pbox)
                         cbox.q.y = curr->ymax;
                     curr = curr->next;
                 }
-                cbox.p.x = int2fixed(cbox.p.x);
-                cbox.q.x = int2fixed(cbox.q.x);
-                cbox.p.y = int2fixed(cbox.p.y);
-                cbox.q.y = int2fixed(cbox.q.y);
+                /* Clamp the values so they won't overflow when converting to fixed. */
+#define SAFE_CONVERT2FIXED(x)  if (x < fixed2int(min_int)) x = fixed2int(min_int); else if (x > fixed2int(max_int)) x = fixed2int(max_int); else x = int2fixed(x);
+                SAFE_CONVERT2FIXED(cbox.p.x);
+                SAFE_CONVERT2FIXED(cbox.p.y);
+                SAFE_CONVERT2FIXED(cbox.q.x);
+                SAFE_CONVERT2FIXED(cbox.q.y);
+#undef SAFE_CONVERT2FIXED
             }
             if (rdev->list.transpose) {
                 fixed temp = cbox.p.x;
@@ -1479,75 +1478,138 @@ clip_get_clipping_box(gx_device * dev, gs_fixed_rect * pbox)
 /* Get bits back from the device. */
 static int
 clip_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
-                        gs_get_bits_params_t * params, gs_int_rect ** unread)
+                        gs_get_bits_params_t * params)
 {
     gx_device_clip *rdev = (gx_device_clip *) dev;
     gx_device *tdev = rdev->target;
     int tx = rdev->translation.x, ty = rdev->translation.y;
     gs_int_rect rect;
-    int code;
 
     rect.p.x = prect->p.x - tx, rect.p.y = prect->p.y - ty;
     rect.q.x = prect->q.x - tx, rect.q.y = prect->q.y - ty;
-    code = (*dev_proc(tdev, get_bits_rectangle))
-        (tdev, &rect, params, unread);
-    if (code > 0) {
-        /* Adjust unread rectangle coordinates */
-        gs_int_rect *list = *unread;
-        int i;
+    return (*dev_proc(tdev, get_bits_rectangle))
+                     (tdev, &rect, params);
+}
 
-        for (i = 0; i < code; ++list, ++i) {
-            list->p.x += tx, list->p.y += ty;
-            list->q.x += tx, list->q.y += ty;
+static int clip_list_enumerate_intersections(gx_clip_list *list, int (*process)(clip_callback_data_t *, int, int, int, int), clip_callback_data_t *pccd, int x, int y, int xe, int ye)
+{
+    int transpose = list->transpose;
+    int code = 0;
+    int yc;
+    /* Start at the last place we succeeded, to try to exploit
+     * locality of reference. */
+    gx_clip_rect *rptr = pccd->last_clip_rect;
+
+    /* If this is the first time through, start at the head. */
+    if (rptr == NULL)
+        rptr = (list->head == NULL ? &list->single : list->head);
+
+    /*
+     * Warp the cursor forwards or backward to the first rectangle row
+     * that could include a given y value.  Assumes rptr is set, and
+     * updates it.  Specifically, after this loop, either rptr == 0 (if
+     * the y value is greater than all y values in the list), or y <
+     * rptr->ymax and either rptr->prev == 0 or y >= rptr->prev->ymax.
+     * Note that y <= rptr->ymin is possible.
+     *
+     * In the first case below, the while loop is safe because if there
+     * is more than one rectangle, there is a 'stopper' at the end of
+     * the list.
+     */
+    if (y >= rptr->ymax) {
+        /* Bug 706875: The 'stopper' here is a rectangle from (max_int, max_int) to
+         * (max_int, max_int). Hence it doesn't 'stop' cases when y == max_int.
+         * These shouldn't really happen, but let's be sure. */
+        if (y == max_int)
+            return 0;
+        if ((rptr = rptr->next) != NULL)
+            while (y >= rptr->ymax)
+                rptr = rptr->next;
+    } else
+        while (rptr->prev != NULL && y < rptr->prev->ymax)
+            rptr = rptr->prev;
+    /* If we've run out, bale! */
+    if (rptr == NULL || (yc = rptr->ymin) >= ye)
+        return 0;
+    if (yc < y)
+        yc = y;
+
+    do {
+        const int ymax = rptr->ymax;
+        int yec = min(ymax, ye);
+
+        do {
+            int xc = rptr->xmin;
+            int xec = rptr->xmax;
+
+            if (xc < x)
+                xc = x;
+            if (xec > xe)
+                xec = xe;
+            if (xec > xc) {
+                if (transpose)
+                    code = process(pccd, yc, xc, yec, xec);
+                else
+                    code = process(pccd, xc, yc, xec, yec);
+                if (code < 0)
+                    return code;
+            }
+            pccd->last_clip_rect = rptr;
+            rptr = rptr->next;
+            if (rptr == NULL)
+                return 0;
         }
-    }
-    return code;
+        while (rptr->ymax == ymax);
+    } while ((yc = rptr->ymin) < ye);
+    return 0;
 }
 
 static int
-clip_call_fill_path(clip_callback_data_t * pccd, int xc, int yc, int xec, int yec)
+do_clip_call_fill_path(clip_callback_data_t *pccd, int xc, int yc, int xec, int yec)
 {
     gx_device *tdev = pccd->tdev;
-    dev_proc_fill_path((*proc));
+    gx_clip_path cpath;
+    gs_fixed_rect rect;
     int code;
-    gx_clip_path cpath_intersection;
-    gx_clip_path *pcpath = (gx_clip_path *)pccd->pcpath;
+    dev_proc_fill_path((*proc));
 
-    /* Previously the code here tested for pcpath != NULL, but
-     * we can commonly (such as from clist_playback_band) be
-     * called with a non-NULL, but still invalid clip path.
-     * Detect this by the list having at least one entry in it. */
-    if (pcpath != NULL && pcpath->rect_list->list.count != 0) {
-        gx_path rect_path;
-        code = gx_cpath_init_local_shared_nested(&cpath_intersection, pcpath, pccd->ppath->memory, 1);
-        if (code < 0)
-            return code;
-        gx_path_init_local(&rect_path, pccd->ppath->memory);
-        code = gx_path_add_rectangle(&rect_path, int2fixed(xc), int2fixed(yc), int2fixed(xec), int2fixed(yec));
-        if (code < 0)
-            return code;
-        code = gx_cpath_intersect(&cpath_intersection, &rect_path,
-                                  gx_rule_winding_number, (gs_gstate *)(pccd->pgs));
-        gx_path_free(&rect_path, "clip_call_fill_path");
-    } else {
-        gs_fixed_rect clip_box;
-        clip_box.p.x = int2fixed(xc);
-        clip_box.p.y = int2fixed(yc);
-        clip_box.q.x = int2fixed(xec);
-        clip_box.q.y = int2fixed(yec);
-        gx_cpath_init_local(&cpath_intersection, pccd->ppath->memory);
-        code = gx_cpath_from_rectangle(&cpath_intersection, &clip_box);
-    }
+    rect.p.x = int2fixed(xc);
+    rect.p.y = int2fixed(yc);
+    rect.q.x = int2fixed(xec);
+    rect.q.y = int2fixed(yec);
+
+    /* The cpath calls might look expensive, but actually are
+     * achieved without any allocations. */
+    gx_cpath_init_local(&cpath, pccd->ppath->memory);
+    code = gx_cpath_from_rectangle(&cpath, &rect);
     if (code < 0)
         return code;
     proc = dev_proc(tdev, fill_path);
     if (proc == NULL)
         proc = gx_default_fill_path;
     code = (*proc)(pccd->tdev, pccd->pgs, pccd->ppath, pccd->params,
-                   pccd->pdcolor, &cpath_intersection);
-    gx_cpath_free(&cpath_intersection, "clip_call_fill_path");
+                   pccd->pdcolor, &cpath);
+    /* We could call gx_cpath_free here, but actually everything
+     * is allocated on the stack, so it serves no purpose. */
+    /* gx_cpath_free(&cpath, "clip_call_fill_path"); */
+
     return code;
 }
+
+static int
+clip_call_fill_path(clip_callback_data_t * pccd, int xc, int yc, int xec, int yec)
+{
+    gx_clip_path *pcpath = (gx_clip_path *)pccd->pcpath;
+
+    /* Previously the code here tested for pcpath != NULL, but
+     * we can commonly (such as from clist_playback_band) be
+     * called with a non-NULL, but still invalid clip path.
+     * Detect this by the list having at least one entry in it. */
+    if (pcpath != NULL && pcpath->rect_list->list.count != 0)
+        return clip_list_enumerate_intersections(&pcpath->rect_list->list, do_clip_call_fill_path, pccd,  xc, yc, xec, yec);
+
+    return do_clip_call_fill_path(pccd, xc, yc, xec, yec);
+}
 static int
 clip_fill_path(gx_device * dev, const gs_gstate * pgs,
                gx_path * ppath, const gx_fill_params * params,
@@ -1563,6 +1625,7 @@ clip_fill_path(gx_device * dev, const gs_gstate * pgs,
     ccdata.params = params;
     ccdata.pdcolor = pdcolor;
     ccdata.pcpath = pcpath;
+    ccdata.last_clip_rect = NULL;
     clip_get_clipping_box(dev, &box);
     return clip_enumerate(rdev,
                           fixed2int(box.p.x),
