@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -960,7 +960,7 @@ static void
 cmap_gray_direct(frac gray, gx_device_color * pdc, const gs_gstate * pgs,
                  gx_device * dev, gs_color_select_t select)
 {
-    uchar i, ncomps = dev->color_info.num_components;
+    uchar i, nc, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_index color;
@@ -971,12 +971,15 @@ cmap_gray_direct(frac gray, gx_device_color * pdc, const gs_gstate * pgs,
     cmprocs = dev_proc(dev, get_color_mapping_procs)(dev, &cmdev);
     cmprocs->map_gray(cmdev, gray, cm_comps);
 
+    nc = ncomps;
+    if (device_encodes_tags(dev))
+        nc--;
     /* apply the transfer function(s); convert to color values */
     if (pgs->effective_transfer_non_identity_count == 0) {
-        for (i = 0; i < ncomps; i++)
+        for (i = 0; i < nc; i++)
             cv[i] = frac2cv(cm_comps[i]);
     } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-        for (i = 0; i < ncomps; i++) {
+        for (i = 0; i < nc; i++) {
             cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
             cv[i] = frac2cv(cm_comps[i]);
@@ -987,16 +990,20 @@ cmap_gray_direct(frac gray, gx_device_color * pdc, const gs_gstate * pgs,
             if (i < ncomps)
                 cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
-            for (i = 0; i < ncomps; i++)
+            for (i = 0; i < nc; i++)
                 cv[i] = frac2cv(cm_comps[i]);
         } else {
-            for (i = 0; i < ncomps; i++) {
+            for (i = 0; i < nc; i++) {
                 cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                             (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
                 cv[i] = frac2cv(cm_comps[i]);
             }
         }
     }
+    /* Copy tags untransformed. */
+    if (nc < ncomps)
+        cv[nc] = cm_comps[nc];
+
     /* encode as a color index */
     color = dev_proc(dev, encode_color)(dev, cv);
 
@@ -1016,7 +1023,7 @@ static void
 cmap_rgb_halftoned(frac r, frac g, frac b, gx_device_color * pdc,
      const gs_gstate * pgs, gx_device * dev, gs_color_select_t select)
 {
-    uchar i, ncomps = dev->color_info.num_components;
+    uchar i, nc, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
     const gx_device *cmdev;
     const gx_cm_color_map_procs *cmprocs;
@@ -1025,15 +1032,20 @@ cmap_rgb_halftoned(frac r, frac g, frac b, gx_device_color * pdc,
     cmprocs = dev_proc(dev, get_color_mapping_procs)(dev, &cmdev);
     cmprocs->map_rgb(cmdev, pgs, r, g, b, cm_comps);
 
+    nc = ncomps;
+    if (device_encodes_tags(dev))
+        nc--;
     /* apply the transfer function(s); convert to color values */
     if (pgs->effective_transfer_non_identity_count != 0) {
+        int n = 0;
         if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-            for (i = 0; i < ncomps; i++)
-                cm_comps[i] = gx_map_color_frac(pgs,
+            n = nc < 3 ? nc : 3;
+
+        for (i = 0; i < n; i++)
+            cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-        else
-            for (i = 0; i < ncomps; i++)
-                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+        for (; i < n; i++)
+            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
     }
 
@@ -1046,7 +1058,7 @@ static void
 cmap_rgb_direct(frac r, frac g, frac b, gx_device_color * pdc,
      const gs_gstate * pgs, gx_device * dev, gs_color_select_t select)
 {
-    uchar i, ncomps = dev->color_info.num_components;
+    uchar i, nc, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_index color;
@@ -1057,34 +1069,45 @@ cmap_rgb_direct(frac r, frac g, frac b, gx_device_color * pdc,
     cmprocs = dev_proc(dev, get_color_mapping_procs)(dev, &cmdev);
     cmprocs->map_rgb(cmdev, pgs, r, g, b, cm_comps);
 
+    nc = ncomps;
+    if (device_encodes_tags(dev))
+        nc--;
     /* apply the transfer function(s); convert to color values */
-    if (pgs->effective_transfer_non_identity_count == 0) {
-        for (i = 0; i < ncomps; i++)
-            cv[i] = frac2cv(cm_comps[i]);
-    } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-        for (i = 0; i < ncomps; i++) {
-            cm_comps[i] = gx_map_color_frac(pgs,
-                                cm_comps[i], effective_transfer[i]);
-            cv[i] = frac2cv(cm_comps[i]);
-        }
-    else
-        for (i = 0; i < ncomps; i++) {
+    if (pgs->effective_transfer_non_identity_count != 0) {
+        int n = 0;
+        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+            n = nc < 3 ? nc : 3;
+        for (i = 0; i < n; i++)
+            cm_comps[i] = gx_map_color_frac(pgs, cm_comps[i],
+                                            effective_transfer[i]);
+        for (; i < nc; i++)
             cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
-            cv[i] = frac2cv(cm_comps[i]);
-        }
-
-    /* encode as a color index */
-    color = dev_proc(dev, encode_color)(dev, cv);
-
-    /* check if the encoding was successful; we presume failure is rare */
-    if (color != gx_no_color_index) {
-        color_set_pure(pdc, color);
-        return;
     }
-    if (gx_render_device_DeviceN(cm_comps, pdc, dev, gx_select_dev_ht(pgs),
+
+    if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0)) {
+        for (i = 0; i < nc; i++)
+            pdc->colors.devn.values[i] = frac2cv(cm_comps[i]);
+        if (i < ncomps)
+            pdc->colors.devn.values[i] = cm_comps[i];
+        pdc->type = gx_dc_type_devn;
+    } else {
+        for (i = 0; i < nc; i++)
+            cv[i] = frac2cv(cm_comps[i]);
+        if (i < ncomps)
+            cv[i] = cm_comps[i];
+        /* encode as a color index */
+        color = dev_proc(dev, encode_color)(dev, cv);
+
+        /* check if the encoding was successful; we presume failure is rare */
+        if (color != gx_no_color_index) {
+            color_set_pure(pdc, color);
+            return;
+        }
+        if (gx_render_device_DeviceN(cm_comps, pdc, dev, gx_select_dev_ht(pgs),
                                         &pgs->screen_phase[select]) == 1)
-        gx_color_load_select(pdc, pgs, dev, select);
+            gx_color_load_select(pdc, pgs, dev, select);
+    }
 }
 
 /* ------ Render CMYK color. ------ */
@@ -1094,7 +1117,7 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
      const gs_gstate * pgs, gx_device * dev, gs_color_select_t select,
      const gs_color_space *source_pcs)
 {
-    uchar i, ncomps = dev->color_info.num_components;
+    uchar i, nc, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_index color;
@@ -1148,13 +1171,20 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
     }
     /* if output device supports devn, we need to make sure we send it the
        proper color type */
+    nc = ncomps;
+    if (device_encodes_tags(dev))
+        nc--;
     if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0)) {
-        for (i = 0; i < ncomps; i++)
+        for (i = 0; i < nc; i++)
             pdc->colors.devn.values[i] = frac2cv(cm_comps[i]);
+        if (i < ncomps)
+            pdc->colors.devn.values[i] = cm_comps[i];
         pdc->type = gx_dc_type_devn;
     } else {
-        for (i = 0; i < ncomps; i++)
+        for (i = 0; i < nc; i++)
             cv[i] = frac2cv(cm_comps[i]);
+        if (i < ncomps)
+            cv[i] = cm_comps[i];
         color = dev_proc(dev, encode_color)(dev, cv);
         if (color != gx_no_color_index)
             color_set_pure(pdc, color);
@@ -1379,13 +1409,14 @@ cmap_separation_halftoned(frac all, gx_device_color * pdc,
 
     /* apply the transfer function(s); convert to color values */
     if (pgs->effective_transfer_non_identity_count != 0) {
+        int n = 0;
         if (additive)
-            for (i = 0; i < ncomps; i++)
-                cm_comps[i] = gx_map_color_frac(pgs,
+            n = ncomps < 3 ? ncomps : 3;
+        for (i = 0; i < n; i++)
+            cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-        else
-            for (i = 0; i < ncomps; i++)
-                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+        for (; i < ncomps; i++)
+            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
     }
 
@@ -1398,7 +1429,7 @@ static void
 cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
                  gx_device * dev, gs_color_select_t select, const gs_color_space *pcs)
 {
-    uint i, ncomps = dev->color_info.num_components;
+    uint i, nc, ncomps = dev->color_info.num_components;
     bool additive = dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE;
     frac comp_value = all;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
@@ -1408,6 +1439,7 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
     gsicc_rendering_param_t render_cond;
     cmm_dev_profile_t *dev_profile = NULL;
     cmm_profile_t *des_profile = NULL;
+    int num_additives = additive ? dev_proc(dev, dev_spec_op)(dev, gxdso_is_sep_supporting_additive_device, NULL, 0) : 0;
 
     dev_proc(dev, get_profile)(dev,  &dev_profile);
     gsicc_extract_profile(dev->graphics_type_tag,
@@ -1417,8 +1449,9 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
          * Invert the photometric interpretation for additive
          * color spaces because separations are always subtractive.
          */
-        if (additive)
+        if (additive && num_additives <= 0) {
             comp_value = frac_1 - comp_value;
+        }
 
         /* Use the "all" value for all components */
         for (i = 0; i < pgs->color_component_map.num_colorants; i++)
@@ -1446,7 +1479,7 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
     /* Check if we have the standard colorants.  If yes, then we will apply
       ICC color management to those colorants. */
     if (devicen_has_cmyk(dev, des_profile) && des_profile->data_cs == gsCMYK &&
-        !named_color_supported(pgs)) {
+        !named_color_supported(pgs) && pgs->color_component_map.sep_type != SEP_ALL) {
         /* We need to do a CMYK to CMYK conversion here.  This will always
            use the default CMYK profile and the device's output profile.
            We probably need to add some checking here
@@ -1458,21 +1491,33 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
     }
 
     /* apply the transfer function(s); convert to color values */
-    if (pgs->effective_transfer_non_identity_count == 0)
-        for (i = 0; i < ncomps; i++)
-            cv[i] = frac2cv(cm_comps[i]);
-    else if (additive)
-        for (i = 0; i < ncomps; i++) {
+    nc = ncomps;
+    if (device_encodes_tags(dev))
+        nc--;
+    if (pgs->effective_transfer_non_identity_count != 0) {
+        int n = 0;
+        if (additive)
+            n = nc < 3 ? nc : 3;
+        for (i = 0; i < n; i++)
             cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-            cv[i] = frac2cv(cm_comps[i]);
-        }
-    else
-        for (i = 0; i < ncomps; i++) {
+        for (; i < nc; i++)
             cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
-            cv[i] = frac2cv(cm_comps[i]);
-        }
+    }
+    for (i = 0; i < nc; i++)
+        cv[i] = frac2cv(cm_comps[i]);
+    /* For additive devices, we should invert the process colors
+     * here! But how do we know how many process colors we have? For
+     * now we'll have to ask the device using a dso. */
+    if (additive) {
+        int j;
+        for (j = 0; j < num_additives; j++)
+            cv[j] = 65535 - cv[j];
+    }
+    /* Copy tags untransformed. */
+    if (nc < ncomps)
+        cv[nc] = cm_comps[nc];
 
     if (use_rgb2dev_icc && pgs->icc_manager->default_rgb != NULL) {
         /* After the transfer function go ahead and do the mapping from RGB to
@@ -1507,6 +1552,16 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
         for (i = 0; i < ncomps; i++)
             pdc->colors.devn.values[i] = cv[i];
         pdc->type = gx_dc_type_devn;
+
+        /* Let device set the tags if present */
+        if (device_encodes_tags(dev)) {
+            const gx_device *cmdev;
+            const gx_cm_color_map_procs *cmprocs;
+
+            cmprocs = dev_proc(dev, get_color_mapping_procs)(dev, &cmdev);
+            cmprocs->map_cmyk(cmdev, 0, 0, 0, 0, cm_comps);
+            pdc->colors.devn.values[ncomps - 1] = cm_comps[ncomps - 1];
+        }
         return;
     }
 
@@ -1560,13 +1615,14 @@ cmap_devicen_halftoned(const frac * pcc,
     }
     /* apply the transfer function(s); convert to color values */
     if (pgs->effective_transfer_non_identity_count != 0) {
+        int n = 0;
         if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-            for (i = 0; i < ncomps; i++)
-                cm_comps[i] = gx_map_color_frac(pgs,
+            n = ncomps < 3 ? ncomps : 3;
+        for (i = 0; i < n; i++)
+            cm_comps[i] = gx_map_color_frac(pgs,
                                 cm_comps[i], effective_transfer[i]);
-        else
-            for (i = 0; i < ncomps; i++)
-                cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
+        for (; i < ncomps; i++)
+            cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                         (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
     }
 
@@ -1576,6 +1632,19 @@ cmap_devicen_halftoned(const frac * pcc,
         gx_color_load_select(pdc, pgs, dev, select);
 }
 
+static void
+encode_tags(const gx_device *dev, gx_device_color *pdc)
+{
+    const gx_device *cmdev;
+    const gx_cm_color_map_procs *cmprocs;
+    uchar ncomps = dev->color_info.num_components;
+    frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
+
+    cmprocs = dev_proc(dev, get_color_mapping_procs)(dev, &cmdev);
+    cmprocs->map_cmyk(cmdev, 0, 0, 0, 0, cm_comps);
+    pdc->colors.devn.values[ncomps - 1] = cm_comps[ncomps - 1];
+}
+
 /*
  * This routine is called to map a DeviceN colorspace to a DeviceN
  * output device which does not require halftoning.
@@ -1585,13 +1654,14 @@ cmap_devicen_direct(const frac * pcc,
     gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
     gs_color_select_t select, const gs_color_space *pcs)
 {
-    uchar i, ncomps = dev->color_info.num_components;
+    uchar i, nc, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
     gx_color_index color;
     gsicc_rendering_param_t render_cond;
     cmm_dev_profile_t *dev_profile = NULL;
     cmm_profile_t *des_profile = NULL;
+    int additive = dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE;
 
     if (pcs->params.device_n.all_none == true) {
         color_set_null(pdc);
@@ -1632,39 +1702,70 @@ cmap_devicen_direct(const frac * pcc,
            through the named color mapping.  */
         devicen_sep_icc_cmyk(cm_comps, pgs, pcs, dev);
     }
+    nc = ncomps;
+    if (device_encodes_tags(dev))
+        nc--;
     /* apply the transfer function(s); convert to color values.
        assign directly if output device supports devn */
     if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0)) {
         if (pgs->effective_transfer_non_identity_count == 0)
-            for (i = 0; i < ncomps; i++)
+            for (i = 0; i < nc; i++)
                 pdc->colors.devn.values[i] = frac2cv(cm_comps[i]);
-        else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-            for (i = 0; i < ncomps; i++)
+        else {
+            int n = 0;
+            if (additive)
+                n = nc < 3 ? nc : 3;
+            for (i = 0; i < n; i++)
                 pdc->colors.devn.values[i] = frac2cv(gx_map_color_frac(pgs,
                                     cm_comps[i], effective_transfer[i]));
-        else
-            for (i = 0; i < ncomps; i++)
+            for (; i < nc; i++)
                 pdc->colors.devn.values[i] = frac2cv(frac_1 - gx_map_color_frac(pgs,
                             (frac)(frac_1 - cm_comps[i]), effective_transfer[i]));
+        }
+        if (i < ncomps)
+            pdc->colors.devn.values[i] = cm_comps[i];
         pdc->type = gx_dc_type_devn;
+        /* For additive devices, we should invert the process colors
+         * here! But how do we know how many process colors we have?
+         * Ask the device using a dso. */
+        if (additive) {
+            int j, n = dev_proc(dev, dev_spec_op)(dev, gxdso_is_sep_supporting_additive_device, NULL, 0);
+            for (j = 0; j < n; j++)
+                pdc->colors.devn.values[j] = 65535 - pdc->colors.devn.values[j];
+        }
+
+        /* Let device set the tags if present */
+        if (device_encodes_tags(dev))
+            encode_tags(dev, pdc);
+
         return;
     }
 
-    if (pgs->effective_transfer_non_identity_count == 0)
-        for (i = 0; i < ncomps; i++)
-            cv[i] = frac2cv(cm_comps[i]);
-    else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
-        for (i = 0; i < ncomps; i++) {
+    if (pgs->effective_transfer_non_identity_count != 0) {
+        int n = 0;
+        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+            n = nc < 3 ? nc : 3;
+        for (i = 0; i < n; i++)
             cm_comps[i] = gx_map_color_frac(pgs,
                                     cm_comps[i], effective_transfer[i]);
-            cv[i] = frac2cv(cm_comps[i]);
-        }
-    else
-        for (i = 0; i < ncomps; i++) {
+        for (; i < nc; i++)
             cm_comps[i] = frac_1 - gx_map_color_frac(pgs,
                             (frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
-            cv[i] = frac2cv(cm_comps[i]);
-        }
+    }
+    if (nc < ncomps)
+        encode_tags(dev, pdc);
+    /* For additive devices, we should invert the process colors
+     * here! But how do we know how many process colors we have?
+     * Ask the device using a dso. */
+    if (additive) {
+        int j, n = dev_proc(dev, dev_spec_op)(dev, gxdso_is_sep_supporting_additive_device, NULL, 0);
+        for (j = 0; j < n; j++)
+            cm_comps[j] = frac_1 - cm_comps[j];
+    }
+    for (i = 0; i < nc; i++)
+        cv[i] = frac2cv(cm_comps[i]);
+    if(i < ncomps)
+        cv[i] = cm_comps[i];
     /* encode as a color index */
     color = dev_proc(dev, encode_color)(dev, cv);
     /* check if the encoding was successful; we presume failure is rare */
@@ -1705,7 +1806,10 @@ gs_identity_transfer(double value, const gx_transfer_map * pmap)
 float
 gs_mapped_transfer(double value, const gx_transfer_map * pmap)
 {
-    return gx_map_color_float(pmap, value);
+    int index = (int)((value) * (transfer_map_size) + 0.5);
+    if (index > transfer_map_size - 1)
+        index = transfer_map_size - 1;
+    return frac2float(pmap->values[index]);
 }
 
 /* Set a transfer map to the identity map. */
@@ -1782,7 +1886,7 @@ gx_default_w_b_mono_encode_color(gx_device *dev, const gx_color_value cv[])
 
 int
 gx_default_w_b_mono_decode_color(gx_device * dev, gx_color_index color,
-                                 gx_color_value pgray[1])
+                                 gx_color_value pgray[])
 {				/* Map 0 to max_value, 1 to 0. */
     pgray[0] = -(gx_color_value) color;
     return 0;
@@ -1818,7 +1922,7 @@ gx_default_b_w_mono_encode_color(gx_device *dev, const gx_color_value cv[])
 
 int
 gx_default_b_w_mono_decode_color(gx_device * dev, gx_color_index color,
-                                 gx_color_value pgray[1])
+                                 gx_color_value pgray[])
 {				/* Map 0 to max_value, 1 to 0. */
     pgray[0] = -((gx_color_value) color ^ 1);
     return 0;
@@ -1883,7 +1987,7 @@ gx_default_8bit_map_gray_color(gx_device * dev, const gx_color_value cv[])
 
 int
 gx_default_8bit_map_color_gray(gx_device * dev, gx_color_index color,
-                              gx_color_value pgray[1])
+                              gx_color_value pgray[])
 {
     pgray[0] = (gx_color_value)(color * gx_max_color_value / 255);
     return 0;
@@ -1975,7 +2079,7 @@ cmyk_1bit_map_color_rgb(gx_device * dev, gx_color_index color,
 
 int
 cmyk_1bit_map_color_cmyk(gx_device * dev, gx_color_index color,
-                        gx_color_value pcv[4])
+                        gx_color_value pcv[])
 {
     pcv[0] = (color & 8 ? 0 : gx_max_color_value);
     pcv[1] = (color & 4 ? 0 : gx_max_color_value);
@@ -2031,7 +2135,7 @@ cmyk_8bit_map_color_rgb(gx_device * dev, gx_color_index color,
 
 int
 cmyk_8bit_map_color_cmyk(gx_device * dev, gx_color_index color,
-                        gx_color_value pcv[4])
+                        gx_color_value pcv[])
 {
     pcv[0] = gx_color_value_from_byte((color >> 24) & 0xff);
     pcv[1] = gx_color_value_from_byte((color >> 16) & 0xff);
@@ -2042,7 +2146,7 @@ cmyk_8bit_map_color_cmyk(gx_device * dev, gx_color_index color,
 
 int
 cmyk_16bit_map_color_cmyk(gx_device * dev, gx_color_index color,
-                          gx_color_value pcv[4])
+                          gx_color_value pcv[])
 {
     pcv[0] = ((color >> 24) >> 24) & 0xffff;
     pcv[1] = ((color >> 16) >> 16) & 0xffff;
@@ -2170,6 +2274,8 @@ cmapper_transfer_add(gx_cmapper_t *data)
     gx_color_index color;
 
     /* apply the transfer function(s) */
+    if (device_encodes_tags(dev))
+        ncomps--;
     for (i = 0; i < ncomps; i++) {
         frac_value = cv2frac(pconc[i]);
         frac_value = gx_map_color_frac(pgs,
@@ -2217,6 +2323,8 @@ cmapper_transfer_sub(gx_cmapper_t *data)
     gx_color_index color;
 
     /* apply the transfer function(s) */
+    if (device_encodes_tags(dev))
+        ncomps--;
     for (i = 0; i < ncomps; i++) {
         frac_value = cv2frac(pconc[i]);
         frac_value = frac_1 - gx_map_color_frac(pgs,
@@ -2323,7 +2431,7 @@ cmap_transfer_halftone(gx_color_value *pconc, gx_device_color * pdc,
      const gs_gstate * pgs, gx_device * dev, bool has_transfer,
      bool has_halftone, gs_color_select_t select)
 {
-    uchar ncomps = dev->color_info.num_components;
+    uchar nc, ncomps = dev->color_info.num_components;
     frac frac_value;
     uchar i;
     frac cv_frac[GX_DEVICE_COLOR_MAX_COMPONENTS];
@@ -2331,12 +2439,15 @@ cmap_transfer_halftone(gx_color_value *pconc, gx_device_color * pdc,
     gx_color_value color_val[GX_DEVICE_COLOR_MAX_COMPONENTS];
 
     /* apply the transfer function(s) */
+    nc = ncomps;
+    if (device_encodes_tags(dev))
+        nc--;
     if (has_transfer) {
         if (pgs->effective_transfer_non_identity_count == 0) {
-            for (i = 0; i < ncomps; i++)
+            for (i = 0; i < nc; i++)
                 cv_frac[i] = cv2frac(pconc[i]);
         } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE) {
-            for (i = 0; i < ncomps; i++) {
+            for (i = 0; i < nc; i++) {
                 frac_value = cv2frac(pconc[i]);
                 cv_frac[i] = gx_map_color_frac(pgs,
                                     frac_value, effective_transfer[i]);
@@ -2344,7 +2455,7 @@ cmap_transfer_halftone(gx_color_value *pconc, gx_device_color * pdc,
         } else {
             if (gx_get_opmsupported(dev) == GX_CINFO_OPMSUPPORTED) {  /* CMYK-like color space */
                 uint k = dev->color_info.black_component;
-                for (i = 0; i < ncomps; i++) {
+                for (i = 0; i < nc; i++) {
                     frac_value = cv2frac(pconc[i]);
                     if (i == k) {
                         cv_frac[i] = frac_1 - gx_map_color_frac(pgs,
@@ -2354,19 +2465,23 @@ cmap_transfer_halftone(gx_color_value *pconc, gx_device_color * pdc,
                     }
                 }
             } else {
-                for (i = 0; i < ncomps; i++) {
+                for (i = 0; i < nc; i++) {
                     frac_value = cv2frac(pconc[i]);
                     cv_frac[i] = frac_1 - gx_map_color_frac(pgs,
                                 (frac)(frac_1 - frac_value), effective_transfer[i]);
                 }
             }
         }
+        if (nc < ncomps)
+            cv_frac[nc] = pconc[nc];
     } else {
         if (has_halftone) {
             /* We need this to be in frac form */
-            for (i = 0; i < ncomps; i++) {
+            for (i = 0; i < nc; i++) {
                 cv_frac[i] = cv2frac(pconc[i]);
             }
+            if (nc < ncomps)
+                cv_frac[nc] = pconc[nc];
         }
     }
     /* Halftoning */
@@ -2377,9 +2492,11 @@ cmap_transfer_halftone(gx_color_value *pconc, gx_device_color * pdc,
     } else {
         /* We have a frac value from the transfer function.  Do the encode.
            which does not take a frac value...  */
-        for (i = 0; i < ncomps; i++) {
+        for (i = 0; i < nc; i++) {
             color_val[i] = frac2cv(cv_frac[i]);
         }
+        if (i < ncomps)
+            color_val[i] = cv_frac[i];
         color = dev_proc(dev, encode_color)(dev, &(color_val[0]));
         /* check if the encoding was successful; we presume failure is rare */
         if (color != gx_no_color_index)
@@ -2396,6 +2513,8 @@ cmap_transfer(gx_color_value *pconc, const gs_gstate * pgs, gx_device * dev)
     uchar i;
 
     /* apply the transfer function(s) */
+    if (device_encodes_tags(dev))
+        ncomps--;
     if (pgs->effective_transfer_non_identity_count == 0) {
         /* No transfer function to apply */
     } else if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
