@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -70,18 +70,20 @@ RELOC_PTRS_END
 static
 ENUM_PTRS_WITH(device_clip_enum_ptrs, gx_device_clip *cptr)
 {
-    if (index < st_clip_list_max_ptrs + 2)
+    if (index < st_clip_list_max_ptrs + 3)
         return ENUM_USING(st_clip_list, &cptr->list,
-                          sizeof(gx_clip_list), index - 2);
+                          sizeof(gx_clip_list), index - 3);
     return ENUM_USING(st_device_forward, vptr,
                       sizeof(gx_device_forward),
-                      index - (st_clip_list_max_ptrs + 2));
+                      index - (st_clip_list_max_ptrs + 3));
 }
 case 0:
 ENUM_RETURN((cptr->current == &cptr->list.single ? NULL :
              (void *)cptr->current));
 case 1:
 ENUM_RETURN((cptr->cpath));
+case 2:
+ENUM_RETURN((cptr->rect_list));
 ENUM_PTRS_END
 static
 RELOC_PTRS_WITH(device_clip_reloc_ptrs, gx_device_clip *cptr)
@@ -91,6 +93,7 @@ RELOC_PTRS_WITH(device_clip_reloc_ptrs, gx_device_clip *cptr)
     else
         RELOC_PTR(gx_device_clip, current);
     RELOC_PTR(gx_device_clip, cpath);
+    RELOC_PTR(gx_device_clip, rect_list);
     RELOC_USING(st_clip_list, &cptr->list, sizeof(gx_clip_list));
     RELOC_USING(st_device_forward, vptr, sizeof(gx_device_forward));
 }
@@ -175,8 +178,10 @@ gx_cpath_init_contained_shared(gx_clip_path * pcpath,
 {
     if (shared) {
         if (shared->path.segments == &shared->path.local_segments) {
+#ifdef DEBUG
             lprintf1("Attempt to share (local) segments of clip path "PRI_INTPTR"!\n",
                      (intptr_t)shared);
+#endif
             return_error(gs_error_Fatal);
         }
         *pcpath = *shared;
@@ -233,8 +238,10 @@ gx_cpath_init_local_shared_nested(gx_clip_path * pcpath,
     if (shared) {
         if ((shared->path.segments == &shared->path.local_segments) &&
             !safely_nested) {
+#ifdef DEBUG
             lprintf1("Attempt to share (local) segments of clip path "PRI_INTPTR"!\n",
                      (intptr_t)shared);
+#endif
             return_error(gs_error_Fatal);
         }
         pcpath->path = shared->path;
@@ -686,13 +693,13 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
                 new_box.q.y = int2fixed(fixed2int_pixround(new_box.q.y + adjust_yu));
             }
             /* Intersect the two rectangles if necessary. */
-            if (old_box.p.x > new_box.p.x)
+            if (old_box.p.x >= new_box.p.x)
                 new_box.p.x = old_box.p.x, ++changed;
-            if (old_box.p.y > new_box.p.y)
+            if (old_box.p.y >= new_box.p.y)
                 new_box.p.y = old_box.p.y, ++changed;
-            if (old_box.q.x < new_box.q.x)
+            if (old_box.q.x <= new_box.q.x)
                 new_box.q.x = old_box.q.x, ++changed;
-            if (old_box.q.y < new_box.q.y)
+            if (old_box.q.y <= new_box.q.y)
                 new_box.q.y = old_box.q.y, ++changed;
             /* Check for a degenerate rectangle. */
             if (new_box.q.x < new_box.p.x || new_box.q.y < new_box.p.y)
