@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -107,7 +107,7 @@ gx_color_index
 gx_device_black(gx_device *dev)
 {
     if (dev->cached_colors.black == gx_no_color_index) {
-        uchar i, ncomps = dev->color_info.num_components;
+        uchar i, nc, ncomps = dev->color_info.num_components;
         frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
         gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
         const gx_device *cmdev;
@@ -117,8 +117,13 @@ gx_device_black(gx_device *dev)
         /* Get color components for black (gray = 0) */
         cmprocs->map_gray(cmdev, frac_0, cm_comps);
 
-        for (i = 0; i < ncomps; i++)
+        nc = ncomps;
+        if (device_encodes_tags(dev))
+            nc--;
+        for (i = 0; i < nc; i++)
             cv[i] = frac2cv(cm_comps[i]);
+        if (i < ncomps)
+            cv[i] = cm_comps[i];
 
         dev->cached_colors.black = dev_proc(dev, encode_color)(dev, cv);
     }
@@ -128,7 +133,7 @@ gx_color_index
 gx_device_white(gx_device *dev)
 {
     if (dev->cached_colors.white == gx_no_color_index) {
-        uchar i, ncomps = dev->color_info.num_components;
+        uchar i, nc, ncomps = dev->color_info.num_components;
         frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
         gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
         const gx_device *cmdev;
@@ -138,8 +143,13 @@ gx_device_white(gx_device *dev)
         /* Get color components for white (gray = 1) */
         cmprocs->map_gray(cmdev, frac_1, cm_comps);
 
-        for (i = 0; i < ncomps; i++)
+        nc = ncomps;
+        if (device_encodes_tags(dev))
+            nc--;
+        for (i = 0; i < nc; i++)
             cv[i] = frac2cv(cm_comps[i]);
+        if (i < ncomps)
+            cv[i] = cm_comps[i];
 
         dev->cached_colors.white = dev_proc(dev, encode_color)(dev, cv);
     }
@@ -318,13 +328,15 @@ gx_dc_no_write(
 static int
 gx_dc_no_read(
     gx_device_color *       pdevc,
-    const gs_gstate        * pgs,                /* ignored */
+    const gs_gstate       * pgs,                /* ignored */
     const gx_device_color * prior_devc,         /* ignored */
     const gx_device *       dev,                /* ignored */
     int64_t		    offset,             /* ignored */
     const byte *            pdata,              /* ignored */
     uint                    size,               /* ignored */
-    gs_memory_t *           mem )               /* ignored */
+    gs_memory_t *           mem,                /* ignored */
+    int                     x0,                 /* ignored */
+    int                     y0)                 /* ignored */
 {
     pdevc->type = gx_dc_type_none;
     return 0;
@@ -345,13 +357,15 @@ gx_dc_cannot_write(
 int
 gx_dc_cannot_read(
     gx_device_color *       pdevc,
-    const gs_gstate  *       pgs,                /* ignored */
+    const gs_gstate *       pgs,                /* ignored */
     const gx_device_color * prior_devc,         /* ignored */
     const gx_device *       dev,                /* ignored */
     int64_t		    offset,             /* ignored */
     const byte *            pdata,              /* ignored */
     uint                    size,               /* ignored */
-    gs_memory_t *           mem )               /* ignored */
+    gs_memory_t *           mem,                /* ignored */
+    int                     x0,                 /* ignored */
+    int                     y0)                 /* ignored */
 {
     return_error(gs_error_unknownerror);
 }
@@ -401,13 +415,15 @@ gx_dc_null_equal(const gx_device_color * pdevc1, const gx_device_color * pdevc2)
 static int
 gx_dc_null_read(
     gx_device_color *       pdevc,
-    const gs_gstate  *       pgs,                /* ignored */
+    const gs_gstate *       pgs,                /* ignored */
     const gx_device_color * prior_devc,         /* ignored */
     const gx_device *       dev,                /* ignored */
     int64_t		    offset,             /* ignored */
     const byte *            pdata,              /* ignored */
     uint                    size,               /* ignored */
-    gs_memory_t *           mem )               /* ignored */
+    gs_memory_t *           mem,                /* ignored */
+    int                     x0,                 /* ignored */
+    int                     y0)                 /* ignored */
 {
     pdevc->type = gx_dc_type_null;
     return 0;
@@ -599,10 +615,7 @@ gx_devn_write_color(
     }
 
     /* Now the tag */
-    if (dev->graphics_type_tag & GS_DEVICE_ENCODES_TAGS)
-        pdata[num_bytes_temp - 1] = (dev->graphics_type_tag & ~GS_DEVICE_ENCODES_TAGS);
-    else
-        pdata[num_bytes_temp - 1] = GS_UNTOUCHED_TAG;
+    pdata[num_bytes_temp - 1] = device_current_tag(dev);
 
     /* Now the data */
     for (i = 0; i < ncomps; i++) {
@@ -768,13 +781,15 @@ gx_devn_read_color(
 static int
 gx_dc_devn_read(
     gx_device_color *       pdevc,
-    const gs_gstate  *       pgs,                /* ignored */
+    const gs_gstate *       pgs,                /* ignored */
     const gx_device_color * prior_devc,         /* ignored */
     const gx_device *       dev,
     int64_t                 offset,             /* ignored */
     const byte *            pdata,
     uint                    size,
-    gs_memory_t *           mem )               /* ignored */
+    gs_memory_t *           mem,                /* ignored */
+    int                     x0,                 /* ignored */
+    int                     y0)                 /* ignored */
 {
     pdevc->type = gx_dc_type_devn;
     return gx_devn_read_color(&(pdevc->colors.devn.values[0]), &(pdevc->tag),
@@ -977,13 +992,15 @@ gx_dc_pure_write(
 static int
 gx_dc_pure_read(
     gx_device_color *       pdevc,
-    const gs_gstate        * pgs,                /* ignored */
+    const gs_gstate       * pgs,                /* ignored */
     const gx_device_color * prior_devc,         /* ignored */
     const gx_device *       dev,
     int64_t		    offset,             /* ignored */
     const byte *            pdata,
     uint                    size,
-    gs_memory_t *           mem )               /* ignored */
+    gs_memory_t *           mem,                /* ignored */
+    int                     x0,                 /* ignored */
+    int                     y0)                 /* ignored */
 {
     pdevc->type = gx_dc_type_pure;
     return gx_dc_read_color(&pdevc->colors.pure, dev, pdata, size);
