@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -20,6 +20,7 @@
 #  define gxdevsop_INCLUDED
 
 #include "gxdevcli.h"
+#include "gsovrc.h"
 
 /* This file enumerates a series of device specific operations, that can be
  * performed using the 'dev_spec_op' procedure in the gs_device structure.
@@ -134,6 +135,39 @@ typedef struct pattern_accum_param_t {
     int pinst_id;
 }pattern_accum_param_s;
 
+/* Structure used for device specific color setting */
+typedef struct color_replace_s {
+    gx_device_color *pdc;
+    const gs_color_space *pcs;
+    const gs_client_color *pcc;
+    const gs_gstate *pgs;  /* Perhaps needed for named color profile information */
+    cmm_profile_t *pdf14_iccprofile;
+} color_replace_t;
+
+/* Operation for pdf14 device to perform when in fill stroke
+   commands occurring in gdevabuf.c (alphabits) device. This
+   lets us get the pdf14 device setup for the stroke fill
+   operation, and to clean up following the operation */
+typedef enum {
+    OP_FS_TRANS_PREFILL,
+    OP_FS_TRANS_PRESTROKE,
+    OP_FS_TRANS_POSTSTROKE,
+    OP_FS_TRANS_CLEANUP
+} OP_FS_TRANS;
+
+/* Structure used for overprint communication coming from alpha buffer to owner */
+typedef struct overprint_abuf_state_s {
+    /* Client filled data that changes each call. */
+    OP_FS_TRANS op_trans; /* What are we doing? */
+    /* Client filled data that remains constant. */
+    const gs_gstate *pgs;          /* From client (abuf) */
+    gx_path *ppath;
+    const gx_clip_path *pcpath;
+    gs_log2_scale_point alpha_buf_path_scale;
+    /* Callee storage. */
+    unsigned char storage[256];
+} overprint_abuf_state_t;
+
 enum {
     /* All gxdso_ keys must be defined in this structure.
      * Do NOT rely on your particular gxdso_ having a particular value.
@@ -450,13 +484,54 @@ enum {
      */
     gxdso_overprintsim_state,
 
-    /* Get/Set information about the overprint op state.
-     *     data = overprint op state to set (OP_STATE_{NONE, FILL, STROKE}) or -1
-     *            for no change.
-     *     size = 0
-     * Returns -1 if unhandled, otherwise the current state.
+    /* Used for handing state settings between the abuf device and the
+     * transparency and overprint compositor.
+     *     data = pointer to overprint_abuf_state_t structure.
+     *            op_trans will be OP_FS_TRANS_PREFILL on the first call.
+     *            In normal operation it will then be OP_FS_TRANS_PRESTROKE
+     *            and then OP_FS_TRANS_POSTSTROKE on subsequent calls. The
+     *            contents of the structure will otherwise remain constant
+     *            between calls.
+     *            In the event of an error return from this call, no further
+     *            calls will be made on with this param block (i.e. the
+     *            callee must tidy itself up).
+     *            In the event of an error in the caller between calls to
+     *            this, an OP_FS_TRANS_CLEANUP call will be made.
+     *     size = size of structure
+     * Returns error. */
+    gxdso_abuf_optrans,
+
+    /* Color replacement method.  Intercepts remap color method(s) to
+     *  enable the device map source colors to device colors directly
+     *  in a method defined in the device.
+     *       data = color_replace_t
+     *       size = sizeof(color_replace_t)
+     *
+     *  Returns 0 is no replacment is made.
+     *  Returns >0 if replacement occurred. */
+    gxdso_replacecolor,
+
+    /* Used solely (for now at least) by pdfwrite. We need to tell pdfwrite that Image and Form
+     * XObjects, have an /OC entry and what the object number of the optional content entry is.
+     * We should already have defined the optional content dictionary using a pdfmark.
      */
-    gxdso_overprint_op,
+    gxdso_pending_optional_content,
+
+    /* Called to give a device the chance to update the color information after put_params
+     * has reset it. This enables device specific changes to not interfere when devices are
+     * cloned for NumRenderingThreads etc. */
+    gxdso_adjust_colors,
+
+    /* Called to ask if we are an additive device that would like to support separations
+     * natively. Should return the number of additive process colors in use.
+     */
+    gxdso_is_sep_supporting_additive_device,
+
+    /* Communicate the start of text with a 'clip' render mode, and the grestore
+     * which restores it away. We send data of 1 for the start of text in a clipping
+     * mode and 0 when we grestore back to a non-clip mode.
+     */
+    gxdso_hilevel_text_clip,
 
     /* Add new gxdso_ keys above this. */
     gxdso_pattern__LAST
