@@ -1,14 +1,16 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  1305 Grant Avenue - Suite 200,
-   Novato, CA 94945, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -33,9 +35,13 @@
 #include "gsimage.h"
 #include "gsbittab.h"
 #include "gzpath.h"
+#include "gxdevsop.h"
 
 #include "gxfapi.h"
 
+#define FAPI_ROUND(v) (v >= 0 ? v + 0.5 : v - 0.5)
+#define FAPI_ROUND_TO_FRACINT(v) ((fracint)FAPI_ROUND(v))
+
 extern_gs_get_fapi_server_inits();
 
 /* FIXME */
@@ -55,6 +61,7 @@ gs_fapi_renderer_retcode(gs_memory_t *mem, gs_fapi_server *I,
 
 typedef struct gs_fapi_outline_handler_s
 {
+    gs_fapi_server *fserver;
     struct gx_path_s *path;
     fixed x0;
     fixed y0;
@@ -97,29 +104,33 @@ add_move(gs_fapi_path *I, int64_t x, int64_t y)
 {
     gs_fapi_outline_handler *olh = (gs_fapi_outline_handler *) I->olh;
 
-    x = import_shift(x, I->shift) + olh->x0;
-    y = -import_shift(y, I->shift) + olh->y0;
-
-    if (x > (int64_t) max_fixed) {
-        x = (int64_t) max_fixed;
-    }
-    else if (x < (int64_t) min_fixed) {
-        x = (int64_t) min_fixed;
+    x = import_shift(x, I->shift);
+    y = -import_shift(y, I->shift);
+    if (olh->fserver->transform_outline) {
+        gs_point pt;
+        I->gs_error = gs_distance_transform((double)fixed2float((float)x), (double)fixed2float((float)y), &olh->fserver->outline_mat, &pt);
+        if (I->gs_error < 0)
+            return I->gs_error;
+        x = float2fixed(pt.x);
+        y = float2fixed(pt.y);
     }
+    x += olh->x0;
+    y += olh->y0;
 
-    if (y > (int64_t) max_fixed) {
-        y = (int64_t) max_fixed;
+    if (x > (int64_t) max_coord_fixed || x < (int64_t) min_coord_fixed
+     || y > (int64_t) max_coord_fixed || y < (int64_t) min_coord_fixed) {
+         I->gs_error = gs_error_undefinedresult;
     }
-    else if (y < (int64_t) min_fixed) {
-        y = (int64_t) min_fixed;
+    else {
+
+        if (olh->need_close && olh->close_path)
+            if ((I->gs_error = add_closepath(I)) < 0)
+                return (I->gs_error);
+        olh->need_close = false;
+
+/*        dprintf2("%f %f moveto\n", fixed2float(x), fixed2float(y)); */
+        I->gs_error = gx_path_add_point(olh->path, (fixed) x, (fixed) y);
     }
-
-    if (olh->need_close && olh->close_path)
-        if ((I->gs_error = add_closepath(I)) < 0)
-            return (I->gs_error);
-    olh->need_close = false;
-    I->gs_error = gx_path_add_point(olh->path, (fixed) x, (fixed) y);
-
     return (I->gs_error);
 }
 
@@ -128,24 +139,29 @@ add_line(gs_fapi_path *I, int64_t x, int64_t y)
 {
     gs_fapi_outline_handler *olh = (gs_fapi_outline_handler *) I->olh;
 
-    x = import_shift(x, I->shift) + olh->x0;
-    y = -import_shift(y, I->shift) + olh->y0;
-    if (x > (int64_t) max_fixed) {
-        x = (int64_t) max_fixed;
-    }
-    else if (x < (int64_t) min_fixed) {
-        x = (int64_t) min_fixed;
+    x = import_shift(x, I->shift);
+    y = -import_shift(y, I->shift);
+    if (olh->fserver->transform_outline) {
+        gs_point pt;
+        I->gs_error = gs_distance_transform((double)fixed2float((float)x), (double)fixed2float((float)y), &olh->fserver->outline_mat, &pt);
+        if (I->gs_error < 0)
+            return I->gs_error;
+        x = float2fixed(pt.x);
+        y = float2fixed(pt.y);
     }
+    x += olh->x0;
+    y += olh->y0;
 
-    if (y > (int64_t) max_fixed) {
-        y = (int64_t) max_fixed;
-    }
-    else if (y < (int64_t) min_fixed) {
-        y = (int64_t) min_fixed;
+    if (x > (int64_t) max_coord_fixed || x < (int64_t) min_coord_fixed
+     || y > (int64_t) max_coord_fixed || y < (int64_t) min_coord_fixed) {
+         I->gs_error = gs_error_undefinedresult;
     }
+    else {
+        olh->need_close = true;
 
-    olh->need_close = true;
-    I->gs_error = gx_path_add_line_notes(olh->path, (fixed) x, (fixed) y, 0);
+/*        dprintf2("%f %f lineto\n", fixed2float(x), fixed2float(y)); */
+        I->gs_error = gx_path_add_line_notes(olh->path, (fixed) x, (fixed) y, 0);
+    }
     return (I->gs_error);
 }
 
@@ -155,57 +171,53 @@ add_curve(gs_fapi_path *I, int64_t x0, int64_t y0, int64_t x1, int64_t y1,
 {
     gs_fapi_outline_handler *olh = (gs_fapi_outline_handler *) I->olh;
 
-    x0 = import_shift(x0, I->shift) + olh->x0;
-    y0 = -import_shift(y0, I->shift) + olh->y0;
-    x1 = import_shift(x1, I->shift) + olh->x0;
-    y1 = -import_shift(y1, I->shift) + olh->y0;
-    x2 = import_shift(x2, I->shift) + olh->x0;
-    y2 = -import_shift(y2, I->shift) + olh->y0;
+    x0 = import_shift(x0, I->shift);
+    y0 = -import_shift(y0, I->shift);
+    x1 = import_shift(x1, I->shift);
+    y1 = -import_shift(y1, I->shift);
+    x2 = import_shift(x2, I->shift);
+    y2 = -import_shift(y2, I->shift);
 
-    if (x0 > (int64_t) max_fixed) {
-        x0 = (int64_t) max_fixed;
-    }
-    else if (x0 < (int64_t) min_fixed) {
-        x0 = (int64_t) min_fixed;
+    if (olh->fserver->transform_outline) {
+        gs_point pt;
+        I->gs_error = gs_distance_transform((double)fixed2float((float)x0), (double)fixed2float((float)y0), &olh->fserver->outline_mat, &pt);
+        if (I->gs_error < 0)
+            return I->gs_error;
+        x0 = float2fixed(pt.x);
+        y0 = float2fixed(pt.y);
+        I->gs_error = gs_distance_transform((double)fixed2float((float)x1), (double)fixed2float((float)y1), &olh->fserver->outline_mat, &pt);
+        if (I->gs_error < 0)
+            return I->gs_error;
+        x1 = float2fixed(pt.x);
+        y1 = float2fixed(pt.y);
+        I->gs_error = gs_distance_transform((double)fixed2float((float)x2), (double)fixed2float((float)y2), &olh->fserver->outline_mat, &pt);
+        if (I->gs_error < 0)
+            return I->gs_error;
+        x2 = float2fixed(pt.x);
+        y2 = float2fixed(pt.y);
     }
+    x0 += olh->x0;
+    y0 += olh->y0;
+    x1 += olh->x0;
+    y1 += olh->y0;
+    x2 += olh->x0;
+    y2 += olh->y0;
 
-    if (y0 > (int64_t) max_fixed) {
-        y0 = (int64_t) max_fixed;
-    }
-    else if (y0 < (int64_t) min_fixed) {
-        y0 = (int64_t) min_fixed;
-    }
-    if (x1 > (int64_t) max_fixed) {
-        x1 = (int64_t) max_fixed;
-    }
-    else if (x1 < (int64_t) min_fixed) {
-        x1 = (int64_t) min_fixed;
+    if (x0 > (int64_t) max_coord_fixed || x0 < (int64_t) min_coord_fixed
+     || y0 > (int64_t) max_coord_fixed || y0 < (int64_t) min_coord_fixed
+     || x1 > (int64_t) max_coord_fixed || x1 < (int64_t) min_coord_fixed
+     || y1 > (int64_t) max_coord_fixed || y1 < (int64_t) min_coord_fixed
+     || x2 > (int64_t) max_coord_fixed || x2 < (int64_t) min_coord_fixed
+     || y2 > (int64_t) max_coord_fixed || y2 < (int64_t) min_coord_fixed)
+    {
+        I->gs_error = gs_error_undefinedresult;
     }
+    else {
+        olh->need_close = true;
 
-    if (y1 > (int64_t) max_fixed) {
-        y1 = (int64_t) max_fixed;
+/*        dprintf6("%f %f %f %f %f %f curveto\n", fixed2float(x0), fixed2float(y0), fixed2float(x1), fixed2float(y1), fixed2float(x2), fixed2float(y2));*/
+        I->gs_error = gx_path_add_curve_notes(olh->path, (fixed) x0, (fixed) y0, (fixed) x1, (fixed) y1, (fixed) x2, (fixed) y2, 0);
     }
-    else if (y1 < (int64_t) min_fixed) {
-        y1 = (int64_t) min_fixed;
-    }
-    if (x2 > (int64_t) max_fixed) {
-        x2 = (int64_t) max_fixed;
-    }
-    else if (x2 < (int64_t) min_fixed) {
-        x2 = (int64_t) min_fixed;
-    }
-
-    if (y2 > (int64_t) max_fixed) {
-        y2 = (int64_t) max_fixed;
-    }
-    else if (y2 < (int64_t) min_fixed) {
-        y2 = (int64_t) min_fixed;
-    }
-
-    olh->need_close = true;
-    I->gs_error =
-        gx_path_add_curve_notes(olh->path, (fixed) x0, (fixed) y0, (fixed) x1,
-                                (fixed) y1, (fixed) x2, (fixed) y2, 0);
     return (I->gs_error);
 }
 
@@ -261,9 +273,7 @@ using_transparency_pattern(gs_gstate *pgs)
 {
     gx_device *dev = gs_currentdevice_inline(pgs);
 
-    return ((!gs_color_writes_pure(pgs))
-            && dev_proc(dev, begin_transparency_group) != gx_default_begin_transparency_group
-            && dev_proc(dev, end_transparency_group) != gx_default_end_transparency_group);
+    return ((!gs_color_writes_pure(pgs)) && dev_proc(dev, dev_spec_op)(dev, gxdso_supports_pattern_transparency, NULL, 0));
 }
 
 static inline bool
@@ -285,7 +295,7 @@ recreate_multiple_master(gs_font_base *pbfont)
             changed = (memcmp(I->face.WeightVector.values, pfont1->data.WeightVector.values,
                              pfont1->data.WeightVector.count * sizeof(pfont1->data.WeightVector.values[0])) != 0);
         }
-        
+
         if (changed) {
             r = (I->set_mm_weight_vector(I, &I->ff, pfont1->data.WeightVector.values, pfont1->data.WeightVector.count) == gs_error_invalidaccess);
             I->face.WeightVector.count = pfont1->data.WeightVector.count;
@@ -428,7 +438,14 @@ gs_fapi_prepare_font(gs_font *pfont, gs_fapi_server *I, int subfont, const char
         < 0)
         return code;
     pbfont->FAPI_font_data = I->ff.server_font_data;    /* Save it back to GS font. */
-    if (I->ff.server_font_data != 0) {
+
+    /* We only want to "refine" the FontBBox for fonts where we allow FAPI to be
+       treated as a "black box", handing over the entire font to the FAPI server.
+       That is, fonts where we give the server either the file, or a buffer with
+       the entire font description in it.
+     */
+    if (I->ff.server_font_data != 0
+        && (font_file_path != NULL || full_font_buf != NULL)) {
         if ((code =
              gs_fapi_renderer_retcode(mem, I,
                                       I->get_font_bbox(I, &I->ff,
@@ -561,12 +578,16 @@ outline_char(gs_memory_t *mem, gs_fapi_server *I, int import_shift_v,
 {
     gs_fapi_path path_interface = path_interface_stub;
     gs_fapi_outline_handler olh;
-    int code;
+    int code = 0;
     gs_gstate *pgs = penum_s->pgs;
+    struct gx_path_s path1;
 
-    olh.path = path;
-    olh.x0 = pgs->ctm.tx_fixed;
-    olh.y0 = pgs->ctm.ty_fixed;
+    (void)gx_path_init_local(&path1, mem);
+
+    olh.fserver = I;
+    olh.path = &path1;
+    olh.x0 = pgs->ctm.tx_fixed - float2fixed(penum_s->fapi_glyph_shift.x);
+    olh.y0 = pgs->ctm.ty_fixed - float2fixed(penum_s->fapi_glyph_shift.y);
     olh.close_path = close_path;
     olh.need_close = false;
     path_interface.olh = &olh;
@@ -576,15 +597,22 @@ outline_char(gs_memory_t *mem, gs_fapi_server *I, int import_shift_v,
                                   I->get_char_outline(I,
                                                       &path_interface))) < 0
         || path_interface.gs_error != 0) {
-        if (path_interface.gs_error != 0)
-            return path_interface.gs_error;
-        else
-            return code;
+        if (path_interface.gs_error != 0) {
+            code = path_interface.gs_error;
+            goto done;
+        }
+        else {
+            goto done;
+        }
     }
     if (olh.need_close && olh.close_path)
         if ((code = add_closepath(&path_interface)) < 0)
-            return code;
-    return 0;
+            goto done;
+    code = gx_path_copy(&path1, path);
+done:
+    code = code >= 0 || code == gs_error_undefinedresult ? 0 : code;
+    gx_path_free(&path1, "outline_char");
+    return code;
 }
 
 static void
@@ -658,29 +686,35 @@ fapi_copy_mono(gx_device *dev1, gs_fapi_raster *rast, int dx, int dy)
 /* Allocate the line buffer for bolding.  We need 2 + bold scan lines. */
 static byte *
 alloc_bold_lines(gs_memory_t *mem, uint width, int bold, client_name_t cname)
-{       return gs_alloc_byte_array(mem, 2 + bold, bitmap_raster(width + bold),
-                                   cname);
+{
+    return gs_alloc_byte_array(mem, 2 + bold, bitmap_raster(width + bold), cname);
 }
 
 /* Merge one (aligned) scan line into another, for vertical smearing. */
-static void
-bits_merge(byte *dest, const byte *src, uint nbytes)
-{       long *dp = (long *)dest;
+void
+gx_fapi_bits_merge(byte *dest, const byte *src, uint nbytes)
+{
+        long *dp = (long *)dest;
         const long *sp = (const long *)src;
         uint n = (nbytes + sizeof(long) - 1) >> ARCH_LOG2_SIZEOF_LONG;
 
-        for ( ; n >= 4; sp += 4, dp += 4, n -= 4 )
-          dp[0] |= sp[0], dp[1] |= sp[1], dp[2] |= sp[2], dp[3] |= sp[3];
-        for ( ; n; ++sp, ++dp, --n )
-          *dp |= *sp;
+        for ( ; n >= 4; sp += 4, dp += 4, n -= 4 ) {
+            dp[0] |= sp[0];
+            dp[1] |= sp[1];
+            dp[2] |= sp[2];
+            dp[3] |= sp[3];
+        }
+        for ( ; n; ++sp, ++dp, --n ) {
+            *dp |= *sp;
+        }
 }
 
 /* Smear a scan line horizontally.  Note that the output is wider than */
 /* the input by the amount of bolding (smear_width). */
-static void
-bits_smear_horizontally(byte *dest, const byte *src, uint width,
-  uint smear_width)
-{       uint bits_on = 0;
+void
+gx_fapi_bits_smear_horizontally(byte *dest, const byte *src, uint width, uint smear_width)
+{
+        uint32_t bits_on = 0;
         const byte *sp = src;
         uint sbyte = *sp;
         byte *dp = dest;
@@ -689,77 +723,95 @@ bits_smear_horizontally(byte *dest, const byte *src, uint width,
         const byte *zp = src;
         uint zmask = 0x80;
         uint i = 0;
+        uint stop;
 
         /* Process the first smear_width bits. */
-        { uint stop = min(smear_width, width);
+        stop = min(smear_width, width);
 
-          for ( ; i < stop; ++i ) {
-            if ( sbyte & sdmask )
+        for ( ; i < stop; ++i ) {
+            if ( sbyte & sdmask ) {
               bits_on++;
-            else if ( bits_on )
+            }
+            else if ( bits_on ) {
               dbyte |= sdmask;
-            if ( (sdmask >>= 1) == 0 )
-              sdmask = 0x80, *dp++ = dbyte, dbyte = sbyte = *++sp;
-          }
+            }
+            if ( (sdmask >>= 1) == 0 ) {
+                sdmask = 0x80;
+                *dp++ = dbyte;
+                dbyte = sbyte = *++sp;
+            }
         }
 
         /* Process all but the last smear_width bits. */
-        { for ( ; i < width; ++i ) {
-            if ( sbyte & sdmask )
-              bits_on++;
-            else if ( bits_on )
-              dbyte |= sdmask;
-            if ( *zp & zmask )
-              --bits_on;
+        for ( ; i < width; ++i ) {
+            if ( sbyte & sdmask ) {
+                /* In practice, bits_on should never overflow,
+                   but if it doesn, handle it gracefully
+                 */
+                bits_on = (uint32_t)(((uint64_t)bits_on + 1) & 0xffffffff);
+            }
+            else if ( bits_on ) {
+                dbyte |= sdmask;
+            }
+            if ( *zp & zmask && bits_on > 0) {
+                --bits_on;
+            }
             if ( (sdmask >>= 1) == 0 ) {
-              sdmask = 0x80;
-              *dp++ = dbyte;
-on:           switch ( (dbyte = sbyte = *++sp) ) {
-                case 0xff:
-                  if ( width - i <= 8 )
-                    break;
-                  *dp++ = 0xff;
-                  bits_on += 8 -
-                    byte_count_bits[(*zp & (zmask - 1)) + (zp[1] & -(int)zmask)];
-                  ++zp;
-                  i += 8;
-                  goto on;
-                case 0:
-                  if ( bits_on || width - i <= 8 )
-                    break;
-                  *dp++ = 0;
-                  /* We know there can't be any bits to be zeroed, */
-                  /* because bits_on can't go negative. */
-                  ++zp;
-                  i += 8;
-                  goto on;
-                default:
-                  ;
-              }
+                sdmask = 0x80;
+                *dp++ = dbyte;
+on:             switch ( (dbyte = sbyte = *++sp) ) {
+                  case 0xff:
+                    if ( width - i <= 8 )
+                        break;
+                    *dp++ = 0xff;
+                    bits_on += 8 - byte_count_bits[(*zp & (zmask - 1)) + (zp[1] & -(int)zmask)];
+                    ++zp;
+                    i += 8;
+                    goto on;
+                  case 0:
+                    if ( bits_on || width - i <= 8 )
+                      break;
+                    *dp++ = 0;
+                    /* We know there can't be any bits to be zeroed, */
+                    /* because bits_on can't go negative. */
+                    ++zp;
+                    i += 8;
+                    goto on;
+                  default:
+                    ;
+                }
+            }
+            if ( (zmask >>= 1) == 0 ) {
+              zmask = 0x80;
+              ++zp;
             }
-            if ( (zmask >>= 1) == 0 )
-              zmask = 0x80, ++zp;
-          }
         }
 
         /* Process the last smear_width bits. */
         /****** WRONG IF width < smear_width ******/
-        { uint stop = width + smear_width;
+        stop = width + smear_width;
 
-          for ( ; i < stop; ++i ) {
-            if ( bits_on )
-              dbyte |= sdmask;
-            if ( (sdmask >>= 1) == 0 )
-              sdmask = 0x80, *dp++ = dbyte, dbyte = 0;
-            if ( *zp & zmask )
-              --bits_on;
-            if ( (zmask >>= 1) == 0 )
-              zmask = 0x80, ++zp;
-          }
+        for ( ; i < stop; ++i ) {
+            if ( bits_on ) {
+                dbyte |= sdmask;
+            }
+            if ( (sdmask >>= 1) == 0 ) {
+                sdmask = 0x80;
+                *dp++ = dbyte;
+                dbyte = 0;
+            }
+            if ( *zp & zmask && bits_on > 0) {
+                --bits_on;
+            }
+            if ( (zmask >>= 1) == 0 ) {
+                zmask = 0x80;
+                ++zp;
+            }
         }
 
-        if ( sdmask != 0x80 )
+        if ( sdmask != 0x80 ) {
           *dp = dbyte;
+        }
 }
 
 static const int frac_pixel_shift = 4;
@@ -776,7 +828,7 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
     gs_gstate *penum_pgs = (gs_gstate *) penum->pgs;
     int code;
     const gx_clip_path *pcpath = pgs->clip_path;
-    const gx_drawing_color *pdcolor = penum->pdcolor;
+    const gx_drawing_color *pdcolor = gs_currentdevicecolor_inline(penum->pgs);
     int rast_orig_x = rast->orig_x;
     int rast_orig_y = -rast->orig_y;
     gs_font_base *pbfont = (gs_font_base *)pfont;
@@ -849,7 +901,7 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
         code = gs_gstate_color_load(pgs);
         if (code < 0)
             return code;
-        
+
         code = gx_image_fill_masked(dev, r, 0, dstr, gx_no_bitmap_id,
                                     (int)dx, (int)dy,
                                     rast->width, rast->height,
@@ -872,7 +924,7 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
         byte *bold_lines = NULL;
         byte *line = NULL;
         int ascent = 0;
-        
+
         pie = gs_image_enum_alloc(mem, "image_char(image_enum)");
         if (!pie) {
             return_error(gs_error_VMerror);
@@ -934,16 +986,16 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
                             memcpy(line, r + y * rast->line_step, rast->line_step);
                             memset(line + rast->line_step, 0x00, (dest_raster + 1) -  rast->line_step);
 
-                            bits_smear_horizontally(merged_line(y), line, rast->width, bold);
+                            gx_fapi_bits_smear_horizontally(merged_line(y), line, rast->width, bold);
                             /* Now re-establish the invariant -- see below. */
                             kmask = 1;
 
                             for ( ; (y & kmask) == kmask && y - kmask >= y0;
                                   kmask = (kmask << 1) + 1) {
 
-                                bits_merge(merged_line(y - kmask), merged_line(y - (kmask >> 1)), dest_bytes);
+                                gx_fapi_bits_merge(merged_line(y - kmask), merged_line(y - (kmask >> 1)), dest_bytes);
                             }
-                        }   
+                        }
 
                        /*
                         * As of this point in the loop, we maintain the following
@@ -968,7 +1020,7 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_gstate *pgs, gs_show_enum *penum,
                             first = false;
                           }
                           else
-                            bits_merge(bold_lines, merged_line(iy), dest_bytes);
+                            gx_fapi_bits_merge(bold_lines, merged_line(iy), dest_bytes);
                         }
                         code = gs_image_next(pie, bold_lines, dest_bytes, &used);
                     }
@@ -1228,6 +1280,7 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
         return 0;
 
     I->use_outline = false;
+    I->transform_outline = false;
 
     if (penum == 0)
         return_error(gs_error_undefined);
@@ -1250,7 +1303,10 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
             (pbfont->dir->ccache.upper >> 1) : MAX_CCACHE_TEMP_BITMAP_BITS;
     }
 
-    I->grid_fit = pbfont->dir->grid_fit_tt;
+    if (pbfont->memory->gs_lib_ctx->font_dir != NULL)
+        I->grid_fit = pbfont->memory->gs_lib_ctx->font_dir->grid_fit_tt;
+    else
+        I->grid_fit = pbfont->dir->grid_fit_tt;
 
     /* Compute the scale : */
     if (!SHOW_IS(penum, TEXT_DO_NONE) && !I->use_outline) {
@@ -1296,7 +1352,7 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
     }
 
    scale = 1 << I->frac_shift;
-  retry_oversampling:
+retry_oversampling:
     if (I->face.font_id != pbfont->id ||
         !MTX_EQ((&I->face.ctm), ctm) ||
         I->face.log2_scale.x != log2_scale.x ||
@@ -1310,10 +1366,9 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
         , {1, 1}
         , true
         };
-        gs_matrix scale_mat, scale_ctm;
 
+        gs_matrix lctm, scale_mat, scale_ctm;
         I->face.font_id = pbfont->id;
-        I->face.ctm = *ctm;
         I->face.log2_scale = log2_scale;
         I->face.align_to_pixels = align_to_pixels;
         I->face.HWResolution[0] = dev->HWResolution[0];
@@ -1326,7 +1381,9 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
         /* We apply the entire transform to the glyph (that is ctm x FontMatrix)
          * at render time.
          */
-
+        lctm = *ctm;
+retry_scaling:
+        I->face.ctm = lctm;
         memset(&scale_ctm, 0x00, sizeof(gs_matrix));
         scale_ctm.xx = dev->HWResolution[0] / 72;
         scale_ctm.yy = dev->HWResolution[1] / 72;
@@ -1334,7 +1391,7 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
         if ((code = gs_matrix_invert((const gs_matrix *)&scale_ctm, &scale_ctm)) < 0)
             return code;
 
-        if ((code = gs_matrix_multiply(ctm, &scale_ctm, &scale_mat)) < 0)  /* scale_mat ==  CTM - resolution scaling */
+        if ((code = gs_matrix_multiply(&lctm, &scale_ctm, &scale_mat)) < 0)  /* scale_mat ==  CTM - resolution scaling */
             return code;
 
         if ((code = I->get_fontmatrix(I, &scale_ctm)) < 0)
@@ -1346,18 +1403,38 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
         if ((code = gs_matrix_multiply(&scale_mat, &scale_ctm, &scale_mat)) < 0)  /* scale_mat ==  CTM - resolution scaling - FontMatrix scaling */
             return code;
 
-        font_scale.matrix[0] =
-            (fracint) (scale_mat.xx * FontMatrix_div * scale + 0.5);
-        font_scale.matrix[1] =
-            -(fracint) (scale_mat.xy * FontMatrix_div * scale + 0.5);
-        font_scale.matrix[2] =
-            (fracint) (scale_mat.yx * FontMatrix_div * scale + 0.5);
-        font_scale.matrix[3] =
-            -(fracint) (scale_mat.yy * FontMatrix_div * scale + 0.5);
-        font_scale.matrix[4] =
-            (fracint) (scale_mat.tx * FontMatrix_div * scale + 0.5);
-        font_scale.matrix[5] =
-            (fracint) (scale_mat.ty * FontMatrix_div * scale + 0.5);
+        if (((int64_t)(scale_mat.xx * FontMatrix_div * scale + 0.5)) != ((int32_t)(scale_mat.xx * FontMatrix_div * scale + 0.5))
+        ||  ((int64_t)(scale_mat.xy * FontMatrix_div * scale + 0.5)) != ((int32_t)(scale_mat.xy * FontMatrix_div * scale + 0.5))
+        ||  ((int64_t)(scale_mat.yx * FontMatrix_div * scale + 0.5)) != ((int32_t)(scale_mat.yx * FontMatrix_div * scale + 0.5))
+        ||  ((int64_t)(scale_mat.yy * FontMatrix_div * scale + 0.5)) != ((int32_t)(scale_mat.yy * FontMatrix_div * scale + 0.5))) {
+            /* Overflow
+               If the scaling is large enough to overflow the 16.16 representation, we forcibly produce an outline
+               unscaled except an arbitrary "midrange" scale (chosen to avoid under/overflow issues). And
+               then scale the points as we create the Ghostscript path outline_char().
+               If the glyph is this large, we're really not worried about hinting or dropout detection etc.
+             */
+
+            memset(&lctm, 0x00, sizeof(gs_matrix));
+            lctm.xx = 256.0;
+            lctm.yy = 256.0;
+            I->transform_outline = true;
+            I->use_outline = true;
+            if ((code = gs_matrix_invert((const gs_matrix *)&lctm, &scale_ctm)) < 0)
+                 return code;
+            if ((code = gs_matrix_multiply(ctm, &scale_ctm, &scale_mat)) < 0)  /* scale_mat ==  CTM - resolution scaling */
+                return code;
+
+            I->outline_mat = scale_mat;
+            goto retry_scaling;
+        }
+        else {
+            font_scale.matrix[0] =  FAPI_ROUND_TO_FRACINT(scale_mat.xx * FontMatrix_div * scale);
+            font_scale.matrix[1] = -FAPI_ROUND_TO_FRACINT(scale_mat.xy * FontMatrix_div * scale);
+            font_scale.matrix[2] =  FAPI_ROUND_TO_FRACINT(scale_mat.yx * FontMatrix_div * scale);
+            font_scale.matrix[3] = -FAPI_ROUND_TO_FRACINT(scale_mat.yy * FontMatrix_div * scale);
+            font_scale.matrix[4] =  FAPI_ROUND_TO_FRACINT(scale_mat.tx * FontMatrix_div * scale);
+            font_scale.matrix[5] =  FAPI_ROUND_TO_FRACINT(scale_mat.ty * FontMatrix_div * scale);
+        }
 
         /* Note: the ctm mapping here is upside down. */
         font_scale.HWResolution[0] =
@@ -1374,6 +1451,7 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
                       (double)font_scale.matrix[3]) == 0.0)) {
 
             /* If the matrix is degenerate, force a scale to 1 unit. */
+            memset(&font_scale.matrix, 0x00, sizeof(font_scale.matrix));
             if (!font_scale.matrix[0])
                 font_scale.matrix[0] = 1;
             if (!font_scale.matrix[3])
@@ -1485,7 +1563,7 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
     }
     if (cr.metrics_type == gs_fapi_metrics_notdef && !bVertical) {
         code =
-            I->ff.fapi_get_metrics(&I->ff, &enc_char_name_string, index, sbw,
+            I->ff.fapi_get_metrics(&I->ff, &enc_char_name_string, (int)index, sbw,
                                    bVertical);
         if (code < 0)
             return code;
@@ -1695,7 +1773,7 @@ gs_fapi_do_char(gs_font *pfont, gs_gstate *pgs, gs_text_enum_t *penum, char *fon
             I->ff.fapi_set_cache(penum, pbfont, &enc_char_name_string, index,
                                  sbw + 2, &char_bbox, sbwp, &imagenow);
     }
-    
+
     /* If we can render the glyph now, do so.
      * we may not be able to in the PS world if there's a CDevProc in the font
      * in which case gs_fapi_finish_render() will be called from the PS
@@ -1740,18 +1818,18 @@ gs_fapi_get_font_info(gs_font *pfont, gs_fapi_font_info item, int index,
 int
 gs_fapi_passfont(gs_font *pfont, int subfont, char *font_file_path,
                  gs_string *full_font_buf, char *fapi_request, char *xlatmap,
-                 char **fapi_id, gs_fapi_get_server_param_callback get_server_param_cb)
+                 char **fapi_id, char **decodingID, gs_fapi_get_server_param_callback get_server_param_cb)
 {
     gs_font_base *pbfont;
     int code = 0;
     gs_fapi_server *I, **list;
     bool free_params = false;
-    gs_memory_t *mem = pfont->memory; 
-    const char *decodingID = NULL;
+    gs_memory_t *mem = pfont->memory;
+    const char *ldecodingID = NULL;
     bool do_restart = false;
-    
+
     list = gs_fapi_get_server_list(mem);
-    
+
     if (!list) /* Should never happen */
       return_error(gs_error_unregistered);
 
@@ -1820,8 +1898,10 @@ gs_fapi_passfont(gs_font *pfont, int subfont, char *font_file_path,
         pbfont->FAPI = I;       /* we need the FAPI server during this stage */
         code =
             gs_fapi_prepare_font(pfont, I, subfont, font_file_path,
-                                 full_font_buf, xlatmap, &decodingID);
+                                 full_font_buf, xlatmap, &ldecodingID);
         if (code >= 0) {
+            if (decodingID != NULL)
+                *decodingID = (char *)ldecodingID;
             (*fapi_id) = (char *)I->ig.d->subtype;
             return 0;
         }
