@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -124,10 +124,10 @@ typedef ulong ufixed;		/* only used in a very few places */
 #define fixed2float(x) ((x)*(1.0/fixed_scale))
 
 /* Rounding and truncation on fixeds */
-#define fixed_floor(x) ((x)&(-1L<<_fixed_shift))
-#define fixed_rounded(x) (((x)+_fixed_round_v)&(-1L<<_fixed_shift))
-#define fixed_ceiling(x) (((x)+_fixed_fraction_v)&(-1L<<_fixed_shift))
-#define fixed_pixround(x) (((x)+_fixed_pixround_v)&(-1L<<_fixed_shift))
+#define fixed_floor(x) ((x)&(-(1L<<_fixed_shift)))
+#define fixed_rounded(x) (((x)+_fixed_round_v)&(-(1L<<_fixed_shift)))
+#define fixed_ceiling(x) (((x)+_fixed_fraction_v)&(-(1L<<_fixed_shift)))
+#define fixed_pixround(x) (((x)+_fixed_pixround_v)&(-(1L<<_fixed_shift)))
 #define fixed_fraction(x) ((int)(x)&_fixed_fraction_v)
 /* I don't see how to do truncation towards 0 so easily.... */
 #define fixed_truncated(x) ((x) < 0 ? fixed_ceiling(x) : fixed_floor(x))
