@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -45,7 +45,7 @@ private_st_gs_image3x();
 
 /* Define the image type for ImageType 3x images. */
 const gx_image_type_t gs_image_type_3x = {
-    &st_gs_image3x, gx_begin_image3x, gx_data_image_source_size,
+    &st_gs_image3x, gx_begin_image3x,
     gx_image_no_sput, gx_image_no_sget, gx_image_default_release,
     IMAGE3X_IMAGETYPE
 };
@@ -132,7 +132,7 @@ gx_begin_image3x_generic(gx_device * dev,
                         const gx_clip_path *pcpath, gs_memory_t *mem,
                         image3x_make_mid_proc_t make_mid,
                         image3x_make_mcde_proc_t make_mcde,
-                        gx_image_enum_common_t **pinfo)
+                        gx_image_enum_common_t **pinfo, char *OC)
 {
     const gs_image3x_t *pim = (const gs_image3x_t *)pic;
     gx_image3x_enum_t *penum;
@@ -144,6 +144,7 @@ gx_begin_image3x_generic(gx_device * dev,
     gs_int_point origin[2];
     int code;
     int i;
+    gs_color_space *pmcs = NULL;
 
     /* Validate the parameters. */
     if (pim->Height <= 0)
@@ -207,9 +208,6 @@ gx_begin_image3x_generic(gx_device * dev,
          * The mask data has to be defined in a DevicePixel color space
          * of the correct depth so that no color mapping will occur.
          */
-        /****** FREE COLOR SPACE ON ERROR OR AT END ******/
-        gs_color_space *pmcs;
-
         if (penum->mask[i].depth == 0) {	/* mask not supplied */
             midev[i] = 0;
             minfo[i] = 0;
@@ -217,19 +215,19 @@ gx_begin_image3x_generic(gx_device * dev,
         }
         code = gs_cspace_new_DevicePixel(mem, &pmcs, penum->mask[i].depth);
         if (code < 0)
-            return code;
+            goto out1;
         mrect.p.x = mrect.p.y = 0;
         mrect.q.x = penum->mask[i].width;
         mrect.q.y = penum->mask[i].height;
         if ((code = gs_matrix_multiply(&mask[i].matrix, pmat, &mat)) < 0 ||
             (code = gs_bbox_transform(&mrect, &mat, &mrect)) < 0
             )
-            return code;
+            goto out1;
 
         /* Bug 700438: If the rectangle is out of range, bail */
         if (mrect.p.x >= (double)INT_MAX || mrect.q.x <= (double)INT_MIN ||
             mrect.p.y >= (double)INT_MAX || mrect.q.y <= (double)INT_MIN) {
-                code = gs_note_error(gs_error_rangecheck);
+            code = gs_note_error(gs_error_rangecheck);
             goto out1;
         }
 
@@ -292,6 +290,8 @@ gx_begin_image3x_generic(gx_device * dev,
         }
         midev[i] = mdev;
         minfo[i] = penum->mask[i].info;
+        rc_decrement_only(pmcs, "gx_begin_image3x_generic(pmcs)");
+        pmcs = NULL;
     }
     gs_image_t_init(&pixel.image, pim->ColorSpace);
     {
@@ -301,6 +301,9 @@ gx_begin_image3x_generic(gx_device * dev,
         pixel.image.type = type1;
         pixel.image.image_parent_type = gs_image_type3x;
     }
+    if (minfo[0] != NULL)
+        minfo[0]->OC = OC;
+
     code = make_mcde(dev, pgs, pmat, (const gs_image_common_t *)&pixel.image,
                      prect, pdcolor, pcpath, mem, &penum->pixel.info,
                      &pcdev, midev, minfo, origin, pim);
@@ -365,6 +368,7 @@ gx_begin_image3x_generic(gx_device * dev,
                        "gx_begin_image3x(mask[0].mdev)");
     }
   out1:
+    rc_decrement(pmcs, "gx_begin_image3x_generic(pmcs)");
     gs_free_object(mem, penum->mask[0].data, "gx_begin_image3x(mask[0].data)");
     gs_free_object(mem, penum->mask[1].data, "gx_begin_image3x(mask[1].data)");
     gs_free_object(mem, penum->pixel.data, "gx_begin_image3x(pixel.data)");
@@ -515,7 +519,7 @@ make_midx_default(gx_device **pmidev, gx_device *dev, int width, int height,
             return_error(gs_error_VMerror);
     if (mdproto == 0)
         return_error(gs_error_rangecheck);
-    midev = gs_alloc_struct(mem, gx_device_memory, &st_device_memory,
+    midev = gs_alloc_struct_immovable(mem, gx_device_memory, &st_device_memory,
                             "make_mid_default");
     if (midev == 0)
         return_error(gs_error_VMerror);
@@ -597,7 +601,7 @@ gx_begin_image3x(gx_device * dev,
 {
     return gx_begin_image3x_generic(dev, pgs, pmat, pic, prect, pdcolor,
                                     pcpath, mem, make_midx_default,
-                                    make_mcdex_default, pinfo);
+                                    make_mcdex_default, pinfo, 0);
 }
 
 /* Process the next piece of an ImageType 3 image. */
