@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -228,7 +228,7 @@ gx_pattern_accum_alloc(gs_memory_t * mem, gs_memory_t * storage_memory,
     size_t max_pattern_bitmap = tdev->MaxPatternBitmap == 0 ? MaxPatternBitmap_DEFAULT :
                                 tdev->MaxPatternBitmap;
 
-    pinst->is_planar = tdev->is_planar;
+    pinst->num_planar_planes = tdev->num_planar_planes;
     /*
      * If the target device can accumulate a pattern stream and the language
      * client supports high level patterns (ps and pdf only) we don't need a
@@ -247,9 +247,13 @@ gx_pattern_accum_alloc(gs_memory_t * mem, gs_memory_t * storage_memory,
         dev_spec_op))((gx_device *)pinst->saved->device,
         gxdso_pattern_can_accum, pinst, 0) == 1)
         force_no_clist = 1; /* Set only for first time through */
-    if (force_no_clist || (size < max_pattern_bitmap && !pinst->is_clist)
-        || pinst->templat.PaintType != 1 ) {
-        gx_device_pattern_accum *adev = gs_alloc_struct(mem, gx_device_pattern_accum,
+    /* If the blend mode in use is not Normal, then we CANNOT use a tile. What
+     * if the blend mode changes half way through the tile? We simply must use
+     * a clist. */
+    if (force_no_clist ||
+        (((size < max_pattern_bitmap && !pinst->is_clist)
+           || pinst->templat.PaintType != 1) && !pinst->templat.BM_Not_Normal)) {
+        gx_device_pattern_accum *adev = gs_alloc_struct_immovable(mem, gx_device_pattern_accum,
                         &st_device_pattern_accum, cname);
         if (adev == 0)
             return 0;
@@ -295,9 +299,10 @@ gx_pattern_accum_alloc(gs_memory_t * mem, gs_memory_t * storage_memory,
     }
     fdev->log2_align_mod = tdev->log2_align_mod;
     fdev->pad = tdev->pad;
-    fdev->is_planar = tdev->is_planar;
+    fdev->num_planar_planes = tdev->num_planar_planes;
     fdev->graphics_type_tag = tdev->graphics_type_tag;
     fdev->interpolate_control = tdev->interpolate_control;
+    fdev->non_strict_bounds = tdev->non_strict_bounds;
     gx_device_forward_fill_in_procs(fdev);
     return fdev;
 }
@@ -423,12 +428,13 @@ pattern_accum_open(gx_device * dev)
 #undef PDSET
                     bits->color_info = padev->color_info;
                     bits->bitmap_memory = mem;
-                    if (target->is_planar > 0)
+
+                    if (target->num_planar_planes > 0)
                     {
                         gx_render_plane_t planes[GX_DEVICE_COLOR_MAX_COMPONENTS];
-                        uchar num_comp = padev->color_info.num_components;
+                        uchar num_comp = padev->num_planar_planes;
                         uchar i;
-                        int depth = target->color_info.depth / target->color_info.num_components;
+                        int depth = target->color_info.depth / num_comp;
                         for (i = 0; i < num_comp; i++)
                         {
                             planes[i].shift = depth * (num_comp - 1 - i);
@@ -858,7 +864,7 @@ gx_pattern_alloc_cache(gs_memory_t * mem, uint num_tiles, ulong max_bits)
         tiles->index = i;
         tiles->cdev = NULL;
         tiles->ttrans = NULL;
-        tiles->is_planar = false;
+        tiles->num_planar_planes = 0;
     }
     return pcache;
 }
@@ -907,11 +913,11 @@ gstate_set_pattern_cache(gs_gstate * pgs, gx_pattern_cache * pcache)
 /* This will not free a pattern if it is 'locked' which should only be for */
 /* a stroke pattern during fill_stroke_path.                               */
 static void
-gx_pattern_cache_free_entry(gx_pattern_cache * pcache, gx_color_tile * ctile)
+gx_pattern_cache_free_entry(gx_pattern_cache * pcache, gx_color_tile * ctile, bool free_dummy)
 {
     gx_device *temp_device;
 
-    if ((ctile->id != gx_no_bitmap_id) && !ctile->is_dummy && !ctile->is_locked) {
+    if ((ctile->id != gx_no_bitmap_id) && (!ctile->is_dummy || free_dummy) && !ctile->is_locked) {
         gs_memory_t *mem = pcache->memory;
 
         /*
@@ -949,7 +955,7 @@ gx_pattern_cache_free_entry(gx_pattern_cache * pcache, gx_color_tile * ctile)
 
         if (ctile->ttrans != NULL) {
             if_debug2m('v', mem,
-                       "[v*] Freeing trans pattern from cache, uid = %ld id = %u\n",
+                       "[v*] Freeing trans pattern from cache, uid = %ld id = %ld\n",
                        ctile->uid.id, ctile->id);
             if ( ctile->ttrans->pdev14 == NULL) {
                 /* This can happen if we came from the clist */
@@ -1037,7 +1043,7 @@ gx_pattern_cache_ensure_space(gs_gstate * pgs, size_t needed)
     while (pcache->bits_used + needed > pcache->max_bits &&
            pcache->bits_used != 0) {
         pcache->next = (pcache->next + 1) % pcache->num_tiles;
-        gx_pattern_cache_free_entry(pcache, &pcache->tiles[pcache->next]);
+        gx_pattern_cache_free_entry(pcache, &pcache->tiles[pcache->next], false);
         /* since a pattern may be temporarily locked (stroke pattern for fill_stroke_path) */
         /* we may not have freed all entries even though we've scanned the entire cache.   */
         /* The following check for wrapping prevents infinite loop if stroke pattern was   */
@@ -1066,11 +1072,11 @@ gx_pattern_cache_update_used(gs_gstate *pgs, size_t used)
 static void make_bitmap(gx_strip_bitmap *, const gx_device_memory *, gx_bitmap_id, const gs_memory_t *);
 int
 gx_pattern_cache_add_entry(gs_gstate * pgs,
-                   gx_device_forward * fdev, gx_color_tile ** pctile, gs_gstate* saved)
+                   gx_device_forward * fdev, gx_color_tile ** pctile)
 {
     gx_pattern_cache *pcache;
     const gs_pattern1_instance_t *pinst;
-    ulong used = 0, mask_used = 0, trans_used = 0;
+    size_t used = 0, mask_used = 0, trans_used = 0;
     gx_bitmap_id id;
     gx_color_tile *ctile;
     int code = ensure_pattern_cache(pgs);
@@ -1162,9 +1168,9 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
     }
     id = pinst->id;
     ctile = gx_pattern_cache_find_tile_for_id(pcache, id);
-    gx_pattern_cache_free_entry(pcache, ctile);         /* ensure that this cache slot is empty */
+    gx_pattern_cache_free_entry(pcache, ctile, false);         /* ensure that this cache slot is empty */
     ctile->id = id;
-    ctile->is_planar = pinst->is_planar;
+    ctile->num_planar_planes = pinst->num_planar_planes;
     ctile->depth = fdev->color_info.depth;
     ctile->uid = pinst->templat.uid;
     ctile->tiling_type = pinst->templat.TilingType;
@@ -1174,12 +1180,7 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
     ctile->has_overlap = pinst->has_overlap;
     ctile->is_dummy = false;
     ctile->is_locked = false;
-    if (pinst->templat.uses_transparency) {
-        /* to work with pdfi get the blend mode out of the saved pgs device */
-        ctile->blending_mode = ((pdf14_device*)(saved->device))->blend_mode;
-    }
-    else
-        ctile->blending_mode = 0;
+    ctile->blending_mode = 0;
     ctile->trans_group_popped = false;
     if (dev_proc(fdev, open_device) != pattern_clist_open_device) {
         if (mbits != 0) {
@@ -1194,7 +1195,7 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
             ctile->tmask.data = 0;
         if (trans != 0) {
             if_debug2m('v', pgs->memory,
-                       "[v*] Adding trans pattern to cache, uid = %ld id = %u\n",
+                       "[v*] Adding trans pattern to cache, uid = %ld id = %ld\n",
                        ctile->uid.id, ctile->id);
             ctile->ttrans = trans;
         }
@@ -1255,7 +1256,7 @@ gx_pattern_cache_get_entry(gs_gstate * pgs, gs_id id, gx_color_tile ** pctile)
         return code;
     pcache = pgs->pattern_cache;
     ctile = gx_pattern_cache_find_tile_for_id(pcache, id);
-    gx_pattern_cache_free_entry(pgs->pattern_cache, ctile);
+    gx_pattern_cache_free_entry(pgs->pattern_cache, ctile, false);
     ctile->id = id;
     *pctile = ctile;
     return 0;
@@ -1282,7 +1283,7 @@ gx_pattern_cache_add_dummy_entry(gs_gstate *pgs,
         return code;
     pcache = pgs->pattern_cache;
     ctile = gx_pattern_cache_find_tile_for_id(pcache, id);
-    gx_pattern_cache_free_entry(pcache, ctile);
+    gx_pattern_cache_free_entry(pcache, ctile, false);
     ctile->id = id;
     ctile->depth = depth;
     ctile->uid = pinst->templat.uid;
@@ -1324,7 +1325,7 @@ dump_raw_pattern(int height, int width, int n_chan, int depth,
     byte *curr_ptr = Buffer;
     int plane_offset;
 
-    is_planar = mdev->is_planar;
+    is_planar = mdev->num_planar_planes > 0;
     max_bands = ( n_chan < 57 ? n_chan : 56);   /* Photoshop handles at most 56 bands */
     if (is_planar) {
         gs_snprintf(full_file_name, sizeof(full_file_name), "%d)PATTERN_PLANE_%dx%dx%d.raw", global_pat_index,
@@ -1400,7 +1401,7 @@ make_bitmap(register gx_strip_bitmap * pbm, const gx_device_memory * mdev,
     pbm->rep_height = pbm->size.y = mdev->height;
     pbm->id = id;
     pbm->rep_shift = pbm->shift = 0;
-    pbm->num_planes = (mdev->is_planar ? mdev->color_info.num_components : 1);
+    pbm->num_planes = mdev->num_planar_planes ? mdev->num_planar_planes : 1;
 
         /* Lets dump this for debug purposes */
 
@@ -1431,7 +1432,23 @@ gx_pattern_cache_winnow(gx_pattern_cache * pcache,
 
         ctile->is_locked = false;		/* force freeing */
         if (ctile->id != gx_no_bitmap_id && (*proc) (ctile, proc_data))
-            gx_pattern_cache_free_entry(pcache, ctile);
+            gx_pattern_cache_free_entry(pcache, ctile, false);
+    }
+}
+
+void
+gx_pattern_cache_flush(gx_pattern_cache * pcache)
+{
+    uint i;
+
+    if (pcache == 0)            /* no cache created yet */
+        return;
+    for (i = 0; i < pcache->num_tiles; ++i) {
+        gx_color_tile *ctile = &pcache->tiles[i];
+
+        ctile->is_locked = false;		/* force freeing */
+        if (ctile->id != gx_no_bitmap_id)
+            gx_pattern_cache_free_entry(pcache, ctile, true);
     }
 }
 
@@ -1526,9 +1543,9 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
     if (code < 0)
         goto fail;
     if (pinst->templat.uses_transparency) {
-        if_debug1m('v', mem, "gx_pattern_load: pushing the pdf14 compositor device into this graphics state pat_id = %u\n", pinst->id);
+        if_debug1m('v', mem, "gx_pattern_load: pushing the pdf14 compositor device into this graphics state pat_id = %ld\n", pinst->id);
         if ((code = gs_push_pdf14trans_device(saved, true, false, 0, 0)) < 0)   /* spot_color_count taken from pdf14 target values */
-            return code;
+            goto fail;
         saved->device->is_open = true;
     } else {
         /* For colored patterns we clear the pattern device's
@@ -1602,7 +1619,7 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
     }
     /* We REALLY don't like the following cast.... */
     code = gx_pattern_cache_add_entry((gs_gstate *)pgs,
-                adev, &ctile, saved);
+                adev, &ctile);
     if (code >= 0) {
         if (!gx_pattern_cache_lookup(pdc, pgs, dev, select)) {
             mlprintf(mem, "Pattern cache lookup failed after insertion!\n");
@@ -1647,7 +1664,7 @@ fail:
     }
     dev_proc(adev, close_device)((gx_device *)adev);
     gx_device_set_target(adev, NULL);
-    rc_decrement(adev, "gx_pattern_load");
+    gx_device_retain((gx_device *)adev, false);
     gs_gstate_free_chain(saved);
     return code;
 }
