@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -75,7 +75,7 @@ gx_monitor_t *gx_monitor_label(gx_monitor_t *mon, const char *name);
 #define gx_monitor_label(A,B) (A)
 #endif
 
-#define gx_monitor_enter(sema)  gp_monitor_enter(&(sema)->native)
-#define gx_monitor_leave(sema)  gp_monitor_leave(&(sema)->native)
+#define gx_monitor_enter(mon)  gp_monitor_enter(&(mon)->native)
+#define gx_monitor_leave(mon)  gp_monitor_leave(&(mon)->native)
 
 #endif /* !defined(gxsync_INCLUDED) */
