@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -300,7 +300,7 @@ gs_type1_endchar(gs_type1_state * pcis)
              * This is like Acrobat Reader behaves.
              */
             char buf0[gs_font_name_max + 1], buf1[30];
-            int l0 = min(pcis->pfont->font_name.size, sizeof(buf0) - 1);
+            int l0 = min(pcis->pfont->font_name.size - 1, sizeof(buf0) - 1);
             int l1 = min(gstr.size, sizeof(buf1) - 1);
 
             memcpy(buf0, pcis->pfont->font_name.chars, l0);
@@ -308,7 +308,7 @@ gs_type1_endchar(gs_type1_state * pcis)
             memcpy(buf1, gstr.data, l1);
             buf1[l1] = 0;
             emprintf2(pfont->memory,
-                      "The font '%s' misses the glyph '%s' . Continue skipping the glyph.",
+                      " Error processing accented character.\n    The font '%s' does not contain the accent glyph '%s'. Skipping the SEAC.\n\n",
                       buf0,
                       buf1);
             return 0;
@@ -346,7 +346,7 @@ type1_cis_get_metrics(const gs_type1_state * pcis, double psbw[4])
  * This is exported only for the benefit of font copying.
  */
 int
-gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
+gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont, /* lgtm[cpp/use-of-goto] */
                      const gs_glyph_data_t *pgd, gs_char *chars)
 {
     gs_type1_data *const pdata = &pfont->data;
@@ -389,13 +389,15 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
     for (;;) {
         uint c0;
 
-        if (cip > end)
+        if (cip >= end) {
              /* We used to treat buffer overrun as a simple invalid font, now we assume that
-             * there is an implicit endcharr.
+             * there is an implicit endchar/return.
              * Part of bug #693170 where the fonts are invalid (no endchar on some glyphs).
              */
+            if (call_depth > 0)
+                goto c_return;
             goto out;
-
+        }
         c0 = *cip++;
 
         charstring_next(c0, state, c, encrypted);
@@ -457,6 +459,7 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
             else {
                 c = fixed2int_var(*csp) + pdata->gsubrNumberBias;
             }
+            CS_CHECK_IPSTACK(ipsp + 1, ipstack);
             code = pdata->procs.subr_data
                 (pfont, c, true, &ipsp[1].cs_data);
             if (code < 0)
@@ -477,6 +480,7 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
             else {
                 c = fixed2int_var(*csp) + pdata->subroutineNumberBias;
             }
+            CS_CHECK_IPSTACK(ipsp + 1, ipstack);
             code = pdata->procs.subr_data
                 (pfont, c, false, &ipsp[1].cs_data);
             if (code < 0)
@@ -490,11 +494,13 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
             end = ipsp->cs_data.bits.data + ipsp->cs_data.bits.size;
             goto call;
         case c_return:
+c_return:
             if (call_depth == 0)
                 return (gs_note_error(gs_error_invalidfont));
             else
                 call_depth--;
             gs_glyph_data_free(&ipsp->cs_data, "gs_type1_piece_codes");
+            CS_CHECK_IPSTACK(ipsp, ipstack);
             --ipsp;
             if (ipsp < ipstack)
                 return (gs_note_error(gs_error_invalidfont));
@@ -542,21 +548,26 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
             case ce1_seac:
                 goto do_seac;
             case ce1_callothersubr:
-                switch (fixed2int_var(*csp)) {
-                default:
-                    goto out;
-                case 3:
-                    if (csp >= &(cstack[1]))
-                        csp -= 2;
-                    goto top;
-                case 12:
-                case 13:
-                case 14:
-                case 15:
-                case 16:
-                case 17:
-                case 18:
-                    cnext;
+                if (CS_CHECK_CSTACK_BOUNDS(csp, cstack)) {
+                    switch (fixed2int_var(*csp)) {
+                    default:
+                        goto out;
+                    case 3:
+                        if (csp >= &(cstack[1]))
+                            csp -= 2;
+                        goto top;
+                    case 12:
+                    case 13:
+                    case 14:
+                    case 15:
+                    case 16:
+                    case 17:
+                    case 18:
+                        cnext;
+                    }
+                }
+                else {
+                    return_error(gs_error_invalidfont);
                 }
             }
         }
