@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -59,6 +59,9 @@ s_CFD_init(stream_state * st)
     if (raster < 0)
         return ERRC;
 
+    if (ss->Columns <= 0)
+        return ERRC;
+
     s_hcd_init_inline(ss);
     /* Because skip_white_pixels can look as many as 4 bytes ahead, */
     /* we need to allow 4 extra bytes at the end of the row buffers. */
@@ -211,17 +214,18 @@ static inline int skip_data(stream_CFD_state *ss, stream_cursor_read *pr, int rl
 
 static inline int invert_data(stream_CFD_state *ss, stream_cursor_read *pr, int *rlen, byte black_byte)
 {
+    byte *qlim = ss->lbuf + ss->raster + CFD_BUFFER_SLOP;
     cfd_declare_state;
     cfd_load_state();
     (void)rlimit;
 
-    if (q >= ss->lbuf + ss->raster + CFD_BUFFER_SLOP || q < ss->lbufstart) {
+    if (q >= qlim || q < ss->lbufstart) {
         return(-1);
     }
 
     if ( (*rlen) > qbit )
     {
-        if (q + ((*rlen - qbit) >> 3) > ss->lbuf + ss->raster + CFD_BUFFER_SLOP) {
+        if (q + ((*rlen - qbit) >> 3) > qlim) {
             return(-1);
         }
 
@@ -233,6 +237,10 @@ static inline int invert_data(stream_CFD_state *ss, stream_cursor_read *pr, int
         }
         (*rlen) -= qbit;
 
+        if (q + ((*rlen) >> 3) >= qlim) {
+            return(-1);
+        }
+
         switch ( (*rlen) >> 3 )
         {
           case 7:         /* original rlen possibly >= 64 */
