@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -236,7 +236,10 @@ static void sjpx_info_callback(const char *msg, void *ptr)
 
 static void sjpx_warning_callback(const char *msg, void *ptr)
 {
+#ifdef DEBUG
+	/* prevent too many messages during normal build */
 	dlprintf1("openjpeg warning: %s", msg);
+#endif
 }
 
 /* initialize the stream */
@@ -435,25 +438,43 @@ static int decode_image(stream_jpxd_state * const state)
     {
         case OPJ_CLRSPC_GRAY:
             state->colorspace = gs_jpx_cs_gray;
-            break;
-        case OPJ_CLRSPC_UNKNOWN: /* make the best guess based on number of channels */
-        {
-            if (numprimcomp < 3)
-            {
-                state->colorspace = gs_jpx_cs_gray;
+            if (numprimcomp > 1) {
+                dmprintf1(state->memory, "openjpeg warning: Ignoring extra components for %d component Gray data.\n", numprimcomp);
+                numprimcomp = 1;
             }
-            else if (numprimcomp == 4)
-            {
+            break;
+        case OPJ_CLRSPC_SRGB:
+        case OPJ_CLRSPC_SYCC:
+        case OPJ_CLRSPC_EYCC:
+            state->colorspace = gs_jpx_cs_rgb;
+            if (numprimcomp > 3) {
+                dmprintf1(state->memory, "openjpeg warning: Ignoring extra components for %d component RGB data.\n", numprimcomp);
+                numprimcomp = 3;
+            }
+           break;
+        case OPJ_CLRSPC_CMYK:
+            state->colorspace = gs_jpx_cs_cmyk;
+            if (numprimcomp > 4) {
+                dmprintf1(state->memory, "openjpeg warning: Ignoring extra components for %d component CMYK data.\n", numprimcomp);
+                numprimcomp = 4;
+            }
+            break;
+        default:
+        case OPJ_CLRSPC_UNSPECIFIED:
+        case OPJ_CLRSPC_UNKNOWN:
+            if (numprimcomp == 1) {
+                dmprintf1(state->memory, "openjpeg warning: unspec CS. %d component so assuming gray.\n", numprimcomp);
+                state->colorspace = gs_jpx_cs_gray;
+            } else if (numprimcomp == 4) {
+                dmprintf1(state->memory, "openjpeg warning: unspec CS. %d components so assuming CMYK.\n", numprimcomp);
                 state->colorspace = gs_jpx_cs_cmyk;
-             }
-            else
-            {
+            } else {
+                /* Note, numprimcomp > 4 possible here. Bug 694909.
+                   Trust that it is RGB though.  Do not set numprimcomp */
+                dmprintf1(state->memory, "openjpeg warning: unspec CS. %d components. Assuming data RGB.\n", numprimcomp);
                 state->colorspace = gs_jpx_cs_rgb;
             }
             break;
-        }
-        default: /* OPJ_CLRSPC_SRGB, OPJ_CLRSPC_SYCC, OPJ_CLRSPC_EYCC */
-            state->colorspace = gs_jpx_cs_rgb;
     }
 
     state->alpha_comp = -1;
@@ -560,12 +581,12 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
                             for (b=0; b<bytepp1; b++)
                                 *row++ = (((*(state->pdata[compno]) << shift_bit) >> (8*(bytepp1-b-1))))
                                                         + (b==0 ? state->sign_comps[compno] : 0); /* split and shift input int to output bytes */
-                            state->pdata[compno]++; 
+                            state->pdata[compno]++;
                         }
                     }
                 }
                 else
-                {   
+                {
                     /* shift_bit = 0, bpp < 8 */
                     int bt=0;
                     int bit_pos = 0;
@@ -600,7 +621,10 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
                 {
                     for (i = 0; i < state->width; i++)
                     {
-                        int in_offset_scaled = (y_offset/state->image->comps[state->alpha_comp].dy*state->width + i)/state->image->comps[state->alpha_comp].dx;
+                        int dx = state->image->comps[state->alpha_comp].dx;
+                        int dy = state->image->comps[state->alpha_comp].dy;
+                        int w = state->image->comps[state->alpha_comp].w;
+                        int in_offset_scaled = (y_offset/dy * w) + i / dx;
                         for (b=0; b<bytepp1; b++)
                             *row++ = (((state->image->comps[state->alpha_comp].data[in_offset_scaled] << shift_bit) >> (8*(bytepp1-b-1))))
                                                                      + (b==0 ? state->sign_comps[state->alpha_comp] : 0);
@@ -612,7 +636,10 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
                     {
                         for (compno=0; compno<img_numcomps; compno++)
                         {
-                            int in_offset_scaled = (y_offset/state->image->comps[compno].dy*state->width + i)/state->image->comps[compno].dx;
+                            int dx = state->image->comps[compno].dx;
+                            int dy = state->image->comps[compno].dy;
+                            int w = state->image->comps[compno].w;
+                            int in_offset_scaled = (y_offset/dy * w) + i / dx;
                             for (b=0; b<bytepp1; b++)
                                 *row++ = (((state->image->comps[compno].data[in_offset_scaled] << shift_bit) >> (8*(bytepp1-b-1))))
                                                                                 + (b==0 ? state->sign_comps[compno] : 0);
@@ -631,7 +658,10 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
                 {
                     for (b=0; b<ppbyte1; b++)
                     {
-                        int in_offset_scaled = (y_offset/state->image->comps[compno].dy*state->width + i)/state->image->comps[compno].dx;
+                        int dx = state->image->comps[compno].dx;
+                        int dy = state->image->comps[compno].dy;
+                        int w = state->image->comps[compno].w;
+                        int in_offset_scaled = (y_offset/dy * w) + i / dx;
                         bt = bt<<state->bpp;
                         bt += state->image->comps[compno].data[in_offset_scaled] + state->sign_comps[compno];
                     }
@@ -678,15 +708,26 @@ s_opjd_process(stream_state * ss, stream_cursor_read * pr,
     int locked = 0;
     int code;
 
-    if (in_size > 0) 
+    if (in_size > 0)
     {
+        if (state->PassThrough && state->PassThroughfn) {
+            if (state->PassThrough && state->PassThroughfn && !state->StartedPassThrough) {
+                state->StartedPassThrough = 1;
+                (state->PassThroughfn)(state->device, NULL, 1);
+            }
+            (state->PassThroughfn)(state->device, (byte *)pr->ptr + 1, (byte *)pr->limit - (byte *)pr->ptr);
+        }
+
         /* buffer available data */
         code = opj_lock(ss->memory);
         if (code < 0) return code;
         locked = 1;
 
         code = s_opjd_accumulate_input(state, pr);
-        if (code < 0) return code;
+        if (code < 0) {
+            (void)opj_unlock(ss->memory);
+            return code;
+        }
 
         if (state->codec == NULL) {
             /* state->sb.size is non-zero after successful
@@ -703,7 +744,7 @@ s_opjd_process(stream_state * ss, stream_cursor_read * pr,
         }
     }
 
-    if (last == 1) 
+    if (last == 1)
     {
         if (state->image == NULL)
         {
@@ -755,6 +796,10 @@ s_opjd_set_defaults(stream_state * ss) {
 
     state->alpha = false;
     state->colorspace = gs_jpx_cs_rgb;
+    state->StartedPassThrough = 0;
+    state->PassThrough = 0;
+    state->PassThroughfn = NULL;
+    state->device = (void *)NULL;
 }
 
 /* stream release.
@@ -765,6 +810,10 @@ s_opjd_release(stream_state *ss)
 {
     stream_jpxd_state *const state = (stream_jpxd_state *) ss;
 
+    if (state->PassThrough && state->PassThroughfn && state->StartedPassThrough) {
+        state->StartedPassThrough = 0;
+        (state->PassThroughfn)(state->device, NULL, 0);
+    }
     /* empty stream or failed to accumulate */
     if (state->codec == NULL)
         return;
@@ -774,11 +823,11 @@ s_opjd_release(stream_state *ss)
     /* free image data structure */
     if (state->image)
         opj_image_destroy(state->image);
-    
+
     /* free stream */
     if (state->stream)
         opj_stream_destroy(state->stream);
-		
+
     /* free decoder handle */
     if (state->codec)
 	opj_destroy_codec(state->codec);
