@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -35,7 +35,7 @@
 #    define unlink(fname) delete(fname)
 #  endif
 #else
-#if !defined(const)
+#if !defined(const) && !defined(_WIN32) && !defined(_WIN64)
 /*
  * Other systems may or may not declare unlink in stdio.h;
  * if they do, the declaration will be compatible with this one, as long
