@@ -113,9 +113,9 @@ copies.  */
 
 /* Margins are left, bottom, right, top. */
 /* left bottom right top */
-#define MJ700V2C_MARGINS_A4      0.118, 0.52, 0.118, 0.33465
-#define MJ6000C_MARGINS_A2      0.948, 0.52, 1.969, 0.33465
-#define MJ8000C_MARGINS_A2      0.194, 0.52, 0.194, 0.33465
+#define MJ700V2C_MARGINS_A4     0.118f, 0.52f, 0.118f, 0.33465f
+#define MJ6000C_MARGINS_A2      0.948f, 0.52f, 1.969f, 0.33465f
+#define MJ8000C_MARGINS_A2      0.194f, 0.52f, 0.194f, 0.33465f
 
 /* Define bits-per-pixel for generic drivers - default is 24-bit mode */
 #ifndef BITSPERPIXEL
@@ -208,96 +208,81 @@ typedef struct gx_device_mj_s gx_device_mj;
          (bpp == 32 ? 3 : 1), dns, r, g, b, k, drct, mcrwv, dtsz \
 }
 
-#define mj_colour_procs(proc_colour_open, proc_get_params, proc_put_params)\
-{	proc_colour_open,\
-        gx_default_get_initial_matrix,\
-        gx_default_sync_output,\
-        gdev_prn_output_page,\
-        gdev_prn_close,\
-        gdev_mjc_map_rgb_color,\
-        gdev_mjc_map_color_rgb,\
-        NULL,	/* fill_rectangle */\
-        NULL,	/* tile_rectangle */\
-        NULL,	/* copy_mono */\
-        NULL,	/* copy_color */\
-        NULL,	/* draw_line */\
-        gx_default_get_bits,\
-        proc_get_params,\
-        proc_put_params,\
-        NULL,	/* map_cmyk_color */\
-        NULL,	/* get_xfont_procs */\
-        NULL,	/* get_xfont_device */\
-        NULL,	/* map_rgb_alpha_color */\
-        NULL,   /* get_page_device */\
-        NULL,	/* get_alpha_bits */\
-        NULL,   /* copy_alpha */\
-        NULL,	/* get_band */\
-        NULL,	/* copy_rop */\
-        NULL,	/* fill_path */\
-        NULL,	/* stroke_path */\
-        NULL,	/* fill_mask */\
-        NULL,	/* fill_trapezoid */\
-        NULL,	/* fill_parallelogram */\
-        NULL,	/* fill_triangle */\
-        NULL,	/* draw_thin_line */\
-        NULL,	/* begin_image */\
-        NULL,	/* image_data */\
-        NULL,	/* end_image */\
-        NULL,	/* strip_tile_rectangle */\
-        NULL,	/* strip_copy_rop, */\
-        NULL,	/* get_clipping_box */\
-        NULL,	/* begin_typed_image */\
-        NULL,	/* get_bits_rectangle */\
-        NULL,	/* map_color_rgb_alpha */\
-        NULL,	/* create_compositor */\
-        NULL,	/* get_hardware_params */\
-        NULL,	/* text_begin */\
-        NULL,	/* finish_copydevice */\
-        NULL, 	/* begin_transparency_group */\
-        NULL, 	/* end_transparency_group */\
-        NULL, 	/* begin_transparency_mask */\
-        NULL, 	/* end_transparency_mask */\
-        NULL, 	/* discard_transparency_layer */\
-        NULL,   /* get_color_mapping_procs */\
-        NULL,   /* get_color_comp_index */\
-        gdev_mjc_encode_color,\
-        gdev_mjc_decode_color\
+static void
+mj_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs(dev);
+
+    set_dev_proc(dev, map_rgb_color, gdev_mjc_map_rgb_color);
+    set_dev_proc(dev, map_color_rgb, gdev_mjc_map_color_rgb);
+    set_dev_proc(dev, get_params, mj_get_params);
+    set_dev_proc(dev, encode_color, gdev_mjc_encode_color);
+    set_dev_proc(dev, decode_color, gdev_mjc_decode_color);
 }
 
-static gx_device_procs mj700v2c_procs =
-mj_colour_procs(mj700v2c_open, mj_get_params, mj700v2c_put_params);
+static void
+mj700v2c_initialize_device_procs(gx_device *dev)
+{
+    mj_initialize_device_procs(dev);
 
-static gx_device_procs mj500c_procs =
-mj_colour_procs(mj500c_open, mj_get_params, mj500c_put_params);
+    set_dev_proc(dev, open_device, mj700v2c_open);
+    set_dev_proc(dev, put_params, mj700v2c_put_params);
+}
 
-static gx_device_procs mj6000c_procs =
-mj_colour_procs(mj6000c_open, mj_get_params, mj700v2c_put_params);
+static void
+mj500c_initialize_device_procs(gx_device *dev)
+{
+    mj_initialize_device_procs(dev);
 
-static gx_device_procs mj8000c_procs =
-mj_colour_procs(mj8000c_open, mj_get_params, mj700v2c_put_params);
+    set_dev_proc(dev, open_device, mj500c_open);
+    set_dev_proc(dev, put_params, mj500c_put_params);
+}
+
+static void
+mj6000c_initialize_device_procs(gx_device *dev)
+{
+    mj_initialize_device_procs(dev);
+
+    set_dev_proc(dev, open_device, mj6000c_open);
+    set_dev_proc(dev, put_params, mj700v2c_put_params);
+}
+
+static void
+mj8000c_initialize_device_procs(gx_device *dev)
+{
+    mj_initialize_device_procs(dev);
+
+    set_dev_proc(dev, open_device, mj8000c_open);
+    set_dev_proc(dev, put_params, mj700v2c_put_params);
+}
 
 gx_device_mj far_data gs_mj700v2c_device =
-mjcmyk_device(mj700v2c_procs, "mj700v2c", 360, 360, BITSPERPIXEL,
-          mj700v2c_print_page, 1024, 1024, 1024, 1024, 1024, 0, 1, 1);
+mjcmyk_device(mj700v2c_initialize_device_procs, "mj700v2c",
+              360, 360, BITSPERPIXEL,
+              mj700v2c_print_page,
+              1024, 1024, 1024, 1024, 1024, 0, 1, 1);
 
 gx_device_mj far_data gs_mj500c_device =
-mjcmy_device(mj500c_procs, "mj500c", 360, 360, BITSPERPIXEL,
-          mj500c_print_page, 1024, 1024, 1024, 1024, 1024, 0, 1, 1);
+mjcmy_device(mj500c_initialize_device_procs, "mj500c",
+             360, 360, BITSPERPIXEL,
+             mj500c_print_page, 1024, 1024, 1024, 1024, 1024, 0, 1, 1);
 
 gx_device_mj far_data gs_mj6000c_device =
-mjcmyk_device(mj6000c_procs, "mj6000c", 360, 360, BITSPERPIXEL,
-          mj6000c_print_page, 1024, 1024, 1024, 1024, 1024, 0, 1, 1);
+mjcmyk_device(mj6000c_initialize_device_procs, "mj6000c",
+              360, 360, BITSPERPIXEL,
+              mj6000c_print_page, 1024, 1024, 1024, 1024, 1024, 0, 1, 1);
 
 gx_device_mj far_data gs_mj8000c_device =
-mjcmyk_device(mj8000c_procs, "mj8000c", 360, 360, BITSPERPIXEL,
-          mj8000c_print_page, 1024, 1024, 1024, 1024, 1024, 0, 1, 1);
+mjcmyk_device(mj8000c_initialize_device_procs, "mj8000c",
+              360, 360, BITSPERPIXEL,
+              mj8000c_print_page, 1024, 1024, 1024, 1024, 1024, 0, 1, 1);
 
 /* Get the paper size code, based on width and height. */
 static int
 gdev_mjc_paper_size(gx_device *dev)
 {
-  int width = dev->MediaSize[0];
-  int height = dev->MediaSize[1];
+  int width = (int)dev->MediaSize[0];
+  int height = (int)dev->MediaSize[1];
 
   if (width == 1190 && height == 1684)
     return PAPER_SIZE_A2;
@@ -333,8 +318,8 @@ mj8000c_open(gx_device * pdev)
 static int
 mj_open(gx_device *pdev, int ptype)
 {       /* Change the margins if necessary. */
-  int xdpi = pdev->x_pixels_per_inch;
-  int ydpi = pdev->y_pixels_per_inch;
+  int xdpi = (int)pdev->x_pixels_per_inch;
+  int ydpi = (int)pdev->y_pixels_per_inch;
 
   static const float mj_margin[4] = { MJ700V2C_MARGINS_A4 };
   static const float mj6000c_a2[4] = { MJ6000C_MARGINS_A2 };
@@ -365,7 +350,7 @@ mj_open(gx_device *pdev, int ptype)
   gx_device_set_margins(pdev, m, true);
 
   if (mj->colorcomp == 3)
-    mj->density = mj->density * 720 / ydpi * 1.5;
+    mj->density = (int)(mj->density * 720 / ydpi) * 1.5;
   else
     mj->density = mj->density * 720 / ydpi;
 
@@ -440,6 +425,7 @@ mj_put_params(gx_device *pdev,  gs_param_list *plist, int ptype)
         code = put_param_int(plist, "Magenta", &magenta, 0, INT_MAX, code);
         code = put_param_int(plist, "Yellow", &yellow, 0, INT_MAX, code);
         code = put_param_int(plist, "Black", &black, 0, INT_MAX, code);
+        (void) code;
 
         if ((code = param_read_bool(plist,
                                      (param_name = "Unidirectional"),
@@ -667,7 +653,7 @@ mj_raster_cmd(int c_id, int in_size, byte* in, byte* buf2,
        ** walk forward, looking for matches:
        */
 
-      for( q++ ; *q == *p && q < in_end ; q++ ) {
+      for( q++ ; q < in_end && *q == *p ; q++ ) {
         if( (q-p) >= 128 ) {
           if( p > inp ) {
             count = p - inp;
@@ -974,12 +960,12 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
   int *errors[2];
   byte *data[4], *plane_data[4][4], *out_data;
   byte *out_row;
-  word *storage;
+  word *storage = NULL;
   uint storage_size_words;
   uint mj_tmp_buf_size;
-  byte* mj_tmp_buf;
+  byte* mj_tmp_buf = NULL;
   int xtalbuff_size;
-  short *xtalbuff;
+  short *xtalbuff = NULL;
   short *Cbar[16];
   short *Mbar[16];
   short *Ybar[16];
@@ -988,6 +974,7 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
   short *Mbuf[2];
   short *Ybuf[2];
   short *Kbuf[2];
+  int code = 0;
 
   /* Tricks and cheats ... */
   if (num_comps == 3) num_comps = 4;            /* print CMYK */
@@ -998,7 +985,6 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
   plane_size = calc_buffsize(line_size, storage_bpp);
 
   if (bits_per_pixel == 1) {            /* Data printed direct from i/p */
-    databuff_size = 0;                  /* so no data buffer required, */
     outbuff_size = plane_size * 4;      /* but need separate output buffers */
   }
 
@@ -1017,10 +1003,11 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
 /* NOZ */
   xtalbuff_size = plane_size*8 + 64;
   xtalbuff = (short *) gs_malloc(pdev->memory->non_gc_memory,  xtalbuff_size*(16*4+2*4) , W, "mj_colour_print_barrier");
-  memset(xtalbuff, 0, xtalbuff_size*(16*4+2*4) * W);
-  {
+  if (xtalbuff) {
         int i;
         short *p = xtalbuff + 16;
+
+        memset(xtalbuff, 0, xtalbuff_size*(16*4+2*4) * W);
         for ( i = 0 ; i < 16 ; i++ ) {
                 Cbar[i] = p;
                 p += xtalbuff_size;
@@ -1053,7 +1040,10 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
         p += xtalbuff_size;
         Kbuf[1] = p;
         p += xtalbuff_size;
+        (void) p;
   }
+  else
+      goto cleanup;
 
   storage = (word *) gs_malloc(pdev->memory->non_gc_memory, storage_size_words, W, "mj_colour_print_page");
 
@@ -1080,8 +1070,10 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
    *   plane_data:  4  (scan direction and alternating buffers)
    */
 
-  if (storage == NULL || mj_tmp_buf == NULL) /* can't allocate working area */
-    return_error(gs_error_VMerror);
+  if (storage == NULL || mj_tmp_buf == NULL) { /* can't allocate working area */
+    code = gs_note_error(gs_error_VMerror);
+    goto cleanup;
+  }
   else {
     int i;
     byte *p = out_data = out_row = (byte *)storage;
@@ -1105,7 +1097,6 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
       p += plane_size;
     }
     if (bits_per_pixel == 1) {
-      out_data = out_row = p;	  /* size is outbuff_size * 4 */
       data[1] += databuff_size;   /* coincides with plane_data pointers */
       data[3] += databuff_size;
     }
@@ -1174,17 +1165,14 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
 #define MOFFSET (pdev->t_margin - MJ700V2C_PRINT_LIMIT) /* Print position */
 
   {
-    int MJ_MARGIN_MM = 55;
-    uint top_skip = ( MJ_MARGIN_MM  * pdev->y_pixels_per_inch ) / 254;
-    top_skip = (top_skip ^ (-1)) & 65536;
     gp_fwrite("\033(V\2\0\0\0",sizeof(byte), 7, prn_stream);
     gp_fwrite("\033(v\2\0\0\xff",sizeof(byte), 7, prn_stream);
   }
 
   /* Send each scan line in turn */
   {
-    long int lend = pdev->height -
-      (dev_t_margin_points(pdev) + dev_b_margin_points(pdev));
+    long int lend = (int)(pdev->height -
+      (dev_t_margin_points(pdev) + dev_b_margin_points(pdev)));
     int cErr, mErr, yErr, kErr;
     int this_pass, i;
     long int lnum;
@@ -1414,12 +1402,13 @@ mj_print_page(gx_device_printer * pdev, gp_file * prn_stream, int ptype)
     gp_fputs("\f\033@", prn_stream);
     gp_fflush(prn_stream);
   }
+cleanup:
   /* free temporary storage */
   gs_free(pdev->memory->non_gc_memory, (char *) storage, storage_size_words, W, "mj_colour_print_page");
   gs_free(pdev->memory->non_gc_memory, (char *) mj_tmp_buf, mj_tmp_buf_size, W, "mj_raster_buffer");
   gs_free(pdev->memory->non_gc_memory, (char *) xtalbuff , xtalbuff_size*(16*4+2*4) , W, "mj_colour_print_barrier");
 
-  return 0;
+  return code;
 }
 
 static void
@@ -1441,14 +1430,6 @@ mj_color_correct(gx_color_value *Rptr ,gx_color_value *Gptr , gx_color_value *Bp
                         *Gptr = M;
                         *Bptr = Y;
                         return;
-                } else if (G>B) {				/* R=G>B */
-                        D = G-B;
-                        Wa  = R;
-                        H  = 256;
-                } else {						/* B>R=G */
-                        D = G-B;
-                        Wa = R;
-                        H = 1024;
                 }
         }
 
@@ -1504,7 +1485,10 @@ mj_color_correct(gx_color_value *Rptr ,gx_color_value *Gptr , gx_color_value *Bp
         if (Y<0)
                 Y=0;
 
-        if(H>256 && H<1024){		/* green correct */
+        /* 2019-10-29 this used to be 'if(H>256 && H<1024)', which can then go
+        beyond bounds of the 512-element grnsep2[]. So have patched up to avoid
+        this, but without any proper idea about what's going on. */
+        if(H>256 && H<768){		/* green correct */
                 short	work;
                 work=(((long)grnsep[M]*(long)grnsep2[H-256])>>16);
                 C+=work;
@@ -1613,12 +1597,12 @@ gdev_mjc_map_color_rgb(gx_device *pdev, gx_color_index color,
           prgb[2] = -(c >> 2);
         }
       else
-        { gx_color_value value = (gx_color_value)color ^ 0xff;
+        { gx_color_index value = (gx_color_index)color ^ 0xff;
           prgb[0] = prgb[1] = prgb[2] = (value << 8) + value;
         }
     break;
   case 16:
-    { gx_color_value c = (gx_color_value)color ^ 0xffff;
+    { gx_color_index c = (gx_color_index)color ^ 0xffff;
       ushort value = c >> 11;
       prgb[0] = ((value << 11) + (value << 6) + (value << 1) +
                  (value >> 4)) >> (16 - gx_color_value_bits);
@@ -1631,7 +1615,7 @@ gdev_mjc_map_color_rgb(gx_device *pdev, gx_color_index color,
     }
     break;
   case 24:
-    { gx_color_value c = (gx_color_value)color ^ 0xffffff;
+    { gx_color_index c = (gx_color_index)color ^ 0xffffff;
       prgb[0] = gx_color_value_from_byte(c >> 16);
       prgb[1] = gx_color_value_from_byte((c >> 8) & 0xff);
       prgb[2] = gx_color_value_from_byte(c & 0xff);
