@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /*
@@ -87,7 +87,8 @@ static dev_proc_print_page(escp2_print_page);
 
 /* Stylus 800 device */
 const gx_device_printer far_data gs_st800_device =
-  prn_device(prn_bg_procs, "st800",	/* The print_page proc is compatible with allowing bg printing */
+  /* The print_page proc is compatible with allowing bg printing */
+  prn_device(gdev_prn_initialize_device_procs_mono_bg, "st800",
         DEFAULT_WIDTH_10THS,
         DEFAULT_HEIGHT_10THS,
         X_DPI, Y_DPI,
@@ -96,7 +97,8 @@ const gx_device_printer far_data gs_st800_device =
 
 /* AP3250 device */
 const gx_device_printer far_data gs_ap3250_device =
-  prn_device(prn_bg_procs, "ap3250",	/* The print_page proc is compatible with allowing bg printing */
+  /* The print_page proc is compatible with allowing bg printing */
+  prn_device(gdev_prn_initialize_device_procs_mono_bg, "ap3250",
         DEFAULT_WIDTH_10THS,
         DEFAULT_HEIGHT_10THS,
         X_DPI, Y_DPI,
@@ -160,7 +162,7 @@ escp2_print_page(gx_device_printer *pdev, gp_file *prn_stream)
         ** margin measured from the *top* of the page:
         */
 
-        fwrite("\033(U\001\0\n\033(C\002\0t\020\033(c\004\0\0\0t\020",
+        gp_fwrite("\033(U\001\0\n\033(C\002\0t\020\033(c\004\0\0\0t\020",
                                                         1, 22, prn_stream);
 #endif
 
