@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /*
@@ -228,14 +228,20 @@ static dev_proc_open_device(imagen_prn_open);
 static dev_proc_close_device(imagen_prn_close);
 
 /* Since the print_page doesn't alter the device, this device can print in the background */
-gx_device_procs imagen_procs =
-        prn_procs(imagen_prn_open, gdev_prn_bg_output_page, imagen_prn_close);
+static void
+imagen_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_mono_bg(dev);
+
+    set_dev_proc(dev, open_device, imagen_prn_open);
+    set_dev_proc(dev, close_device, imagen_prn_close);
+}
 
 #define ppdev ((gx_device_printer *)pdev)
 
 /*-------------------------------------------*/
 const gx_device_printer far_data gs_imagen_device =
-  prn_device(/*prn_std_procs*/ imagen_procs,
+  prn_device(imagen_initialize_device_procs,
         "imagen",
         WIDTH_10THS,
         HEIGHT_10THS,
