@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 #include "stdint_.h"
@@ -32,11 +32,13 @@
 
 #include "gdevpdfimg.h"
 
-#define	    COMPRESSION_NONE	1	/* dump mode */
-#define	    COMPRESSION_LZW		2       /* Lempel-Ziv  & Welch */
-#define	    COMPRESSION_FLATE	3
-#define	    COMPRESSION_JPEG	4
-#define	    COMPRESSION_RLE		5
+enum {
+    COMPRESSION_NONE	= 1,	/* dump mode */
+    COMPRESSION_LZW	= 2,    /* Lempel-Ziv & Welch */
+    COMPRESSION_FLATE	= 3,
+    COMPRESSION_JPEG	= 4,
+    COMPRESSION_RLE	= 5
+};
 
 static struct compression_string {
     unsigned char id;
@@ -55,19 +57,23 @@ int PCLm_close(gx_device * pdev);
 
 
 /* ------ The pdfimage8 device ------ */
+static void
+pdfimage8_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_gray(dev);
 
-static const gx_device_procs pdfimage8_procs =
-prn_color_params_procs(pdf_image_open,
-                       gdev_prn_output_page_seekable,
-                       pdf_image_close,
-                       gx_default_gray_map_rgb_color,
-                       gx_default_gray_map_color_rgb,
-                       pdf_image_get_params_downscale,
-                       pdf_image_put_params_downscale);
+    set_dev_proc(dev, open_device, pdf_image_open);
+    set_dev_proc(dev, output_page, gdev_prn_output_page);
+    set_dev_proc(dev, close_device, pdf_image_close);
+    set_dev_proc(dev, get_params, pdf_image_get_params_downscale);
+    set_dev_proc(dev, put_params, pdf_image_put_params_downscale);
+    set_dev_proc(dev, encode_color, gx_default_8bit_map_gray_color);
+    set_dev_proc(dev, decode_color, gx_default_8bit_map_color_gray);
+}
 
 const gx_device_pdf_image gs_pdfimage8_device = {
     prn_device_body(gx_device_pdf_image,
-                    pdfimage8_procs,
+                    pdfimage8_initialize_device_procs,
                     "pdfimage8",
                     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                     600, 600,   /* 600 dpi by default */
@@ -84,19 +90,21 @@ const gx_device_pdf_image gs_pdfimage8_device = {
 };
 
 /* ------ The pdfimage24 device ------ */
+static void
+pdfimage24_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_rgb(dev);
 
-static const gx_device_procs pdfimage24_procs =
-prn_color_params_procs(pdf_image_open,
-                       gdev_prn_output_page_seekable,
-                       pdf_image_close,
-                       gx_default_rgb_map_rgb_color,
-                       gx_default_rgb_map_color_rgb,
-                       pdf_image_get_params_downscale,
-                       pdf_image_put_params_downscale);
+    set_dev_proc(dev, open_device, pdf_image_open);
+    set_dev_proc(dev, output_page, gdev_prn_output_page);
+    set_dev_proc(dev, close_device, pdf_image_close);
+    set_dev_proc(dev, get_params, pdf_image_get_params_downscale);
+    set_dev_proc(dev, put_params, pdf_image_put_params_downscale);
+}
 
 const gx_device_pdf_image gs_pdfimage24_device = {
     prn_device_body(gx_device_pdf_image,
-                    pdfimage24_procs,
+                    pdfimage24_initialize_device_procs,
                     "pdfimage24",
                     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                     600, 600,   /* 600 dpi by default */
@@ -113,17 +121,21 @@ const gx_device_pdf_image gs_pdfimage24_device = {
 };
 
 /* ------ The pdfimage32 device ------ */
+static void
+pdfimage32_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_cmyk8(dev);
 
-static const gx_device_procs pdfimage32_procs = {
-    pdf_image_open, NULL, NULL, gdev_prn_output_page_seekable, pdf_image_close,
-    NULL, cmyk_8bit_map_color_cmyk, NULL, NULL, NULL, NULL, NULL, NULL,
-    pdf_image_get_params_downscale_cmyk, pdf_image_put_params_downscale_cmyk,
-    cmyk_8bit_map_cmyk_color, NULL, NULL, NULL, gx_page_device_get_page_device
-};
+    set_dev_proc(dev, open_device, pdf_image_open);
+    set_dev_proc(dev, output_page, gdev_prn_output_page);
+    set_dev_proc(dev, close_device, pdf_image_close);
+    set_dev_proc(dev, get_params, pdf_image_get_params_downscale_cmyk);
+    set_dev_proc(dev, put_params, pdf_image_put_params_downscale_cmyk);
+}
 
 const gx_device_pdf_image gs_pdfimage32_device = {
     prn_device_body(gx_device_pdf_image,
-                    pdfimage32_procs,
+                    pdfimage32_initialize_device_procs,
                     "pdfimage32",
                     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                     600, 600,   /* 600 dpi by default */
@@ -188,8 +200,6 @@ static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
                          gp_file *file)
 {
     gx_device_printer *const pdev = (gx_device_printer *)pdf_dev;
-    cmm_dev_profile_t *profile_struct;
-    gsicc_rendering_param_t rendering_params;
     int code;
     pdfimage_page *page;
 
@@ -201,39 +211,11 @@ static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
 
     if (gdev_prn_file_is_new(pdev)) {
         /* Set up the icc link settings at this time */
-        code = dev_proc(pdev, get_profile)((gx_device *)pdev, &profile_struct);
-        if (code < 0)
-            return_error(gs_error_undefined);
-        if (profile_struct->postren_profile != NULL) {
-            rendering_params.black_point_comp = gsBLACKPTCOMP_ON;
-            rendering_params.graphics_type_tag = GS_UNKNOWN_TAG;
-            rendering_params.override_icc = false;
-            rendering_params.preserve_black = gsBLACKPRESERVE_OFF;
-            rendering_params.rendering_intent = gsRELATIVECOLORIMETRIC;
-            rendering_params.cmm = gsCMM_DEFAULT;
-            if (profile_struct->oi_profile != NULL) {
-                pdf_dev->icclink = gsicc_alloc_link_dev(pdev->memory,
-                    profile_struct->oi_profile, profile_struct->postren_profile,
-                    &rendering_params);
-            } else if (profile_struct->link_profile != NULL) {
-                pdf_dev->icclink = gsicc_alloc_link_dev(pdev->memory,
-                    profile_struct->link_profile, profile_struct->postren_profile,
-                    &rendering_params);
-            } else {
-                pdf_dev->icclink = gsicc_alloc_link_dev(pdev->memory,
-                    profile_struct->device_profile[GS_DEFAULT_DEVICE_PROFILE], profile_struct->postren_profile,
-                    &rendering_params);
-            }
-            if (pdf_dev->icclink == NULL) {
-                gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
-                return_error(gs_error_VMerror);
-            }
-            /* If it is identity, release it now and set link to NULL */
-            if (pdf_dev->icclink->is_identity) {
-                pdf_dev->icclink->procs.free_link(pdf_dev->icclink);
-                gsicc_free_link_dev(pdev->memory, pdf_dev->icclink);
-                pdf_dev->icclink = NULL;
-            }
+        code = gx_downscaler_create_post_render_link((gx_device *)pdev,
+                                                     &pdf_dev->icclink);
+        if (code < 0) {
+            gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
+            return code;
         }
 
         /* Set up the stream and insert the file header */
@@ -256,6 +238,7 @@ static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
 
         stream_puts(pdf_dev->strm, "%PDF-1.3\n");
         stream_puts(pdf_dev->strm, "%\307\354\217\242\n");
+        pdfimage_write_args_comment(pdf_dev,pdf_dev->strm);
 
         if (pdf_dev->ocr.file_init) {
             code = pdf_dev->ocr.file_init(pdf_dev);
@@ -270,7 +253,6 @@ static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
             }
         }
 
-        pdfimage_write_args_comment(pdf_dev, pdf_dev->strm);
         pdf_dev->Pages = page;
     } else {
         pdfimage_page *current = pdf_dev->Pages;
@@ -278,10 +260,11 @@ static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
             current = current->next;
         current->next = page;
     }
-    page->ImageObjectNumber = (pdf_dev->NumPages * 4) + 3 + pdf_dev->ocr.file_objects;
+    page->ImageObjectNumber = (pdf_dev->NumPages * PDFIMG_OBJS_PER_PAGE) + PDFIMG_STATIC_OBJS + pdf_dev->ocr.file_objects;
     page->LengthObjectNumber = page->ImageObjectNumber + 1;
     page->PageStreamObjectNumber = page->LengthObjectNumber + 1;
     page->PageDictObjectNumber = page->PageStreamObjectNumber + 1;
+    page->PageLengthObjectNumber = page->PageDictObjectNumber + 1;
     page->ImageOffset = stell(pdf_dev->strm);
     pdf_dev->StripHeight = pdf_dev->height;
 
@@ -341,7 +324,7 @@ encode(gx_device *pdev, stream **s, const stream_template *t, gs_memory_t *mem)
         jcdp->Picky = 0;
         jcdp->Relax = 0;
         jcdp->cinfo.image_width = gx_downscaler_scale(pdf_dev->width, pdf_dev->downscale.downscale_factor);
-        jcdp->cinfo.image_height = pdf_dev->StripHeight;
+        jcdp->cinfo.image_height = gx_downscaler_scale(pdf_dev->height, pdf_dev->downscale.downscale_factor);
         switch (pdf_dev->color_info.depth) {
             case 32:
                 jcdp->cinfo.input_components = 4;
@@ -434,6 +417,7 @@ pdf_image_downscale_and_print_page(gx_device_printer *dev,
     pdfimage_page *page = pdf_dev->Pages;
     char Buffer[1024];
     stream *s = pdf_dev->strm;
+    gs_offset_t len;
 
     if (page == NULL)
         return_error(gs_error_undefined);
@@ -566,37 +550,33 @@ pdf_image_downscale_and_print_page(gx_device_printer *dev,
 
     page->PageStreamOffset = stell(pdf_dev->strm);
     pprintd1(pdf_dev->strm, "%d 0 obj\n", page->PageStreamObjectNumber);
-    stream_puts(pdf_dev->strm, "<<\n/Filter/FlateDecode/Length           \n>>\nstream\n");
+    pprintd1(pdf_dev->strm, "<<\n/Filter/FlateDecode/Length %d 0 R\n>>\nstream\n", page->PageLengthObjectNumber);
     stream_pos = stell(pdf_dev->strm);
     encode((gx_device *)pdf_dev, &pdf_dev->strm, &s_zlibE_template, pdf_dev->memory->non_gc_memory);
     if (pdf_dev->ocr.end_page)
         stream_puts(pdf_dev->strm, "q\n");
-    pprintd2(pdf_dev->strm, "%d 0 0 %d 0 0 cm\n/Im1 Do",
-             (int)((width / (pdf_dev->HWResolution[0] / 72)) * factor),
-             (int)((height / (pdf_dev->HWResolution[1] / 72)) * factor));
+    gs_snprintf(Buffer, sizeof(Buffer), "%f 0 0 %f 0 0 cm\n/Im1 Do", ((double)pdf_dev->width / pdf_dev->HWResolution[0]) * 72, ((double)pdf_dev->height / pdf_dev->HWResolution[1]) * 72);
+    stream_puts(pdf_dev->strm, Buffer);
     if (pdf_dev->ocr.end_page) {
         stream_puts(pdf_dev->strm, "\nQ");
         pdf_dev->ocr.end_page(pdf_dev);
     }
     s_close_filters(&pdf_dev->strm, s);
-    {
-        gs_offset_t pos = stell(pdf_dev->strm);
-        gs_offset_t len = pos - stream_pos;
-        sseek(pdf_dev->strm, stream_pos - 21);
-        pprintd1(pdf_dev->strm, "%d", (int)len);
-        sseek(pdf_dev->strm, pos);
-    }
+    len = stell(pdf_dev->strm) - stream_pos;
     stream_puts(pdf_dev->strm, "\nendstream\nendobj\n");
 
+    page->PageLengthOffset = stell(pdf_dev->strm);
+    pprintd2(pdf_dev->strm, "%d 0 obj\n%d\nendobj\n", page->PageLengthObjectNumber, len);
+
     page->PageDictOffset = stell(pdf_dev->strm);
     pprintd1(pdf_dev->strm, "%d 0 obj\n", page->PageDictObjectNumber);
     pprintd1(pdf_dev->strm, "<<\n/Contents %d 0 R\n", page->PageStreamObjectNumber);
     stream_puts(pdf_dev->strm, "/Type /Page\n/Parent 2 0 R\n");
-    gs_sprintf(Buffer, "/MediaBox [0 0 %f %f]\n", ((double)pdf_dev->width / pdf_dev->HWResolution[0]) * 72, ((double)pdf_dev->height / pdf_dev->HWResolution[1]) * 72);
+    gs_snprintf(Buffer, sizeof(Buffer), "/MediaBox [0 0 %f %f]\n", ((double)pdf_dev->width / pdf_dev->HWResolution[0]) * 72, ((double)pdf_dev->height / pdf_dev->HWResolution[1]) * 72);
     stream_puts(pdf_dev->strm, Buffer);
     pprintd1(pdf_dev->strm, "/Resources <<\n/XObject <<\n/Im1 %d 0 R\n>>\n", page->ImageObjectNumber);
     if (pdf_dev->ocr.file_init)
-        stream_puts(pdf_dev->strm, "/Font <<\n/Ft0 3 0 R\n>>\n");
+        pprintd1(pdf_dev->strm, "/Font <<\n/Ft0 %d 0 R\n>>\n", PDFIMG_STATIC_OBJS);
     stream_puts(pdf_dev->strm, ">>\n>>\nendobj\n");
 
     gx_downscaler_fin(&ds);
@@ -622,12 +602,10 @@ static void write_fileID(stream *s, const byte *str, int size)
         stream_cursor_write w;
         int status;
 
-        r.ptr = str - 1;
-        r.limit = r.ptr + size;
-        w.limit = buf + sizeof(buf) - 1;
+        stream_cursor_read_init(&r, str, size);
+
         do {
-            /* One picky compiler complains if we initialize to buf - 1. */
-            w.ptr = buf;  w.ptr--;
+            stream_cursor_write_init(&w, buf, sizeof(buf));
             status = (*templat->process) (st, &r, &w, true);
             stream_write(s, buf, (uint) (w.ptr + 1 - buf));
         }
@@ -690,7 +668,7 @@ static void write_xref_entry (stream *s, gs_offset_t Offset)
     if (Offset > 9999999999){
         Offset = 0;
     }
-    gs_sprintf(O, "%d", Offset);
+    gs_snprintf(O, sizeof(O), "%d", Offset);
     for (i=0; i< (10 - strlen(O)); i++)
         stream_puts(s, "0");
     stream_puts(s, O);
@@ -698,13 +676,13 @@ static void write_xref_entry (stream *s, gs_offset_t Offset)
 }
 
 static void
-pdf_store_default_Producer(char *buf)
+pdf_store_default_Producer(char buf[256])
 {
     int major = (int)(gs_revision / 1000);
     int minor = (int)(gs_revision - (major * 1000)) / 10;
     int patch = gs_revision % 10;
 
-    gs_sprintf(buf, "(%s %d.%02d.%d)", gs_product, major, minor, patch);
+    gs_snprintf(buf, 256, "(%s %d.%02d.%d)", gs_product, major, minor, patch);
 }
 
 static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
@@ -722,7 +700,7 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
         pdf_store_default_Producer(Producer);
 
         pdf_dev->RootOffset = stell(pdf_dev->strm);
-        stream_puts(pdf_dev->strm, "1 0 obj\n<<\n/Pages 2 0 R\n/Type /Catalog\n>>\nendobj\n");
+        stream_puts(pdf_dev->strm, "1 0 obj\n<<\n/Pages 2 0 R\n/Type /Catalog\n/Info 3 0 R\n>>\nendobj\n");
 
         pdf_dev->PagesOffset = stell(pdf_dev->strm);
         pprintd1(pdf_dev->strm, "2 0 obj\n<<\n/Count %d\n", pdf_dev->NumPages);
@@ -750,18 +728,26 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
         tms = *localtime(&t);
 #endif
 
-        gs_sprintf(CreationDate, "(D:%04d%02d%02d%02d%02d%02d%c%02d\'%02d\')",
+        gs_snprintf(CreationDate, sizeof(CreationDate), "(D:%04d%02d%02d%02d%02d%02d%c%02d\'%02d\')",
             tms.tm_year + 1900, tms.tm_mon + 1, tms.tm_mday,
             tms.tm_hour, tms.tm_min, tms.tm_sec,
             timesign, timeoffset / 60, timeoffset % 60);
 
+        pdf_dev->InfoOffset = stell(pdf_dev->strm);
+        stream_puts(pdf_dev->strm, "3 0 obj\n<<\n/Producer");
+        stream_puts(pdf_dev->strm, Producer);
+        stream_puts(pdf_dev->strm, "\n/CreationDate");
+        stream_puts(pdf_dev->strm, CreationDate);
+        stream_puts(pdf_dev->strm, "\n>>\nendobj\n");
+
         pdf_dev->xrefOffset = stell(pdf_dev->strm);
         if (PCLm)
             pprintd1(pdf_dev->strm, "xref\n0 %d\n0000000000 65536 f \n", pdf_dev->NextObject);
         else
-            pprintd1(pdf_dev->strm, "xref\n0 %d\n0000000000 65536 f \n", (pdf_dev->NumPages * 4) + 3 + pdf_dev->ocr.file_objects);
+            pprintd1(pdf_dev->strm, "xref\n0 %d\n0000000000 65536 f \n", (pdf_dev->NumPages * PDFIMG_OBJS_PER_PAGE) + PDFIMG_STATIC_OBJS + pdf_dev->ocr.file_objects);
         write_xref_entry(pdf_dev->strm, pdf_dev->RootOffset);
         write_xref_entry(pdf_dev->strm, pdf_dev->PagesOffset);
+        write_xref_entry(pdf_dev->strm, pdf_dev->InfoOffset);
         if (pdf_dev->ocr.file_objects) {
             int i;
 
@@ -777,9 +763,10 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
                 write_xref_entry(pdf_dev->strm, page->LengthOffset);
                 write_xref_entry(pdf_dev->strm, page->PageStreamOffset);
                 write_xref_entry(pdf_dev->strm, page->PageDictOffset);
+                write_xref_entry(pdf_dev->strm, page->PageLengthOffset);
                 page = page->next;
             }
-            pprintd1(pdf_dev->strm, "trailer\n<<\n/Size %d\n/Root 1 0 R\n/ID [", (pdf_dev->NumPages * 4) + 3 + pdf_dev->ocr.file_objects);
+            pprintd1(pdf_dev->strm, "trailer\n<<\n/Size %d\n/Root 1 0 R\n/ID [", (pdf_dev->NumPages * PDFIMG_OBJS_PER_PAGE) + PDFIMG_STATIC_OBJS + pdf_dev->ocr.file_objects);
             pdf_compute_fileID(pdf_dev, fileID, CreationDate, Title, Producer);
             write_fileID(pdf_dev->strm, (const byte *)&fileID, 16);
             write_fileID(pdf_dev->strm, (const byte *)&fileID, 16);
@@ -833,12 +820,8 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
         pdf_dev->Pages = NULL;
         pdf_dev->NumPages = 0;
     }
-    if (pdf_dev->icclink != NULL)
-    {
-        pdf_dev->icclink->procs.free_link(pdf_dev->icclink);
-        gsicc_free_link_dev(pdf_dev->memory, pdf_dev->icclink);
-        pdf_dev->icclink = NULL;
-    }
+    gsicc_free_link_dev(pdf_dev->icclink);
+    pdf_dev->icclink = NULL;
     pdf_dev->RootOffset = 0;
     pdf_dev->PagesOffset = 0;
     pdf_dev->xrefOffset = 0;
@@ -880,7 +863,7 @@ pdf_image_print_page(gx_device_printer * pdev, gp_file * file)
 int
 pdf_image_open(gx_device *pdev)
 {
-    gx_device_pdf_image *ppdev = (gx_device_pdf_image *)pdev;
+    gx_device_pdf_image *ppdev;
     int code;
     bool update_procs = false;
 
@@ -891,6 +874,7 @@ pdf_image_open(gx_device *pdev)
     while(pdev->child)
         pdev = pdev->child;
 
+    ppdev = (gx_device_pdf_image *)pdev;
     ppdev->file = NULL;
     ppdev->Pages = NULL;
     ppdev->NumPages = 0;
@@ -910,7 +894,7 @@ pdf_image_open(gx_device *pdev)
             gx_copy_device_procs(pdev->parent, pdev, &gs_flp_device);
     }
     if (ppdev->OpenOutputFile)
-        code = gdev_prn_open_printer_seekable(pdev, 1, true);
+        code = gdev_prn_open_printer(pdev, 1);
 
     return code;
 }
@@ -965,6 +949,12 @@ pdf_image_get_some_params(gx_device * dev, gs_param_list * plist, int which)
     if (ecode < 0)
         return ecode;
 
+    ecode = param_write_bool(plist, "Tumble", &pdf_dev->Tumble);
+    if (ecode < 0)
+        return ecode;
+    ecode = param_write_bool(plist, "Tumble2", &pdf_dev->Tumble2);
+    if (ecode < 0)
+        return ecode;
     ecode = param_write_int(plist, "StripHeight", &pdf_dev->StripHeight);
     if (ecode < 0)
         return ecode;
@@ -1016,6 +1006,18 @@ pdf_image_put_some_params(gx_device * dev, gs_param_list * plist, int which)
     gs_param_string comprstr;
     const char *param_name;
 
+    code = param_read_bool(plist, param_name = "Tumble", &pdf_dev->Tumble);
+    if (code < 0) {
+        errprintf(pdf_dev->memory, "Invalid Tumble setting\n");
+        param_signal_error(plist, param_name, ecode);
+        return code;
+    }
+    code = param_read_bool(plist, param_name = "Tumble2", &pdf_dev->Tumble2);
+    if (code < 0) {
+        errprintf(pdf_dev->memory, "Invalid Tumble2 setting\n");
+        param_signal_error(plist, param_name, ecode);
+        return code;
+    }
     code = param_read_int(plist, param_name = "StripHeight", &pdf_dev->StripHeight);
     if (code < 0) {
         errprintf(pdf_dev->memory, "Invalid StripHeight setting\n");
@@ -1058,10 +1060,7 @@ pdf_image_put_some_params(gx_device * dev, gs_param_list * plist, int which)
                                           (which & 2 ? GX_DOWNSCALER_PARAMS_TRAP : 0) |
                                           (which & 4 ? GX_DOWNSCALER_PARAMS_ETS : 0)));
         if (code < 0)
-        {
             ecode = code;
-            param_signal_error(plist, param_name, ecode);
-        }
     }
     if (ecode < 0)
         return ecode;
@@ -1090,31 +1089,102 @@ pdf_image_put_params_downscale_cmyk_ets(gx_device * dev, gs_param_list * plist)
     return pdf_image_put_some_params(dev, plist, 7);
 }
 
+static void
+PCLm_get_initial_matrix(gx_device * dev, register gs_matrix * pmat)
+{
+    gx_device_pdf_image *pdev = (gx_device_pdf_image *)dev;
+
+    gx_default_get_initial_matrix(dev, pmat);
+
+    if (pdev->Duplex && (pdev->NumPages & 1)) {
+        /* Duplexing and we are on the back of the page. */
+        if (pdev->Tumble) {
+            if (pdev->Tumble2) {
+                /* Rotate by 180 degrees and flip on X. */
+                pmat->xx = pmat->xx;
+                pmat->xy = -pmat->xy;
+                pmat->yx = -pmat->yx;
+                pmat->yy = -pmat->yy;
+                pmat->ty = pdev->height - pmat->ty;
+            } else {
+                /* Rotate by 180 degrees */
+                pmat->xx = -pmat->xx;
+                pmat->xy = -pmat->xy;
+                pmat->yx = -pmat->yx;
+                pmat->yy = -pmat->yy;
+                pmat->tx = pdev->width - pmat->tx;
+                pmat->ty = pdev->height - pmat->ty;
+            }
+        } else if (pdev->Tumble2) {
+            /* Flip on X */
+            pmat->xx = -pmat->xx;
+            pmat->yx = -pmat->yx;
+            pmat->tx = pdev->width - pmat->tx;
+        }
+    }
+}
+
 /* ------ The PCLm device ------ */
+static void
+PCLm_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_rgb(dev);
+
+    set_dev_proc(dev, open_device, PCLm_open);
+    set_dev_proc(dev, output_page, gdev_prn_output_page);
+    set_dev_proc(dev, get_initial_matrix, PCLm_get_initial_matrix);
+    set_dev_proc(dev, close_device, PCLm_close);
+    set_dev_proc(dev, get_params, pdf_image_get_params_downscale);
+    set_dev_proc(dev, put_params, pdf_image_put_params_downscale);
+}
+
+static void
+PCLm8_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_rgb(dev);
+
+    set_dev_proc(dev, open_device, PCLm_open);
+    set_dev_proc(dev, output_page, gdev_prn_output_page);
+    set_dev_proc(dev, get_initial_matrix, PCLm_get_initial_matrix);
+    set_dev_proc(dev, close_device, PCLm_close);
+    set_dev_proc(dev, get_params, pdf_image_get_params_downscale);
+    set_dev_proc(dev, put_params, pdf_image_put_params_downscale);
+    set_dev_proc(dev, encode_color, gx_default_8bit_map_gray_color);
+    set_dev_proc(dev, decode_color, gx_default_8bit_map_color_gray);
+}
 
 static dev_proc_print_page(PCLm_print_page);
 
-static const gx_device_procs PCLm_procs =
-prn_color_params_procs(PCLm_open,
-                       gdev_prn_output_page_seekable,
-                       PCLm_close,
-                       gx_default_rgb_map_rgb_color,
-                       gx_default_rgb_map_color_rgb,
-                       pdf_image_get_params_downscale,
-                       pdf_image_put_params_downscale);
-
 const gx_device_pdf_image gs_PCLm_device = {
-    prn_device_body(gx_device_pdf_image,
-                    PCLm_procs,
-                    "pclm",
-                    DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-                    600, 600,   /* 600 dpi by default */
-                    0, 0, 0, 0, /* Margins */
-                    3,          /* num components */
-                    24,         /* bits per sample */
-                    255, 255, 256, 256,
-                    PCLm_print_page),
-                    3,
+    prn_device_body_duplex(gx_device_pdf_image,
+                           PCLm_initialize_device_procs,
+                           "pclm",
+                           DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+                           600, 600,   /* 600 dpi by default */
+                           0, 0, 0, 0, /* Margins */
+                           3,          /* num components */
+                           24,         /* bits per sample */
+                           255, 255, 256, 256,
+                           PCLm_print_page),
+    3,
+    GX_DOWNSCALER_PARAMS_DEFAULTS,
+    16, /* StripHeight */
+    0.0, /* QFactor */
+    0  /* JPEGQ */
+};
+
+const gx_device_pdf_image gs_PCLm8_device = {
+    prn_device_body_duplex(gx_device_pdf_image,
+                           PCLm8_initialize_device_procs,
+                           "pclm8",
+                           DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+                           600, 600,   /* 600 dpi by default */
+                           0, 0, 0, 0, /* Margins */
+                           1,          /* num components */
+                           8,         /* bits per sample */
+                           255, 0, 256, 0,
+                           PCLm_print_page),
+    3,
     GX_DOWNSCALER_PARAMS_DEFAULTS,
     16, /* StripHeight */
     0.0, /* QFactor */
@@ -1168,7 +1238,7 @@ PCLm_close_temp_file(gx_device_pdf_image *pdev, PCLm_temp_file_t *ptf, int code)
     }
     if (file) {
         err = gp_ferror(file) | gp_fclose(file);
-        unlink(ptf->file_name);
+        gp_unlink(pdev->memory, ptf->file_name);
         ptf->file = 0;
     }
     return
@@ -1202,7 +1272,7 @@ PCLm_open_temp_stream(gx_device_pdf_image *pdev, PCLm_temp_file_t *ptf)
 int
 PCLm_open(gx_device *pdev)
 {
-    gx_device_pdf_image *ppdev = (gx_device_pdf_image *)pdev;
+    gx_device_pdf_image *ppdev;
     int code;
     bool update_procs = false;
 
@@ -1213,6 +1283,7 @@ PCLm_open(gx_device *pdev)
     while(pdev->child)
         pdev = pdev->child;
 
+    ppdev = (gx_device_pdf_image *)pdev;
     memset(&ppdev->ocr, 0, sizeof(ppdev->ocr));
     ppdev->file = NULL;
     ppdev->Pages = NULL;
@@ -1233,7 +1304,7 @@ PCLm_open(gx_device *pdev)
             gx_copy_device_procs(pdev->parent, pdev, &gs_flp_device);
     }
     if (ppdev->OpenOutputFile)
-        code = gdev_prn_open_printer_seekable(pdev, 1, true);
+        code = gdev_prn_open_printer(pdev, 1);
 
     if(code < 0)
         return code;
@@ -1252,8 +1323,6 @@ static int gdev_PCLm_begin_page(gx_device_pdf_image *pdf_dev,
                          gp_file *file)
 {
     gx_device_printer *const pdev = (gx_device_printer *)pdf_dev;
-    cmm_dev_profile_t *profile_struct;
-    gsicc_rendering_param_t rendering_params;
     int code;
     pdfimage_page *page;
 
@@ -1265,39 +1334,11 @@ static int gdev_PCLm_begin_page(gx_device_pdf_image *pdf_dev,
 
     if (gdev_prn_file_is_new(pdev)) {
         /* Set up the icc link settings at this time */
-        code = dev_proc(pdev, get_profile)((gx_device *)pdev, &profile_struct);
-        if (code < 0)
-            return_error(gs_error_undefined);
-        if (profile_struct->postren_profile != NULL) {
-            rendering_params.black_point_comp = gsBLACKPTCOMP_ON;
-            rendering_params.graphics_type_tag = GS_UNKNOWN_TAG;
-            rendering_params.override_icc = false;
-            rendering_params.preserve_black = gsBLACKPRESERVE_OFF;
-            rendering_params.rendering_intent = gsRELATIVECOLORIMETRIC;
-            rendering_params.cmm = gsCMM_DEFAULT;
-            if (profile_struct->oi_profile != NULL) {
-                pdf_dev->icclink = gsicc_alloc_link_dev(pdev->memory,
-                    profile_struct->oi_profile, profile_struct->postren_profile,
-                    &rendering_params);
-            } else if (profile_struct->link_profile != NULL) {
-                pdf_dev->icclink = gsicc_alloc_link_dev(pdev->memory,
-                    profile_struct->link_profile, profile_struct->postren_profile,
-                    &rendering_params);
-            } else {
-                pdf_dev->icclink = gsicc_alloc_link_dev(pdev->memory,
-                    profile_struct->device_profile[GS_DEFAULT_DEVICE_PROFILE], profile_struct->postren_profile,
-                    &rendering_params);
-            }
-            if (pdf_dev->icclink == NULL) {
-                gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
-                return_error(gs_error_VMerror);
-            }
-            /* If it is identity, release it now and set link to NULL */
-            if (pdf_dev->icclink->is_identity) {
-                pdf_dev->icclink->procs.free_link(pdf_dev->icclink);
-                gsicc_free_link_dev(pdev->memory, pdf_dev->icclink);
-                pdf_dev->icclink = NULL;
-            }
+        code = gx_downscaler_create_post_render_link((gx_device *)pdev,
+                                                     &pdf_dev->icclink);
+        if (code < 0) {
+            gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
+            return code;
         }
 
         /* Set up the stream and insert the file header */
@@ -1321,7 +1362,7 @@ static int gdev_PCLm_begin_page(gx_device_pdf_image *pdf_dev,
         stream_puts(pdf_dev->strm, "%PDF-1.3\n");
         stream_puts(pdf_dev->strm, "%PCLm 1.0\n");
         pdf_dev->Pages = page;
-        pdf_dev->NextObject = 3;
+        pdf_dev->NextObject = 4;
     } else {
         pdfimage_page *current = pdf_dev->Pages;
         while(current->next)
@@ -1399,7 +1440,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev,
     pprintd1(pdf_dev->strm, "%d 0 obj\n", page->PageDictObjectNumber);
     pprintd1(pdf_dev->strm, "<<\n/Contents %d 0 R\n", page->PageStreamObjectNumber);
     stream_puts(pdf_dev->strm, "/Type /Page\n/Parent 2 0 R\n");
-    gs_sprintf(Buffer, "/MediaBox [0 0 %.3f %.3f]\n", ((double)pdf_dev->width / pdf_dev->HWResolution[0]) * 72, ((double)pdf_dev->height / pdf_dev->HWResolution[1]) * 72);
+    gs_snprintf(Buffer, sizeof(Buffer), "/MediaBox [0 0 %.3f %.3f]\n", ((double)pdf_dev->width / pdf_dev->HWResolution[0]) * 72, ((double)pdf_dev->height / pdf_dev->HWResolution[1]) * 72);
     stream_puts(pdf_dev->strm, Buffer);
     stream_puts(pdf_dev->strm, "/Resources <<\n/XObject <<\n");
 
@@ -1418,9 +1459,9 @@ PCLm_downscale_and_print_page(gx_device_printer *dev,
             double adjusted;
             adjusted = height - (row * pdf_dev->StripHeight);
             adjusted = adjusted / (pdf_dev->HWResolution[1] / (factor * 72));
-            gs_sprintf(Buffer, "%.3f 0 0 %.3f 0 0 cm\n/Im%d Do Q\n", (width / (pdf_dev->HWResolution[0] / 72)) * factor, adjusted, row);
+            gs_snprintf(Buffer, sizeof(Buffer), "%.3f 0 0 %.3f 0 0 cm\n/Im%d Do Q\n", (width / (pdf_dev->HWResolution[0] / 72)) * factor, adjusted, row);
         } else
-            gs_sprintf(Buffer, "%.3f 0 0 %.3f 0 %f cm\n/Im%d Do Q\n", (width / (pdf_dev->HWResolution[0] / 72)) * factor, StripDecrement, ((height / (pdf_dev->HWResolution[1] / 72)) * factor) - (StripDecrement * (row + 1)), row);
+            gs_snprintf(Buffer, sizeof(Buffer), "%.3f 0 0 %.3f 0 %f cm\n/Im%d Do Q\n", (width / (pdf_dev->HWResolution[0] / 72)) * factor, StripDecrement, ((height / (pdf_dev->HWResolution[1] / 72)) * factor) - (StripDecrement * (row + 1)), row);
         stream_puts(pdf_dev->temp_stream.strm, Buffer);
         pprintd2(pdf_dev->strm, "/Im%d %d 0 R\n", row, page->ImageObjectNumber + (row * 2));
     }
@@ -1498,7 +1539,10 @@ PCLm_downscale_and_print_page(gx_device_printer *dev,
             stream_puts(pdf_dev->strm, "/Subtype /Image\n");
             pprintd1(pdf_dev->strm, "/Width %d\n", width);
             pprintd1(pdf_dev->strm, "/Height %d\n", Read);
-            stream_puts(pdf_dev->strm, "/ColorSpace /DeviceRGB\n");
+            if (dev->color_info.max_components == 1)
+                stream_puts(pdf_dev->strm, "/ColorSpace /DeviceGray\n");
+            else
+                stream_puts(pdf_dev->strm, "/ColorSpace /DeviceRGB\n");
             stream_puts(pdf_dev->strm, "/BitsPerComponent 8\n");
             switch (pdf_dev->Compression) {
                 case COMPRESSION_FLATE:
@@ -1598,7 +1642,10 @@ PCLm_downscale_and_print_page(gx_device_printer *dev,
         stream_puts(pdf_dev->strm, "/Subtype /Image\n");
         pprintd1(pdf_dev->strm, "/Width %d\n", width);
         pprintd1(pdf_dev->strm, "/Height %d\n", Read);
-        stream_puts(pdf_dev->strm, "/ColorSpace /DeviceRGB\n");
+        if (dev->color_info.max_components == 1)
+            stream_puts(pdf_dev->strm, "/ColorSpace /DeviceGray\n");
+        else
+            stream_puts(pdf_dev->strm, "/ColorSpace /DeviceRGB\n");
         stream_puts(pdf_dev->strm, "/BitsPerComponent 8\n");
         switch (pdf_dev->Compression) {
             case COMPRESSION_FLATE:
