@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 #include "stdint_.h"
@@ -31,28 +31,11 @@
 #include "sjpeg.h"
 
 #include "gdevpdfimg.h"
+#include "tessocr.h"
 
-#define	    COMPRESSION_NONE	1	/* dump mode */
-#define	    COMPRESSION_LZW		2       /* Lempel-Ziv  & Welch */
-#define	    COMPRESSION_FLATE	3
-#define	    COMPRESSION_JPEG	4
-#define	    COMPRESSION_RLE		5
-
-static struct compression_string {
-    unsigned char id;
-    const char *str;
-} compression_strings [] = {
-    { COMPRESSION_NONE, "None" },
-    { COMPRESSION_LZW, "LZW" },     /* Not supported in PCLm */
-    { COMPRESSION_FLATE, "Flate" },
-    { COMPRESSION_JPEG, "JPEG" },
-    { COMPRESSION_RLE, "RLE" },
-    { 0, NULL }
-};
-
-int pdf_ocr_open(gx_device *pdev);
-int pdf_ocr_close(gx_device *pdev);
-
+static dev_proc_initialize_device(pdf_ocr_initialize_device);
+static dev_proc_open_device(pdf_ocr_open);
+static dev_proc_close_device(pdf_ocr_close);
 
 static int
 pdfocr_put_some_params(gx_device * dev, gs_param_list * plist)
@@ -63,14 +46,32 @@ pdfocr_put_some_params(gx_device * dev, gs_param_list * plist)
     gs_param_string langstr;
     const char *param_name;
     size_t len;
+    int engine;
 
     switch (code = param_read_string(plist, (param_name = "OCRLanguage"), &langstr)) {
         case 0:
-            len = langstr.size;
-            if (len >= sizeof(pdf_dev->ocr.language))
-                len = sizeof(pdf_dev->ocr.language)-1;
-            memcpy(pdf_dev->ocr.language, langstr.data, len);
-            pdf_dev->ocr.language[len] = 0;
+                if (gs_is_path_control_active(pdf_dev->memory)
+                && (strlen(pdf_dev->ocr.language) != langstr.size || memcmp(pdf_dev->ocr.language, langstr.data, langstr.size) != 0)) {
+                return_error(gs_error_invalidaccess);
+            }
+            else {
+                len = langstr.size;
+                if (len >= sizeof(pdf_dev->ocr.language))
+                    len = sizeof(pdf_dev->ocr.language)-1;
+                memcpy(pdf_dev->ocr.language, langstr.data, len);
+                pdf_dev->ocr.language[len] = 0;
+            }
+            break;
+        case 1:
+            break;
+        default:
+            ecode = code;
+            param_signal_error(plist, param_name, ecode);
+    }
+
+    switch (code = param_read_int(plist, (param_name = "OCREngine"), &engine)) {
+        case 0:
+            pdf_dev->ocr.engine = engine;
             break;
         case 1:
             break;
@@ -120,6 +121,9 @@ pdfocr_get_some_params(gx_device * dev, gs_param_list * plist)
     if ((code = param_write_string(plist, "OCRLanguage", &langstr)) < 0)
         ecode = code;
 
+    if ((code = param_write_int(plist, "OCREngine", &pdf_dev->ocr.engine)) < 0)
+        ecode = code;
+
     return ecode;
 }
 
@@ -144,19 +148,25 @@ pdfocr_get_params_downscale(gx_device * dev, gs_param_list * plist)
 }
 
 /* ------ The pdfocr8 device ------ */
+static void
+pdfocr8_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_gray(dev);
 
-static const gx_device_procs pdfocr8_procs =
-prn_color_params_procs(pdf_ocr_open,
-                       gdev_prn_output_page_seekable,
-                       pdf_ocr_close,
-                       gx_default_gray_map_rgb_color,
-                       gx_default_gray_map_color_rgb,
-                       pdfocr_get_params_downscale,
-                       pdfocr_put_params_downscale);
+    set_dev_proc(dev, initialize_device, pdf_ocr_initialize_device);
+    set_dev_proc(dev, initialize_device, pdf_ocr_initialize_device);
+    set_dev_proc(dev, open_device, pdf_ocr_open);
+    set_dev_proc(dev, output_page, gdev_prn_output_page_seekable);
+    set_dev_proc(dev, close_device, pdf_ocr_close);
+    set_dev_proc(dev, get_params, pdfocr_get_params_downscale);
+    set_dev_proc(dev, put_params, pdfocr_put_params_downscale);
+    set_dev_proc(dev, encode_color, gx_default_8bit_map_gray_color);
+    set_dev_proc(dev, decode_color, gx_default_8bit_map_color_gray);
+}
 
 const gx_device_pdf_image gs_pdfocr8_device = {
     prn_device_body(gx_device_pdf_image,
-                    pdfocr8_procs,
+                    pdfocr8_initialize_device_procs,
                     "pdfocr8",
                     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                     600, 600,   /* 600 dpi by default */
@@ -173,19 +183,22 @@ const gx_device_pdf_image gs_pdfocr8_device = {
 };
 
 /* ------ The pdfocr24 device ------ */
+static void
+pdfocr24_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_rgb(dev);
 
-static const gx_device_procs pdfocr24_procs =
-prn_color_params_procs(pdf_ocr_open,
-                       gdev_prn_output_page_seekable,
-                       pdf_ocr_close,
-                       gx_default_rgb_map_rgb_color,
-                       gx_default_rgb_map_color_rgb,
-                       pdfocr_get_params_downscale,
-                       pdfocr_put_params_downscale);
+    set_dev_proc(dev, initialize_device, pdf_ocr_initialize_device);
+    set_dev_proc(dev, open_device, pdf_ocr_open);
+    set_dev_proc(dev, output_page, gdev_prn_output_page_seekable);
+    set_dev_proc(dev, close_device, pdf_ocr_close);
+    set_dev_proc(dev, get_params, pdfocr_get_params_downscale);
+    set_dev_proc(dev, put_params, pdfocr_put_params_downscale);
+}
 
 const gx_device_pdf_image gs_pdfocr24_device = {
     prn_device_body(gx_device_pdf_image,
-                    pdfocr24_procs,
+                    pdfocr24_initialize_device_procs,
                     "pdfocr24",
                     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                     600, 600,   /* 600 dpi by default */
@@ -202,17 +215,24 @@ const gx_device_pdf_image gs_pdfocr24_device = {
 };
 
 /* ------ The pdfocr32 device ------ */
+static void
+pdfocr32_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_cmyk8(dev);
 
-static const gx_device_procs pdfocr32_procs = {
-    pdf_ocr_open, NULL, NULL, gdev_prn_output_page_seekable, pdf_ocr_close,
-    NULL, cmyk_8bit_map_color_cmyk, NULL, NULL, NULL, NULL, NULL, NULL,
-    pdfocr_get_params_downscale_cmyk, pdfocr_put_params_downscale_cmyk,
-    cmyk_8bit_map_cmyk_color, NULL, NULL, NULL, gx_page_device_get_page_device
-};
+    set_dev_proc(dev, initialize_device, pdf_ocr_initialize_device);
+    set_dev_proc(dev, open_device, pdf_ocr_open);
+    set_dev_proc(dev, output_page, gdev_prn_output_page_seekable);
+    set_dev_proc(dev, close_device, pdf_ocr_close);
+    set_dev_proc(dev, get_params, pdfocr_get_params_downscale_cmyk);
+    set_dev_proc(dev, put_params, pdfocr_put_params_downscale_cmyk);
+    set_dev_proc(dev, decode_color, cmyk_8bit_map_color_cmyk);
+    set_dev_proc(dev, encode_color, cmyk_8bit_map_cmyk_color);
+}
 
 const gx_device_pdf_image gs_pdfocr32_device = {
     prn_device_body(gx_device_pdf_image,
-                    pdfocr32_procs,
+                    pdfocr32_initialize_device_procs,
                     "pdfocr32",
                     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                     600, 600,   /* 600 dpi by default */
@@ -228,21 +248,27 @@ const gx_device_pdf_image gs_pdfocr32_device = {
     0    /* JPEGQ */
 };
 
+/* These strings have object numbers built into them which are only
+ * correct for a specific value of PDFIMG_STATIC_OBJS (currently 4)
+ * If we add more static objects (eg Metadata) then we need to update
+ * the values in the strings below.
+ */
+
 /* Funky font */
 static const char funky_font[] =
-"3 0 obj\n<</BaseFont/GlyphLessFont/DescendantFonts[4 0 R]"
-"/Encoding/Identity-H/Subtype/Type0/ToUnicode 6 0 R/Type /Font"
+"4 0 obj\n<</BaseFont/GlyphLessFont/DescendantFonts[5 0 R]"
+"/Encoding/Identity-H/Subtype/Type0/ToUnicode 7 0 R/Type /Font"
 ">>\nendobj\n";
 
 static const char funky_font2[] =
-"4 0 obj\n<</BaseFont/GlyphLessFont"
-"/CIDToGIDMap 5 0 R\n/CIDSystemInfo<<\n"
+"5 0 obj\n<</BaseFont/GlyphLessFont"
+"/CIDToGIDMap 6 0 R\n/CIDSystemInfo<<\n"
 "/Ordering (Identity)/Registry (Adobe)/Supplement 0>>"
-"/FontDescriptor 7 0 R/Subtype /CIDFontType2/Type/Font"
+"/FontDescriptor 8 0 R/Subtype /CIDFontType2/Type/Font"
 "/DW 500>>\nendobj\n";
 
 static const char funky_font3[] =
-"5 0 obj\n<</Length 210/Filter/FlateDecode"
+"6 0 obj\n<</Length 210/Filter/FlateDecode"
 ">>stream\n";
 
 static const char funky_font3a[] = {
@@ -279,7 +305,7 @@ static const char funky_font3b[] =
 "endstream\nendobj\n";
 
 static const char funky_font4[] =
-"6 0 obj\n<</Length 353>>\nstream\n"
+"7 0 obj\n<</Length 353>>\nstream\n"
 "/CIDInit /ProcSet findresource begin\n"
 "12 dict begin\n"
 "begincmap\n"
@@ -305,13 +331,13 @@ static const char funky_font4[] =
 "endobj\n";
 
 static const char funky_font5[] =
-"7 0 obj\n"
+"8 0 obj\n"
 "<</Ascent 1000/CapHeight 1000/Descent -1/Flags 5"
-"/FontBBox[0 0 500 1000]/FontFile2 8 0 R/FontName/GlyphLessFont"
+"/FontBBox[0 0 500 1000]/FontFile2 9 0 R/FontName/GlyphLessFont"
 "/ItalicAngle 0/StemV 80/Type/FontDescriptor>>\nendobj\n";
 
 static const char funky_font6[] =
-"8 0 obj\n<</Length 572/Length1 572>>\nstream\n";
+"9 0 obj\n<</Length 572/Length1 572>>\nstream\n";
 
 static const char funky_font6a[] =
 {
@@ -416,7 +442,7 @@ ocr_file_init(gx_device_pdf_image *dev)
     stream_write(dev->strm, funky_font6a, sizeof(funky_font6a));
     stream_write(dev->strm, funky_font6b, sizeof(funky_font6b)-1);
 
-    return ocr_init_api(dev->memory, language, &dev->ocr.state);
+    return ocr_init_api(dev->memory->non_gc_memory, language, dev->ocr.engine, &dev->ocr.state);
 }
 
 static void
@@ -528,20 +554,20 @@ flush_word(gx_device_pdf_image *dev)
 
     size = bbox[3]-bbox[1];
     if (dev->ocr.cur_size != size) {
-        gs_sprintf(buffer, "/Ft0 %.3f Tf", size);
+        gs_snprintf(buffer, sizeof(buffer), "/Ft0 %.3f Tf", size);
         stream_puts(dev->strm, buffer);
         dev->ocr.cur_size = size;
     }
     scale = (bbox[2]-bbox[0]) / size / len * 200;
     if (dev->ocr.cur_scale != scale) {
-        gs_sprintf(buffer, " %.3f Tz", scale);
+        gs_snprintf(buffer, sizeof(buffer), " %.3f Tz", scale);
         stream_puts(dev->strm, buffer);
         dev->ocr.cur_scale = scale;
     }
-    gs_sprintf(buffer, " 1 0 0 1 %.3f %.3f Tm[<", bbox[0], bbox[1]);
+    gs_snprintf(buffer, sizeof(buffer), " 1 0 0 1 %.3f %.3f Tm[<", bbox[0], bbox[1]);
     stream_puts(dev->strm, buffer);
     for (i = 0; i < len; i++) {
-        gs_sprintf(buffer, "%04x", dev->ocr.word_chars[i]);
+        gs_snprintf(buffer, sizeof(buffer), "%04x", dev->ocr.word_chars[i]);
         stream_puts(dev->strm, buffer);
     }
     stream_puts(dev->strm, ">]TJ\n");
@@ -591,19 +617,19 @@ ocr_callback(void *arg, const char *rune_,
 
     size = bbox[3]-bbox[1];
     if (ppdev->ocr.cur_size != size) {
-        gs_sprintf(buffer, "/Ft0 %f Tf ", size);
+        gs_snprintf(buffer, sizeof(buffer), "/Ft0 %f Tf ", size);
         stream_puts(ppdev->strm, buffer);
         ppdev->ocr.cur_size = size;
     }
     scale = (bbox[2]-bbox[0]) / size * 200;
     if (ppdev->ocr.cur_scale != scale) {
-        gs_sprintf(buffer, " %f Tz ", scale);
+        gs_snprintf(buffer, sizeof(buffer), " %f Tz ", scale);
         stream_puts(ppdev->strm, buffer);
         ppdev->ocr.cur_scale = scale;
     }
-    gs_sprintf(buffer, "1 0 0 1 %f %f Tm ", bbox[0], bbox[1]);
+    gs_snprintf(buffer, sizeof(buffer), "1 0 0 1 %f %f Tm ", bbox[0], bbox[1]);
     stream_puts(ppdev->strm, buffer);
-    gs_sprintf(buffer, "<%04x>Tj\n", unicode);
+    gs_snprintf(buffer, sizeof(buffer), "<%04x>Tj\n", unicode);
     stream_puts(ppdev->strm, buffer);
 #else
     bbox[0] = word_bbox[0] * scale / ppdev->ocr.xres;
@@ -682,7 +708,18 @@ ocr_end_page(gx_device_pdf_image *dev)
     return 0;
 }
 
-int
+static int
+pdf_ocr_initialize_device(gx_device *dev)
+{
+    gx_device_pdf_image *ppdev = (gx_device_pdf_image *)dev;
+    const char *default_ocr_lang = "eng";
+
+    ppdev->ocr.language[0] = '\0';
+    strcpy(ppdev->ocr.language, default_ocr_lang);
+    return 0;
+}
+
+static int
 pdf_ocr_open(gx_device *pdev)
 {
     gx_device_pdf_image *ppdev;
@@ -705,7 +742,7 @@ pdf_ocr_open(gx_device *pdev)
     return 0;
 }
 
-int
+static int
 pdf_ocr_close(gx_device *pdev)
 {
     gx_device_pdf_image *pdf_dev;
