@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -57,66 +57,22 @@ struct gx_device_perm_s {
 };
 typedef struct gx_device_perm_s gx_device_perm_t;
 
-static const gx_device_procs perm_procs = {
-    gdev_prn_open,
-    NULL,
-/* Since the print_page doesn't alter the device, this device can print in the background */
-    NULL,
-    gdev_prn_bg_output_page,
-    gdev_prn_close,
-    NULL,
-    NULL,
-    NULL,				/* fill_rectangle */
-    NULL,				/* tile_rectangle */
-    NULL,				/* copy_mono */
-    NULL,				/* copy_color */
-    NULL,				/* draw_line */
-    NULL,				/* get_bits */
-    perm_get_params,		        /* get_params */
-    perm_put_params,		        /* put_params */
-    NULL,				/* map_cmyk_color - not used */
-    NULL,				/* get_xfont_procs */
-    NULL,				/* get_xfont_device */
-    NULL,				/* map_rgb_alpha_color */
-    gx_page_device_get_page_device,	/* get_page_device */
-    NULL,				/* get_alpha_bits */
-    NULL,				/* copy_alpha */
-    NULL,				/* get_band */
-    NULL,				/* copy_rop */
-    NULL,				/* fill_path */
-    NULL,				/* stroke_path */
-    NULL,				/* fill_mask */
-    NULL,				/* fill_trapezoid */
-    NULL,				/* fill_parallelogram */
-    NULL,				/* fill_triangle */
-    NULL,				/* draw_thin_line */
-    NULL,				/* begin_image */
-    NULL,				/* image_data */
-    NULL,				/* end_image */
-    NULL,				/* strip_tile_rectangle */
-    NULL,				/* strip_copy_rop */
-    NULL,				/* get_clipping_box */
-    NULL,				/* begin_typed_image */
-    NULL,				/* get_bits_rectangle */
-    NULL,				/* map_color_rgb_alpha */
-    NULL,				/* create_compositor */
-    NULL,				/* get_hardware_params */
-    NULL,				/* text_begin */
-    NULL,				/* finish_copydevice */
-    NULL,				/* begin_transparency_group */
-    NULL,				/* end_transparency_group */
-    NULL,				/* begin_transparency_mask */
-    NULL,				/* end_transparency_mask */
-    NULL,				/* discard_transparency_layer */
-    perm_get_color_mapping_procs,	/* get_color_mapping_procs */
-    perm_get_color_comp_index,
-    perm_encode_color,		/* encode_color */
-    perm_decode_color		/* decode_color */
+static void
+perm_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_bg(dev);
 
-};
+    set_dev_proc(dev, get_params, perm_get_params);
+    set_dev_proc(dev, put_params, perm_put_params);
+    set_dev_proc(dev, get_color_mapping_procs, perm_get_color_mapping_procs);
+    set_dev_proc(dev, get_color_comp_index, perm_get_color_comp_index);
+    set_dev_proc(dev, encode_color, perm_encode_color);
+    set_dev_proc(dev, decode_color, perm_decode_color);
+}
 
 const gx_device_perm_t gs_perm_device = {
-    prn_device_body_extended(gx_device_perm_t, perm_procs, "permute",
+    prn_device_body_extended(gx_device_perm_t,
+        perm_initialize_device_procs, "permute",
         DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS, 72, 72,
         0, 0, 0, 0,
         GX_DEVICE_COLOR_MAX_COMPONENTS, 4,
@@ -190,7 +146,7 @@ perm_print_page(gx_device_printer *pdev, gp_file *pstream)
             cooked_line[x * 3 + 1] = g;
             cooked_line[x * 3 + 2] = b;
         }
-        fwrite(cooked_line, 1, dev->width * 3, pstream);
+        gp_fwrite(cooked_line, 1, dev->width * 3, pstream);
     }
     gs_free_object(pdev->memory, cooked_line, "perm_print_page");
     gs_free_object(pdev->memory, raw_line, "perm_print_page");
