@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -36,8 +36,6 @@
 /****** THIS IS USELESS.  XGetImage DOES NOT GENERATE EXPOSURE EVENTS. ******/
 #define GET_IMAGE_EXPOSURES 0
 
-/* GC descriptors */
-private_st_device_X();
 
 /* Forward references */
 static int x_copy_image(gx_device_X * xdev, const byte * base, int sourcex,
@@ -54,106 +52,12 @@ static void update_do_flush(gx_device_X *);
 static void do_flush_text(gx_device_X *);
 
 /* Driver procedures */
-/* (External procedures are declared in gdevx.h.) */
-/*extern int gdev_x_open(gx_device_X *);*/
-static dev_proc_open_device(x_open);
-static dev_proc_get_initial_matrix(x_get_initial_matrix);
-static dev_proc_sync_output(x_sync);
-static dev_proc_output_page(x_output_page);
-/*extern int gdev_x_close(gx_device_X *);*/
-static dev_proc_close_device(x_close);
-/*extern dev_proc_map_rgb_color(gdev_x_map_rgb_color);*/
-/*extern dev_proc_map_color_rgb(gdev_x_map_color_rgb);*/
-static dev_proc_fill_rectangle(x_fill_rectangle);
-static dev_proc_copy_mono(x_copy_mono);
-static dev_proc_copy_color(x_copy_color);
-/*extern dev_proc_get_params(gdev_x_get_params);*/
-/*extern dev_proc_put_params(gdev_x_put_params);*/
-static dev_proc_get_page_device(x_get_page_device);
-static dev_proc_strip_tile_rectangle(x_strip_tile_rectangle);
-static dev_proc_get_bits_rectangle(x_get_bits_rectangle);
-/*extern dev_proc_get_xfont_procs(gdev_x_finish_copydevice);*/
-static dev_proc_fillpage(x_fillpage);
 
 /* The device descriptor */
 #define x_device(this_device, dev_body) \
 const gx_device_X this_device = { \
     dev_body, \
-    {				/* std_procs */ \
-        x_open, \
-        x_get_initial_matrix, \
-        x_sync, \
-        x_output_page, \
-        x_close, \
-        gdev_x_map_rgb_color, \
-        gdev_x_map_color_rgb, \
-        x_fill_rectangle, \
-        NULL,			/* tile_rectangle */ \
-        x_copy_mono, \
-        x_copy_color, \
-        NULL,			/* draw_line */ \
-        NULL,			/* get_bits */ \
-        gdev_x_get_params, \
-        gdev_x_put_params, \
-        NULL,			/* map_cmyk_color */ \
-        NULL, \
-        NULL,			/* get_xfont_device */ \
-        NULL,			/* map_rgb_alpha_color */ \
-        x_get_page_device, \
-        NULL,			/* get_alpha_bits */ \
-        NULL,			/* copy_alpha */ \
-        NULL,			/* get_band */ \
-        NULL,			/* copy_rop */ \
-        NULL,			/* fill_path */ \
-        NULL,			/* stroke_path */ \
-        NULL,			/* fill_mask */ \
-        NULL,			/* fill_trapezoid */ \
-        NULL,			/* fill_parallelogram */ \
-        NULL,			/* fill_triangle */ \
-        NULL,			/* draw_thin_line */ \
-        NULL,			/* begin_image */ \
-        NULL,			/* image_data */ \
-        NULL,			/* end_image */ \
-        x_strip_tile_rectangle, \
-        NULL,			/* strip_copy_rop */ \
-        NULL,			/* get_clipping_box */ \
-        NULL,                   /* begin_typed_image */ \
-        x_get_bits_rectangle, \
-        NULL,			/* map_color_rgb_alpha */ \
-        NULL,			/* create_compositor */ \
-        NULL,			/* get_hardware_params */ \
-        NULL,			/* text_begin */ \
-        gdev_x_finish_copydevice, \
-        NULL,			/* begin_transparency_group */ \
-        NULL,                   /* end_transparency_group */ \
-        NULL,                  /* begin_transparency_mask */ \
-        NULL,                    /* end_transparency_mask */ \
-        NULL,                       /* discard_transparency_layer */ \
-        NULL,                  /* get_color_mapping_procs */ \
-        NULL,                     /* get_color_comp_index */ \
-        NULL,                     /* encode_color */ \
-        NULL,                     /* decode_color */ \
-        NULL,                   /* pattern_manage */ \
-        NULL,                       /* fill_rectangle_hl_color */ \
-        NULL,                      /* include_color_space */ \
-        NULL,                    /* fill_linear_color_scanline */ \
-        NULL,                   /* fill_linear_color_trapezoid */ \
-        NULL,                    /* fill_linear_color_triangle */ \
-        NULL,                    /* update_spot_equivalent_colors */ \
-        NULL,                  /* ret_devn_params */ \
-        x_fillpage,              /* fillpage */ \
-        NULL,                      /* push_transparency_state */ \
-        NULL,                      /* pop_transparency_state */ \
-        NULL,                      /* put_image */ \
-        NULL,                      /* dev_spec_op */ \
-        NULL,                      /* copy_planes */ \
-        NULL,                      /* get_profile */ \
-        NULL,        /* set_graphics_type_tag */ \
-        NULL, \
-        NULL, \
-        NULL, \
-        NULL  \
-    }, \
+    { 0 },\
     gx_device_bbox_common_initial(0 /*false*/, 1 /*true*/, 1 /*true*/), \
     0 /*false*/,		/* is_buffered */ \
     1 /*true*/,			/* IsPageDevice */ \
@@ -240,15 +144,36 @@ const gx_device_X this_device = { \
     } \
 };
 
+static void
+x_finalize(const gs_memory_t *cmem, void *vpdev)
+{
+    /* Do an actual close of the window. */
+    gx_device_X *xdev = (gx_device_X *)vpdev;
+
+    if (xdev->dpy) {
+        XCloseDisplay(xdev->dpy);
+        xdev->dpy = NULL;
+    }
+    gx_device_finalize(cmem, vpdev);
+}
+
+gs_public_st_suffix_add1_final(st_device_X, gx_device_X,
+    "gx_device_X", device_x_enum_ptrs, device_x_reloc_ptrs,
+    x_finalize, st_device_bbox, buffer);
+
 x_device(gs_x11_device,
-         std_device_color_stype_body(gx_device_X, 0, "x11", &st_device_X,
+         std_device_color_stype_body(gx_device_X,
+                                     gdev_x_initialize_device_procs,
+                                     "x11", &st_device_X,
                                      FAKE_RES * DEFAULT_WIDTH_10THS / 10,
                                      FAKE_RES * DEFAULT_HEIGHT_10THS / 10,	/* x and y extent (nominal) */
                                      FAKE_RES, FAKE_RES,	/* x and y density (nominal) */
                                      24, 255, 256 ))
 
 x_device(gs_x11alpha_device,
-         std_device_dci_alpha_type_body(gx_device_X, 0, "x11alpha", &st_device_X,
+         std_device_dci_alpha_type_body(gx_device_X,
+                                        gdev_x_initialize_device_procs,
+                                        "x11alpha", &st_device_X,
                                         FAKE_RES * DEFAULT_WIDTH_10THS / 10,
                                         FAKE_RES * DEFAULT_HEIGHT_10THS / 10,	/* x and y extent (nominal) */
                                         FAKE_RES, FAKE_RES,	/* x and y density (nominal) */
@@ -269,7 +194,7 @@ GC gc, XImage * pi, int sx, int sy, int dx, int dy, unsigned w, unsigned h);
   END
 
 /* Open the device.  Most of the code is in gdevxini.c. */
-static int
+int
 x_open(gx_device * dev)
 {
     gx_device_X *xdev = (gx_device_X *) dev;
@@ -295,7 +220,7 @@ x_open(gx_device * dev)
 }
 
 /* Close the device. */
-static int
+int
 x_close(gx_device * dev)
 {
     gx_device_X *xdev = (gx_device_X *) dev;
@@ -306,7 +231,7 @@ x_close(gx_device * dev)
 /* Get initial matrix for X device. */
 /* This conflicts seriously with the code for page devices; */
 /* we only do it if Ghostview is active. */
-static void
+void
 x_get_initial_matrix(gx_device * dev, gs_matrix * pmat)
 {
     gx_device_X *xdev = (gx_device_X *) dev;
@@ -324,7 +249,7 @@ x_get_initial_matrix(gx_device * dev, gs_matrix * pmat)
 }
 
 /* Synchronize the display with the commands already given. */
-static int
+int
 x_sync(gx_device * dev)
 {
     gx_device_X *xdev = (gx_device_X *) dev;
@@ -351,7 +276,7 @@ gdev_x_send_event(gx_device_X *xdev, Atom msg)
 }
 
 /* Output "page" */
-static int
+int
 x_output_page(gx_device * dev, int num_copies, int flush)
 {
     gx_device_X *xdev = (gx_device_X *) dev;
@@ -374,7 +299,7 @@ x_output_page(gx_device * dev, int num_copies, int flush)
 }
 
 /* Fill a rectangle with a color. */
-static int
+int
 x_fill_rectangle(gx_device * dev,
                  int x, int y, int w, int h, gx_color_index gscolor)
 {
@@ -405,7 +330,7 @@ x_fill_rectangle(gx_device * dev,
 }
 
 /* Copy a monochrome bitmap. */
-static int
+int
 x_copy_mono(gx_device * dev,
             const byte * base, int sourcex, int raster, gx_bitmap_id id,
             int x, int y, int w, int h,
@@ -627,7 +552,8 @@ x_copy_image(gx_device_X * xdev, const byte * base, int sourcex, int raster,
     }
     return 0;
 }
-static int
+
+int
 x_copy_color(gx_device * dev,
              const byte * base, int sourcex, int raster, gx_bitmap_id id,
              int x, int y, int w, int h)
@@ -647,14 +573,14 @@ x_copy_color(gx_device * dev,
 
 /* Get the page device.  We reimplement this so that we can make this */
 /* device be a page device conditionally. */
-static gx_device *
+gx_device *
 x_get_page_device(gx_device * dev)
 {
     return (((gx_device_X *) dev)->IsPageDevice ? dev : (gx_device *) 0);
 }
 
 /* Tile a rectangle. */
-static int
+int
 x_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
                        int x, int y, int w, int h,
                        gx_color_index zero, gx_color_index one,
@@ -736,9 +662,9 @@ x_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 }
 
 /* Read bits back from the screen. */
-static int
+int
 x_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
-                     gs_get_bits_params_t * params, gs_int_rect ** unread)
+                     gs_get_bits_params_t * params)
 {
     gx_device_X *xdev = (gx_device_X *) dev;
     int depth = dev->color_info.depth;
@@ -770,7 +696,7 @@ x_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
         !(options & GB_RASTER_ALL)
         )
         return
-            gx_default_get_bits_rectangle(dev, prect, params, unread);
+            gx_default_get_bits_rectangle(dev, prect, params);
     params->options =
         GB_COLORS_NATIVE | GB_ALPHA_NONE | GB_PACKING_CHUNKY |
         GB_RETURN_COPY | GB_OFFSET_0 |
@@ -789,22 +715,6 @@ x_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
         flush_text(xdev);
     } else
         update_do_flush(xdev);
-    /*
-     * If we want a list of unread rectangles, turn on graphics
-     * exposures, and accept exposure events.
-     */
-        /******
-         ****** FOLLOWING IS WRONG.  XGetImage DOES NOT GENERATE
-         ****** EXPOSURE EVENTS.
-         ******/
-#if GET_IMAGE_EXPOSURES
-    if (unread) {
-        XSetGraphicsExposures(xdev->dpy, xdev->gc, True);
-        XGetWindowAttributes(xdev->dpy, xdev->win, &attributes);
-        XSelectInput(xdev->dpy, xdev->win,
-                     attributes.your_event_mask | ExposureMask);
-    }
-#endif /* GET_IMAGE_EXPOSURES */
     /*
      * The X library doesn't provide any way to specify the desired
      * bit or byte ordering for the result, so we may have to swap the
@@ -883,56 +793,13 @@ x_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
         }
         XDestroyImage(image);
     }
-    if (unread) {
-#if GET_IMAGE_EXPOSURES
-        XEvent event;
-#endif /* GET_IMAGE_EXPOSURES */
-
-        *unread = 0;
-#if GET_IMAGE_EXPOSURES
-        /* Read any exposure events. */
-        XWindowEvent(xdev->dpy, xdev->win, ExposureMask, &event);
-        if (event.type == GraphicsExpose) {
-            gs_int_rect *rects = (gs_int_rect *)
-                gs_alloc_bytes(dev->memory, sizeof(gs_int_rect),
-                               "x_get_bits_rectangle");
-            int num_rects = 0;
-
-            for (;;) {
-                if (rects == 0) {
-                    code = gs_note_error(gs_error_VMerror);
-                    break;
-                }
-#define xevent (*(XGraphicsExposeEvent *)&event)
-                rects[num_rects].q.x = xevent.width +
-                    (rects[num_rects].p.x = xevent.x);
-                rects[num_rects].q.y = xevent.height +
-                    (rects[num_rects].p.y = xevent.y);
-                ++num_rects;
-                if (!xevent.count)
-                    break;
-#undef xevent
-                rects = gs_resize_object(dev->memory, rects,
-                                         (num_rects + 1) * sizeof(gs_int_rect),
-                                         "x_get_bits_rectangle");
-            }
-            if (code >= 0) {
-                *unread = rects;
-                code = num_rects;
-            }
-        }
-        /* Restore the window state. */
-        XSetGraphicsExposures(xdev->dpy, xdev->gc, False);
-        XSelectInput(xdev->dpy, xdev->win, attributes.your_event_mask);
-#endif /* GET_IMAGE_EXPOSURES */
-    }
     return code;
 }
 
 /* Supplying a fillpage prevents the erasepage optimisation
    device being installed.
  */
-static int
+int
 x_fillpage(gx_device *dev, gs_gstate * pgs, gx_device_color *pdevc)
 {
     return gx_default_fillpage(dev, pgs, pdevc);
