@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -29,6 +29,7 @@
 #include "gdevpdtf.h"                /* Required to include gdevpdti.h */
 #include "gdevpdti.h"                /* For pdf_charproc_x_offset */
 #include "gsptype1.h"
+#include "gdevpdtt.h"
 
 /* We need this color space type for constructing temporary color spaces. */
 extern const gs_color_space_type gs_color_space_type_Indexed;
@@ -54,7 +55,7 @@ pdf_copy_mask_data(gx_device_pdf * pdev, const byte * base, int sourcex,
                    gs_image_t *pim, pdf_image_writer *piw,
                    int for_pattern)
 {
-    ulong nbytes;
+    uint64_t nbytes;
     int code;
     const byte *row_base;
     int row_step;
@@ -62,7 +63,7 @@ pdf_copy_mask_data(gx_device_pdf * pdev, const byte * base, int sourcex,
 
     gs_image_t_init_mask(pim, true);
     pdf_make_bitmap_image(pim, x, y, w, h);
-    nbytes = ((ulong)w * h + 7) / 8;
+    nbytes = ((uint64_t)w * h + 7) / 8;
 
     if (for_pattern) {
         /*
@@ -137,7 +138,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
     pdf_stream_position_t ipos;
     pdf_resource_t *pres = 0;
     byte invert = 0;
-    bool in_line = false;
+    bool in_line = false, char_proc_begun = false;
     gs_show_enum *show_enum = (gs_show_enum *)pdev->pte;
     int x_offset, y_offset;
     double width;
@@ -181,6 +182,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
                                            &pcp, &ipos);
                 if (code < 0)
                     return code;
+                char_proc_begun = true;
                 y_offset = -y_offset;
                 width = psdf_round(pdev->char_width.x, 100, 10); /* See
                         pdf_write_Widths about rounding. We need to provide
@@ -192,7 +194,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
                 pdf_image_writer_init(&writer);
                 code = pdf_begin_write_image(pdev, &writer, gs_no_id, w, h, NULL, true);
                 if (code < 0)
-                    return code;
+                    goto fail;
                 pres = (pdf_resource_t *) pcp;
                 goto wr;
             } else if (pdev->pte != NULL) {
@@ -216,13 +218,17 @@ pdf_copy_mono(gx_device_pdf *pdev,
             set_image_color(pdev, zero);
     } else if (zero == pdev->black && one == pdev->white) {
         pcs = gs_cspace_new_DeviceGray(pdev->memory);
-        if (pcs == NULL)
-            return_error(gs_error_VMerror);
+        if (pcs == NULL) {
+            code = gs_note_error(gs_error_VMerror);
+            goto fail;
+        }
         gs_image_t_init(&image, pcs);
     } else if (zero == pdev->white && one == pdev->black) {
         pcs = gs_cspace_new_DeviceGray(pdev->memory);
-        if (pcs == NULL)
-            return_error(gs_error_VMerror);
+        if (pcs == NULL) {
+            code = gs_note_error(gs_error_VMerror);
+            goto fail;
+        }
         gs_image_t_init(&image, pcs);
         invert = 0xff;
     } else {
@@ -240,13 +246,14 @@ pdf_copy_mono(gx_device_pdf *pdev,
 
         code = pdf_cspace_init_Device(pdev->memory, &pcs_base, ncomp);
         if (code < 0)
-            return code;
+            goto fail;
         c[0] = psdf_adjust_color_index((gx_device_vector *)pdev, zero);
         c[1] = psdf_adjust_color_index((gx_device_vector *)pdev, one);
         pcs = gs_cspace_alloc(pdev->memory, &gs_color_space_type_Indexed);
         if (pcs == NULL) {
             rc_decrement_cs(pcs_base, "pdf_copy_mono");
-            return_error(gs_error_VMerror);
+            code = gs_note_error(gs_error_VMerror);
+            goto fail;
         }
         pcs->base_space = pcs_base;
         pcs->params.indexed.hival = 1;
@@ -263,18 +270,18 @@ pdf_copy_mono(gx_device_pdf *pdev,
     }
     pdf_make_bitmap_image(&image, x, y, w, h);
     {
-        ulong nbytes = (ulong) ((w + 7) >> 3) * h;
+        uint64_t nbytes = (uint64_t) ((w + 7) >> 3) * h;
 
         code = pdf_open_page(pdev, PDF_IN_STREAM);
         if (code < 0)
-            return code;
+            goto fail;
         in_line = nbytes < pdev->MaxInlineImageSize;
         if (in_line)
             pdf_put_image_matrix(pdev, &image.ImageMatrix, 1.0);
         pdf_image_writer_init(&writer);
         code = pdf_begin_write_image(pdev, &writer, gs_no_id, w, h, NULL, in_line);
         if (code < 0)
-            return code;
+            goto fail;
     }
   wr:
     if (image.ImageMask)
@@ -287,7 +294,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
         code = pdf_color_space_named(pdev, NULL, &cs_value, NULL, pcs,
                                &writer.pin->color_spaces, in_line, NULL, 0, false);
         if (code < 0)
-            return code;
+            goto fail;
         pcsvalue = &cs_value;
     }
     /*
@@ -322,33 +329,37 @@ pdf_copy_mono(gx_device_pdf *pdev,
     code = pdf_begin_image_data(pdev, &writer, (const gs_pixel_image_t *)&image,
                          pcsvalue, 0);
     if (code < 0)
-        return code;
+        goto fail;
 
     code = pdf_copy_mask_bits(writer.binary[0].strm, base, sourcex, raster,
                               w, h, invert);
     if (code < 0)
-        return code;
+        goto fail;
     code = pdf_end_image_binary(pdev, &writer, writer.height);
     if (code < 0)
-        return code;
+        goto fail;
 
     if (!pres) {
         switch ((code = pdf_end_write_image(pdev, &writer))) {
             default:                /* error */
-                return code;
+                goto fail;
             case 1:
-                return 0;
+                code = 0;
+                goto fail;
             case 0:
-                return pdf_do_image(pdev, writer.pres, &image.ImageMatrix,
+                code = pdf_do_image(pdev, writer.pres, &image.ImageMatrix,
                                     true);
+                goto fail;
         }
     }
     writer.end_string = "";        /* no Q */
     switch ((code = pdf_end_write_image(pdev, &writer))) {
     default:                /* error */
+        goto fail;
         return code;
     case 0:                        /* not possible */
-        return_error(gs_error_Fatal);
+        code = gs_note_error(gs_error_Fatal);
+        goto fail;
     case 1:
         break;
     }
@@ -365,6 +376,11 @@ pdf_copy_mono(gx_device_pdf *pdev,
         imat.yy /= h;
         return pdf_do_char_image(pdev, (const pdf_char_proc_t *)pres, &imat);
     }
+
+fail:
+    if (char_proc_begun)
+        (void)pdf_end_char_proc(pdev, &ipos);
+    return code;
 }
 int
 gdev_pdf_copy_mono(gx_device * dev,
@@ -392,7 +408,7 @@ pdf_copy_color_data(gx_device_pdf * pdev, const byte * base, int sourcex,
     int bytes_per_pixel = depth >> 3;
     gs_color_space *pcs;
     cos_value_t cs_value;
-    ulong nbytes;
+    uint64_t nbytes;
     int code = pdf_cspace_init_Device(pdev->memory, &pcs, bytes_per_pixel);
     const byte *row_base;
     int row_step;
@@ -405,7 +421,7 @@ pdf_copy_color_data(gx_device_pdf * pdev, const byte * base, int sourcex,
     gs_image_t_init(pim, pcs);
     pdf_make_bitmap_image(pim, x, y, w, h);
     pim->BitsPerComponent = 8;
-    nbytes = (ulong)w * bytes_per_pixel * h;
+    nbytes = (uint64_t)w * bytes_per_pixel * h;
 
     if (for_pattern == 1) {
         /*
@@ -496,15 +512,65 @@ gdev_pdf_copy_color(gx_device * dev, const byte * base, int sourcex,
 /* Fill a mask. */
 int
 gdev_pdf_fill_mask(gx_device * dev,
-                 const byte * data, int data_x, int raster, gx_bitmap_id id,
+                   const byte * data, int data_x, int raster, gx_bitmap_id id,
                    int x, int y, int width, int height,
                    const gx_drawing_color * pdcolor, int depth,
                    gs_logical_operation_t lop, const gx_clip_path * pcpath)
 {
     gx_device_pdf *pdev = (gx_device_pdf *) dev;
+    pdf_text_enum_t *penum = (pdf_text_enum_t *)pdev->OCR_enum;
 
     if (width <= 0 || height <= 0)
         return 0;
+
+    /* If OCRStage is 'OCR_Rendering' then we are handling an image which is a rendered glyph
+     * that we want to have OCR software process and return a Unicode code point for.
+     * We specifically do *not* want to send the image to the output PDF file!
+     */
+    if (pdev->OCRStage == OCR_Rendering) {
+        int code = 0;
+        ocr_glyph_t *new_glyph = NULL;
+        int index;
+
+        new_glyph = (ocr_glyph_t *)gs_alloc_bytes(pdev->pdf_memory, sizeof(ocr_glyph_t), "");
+        if (new_glyph == NULL)
+            return_error(gs_error_VMerror);
+        new_glyph->data = gs_alloc_bytes(pdev->pdf_memory, raster*height, "");
+        if (new_glyph->data == NULL)
+            return_error(gs_error_VMerror);
+        memcpy(new_glyph->data, data, raster * height);
+        new_glyph->height = height;
+        new_glyph->width = width;
+        new_glyph->raster = raster;
+        new_glyph->x = x;
+        new_glyph->y = y;
+        if (penum != NULL) {
+            new_glyph->char_code = penum->pte_default->returned.current_char;
+            new_glyph->glyph = penum->pte_default->returned.current_glyph;
+        } else {
+            new_glyph->char_code = pdev->OCR_char_code;
+            new_glyph->glyph = pdev->OCR_glyph;
+        }
+        new_glyph->next = NULL;
+        new_glyph->is_space = true;
+        for(index = 0; index < height * raster;index++){
+            if(data[index] != 0x00) {
+                new_glyph->is_space = false;
+                break;
+            }
+        }
+        if (pdev->ocr_glyphs == NULL)
+            pdev->ocr_glyphs = new_glyph;
+        else {
+            ocr_glyph_t *next = pdev->ocr_glyphs;
+
+            while (next->next != NULL)
+                next = next->next;
+            next->next = new_glyph;
+        }
+        return code;
+    }
+
     if (depth > 1 || (!gx_dc_is_pure(pdcolor) != 0 && !(gx_dc_is_pattern1_color(pdcolor))))
         return gx_default_fill_mask(dev, data, data_x, raster, id,
                                     x, y, width, height, pdcolor, depth, lop,
@@ -558,14 +624,14 @@ gdev_pdf_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
     if (!pres) {
         /* Create the Pattern resource. */
         int code;
-        long length_id;
+        int64_t length_id;
         gs_offset_t start, end;
         stream *s;
         gs_image_t image;
         pdf_image_writer writer;
-        long image_bytes = ((long)tw * depth + 7) / 8 * th;
+        int64_t image_bytes = ((int64_t)tw * depth + 7) / 8 * th;
         bool in_line = image_bytes < pdev->MaxInlineImageSize;
-        ulong tile_id =
+        int64_t tile_id =
             (tw == tiles->size.x && th == tiles->size.y ? tiles->id :
              gx_no_bitmap_id);
 
@@ -588,7 +654,7 @@ gdev_pdf_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
         pprintg2(s, "/Matrix[%g 0 0 %g 0 0]", tw / xscale, th / yscale);
         stream_puts(s, "/BBox[0 0 1 1]/XStep 1/YStep 1/Length ");
         length_id = pdf_obj_ref(pdev);
-        pprintld1(s, "%ld 0 R>>stream\n", length_id);
+        pprinti64d1(s, "%"PRId64" 0 R>>stream\n", length_id);
         start = pdf_stell(pdev);
         code = copy_data(pdev, tiles->data, 0, tiles->raster,
                          tile_id, 0, 0, tw, th, &image, &writer, -1);
@@ -604,7 +670,7 @@ gdev_pdf_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
         stream_puts(s, "\nendstream\n");
         pdf_end_resource(pdev, resourcePattern);
         pdf_open_separate(pdev, length_id, resourceNone);
-        pprintld1(pdev->strm, "%ld\n", end - start);
+        pprinti64d1(pdev->strm, "%"PRId64"\n", end - start);
         pdf_end_separate(pdev, resourceNone);
         pres->object->written = true; /* don't write at end of page */
     }
@@ -631,7 +697,7 @@ gdev_pdf_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
             pprintg3(s, " %g %g %g", (int)(color1 >> 16) / 255.0,
                      (int)((color1 >> 8) & 0xff) / 255.0,
                      (int)(color1 & 0xff) / 255.0);
-        pprintld1(s, "/R%ld scn", pdf_resource_id(pres));
+        pprinti64d1(s, "/R%"PRId64" scn", pdf_resource_id(pres));
         pprintg4(s, " %g %g %g %g re f Q\n",
                  x / xscale, y / yscale, w / xscale, h / xscale);
     }
