@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -100,6 +100,7 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  0 /*false*/,			/* fill_overprint */
  0 /*false*/,			/* stroke_overprint */
  1 /* Absolute Colorimetric */, /* rendering intent */
+ 0 /*false */,                  /* user_icc */
  0 /*false*/,                   /* remap_fill_coilor */
  0 /*false*/,                   /* remap_stroke_coilor */
  gs_no_id,			/* halftone_id */
@@ -112,7 +113,11 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  {{0}},				/* xref */
  {{0}},				/* asides */
  {{0}},				/* streams */
- {{0}},				/* pictures */
+ {{0}},				/* ObjStm */
+ gs_no_id,          /* ObjStm_id */
+ NULL,              /* ObjStmOffsets */
+ 0,                 /* NumObjstmObjects */
+ true,             /* doubleXref */
  0,				/* next_id */
  0,				/* Catalog */
  0,				/* Info */
@@ -146,6 +151,7 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  0,				/* articles */
  0,				/* Dests */
  0,				/* EmbeddedFiles */
+ 0,             /* AF (part of embedded files for PDF/A-3) */
  {0},				/* fileID */
          {0, 0},		/* uuid_time */
  0,				/* global_named_objects */
@@ -155,6 +161,9 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  0,				/* font_cache */
  {0, 0},			/* char_width */
  0,				/* clip_path */
+ 0,             /* clipped_text_pending */
+ 0,             /* saved_vgstack_depth_for_textclip */
+ 0,             /* saved_vgstack_bottom_for_textclip */
  0,                             /* PageLabels */
  -1,                            /* PageLabels_current_page */
  0,                             /* PageLabels_current_label */
@@ -197,6 +206,7 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  {0,0,0,0,0,0}, 		/* gs_matrix converting_image_matrix */
  0,				/* image_mask_scale */
  NULL,				/* pres_soft_mask_dict */
+ 0,                 /* pdfmark_dup_stream */
  {0, 0},			/* objname */
  0,				/* OPDFRead_procset_length */
  0,				/* find_resource_param */
@@ -213,6 +223,7 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  true,				/* DetectDuplicateImages */
  false,				/* AllowIncrementalCFF */
  !PDF_FOR_OPDFREAD,		/* WantsToUnicode */
+ !PDF_FOR_OPDFREAD,		/* WantsOptionalContent */
  !PDF_FOR_OPDFREAD,		/* PdfmarkCapable */
  !PDF_FOR_OPDFREAD,		/* WantsPageLabels */
  PDF_FOR_OPDFREAD,		/* AllowPSRepeatFunctions */
@@ -236,11 +247,19 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  NULL,                  /* OCRUnicode */
  0,                     /* OCR_char_code */
  0,                     /* OCR_glyph */
+ NULL,                  /* OCE_penum */
  NULL,                  /* ocr_glyphs */
  0,                     /* initial_pattern_state */
  false,                 /* OmitInfoDate */
  false,                 /* OmitXMP */
- false                  /* OmitID */
+ false,                 /* OmitID */
+ false,                 /* ModifiesPageSize */
+ false,                 /* ModifiesPageOrder */
+ true,                  /* WriteXRefStm */
+ true,                  /* WriteObjStms */
+ NULL,                     /* PendingOC */
+ true,                  /* ToUnicodeForStdEnc */
+ true                   /* EmbedSubstituteFonts */
 };
 
 #else
