@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -360,7 +360,7 @@ attach_cmap_resource(gx_device_pdf *pdev, pdf_font_resource_t *pdfont,
                     (pcmap->WMode ? "/Identity-V" : "/Identity-H"));
         else
             gs_snprintf(pdfont->u.type0.Encoding_name, sizeof(pdfont->u.type0.Encoding_name),
-                        "%ld 0 R", pdf_resource_id(pcmres));
+                        "%"PRId64" 0 R", pdf_resource_id(pcmres));
     } else {
         uint size = 0;
 
@@ -431,7 +431,7 @@ scan_cmap_text(pdf_text_enum_t *pte, void *vbuf)
     gs_font_type0 *const font = (gs_font_type0 *)pte->orig_font; /* Type 0, fmap_CMap */
     /* Not sure. Changed for CDevProc callout. Was pte->current_font */
     gs_text_enum_t scan = *(gs_text_enum_t *)pte;
-    int wmode = font->WMode, code, rcode = 0;
+    int wmode = font->WMode == 0 ? 0 : 1, code, rcode = 0;
     pdf_font_resource_t *pdsubf0 = NULL;
     gs_font *subfont0 = NULL, *saved_subfont = NULL;
     uint index = scan.index, xy_index = scan.xy_index, start_index = index;
@@ -450,7 +450,7 @@ scan_cmap_text(pdf_text_enum_t *pte, void *vbuf)
         uint font_index = 0x7badf00d;
         gs_const_string str;
         pdf_text_process_state_t text_state;
-        pdf_font_resource_t *pdsubf;
+        pdf_font_resource_t *pdsubf = NULL;
         gs_font *subfont = NULL;
         gs_point wxy;
         bool font_change = 0;
@@ -797,17 +797,17 @@ scan_cmap_text(pdf_text_enum_t *pte, void *vbuf)
                 pte->text.operation = save_op;
                 pte->text.data.glyphs = save_data;
             }
-            pte->current_font = subfont0;
             if (!subfont0 || !pdsubf0)
                 /* This should be impossible */
                 return_error(gs_error_invalidfont);
 
+            pte->current_font = subfont0;
             code = gs_matrix_multiply(&subfont0->FontMatrix, &font->FontMatrix, &m3);
             /* We thought that it should be gs_matrix_multiply(&font->FontMatrix, &subfont0->FontMatrix, &m3); */
             if (code < 0)
                 return code;
-            if (pdsubf0->FontType == ft_user_defined  || pdsubf0->FontType == ft_PDF_user_defined  || pdsubf->FontType == ft_encrypted ||
-                pdsubf->FontType == ft_encrypted2)
+            if (pdsubf0->FontType == ft_user_defined  || pdsubf0->FontType == ft_PDF_user_defined  || pdsubf0->FontType == ft_encrypted ||
+                pdsubf0->FontType == ft_encrypted2)
                     pdfont = pdsubf0;
             else {
                 code = pdf_obtain_parent_type0_font_resource(pdev, pdsubf0, font_index0,
