@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -81,6 +81,9 @@ pdf_process_string_aux(pdf_text_enum_t *penum, gs_string *pstr,
     return pdf_process_string(penum, pstr, pfmat, ppts, gdata);
 }
 
+/* #define OCR_DUMP_BITMAP */
+#undef OCR_DUMP_BITMAP
+
 static int OCRText(gx_device_pdf *pdev, gs_glyph glyph, gs_char ch, gs_char *length, byte **unicode)
 {
 #if OCR_VERSION > 0
@@ -93,7 +96,9 @@ static int OCRText(gx_device_pdf *pdev, gs_glyph glyph, gs_char ch, gs_char *len
         byte *bitmap = NULL, *src, *dest, *rowptr, srcmask, destmask;
         void *state;
         const char *language = pdev->ocr_language;
+#ifdef OCR_DUMP_BITMAP
         gp_file *DbgFile;
+#endif
 
         if(language == NULL || language[0] == 0)
             language = "eng";
@@ -183,7 +188,7 @@ static int OCRText(gx_device_pdf *pdev, gs_glyph glyph, gs_char ch, gs_char *len
             next_glyph = next_glyph->next;
         }
 
-#if 0
+#ifdef OCR_DUMP_BITMAP
         DbgFile = gp_fopen(pdev->memory, "d:/temp/bits.txt", "wb+");
         for(row = 0;row < rows;row++) {
             for(column = 0;column < stride;column++) {
@@ -270,21 +275,36 @@ static int OCRText(gx_device_pdf *pdev, gs_glyph glyph, gs_char ch, gs_char *len
          * the stored values returned by the OCR engine.
          */
         if(new_glyph) {
-            *unicode = (byte *)gs_alloc_bytes(pdev->memory, 2 * sizeof(ushort), "temporary Unicode array");
-            if(*unicode == NULL)
-                return_error(gs_error_VMerror);
-            u = (char *)(*unicode);
-            if(new_glyph->is_space) {
-                memset(u, 0x00, 3);
-                u[3] = 0x20;
-            }
-            else {
-                for(ix = 0;ix < 4;ix++) {
-                    u[3 - ix] = (pdev->OCRUnicode[ocr_index] & mask) >> (8 * ix);
-                    mask = mask << 8;
+            if(pdev->OCRUnicode[ocr_index] > 0xFFFF) {
+                *unicode = (byte *)gs_alloc_bytes(pdev->memory, 2 * sizeof(ushort), "temporary Unicode array");
+                if(*unicode == NULL)
+                    return_error(gs_error_VMerror);
+                u = (char *)(*unicode);
+                if(new_glyph->is_space) {
+                    memset(u, 0x00, 3);
+                    u[3] = 0x20;
                 }
+                else{
+                    for(ix = 0;ix < 4;ix++) {
+                        u[3 - ix] = (pdev->OCRUnicode[ocr_index] & mask) >> (8 * ix);
+                        mask = mask << 8;
+                    }
+                }
+                *length = 4;
+            }else{
+                *unicode = (byte *)gs_alloc_bytes(pdev->memory, sizeof(ushort), "temporary Unicode array");
+                if(*unicode == NULL)
+                    return_error(gs_error_VMerror);
+                u = (char *)(*unicode);
+                if(new_glyph->is_space) {
+                    memset(u, 0x00, 2);
+                    u[1] = 0x20;
+                }else{
+                    u[0] = (pdev->OCRUnicode[ocr_index] & 0xFF00) >> 8;
+                    u[1] = (pdev->OCRUnicode[ocr_index] & 0xFF);
+                }
+                *length = 2;
             }
-            *length = 4;
         }
     }
     #endif
@@ -387,10 +407,32 @@ pdf_add_ToUnicode(gx_device_pdf *pdev, gs_font *font, pdf_font_resource_t *pdfon
             length = font->procs.decode_glyph((gs_font *)font, glyph, ch, unicode, length);
         }
 
-        if (pdfont->cmap_ToUnicode != NULL)
-            gs_cmap_ToUnicode_add_pair(pdfont->cmap_ToUnicode, ch, unicode, length);
-        if (length > 2 && pdfont->u.simple.Encoding != NULL)
-            pdfont->TwoByteToUnicode = 0;
+        /* We use this when determining whether we should use an existing ToUnicode
+         * CMap entry, for s aimple font. The basic problem appears to be that the front end ToUnicode
+         * processing is somewhat limited, due to its origins in the PostScript GlyphNames2Unicode
+         * handling. We cannot support more than 4 bytes on input, the bug for 702201 has a ToUnicode
+         * entry which maps the single /ffi glyph to 'f' 'f' and 'i' code points for a total of 6
+         * (3 x UTF-16BE code points) bytes. If we just leave the Encoding alone then Acrobat will use the
+         * /ffi glyph to get the 'correct' answer. This was originally done using a 'TwoByteToUnicode'
+         * flag in the font and if the flag was not true, then we dropped the entire ToUnicode CMap.
+         *
+         * Additionally; bug #708284, the ToUnicode CMap is actually broken, it contains invalid UTF-16BE
+         * entries which actually are 4 bytes long. Acrobat silently ignores the broekn entires (!) but the fact
+         * that we dropped the entire CMap meant that none of the content pasted correctly.
+         *
+         * So... Until we get to the point of preserving input codes in excess of 4 bytes we do some hideous
+         * hackery here and simply drop ToUnicode entries in simple fonts, with a standard encoding, when the
+         * ToUnicode entry is not a single UTF-16BE code point. Acrobat will use the Encoding for the missing entries
+         * or the character code in extremis, which is as good as this is going to get right now.
+         *
+         */
+        if (pdfont->cmap_ToUnicode != NULL) {
+            if (pdfont->u.simple.Encoding != NULL) {
+                if (length <= 2)
+                    gs_cmap_ToUnicode_add_pair(pdfont->cmap_ToUnicode, ch, unicode, length);
+            } else
+                gs_cmap_ToUnicode_add_pair(pdfont->cmap_ToUnicode, ch, unicode, length);
+        }
     }
 
     if (unicode)
@@ -512,17 +554,17 @@ pdf_encode_string_element(gx_device_pdf *pdev, gs_font *font, pdf_font_resource_
         if (code < 0 && code != gs_error_undefined)
             return code;
         if (code == gs_error_undefined) {
-            if (pdev->PDFA != 0 || pdev->PDFX) {
+            if (pdev->PDFA != 0 || pdev->PDFX != 0) {
                 switch (pdev->PDFACompatibilityPolicy) {
                     case 0:
                         emprintf(pdev->memory,
-                             "Requested glyph not present in source font,\n not permitted in PDF/A, reverting to normal PDF output\n");
+                             "Requested glyph not present in source font,\n not permitted in PDF/A or PDF/X, reverting to normal PDF output\n");
                         pdev->AbortPDFAX = true;
                         pdev->PDFA = 0;
                         break;
                     case 1:
                         emprintf(pdev->memory,
-                             "Requested glyph not present in source font,\n not permitted in PDF/A, glyph will not be present in output file\n\n");
+                             "Requested glyph not present in source font,\n not permitted in PDF/A or PDF/X, glyph will not be present in output file\n\n");
                         /* Returning an error causees text processing to try and
                          * handle the glyph by rendering to a bitmap instead of
                          * as a glyph in a font. This will eliminate the problem
@@ -532,7 +574,7 @@ pdf_encode_string_element(gx_device_pdf *pdev, gs_font *font, pdf_font_resource_
                         break;
                     case 2:
                         emprintf(pdev->memory,
-                             "Requested glyph not present in source font,\n not permitted in PDF/A, aborting conversion\n");
+                             "Requested glyph not present in source font,\n not permitted in PDF/A or PDF/X, aborting conversion\n");
                         /* Careful here, only certain errors will bubble up
                          * through the text processing.
                          */
@@ -540,7 +582,7 @@ pdf_encode_string_element(gx_device_pdf *pdev, gs_font *font, pdf_font_resource_
                         break;
                     default:
                         emprintf(pdev->memory,
-                             "Requested glyph not present in source font,\n not permitted in PDF/A, unrecognised PDFACompatibilityLevel,\nreverting to normal PDF output\n");
+                             "Requested glyph not present in source font,\n not permitted in PDF/A or PDF/X, unrecognised PDFACompatibilityLevel,\nreverting to normal PDF output\n");
                         pdev->AbortPDFAX = true;
                         pdev->PDFA = 0;
                         break;
@@ -871,6 +913,7 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
                 if (rect.p.x > rect.q.x || rect.p.y > rect.q.y) {
                     penum->index += pstr->size;
                     text->operation &= ~TEXT_DO_DRAW;
+                    penum->text_clipped = true;
                 }
             }
         } else {
