@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -85,7 +85,7 @@ typedef struct pdf_font_resource_s pdf_font_resource_t;
 /* ---------------- Font accessing ---------------- */
 
 /* Get the object ID of a font resource. */
-long pdf_font_id(const pdf_font_resource_t *pdfont);
+int64_t pdf_font_id(const pdf_font_resource_t *pdfont);
 /* Register charproc fonts with the page or substream. */
 int pdf_used_charproc_resources(gx_device_pdf *pdev, pdf_font_resource_t *pdfont);
 
