@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -769,6 +769,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
                         const gs_const_string *alt_font_name)
 {
     gs_font *const font = (gs_font *)pfont;
+    gs_type42_data *d0 = &pfont->data;
     gs_const_string font_name;
     byte OffsetTable[12];
     uint numTables_stored, numTables, numTables_out;
@@ -825,12 +826,8 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 #define W(a,b,c,d)\
   ( ((a) << 24) + ((b) << 16) + ((c) << 8) + (d))
 
-    READ_SFNTS(pfont, 0, 12, OffsetTable);
-    if (u32(OffsetTable) == W('t','t','c','f')) {
-        READ_SFNTS(pfont, 12, 4, OffsetTable);
-        TTCFontOffset = u32(OffsetTable);
-        READ_SFNTS(pfont, TTCFontOffset, 12, OffsetTable);
-    }
+    TTCFontOffset = d0->subfontOffset;
+    READ_SFNTS(pfont, TTCFontOffset, 12, OffsetTable);
     numTables_stored = U16(OffsetTable + 4);
     for (i = numTables = 0; i < numTables_stored; ++i) {
         byte tab[16];
@@ -845,7 +842,6 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
         length = u32(tab + 12);
         /* Copy the table data now (a rudiment of old code). */
         memcpy(&tables[numTables * 16], tab, 16);
-
         switch (u32(tab)) {
         case W('h','e','a','d'):
             if (length < 54)
@@ -900,13 +896,16 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
             /* falls through */
         case W('c','v','t',' '):
         case W('f','p','g','m'):
-        case W('g','a','s','p'):
-        case W('k','e','r','n'):
         case W('p','r','e','p'):
             break;		/* always copy these if present */
+        case W('D','S','I','G'):
         case W('E','B','D','T'):
         case W('E','B','L','C'):
         case W('E','B','S','C'):
+        case W('G','D','E','F'):
+        case W('G','P','O','S'):
+        case W('g','a','s','p'):
+        case W('k','e','r','n'):
             continue;
         default:
             if (writing_cid)
