@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -42,6 +42,9 @@
 #include "gsicc_cache.h"
 #include <stdlib.h>             /* abs() */
 
+#include "gxfcache.h"
+#include "gxfont.h"
+
 /* ---------------- Device definition ---------------- */
 
 /* Define the default resolution. */
@@ -141,6 +144,7 @@ static dev_proc_fill_mask(pclxl_fill_mask);
 
 static dev_proc_get_params(pclxl_get_params);
 static dev_proc_put_params(pclxl_put_params);
+static dev_proc_text_begin(pclxl_text_begin);
 
 /*static dev_proc_draw_thin_line(pclxl_draw_thin_line); */
 static dev_proc_begin_typed_image(pclxl_begin_typed_image);
@@ -170,6 +174,7 @@ pclxl_initialize_device_procs(gx_device *dev,
     set_dev_proc(dev, fill_triangle, gdev_vector_fill_triangle);
     set_dev_proc(dev, begin_typed_image, pclxl_begin_typed_image);
     set_dev_proc(dev, strip_copy_rop2, pclxl_strip_copy_rop2);
+    set_dev_proc(dev, text_begin, pclxl_text_begin);
 }
 
 static void
@@ -358,7 +363,10 @@ pclxl_can_handle_color_space(const gs_color_space * pcs)
     return !(index == gs_color_space_index_Separation ||
              index == gs_color_space_index_Pattern ||
              index == gs_color_space_index_DeviceN ||
-             index == gs_color_space_index_ICC);
+             index == gs_color_space_index_ICC ||
+             (index <= gs_color_space_index_CIEA &&
+             index >= gs_color_space_index_CIEDEFG)
+             );
 }
 
 /* Test whether we can icclink-transform an image. */
@@ -2808,3 +2816,12 @@ pclxl_put_params(gx_device * dev,       /* I - Device info */
 
     return (0);
 }
+
+static int pclxl_text_begin(gx_device * dev, gs_gstate * pgs,
+                    const gs_text_params_t *text, gs_font * font,
+                    const gx_clip_path * pcpath,
+                    gs_text_enum_t ** ppte)
+{
+    font->dir->ccache.upper = 0;
+    return gx_default_text_begin(dev, pgs, text, font, pcpath, ppte);
+}
