@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2022 Artifex Software, Inc.
+/* Copyright (C) 2019-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* jpgtop.c */
@@ -498,6 +498,9 @@ jpg_impl_process(pl_interp_implementation_t * impl, stream_cursor_read * pr)
         case ii_state_identifying:
         {
             const byte *hdr;
+
+            jpg->jerr.setjmp_buffer = align_setjmp_buffer(&jpg->aligned_jmpbuf);
+
             /* Try and get us 11 bytes */
             code = ensure_bytes(jpg, pr, 11);
             if (code < 0)
@@ -519,8 +522,6 @@ jpg_impl_process(pl_interp_implementation_t * impl, stream_cursor_read * pr)
                 break;
             }
 
-            jpg->jerr.setjmp_buffer = align_setjmp_buffer(&jpg->aligned_jmpbuf);
-
             jpg->cinfo.err = jpeg_std_error(&jpg->jerr.pub);
             jpg->jerr.pub.error_exit = jpg_error_exit;
             if (setjmp(*jpg->jerr.setjmp_buffer)) {
@@ -538,6 +539,7 @@ jpg_impl_process(pl_interp_implementation_t * impl, stream_cursor_read * pr)
             jpg->jsrc.term_source = jpg_term_source;
 
             jpg->state = ii_state_jpeg_header;
+            jpg->y = 0;
             break;
         case ii_state_jpeg_header:
         {
