@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -286,6 +286,8 @@ set_dev_specific_default_palette(pcl_cs_base_t * pbase, /* ignored in this case
         palette[3 * i] = cmy_default[3 * porder[i]];
         palette[3 * i + 1] = cmy_default[3 * porder[i] + 1];
         palette[3 * i + 2] = cmy_default[3 * porder[i] + 2];
+        if (num > 8)
+            palette[3 * i + 3] = cmy_default[3 * porder[i] + 3];
     }
 }
 
@@ -452,16 +454,17 @@ set_default_entries(pcl_cs_indexed_t * pindexed, int start, int num, bool gl2)
             set_dev_specific_default_palette,   /* CMY */
             set_colmet_default_palette, /* colorimetric RGB */
             set_CIELab_default_palette, /* CIE L*a*b* */
-            set_lumchrom_default_palette        /* luminance-
+            set_lumchrom_default_palette,        /* luminance-
                                                  * chrominance */
+            set_dev_specific_default_palette   /* KCMY */
     };
 
     /*
-     * For each color space, 8 palette entries are stored in the canonical
-     * CMY order; any palette entries beyond the first 8 always default to
+     * For each color space, 8 or 16 palette entries are stored in the canonical
+     * CMY order; any palette entries beyond the first 8 or 16 always default to
      * black. These arrays are incorporated into procedures that handle the
      * generation of colors for each color space type. For the bits per index
-     * settings of 1, 2, or >= 3, an order array is provided, to indicate the
+     * settings of 1, 2, 3, or >= 4, an order array is provided, to indicate the
      * order in which the default palette entries should be entered into the
      * palette. Separate arrays are provided for the RGB and CMY color spaces,
      * and for GL/2 default colors.
@@ -469,11 +472,12 @@ set_default_entries(pcl_cs_indexed_t * pindexed, int start, int num, bool gl2)
     static const byte order_1[] = { 0, 7 };
     static const byte cmy_order_2[] = { 0, 1, 2, 7 };
     static const byte cmy_order_3[] = { 0, 1, 2, 3, 4, 5, 6, 7 };
+    static const byte cmy_order_4[] = { 0, 7, 1, 7, 2, 7, 3, 7, 4, 7, 5, 7, 6, 7, 7, 7 };
     static const byte rgb_order_2[] = { 7, 6, 5, 0 };
     static const byte rgb_order_3[] = { 7, 6, 5, 4, 3, 2, 1, 0 };
     static const byte gl2_order_2[] = { 0, 7, 6, 5 };
     static const byte gl2_order_3[] = { 0, 7, 6, 5, 4, 3, 2, 1 };
-    static const byte *cmy_order[3] = { order_1, cmy_order_2, cmy_order_3 };
+    static const byte *cmy_order[4] = { order_1, cmy_order_2, cmy_order_3, cmy_order_4 };
     static const byte *rgb_order[3] = { order_1, rgb_order_2, rgb_order_3 };
     static const byte *gl2_order[3] = { order_1, gl2_order_2, gl2_order_3 };
     int type = pindexed->cid.cspace;
@@ -495,7 +499,12 @@ set_default_entries(pcl_cs_indexed_t * pindexed, int start, int num, bool gl2)
     else
         porder = cmy_order[bits];
 
-    /* set the default colors for up to the first 8 entries */
+    /* If 4-plane simple color KCMY graphics, set cnt for larger palette. */
+    if (type == pcl_cspace_KCMY) {
+        cnt = 15;
+        porder = cmy_order_4;
+    }
+    /* set the default colors for up to the first 16 entries */
     set_default_palette[(int)type] (pindexed->pbase,
                                     pindexed->palette.data,
                                     porder, start, cnt);
