@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -945,7 +945,7 @@ hpgl_print_char(hpgl_state_t * pgls, uint ch)
                    page issue in practice. */
                 code = pcl_mark_page_for_current_pos(pgls);
             }
-            gs_text_release(penum, "hpgl_print_char");
+            gs_text_release(pgs, penum, "hpgl_print_char");
             if (code < 0) {
                 hpgl_free_stick_fonts(pgls);
                 return code;
@@ -1203,6 +1203,7 @@ hpgl_process_buffer(hpgl_state_t * pgls, gs_point * offset)
     hpgl_real_t label_length = 0.0, label_height = 0.0;
     bool vertical = hpgl_text_is_vertical(pgls->g.character.text_path);
     int i, inc;
+    const pcl_font_selection_t *pfs =&pgls->g.font_selection[pgls->g.font_selected];
 
     /* a peculiar side effect of LABEL parsing double byte characters
        is it leaves an extra byte in the buffer - fix that now, and
@@ -1268,7 +1269,25 @@ hpgl_process_buffer(hpgl_state_t * pgls, gs_point * offset)
                         break;
                 }
             hpgl_call(hpgl_ensure_font(pgls));
-            hpgl_get_char_width(pgls, ch, &width);
+            /* Hackery... I believe that this *ought* to be handled by resizing every character in the
+             * proportional font as it is drawn, so that it is the size of mandated character (from the SI operator).
+             * This would mean that the calculation of the width would be correct (num characters * character width)
+             * which is what gets returned in label_length above.
+             * But that isn't what hpgl_print_char does. It does use the scale returned by hpgl_current_scale()
+             * to resize each character as it is drawn, which works perfectly well for a fixed pitch font, but
+             * results in each character being potentially a different size with a proportional font.
+             * Ideally I'd fix hpgl_print_char, but I can't see how to do that. It would mean moving a lot of
+             * logic from here into hpgl_print_char() I think.
+             * So here, pretend that we are not writing in absolute mode, that means the length returned
+             * in label_length will be the actual width of the (unscaled) characters.
+             * Then see below....
+             */
+            if (pgls->g.character.size_mode == hpgl_size_absolute && pfs->params.proportional_spacing) {
+                pgls->g.character.size_mode = hpgl_size_not_set;
+                hpgl_get_char_width(pgls, ch, &width);
+                pgls->g.character.size_mode = hpgl_size_absolute;
+            } else
+                hpgl_get_char_width(pgls, ch, &width);
           acc_ht:hpgl_call(hpgl_get_current_cell_height
                       (pgls, &height));
             if (vertical) {
@@ -1286,6 +1305,23 @@ hpgl_process_buffer(hpgl_state_t * pgls, gs_point * offset)
         }
         hpgl_select_font_pri_alt(pgls, save_index);
     }
+    /* The second part of the hackery. We've calculated the width of the string using 'ordinary'
+     * drawing. We now find out what scaling will be used when drawing the text in 'absolute' mode.
+     * Then we revert back to 'not set' mode, and retrieve that scaling. That gives us the scaling
+     * that ws used (see above) to retrieve label_length.
+     * Now we know the two scalings, we can adjust label_length so that it will be the actual width
+     * of the string we draw. We can then use that to adjust the starting position so that it is
+     * correct.
+     */
+    if (pgls->g.character.size_mode == hpgl_size_absolute && pfs->params.proportional_spacing) {
+        gs_point unset_scale, absolute_scale;
+
+        pgls->g.character.size_mode = hpgl_size_not_set;
+        absolute_scale = hpgl_current_char_scale(pgls);
+        pgls->g.character.size_mode = hpgl_size_absolute;
+        unset_scale = hpgl_current_char_scale(pgls);
+        label_length = label_length * (unset_scale.x / absolute_scale.x);
+    }
     hpgl_call(hpgl_get_character_origin_offset(pgls, pgls->g.label.origin,
                                                label_length, label_height,
                                                offset));
@@ -1499,27 +1535,29 @@ hpgl_LB(hpgl_args_t * pargs, hpgl_state_t * pgls)
                  * restore the initial position, per TRM 23-78.
                  */
                 if (!hpgl_can_concat_labels(pgls)) {
-                    hpgl_call(hpgl_add_point_to_path(pgls,
+                    code = hpgl_add_point_to_path(pgls,
                                                      pgls->g.
                                                      carriage_return_pos.x,
                                                      pgls->g.
                                                      carriage_return_pos.y,
                                                      hpgl_plot_move_absolute,
-                                                     true));
+                                                     true);
                 } else {
                     /* undo the label origin offsets */
-                    hpgl_call(hpgl_add_point_to_path
+                    code = hpgl_add_point_to_path
                               (pgls, lo_offsets.x, lo_offsets.y,
-                               hpgl_plot_move_relative, false));
+                               hpgl_plot_move_relative, false);
                 }
-                /* clear the current path terminating carriage
-                   returns and linefeeds will leave "moveto's" in
-                   the path */
-                hpgl_call(hpgl_clear_current_path(pgls));
                 /* also clean up stick fonts - they are likely to
                    become dangling references in the current font
                    scheme since they don't have a dictionary entry */
                 hpgl_free_stick_fonts(pgls);
+                if (code < 0)
+                    return code;
+                /* clear the current path terminating carriage
+                   returns and linefeeds will leave "moveto's" in
+                   the path */
+                hpgl_call(hpgl_clear_current_path(pgls));
                 return 0;
             }
             /*
@@ -1539,9 +1577,14 @@ hpgl_LB(hpgl_args_t * pargs, hpgl_state_t * pgls)
         if (GL_LB_CH == CR && !pgls->g.transparent_data) {
             gs_point lo_offsets;
 
-            hpgl_call(hpgl_process_buffer(pgls, &lo_offsets));
+            code = hpgl_process_buffer(pgls, &lo_offsets);
             hpgl_destroy_label_buffer(pgls);
-            hpgl_call(hpgl_init_label_buffer(pgls));
+            if (code >= 0)
+                code = hpgl_init_label_buffer(pgls);
+            if (code < 0) {
+                hpgl_free_stick_fonts(pgls);
+                return code;
+            }
         }
     }
     pargs->source.ptr = p;
@@ -1568,6 +1611,8 @@ hpgl_free_stick_fonts(hpgl_state_t * pgls)
     pl_font_t *font = &pgls->g.stick_font[pgls->g.font_selected]
         [pfs->params.proportional_spacing];
 
+    pl_font_glyph_width_cache_remove_nodes(font);
+
     /* no stick fonts - nothing to do */
     if (font->pfont == 0)
         return;
@@ -1585,23 +1630,30 @@ hpgl_print_symbol_mode_char(hpgl_state_t * pgls)
     int saved_origin = pgls->g.label.origin;
     gs_point save_pos = pgls->g.pos;
     gs_point lo_offsets;
+    int code;
 
     hpgl_call(hpgl_gsave(pgls));
     /* HAS this need checking.  I don't know how text direction
        and label origin interact in symbol mode */
     pgls->g.label.origin = 5;
     /* HAS - alot of work for one character */
-    hpgl_call(hpgl_clear_current_path(pgls));
-    hpgl_call(hpgl_init_label_buffer(pgls));
-    hpgl_call(hpgl_buffer_char(pgls, pgls->g.symbol_mode));
-    hpgl_call(hpgl_process_buffer(pgls, &lo_offsets));
-    hpgl_destroy_label_buffer(pgls);
-    hpgl_call(hpgl_grestore(pgls));
+    if (((code = hpgl_clear_current_path(pgls)) >= 0) &&
+        ((code = hpgl_init_label_buffer(pgls)) >= 0)) {
+        if ((code = hpgl_buffer_char(pgls, pgls->g.symbol_mode)) >= 0)
+            code = hpgl_process_buffer(pgls, &lo_offsets);
+        hpgl_destroy_label_buffer(pgls);
+    }
+    {
+        /* must grestore here from earlier gsave */
+        int code2 = hpgl_grestore(pgls);
+        if (code >= 0) code = code2;
+    }
+    hpgl_free_stick_fonts(pgls);
     /* restore the origin */
     pgls->g.label.origin = saved_origin;
-    hpgl_call(hpgl_set_current_position(pgls, &save_pos));
-    hpgl_free_stick_fonts(pgls);
-    return 0;
+    if (code >= 0)
+        hpgl_call(hpgl_set_current_position(pgls, &save_pos));
+    return code;
 }
 
 /* Initialization */
