@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -641,9 +641,12 @@ process_zero_mask_rows(pcl_raster_t * prast, int nrows)
         code = pcl_set_drawing_color(pcs, pcl_pattern_solid_white, 0, true);
         if (code < 0)
             return code;
-        code = gs_setrasterop(pcs->pgs, (gs_rop3_t) rop3_know_S_1((int)0xff));
-        if (code < 0)
-            return code;
+        code = check_rasterops(pcs, (gs_rop3_t) rop3_know_S_1((int)0xff));
+        if (code >= 0) {
+            code = gs_setrasterop(pcs->pgs, (gs_rop3_t) rop3_know_S_1((int)0xff));
+            if (code < 0)
+                return code;
+        }
         while ((nrows-- > 0) && (code >= 0))
             code = gs_image_next(pen, prast->mask_buff, nbytes, &dummy);
 
@@ -731,23 +734,32 @@ process_zero_rows(pcl_raster_t * prast, int nrows)
             tmp_rect.q.x = (double)npixels;
             tmp_rect.q.y = (double)nrows;
             if (invert) {
-                code = gs_setrasterop(pgs,
-                               (gs_rop3_t)
-                               rop3_invert_S(gs_currentrasterop(pgs))
-                    );
-                if (code < 0)
-                    return code;
+                code = check_rasterops(prast->pcs, (gs_rop3_t)
+                               rop3_invert_S(gs_currentrasterop(pgs)));
+                if (code >= 0) {
+                    code = gs_setrasterop(pgs,
+                                   (gs_rop3_t)
+                                   rop3_invert_S(gs_currentrasterop(pgs))
+                        );
+                    if (code < 0)
+                        return code;
+                }
 
                 code = gs_rectfill(pgs, &tmp_rect, 1);
                 if (code < 0)
                     return code;
 
-                code = gs_setrasterop(pgs,
-                               (gs_rop3_t)
-                               rop3_invert_S(gs_currentrasterop(pgs))
-                    );
-                if (code < 0)
-                    return code;
+                code = check_rasterops(prast->pcs, (gs_rop3_t)
+                               rop3_invert_S(gs_currentrasterop(pgs)));
+                if (code >= 0) {
+                    code = gs_setrasterop(pgs,
+                                   (gs_rop3_t)
+                                   rop3_invert_S(gs_currentrasterop(pgs))
+                        );
+                    if (code < 0)
+                        return code;
+                } else
+                    code = 0;
             } else {
                 code = gs_rectfill(pgs, &tmp_rect, 1);
                 if (code < 0)
@@ -1187,6 +1199,9 @@ pcl_start_raster(uint src_width, uint src_height, pcl_state_t * pcs)
     pcl_cs_indexed_t *pindexed = ppalet->pindexed;
     pcl_encoding_type_t penc = pcl_cs_indexed_get_encoding(pindexed);
     pcl_seed_row_t *pseed_rows = 0;
+    int pattern_could_be_transparent =
+            pcs->pattern_transparent &&
+            pcs->pattern_type != pcl_pattern_solid_frgrnd;
 
     /* there can only be one raster object present at a time */
     if (prast != 0)
@@ -1199,7 +1214,7 @@ pcl_start_raster(uint src_width, uint src_height, pcl_state_t * pcs)
 
     prast->pmem = pcs->memory;
 
-    if (pcs->source_transparent || pcs->pattern_transparent)
+    if (pcs->source_transparent || pattern_could_be_transparent)
         prast->transparent = true;
     else
         prast->transparent = false;
@@ -1306,7 +1321,7 @@ pcl_start_raster(uint src_width, uint src_height, pcl_state_t * pcs)
 
     /* see if a mask is required */
     if (!pcs->source_transparent &&
-        pcs->pattern_transparent &&
+        pattern_could_be_transparent &&
         (!prast->indexed ||
          (prast->wht_indx < (1 << prast->nplanes * prast->bits_per_plane)))) {
 
