@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -121,9 +121,9 @@ long int pjl_proc_get_named_resource_size(pl_interp_implementation_t * pli,
                                           char *name);
 
 typedef int (*pjl_proc_get_named_resource_t) (pl_interp_implementation_t * pli,
-                                              char *name, byte * data);
+                                              char *name, byte * data, long int datasize);
 int pjl_proc_get_named_resource(pl_interp_implementation_t * pli, char *name,
-                                byte * data);
+                                byte * data, long int datasize);
 
 typedef int (*pjl_proc_process_t) (pl_interp_implementation_t * pli,
                                    stream_cursor_read * pr);
