@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -210,6 +210,12 @@ struct pl_font_s
     float pts_per_inch;         /* either 72 or 72.307 (for Intellifont) */
     pl_glyph_width_node_t *widths_cache;
     int widths_cache_nitems;
+    char **names;
+    int max_name_index;
+    int next_name_index;
+#ifdef DEBUG
+    char FontName[16];
+#endif
 };
 
 #define private_st_pl_font()	/* in plfont.c */\
