@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -105,11 +105,12 @@ bool pl_main_get_nocache(const gs_memory_t *mem);
 bool pl_main_get_page_set_on_command_line(const gs_memory_t *mem);
 bool pl_main_get_res_set_on_command_line(const gs_memory_t *mem);
 bool pl_main_get_high_level_device(const gs_memory_t *mem);
+bool pl_main_get_rasterop_support(const gs_memory_t *mem);
 void pl_main_get_forced_geometry(const gs_memory_t *mem, const float **resolutions, const long **dimensions);
 int pl_main_get_scanconverter(const gs_memory_t *mem);
 pl_main_instance_t *pl_main_get_instance(const gs_memory_t *mem);
 
-typedef int pl_main_get_codepoint_t(gp_file *, const char **);
+typedef int pl_main_get_codepoint_t(stream *, const char **);
 void pl_main_set_arg_decode(pl_main_instance_t *minst,
                             pl_main_get_codepoint_t *get_codepoint);
 
@@ -121,4 +122,8 @@ char *pl_main_get_pcl_personality(const gs_memory_t *mem);
 
 pl_interp_implementation_t *pl_main_get_pcl_instance(const gs_memory_t *mem);
 pl_interp_implementation_t *pl_main_get_pjl_instance(const gs_memory_t *mem);
+pl_interp_implementation_t *pl_main_get_pdf_instance(const gs_memory_t *mem);
+pl_interp_implementation_t *pl_main_get_xps_instance(const gs_memory_t *mem);
+pl_interp_implementation_t *pl_main_get_language_instance(const gs_memory_t *mem, const char *name);
+
 #endif /* plmain_INCLUDED */
