@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -646,14 +646,21 @@ read_deltarow_bitmap_data(px_bitmap_enum_t * benum, byte ** pdata,
                     uint val = *pin++;
 
                     --avail;
-                    deltarow->row_byte_count--;
-                    deltarow->short_cnt = (val >> 5) + 1;       /* 1 to 8 new bytes to copy */
-                    deltarow->short_offset = val & 0x1f;        /* num to retain from last row, skip */
-                    if (deltarow->short_offset == 0x1f)
-                        deltarow->state = partial_offset;       /* accumulate more offset */
-                    else {
-                        pout += deltarow->short_offset; /* skip keeps old data in row */
-                        deltarow->state = partial_cnt;  /* done with offset do count */
+
+                    if (deltarow->row_byte_count == 0) {
+                        /* duplicate the row */
+                        deltarow->state = next_is_bytecount;
+                        end_of_row = true;
+                    } else {
+                        deltarow->row_byte_count--;
+                        deltarow->short_cnt = (val >> 5) + 1;       /* 1 to 8 new bytes to copy */
+                        deltarow->short_offset = val & 0x1f;        /* num to retain from last row, skip */
+                        if (deltarow->short_offset == 0x1f)
+                            deltarow->state = partial_offset;       /* accumulate more offset */
+                        else {
+                            pout += deltarow->short_offset; /* skip keeps old data in row */
+                            deltarow->state = partial_cnt;  /* done with offset do count */
+                        }
                     }
                     break;
                 }
@@ -662,15 +669,21 @@ read_deltarow_bitmap_data(px_bitmap_enum_t * benum, byte ** pdata,
                     uint offset = *pin++;
 
                     avail--;
-                    deltarow->row_byte_count--;
+                    if (deltarow->row_byte_count == 0) {
+                        /* duplicate the row */
+                        deltarow->state = next_is_bytecount;
+                        end_of_row = true;
+                    } else {
+                        deltarow->row_byte_count--;
 
-                    deltarow->short_offset += offset;
+                        deltarow->short_offset += offset;
 
-                    if (offset == 0xff)
-                        deltarow->state = partial_offset;       /* 0x1f + ff ff ff ff ff + 1 */
-                    else {
-                        pout += deltarow->short_offset; /* skip keeps old data in row */
-                        deltarow->state = partial_cnt;  /* done with offset do count */
+                        if (offset == 0xff)
+                            deltarow->state = partial_offset;       /* 0x1f + ff ff ff ff ff + 1 */
+                        else {
+                            pout += deltarow->short_offset; /* skip keeps old data in row */
+                            deltarow->state = partial_cnt;  /* done with offset do count */
+                        }
                     }
                     break;
                 }
@@ -678,18 +691,24 @@ read_deltarow_bitmap_data(px_bitmap_enum_t * benum, byte ** pdata,
             case partial_cnt:{
                     /* check for possible row overflow */
                     if (pout >= *pdata + benum->data_per_row)
-                        return -1;
-                    *pout++ = *pin++;   /* copy new data into row */
+                        pin++;
+                    else
+                        *pout++ = *pin++;   /* copy new data into row */
                     avail--;
-                    deltarow->row_byte_count--;
-                    deltarow->short_cnt--;
-
                     if (deltarow->row_byte_count == 0) {
                         end_of_row = true;
                         deltarow->state = next_is_bytecount;
-                    } else if (deltarow->short_cnt == 0)
-                        deltarow->state = next_is_cmd;
-                    /* else more bytes to copy */
+                    } else {
+                        deltarow->row_byte_count--;
+                        deltarow->short_cnt--;
+
+                        if (deltarow->row_byte_count == 0) {
+                            end_of_row = true;
+                            deltarow->state = next_is_bytecount;
+                        } else if (deltarow->short_cnt == 0)
+                            deltarow->state = next_is_cmd;
+                        /* else more bytes to copy */
+                    }
                     break;
                 }
 
