@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -288,7 +288,8 @@ trace_array_data(const gs_memory_t * mem, const char *label,
 
 /* Process a buffer of PCL XL commands. */
 int
-px_process(px_parser_state_t * st, px_state_t * pxs, stream_cursor_read * pr)
+px_process(px_parser_state_t * st, /* lgtm [cpp/use-of-goto] */
+           px_state_t * pxs, stream_cursor_read * pr)
 {
     const byte *orig_p = pr->ptr;
     const byte *next_p = orig_p;        /* start of data not copied to saved */
