@@ -1,4 +1,4 @@
-/* Copyright (C) 2018-2022 Artifex Software, Inc.
+/* Copyright (C) 2018-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Top level PDF access routines */
@@ -39,6 +39,7 @@
 #include "pdf_repair.h"
 #include "pdf_xref.h"
 #include "pdf_device.h"
+#include "pdf_mark.h"
 
 #include "gsstate.h"        /* For gs_gstate */
 #include "gsicc_manage.h"  /* For gsicc_init_iccmanager() */
@@ -47,6 +48,10 @@
 #include "gsmchunk.h"
 #endif
 
+#ifndef USE_PDF_PERMISSIONS
+#define USE_PDF_PERMISSIONS 0
+#endif
+
 extern const char gp_file_name_list_separator;
 /*
  * Convenience routine to check if a given string exists in a dictionary
@@ -507,7 +512,7 @@ const char *gs_internal_error_strings[] = {
 #define FIRSTINTERNALERROR gs_error_hit_detected * -1
 #define LASTGSERROR gs_error_handled * -1
 
-void pdfi_verbose_error(pdf_context *ctx, int gs_error, const char *gs_lib_function, int pdfi_error, const char *pdfi_function_name, const char *extra_info)
+void pdfi_verbose_error(pdf_context *ctx, int gs_error, const char *gs_lib_function, int pdfi_error, const char *pdfi_function_name, const char *extra_info, const char *file_line)
 {
     char fallback[] = "unknown graphics library error";
 
@@ -529,9 +534,16 @@ void pdfi_verbose_error(pdf_context *ctx, int gs_error, const char *gs_lib_funct
             }
             errprintf(ctx->memory, "Graphics library error %d (%s) in function '%s'", gs_error, error_string, pdfi_function_name);
             if (gs_lib_function != NULL)
-                errprintf(ctx->memory, " from lib routine '%s'.\n", gs_lib_function);
+                errprintf(ctx->memory, " from lib routine '%s'", gs_lib_function);
+
+#if DEBUG_FILE_LINE==1
+            if (file_line != NULL)
+                errprintf(ctx->memory, "%s'.\n", file_line);
             else
                 errprintf(ctx->memory, ".\n");
+#else
+            errprintf(ctx->memory, ".\n");
+#endif
 
             if (pdfi_error != 0)
                 errprintf(ctx->memory, "\tSetting pdfi error %d - %s.\n", pdfi_error, pdf_error_strings[pdfi_error]);
@@ -539,7 +551,14 @@ void pdfi_verbose_error(pdf_context *ctx, int gs_error, const char *gs_lib_funct
                 errprintf(ctx->memory, "\t%s\n", extra_info);
         } else {
             if (pdfi_error != 0) {
+#if DEBUG_FILE_LINE==1
+                if (file_line != NULL)
+                    errprintf(ctx->memory, "Function '%s' %s set pdfi error %d - %s.\n", pdfi_function_name, file_line, pdfi_error, pdf_error_strings[pdfi_error]);
+                else
+                    errprintf(ctx->memory, "Function '%s' set pdfi error %d - %s.\n", pdfi_function_name, pdfi_error, pdf_error_strings[pdfi_error]);
+#else
                 errprintf(ctx->memory, "Function '%s' set pdfi error %d - %s.\n", pdfi_function_name, pdfi_error, pdf_error_strings[pdfi_error]);
+#endif
                 if (extra_info != NULL)
                     errprintf(ctx->memory, "\t%s\n", extra_info);
             } else {
@@ -550,7 +569,7 @@ void pdfi_verbose_error(pdf_context *ctx, int gs_error, const char *gs_lib_funct
     }
 }
 
-void pdfi_verbose_warning(pdf_context *ctx, int gs_error, const char *gs_lib_function, int pdfi_warning, const char *pdfi_function_name, const char *extra_info)
+void pdfi_verbose_warning(pdf_context *ctx, int gs_error, const char *gs_lib_function, int pdfi_warning, const char *pdfi_function_name, const char *extra_info, const char *file_line)
 {
     char fallback[] = "unknown graphics library error";
 
@@ -573,16 +592,29 @@ void pdfi_verbose_warning(pdf_context *ctx, int gs_error, const char *gs_lib_fun
             outprintf(ctx->memory, "Graphics library error %d (%s) in function '%s'", gs_error, error_string, pdfi_function_name);
             if (gs_lib_function != NULL)
                 outprintf(ctx->memory, " from lib routine '%s'.\n", gs_lib_function);
-            else
-                outprintf(ctx->memory, ".\n");
 
+#if DEBUG_FILE_LINE==1
+            if (file_line != NULL)
+                errprintf(ctx->memory, "%s'.\n", file_line);
+            else
+                errprintf(ctx->memory, ".\n");
+#else
+            errprintf(ctx->memory, ".\n");
+#endif
             if (pdfi_warning != 0)
                 outprintf(ctx->memory, "\tsetting pdfi warning %d - %s.\n", pdfi_warning, pdf_warning_strings[pdfi_warning]);
             if (extra_info != NULL)
                 outprintf(ctx->memory, "\t%s\n", extra_info);
         } else {
             if (pdfi_warning != 0) {
+#if DEBUG_FILE_LINE==1
+                if (file_line != NULL)
+                    outprintf(ctx->memory, "Function '%s' %s set pdfi warning %d - %s.\n", pdfi_function_name, file_line, pdfi_warning, pdf_warning_strings[pdfi_warning]);
+                else
+                    outprintf(ctx->memory, "Function '%s' set pdfi warning %d - %s.\n", pdfi_function_name, pdfi_warning, pdf_warning_strings[pdfi_warning]);
+#else
                 outprintf(ctx->memory, "Function '%s' set pdfi warning %d - %s.\n", pdfi_function_name, pdfi_warning, pdf_warning_strings[pdfi_warning]);
+#endif
                 if (extra_info != NULL)
                     errprintf(ctx->memory, "\t%s\n", extra_info);
             } else {
@@ -593,7 +625,7 @@ void pdfi_verbose_warning(pdf_context *ctx, int gs_error, const char *gs_lib_fun
     }
 }
 
-void pdfi_set_error_var(pdf_context *ctx, int gs_error, const char *gs_lib_function, pdf_error pdfi_error, const char *pdfi_function_name, const char *fmt, ...)
+int pdfi_set_error_var(pdf_context *ctx, int gs_error, const char *gs_lib_function, pdf_error pdfi_error, const char *pdfi_function_name, const char *fmt, ...)
 {
     if (pdfi_error != 0)
         ctx->pdf_errors[pdfi_error / (sizeof(char) * 8)] |= 1 << pdfi_error % (sizeof(char) * 8);
@@ -605,11 +637,18 @@ void pdfi_set_error_var(pdf_context *ctx, int gs_error, const char *gs_lib_funct
         (void)vsnprintf(extra_info, sizeof(extra_info), fmt, args);
         va_end(args);
 
-        pdfi_verbose_error(ctx, gs_error, gs_lib_function, pdfi_error, pdfi_function_name, extra_info);
+        pdfi_verbose_error(ctx, gs_error, gs_lib_function, pdfi_error, pdfi_function_name, extra_info, NULL);
     }
+    if (ctx->args.pdfstoponerror) {
+        if (gs_error < 0)
+            return gs_error;
+        else
+            return gs_error_unknownerror;
+    }
+    return 0;
 }
 
-void pdfi_set_warning_var(pdf_context *ctx, int gs_error, const char *gs_lib_function, pdf_warning pdfi_warning, const char *pdfi_function_name, const char *fmt, ...)
+int pdfi_set_warning_var(pdf_context *ctx, int gs_error, const char *gs_lib_function, pdf_warning pdfi_warning, const char *pdfi_function_name, const char *fmt, ...)
 {
     ctx->pdf_warnings[pdfi_warning / (sizeof(char) * 8)] |= 1 << pdfi_warning % (sizeof(char) * 8);
     if (ctx->args.verbose_warnings) {
@@ -620,16 +659,15 @@ void pdfi_set_warning_var(pdf_context *ctx, int gs_error, const char *gs_lib_fun
         (void)vsnprintf(extra_info, sizeof(extra_info), fmt, args);
         va_end(args);
 
-        pdfi_verbose_warning(ctx, gs_error, gs_lib_function, pdfi_warning, pdfi_function_name, extra_info);
+        pdfi_verbose_warning(ctx, gs_error, gs_lib_function, pdfi_warning, pdfi_function_name, extra_info, NULL);
     }
-}
-
-void pdfi_log_info(pdf_context *ctx, const char *pdfi_function, const char *info)
-{
-#ifdef DEBUG
-    if (!ctx->args.QUIET)
-        outprintf(ctx->memory, "%s", info);
-#endif
+    if (ctx->args.pdfstoponwarning) {
+        if (gs_error < 0)
+            return gs_error;
+        else
+            return gs_error_unknownerror;
+    }
+    return 0;
 }
 
 void
@@ -695,10 +733,14 @@ pdfi_report_errors(pdf_context *ctx)
             char *cs;
 
             cs = (char *)gs_alloc_bytes(ctx->memory, s->length + 1, "temporary string for error report");
-            memcpy(cs, s->data, s->length);
-            cs[s->length] = 0x00;
-            dmprintf1(ctx->memory, "   **** The file was produced by: \n   **** >>>> %s <<<<\n", cs);
-            gs_free_object(ctx->memory, cs, "temporary string for error report");
+            if (cs == NULL) {
+                dmprintf(ctx->memory, "   **** Out of memory while trying to display Producer ****\n");
+            } else {
+                memcpy(cs, s->data, s->length);
+                cs[s->length] = 0x00;
+                dmprintf1(ctx->memory, "   **** The file was produced by: \n   **** >>>> %s <<<<\n", cs);
+                gs_free_object(ctx->memory, cs, "temporary string for error report");
+            }
         }
         pdfi_countdown(s);
     }
@@ -819,12 +861,46 @@ int pdfi_separation_name_from_index(gs_gstate *pgs, gs_separation_name index, un
     return_error(gs_error_undefined);
 }
 
-/* These functions are used by the 'PL' implementation, eventually we will */
-/* need to have custom PostScript operators to process the file or at      */
-/* (least pages from it).                                                  */
+int pdfi_finish_pdf_file(pdf_context *ctx)
+{
+    if (ctx->Root) {
+        if (ctx->device_state.writepdfmarks && ctx->args.preservemarkedcontent && ctx->device_state.WantsOptionalContent) {
+            pdf_obj *o = NULL;
+            int code = 0;
+
+            code = pdfi_dict_knownget_type(ctx, ctx->Root, "OCProperties", PDF_DICT, &o);
+            if (code > 0) {
+                // Build and send the OCProperties structure
+                code = pdfi_pdfmark_from_objarray(ctx, &o, 1, NULL, "OCProperties");
+                pdfi_countdown(o);
+                if (code < 0)
+                    /* Error message ? */
+                ;
+            }
+        }
+    }
+    return 0;
+}
 
 int pdfi_close_pdf_file(pdf_context *ctx)
 {
+    if (ctx->Root) {
+        if (ctx->device_state.writepdfmarks && ctx->args.preservemarkedcontent && ctx->device_state.WantsOptionalContent) {
+            pdf_obj *o = NULL;
+            int code = 0;
+
+            code = pdfi_dict_knownget(ctx, ctx->Root, "OCProperties", &o);
+            if (code > 0) {
+                /* Build and send the OCProperties structure */
+                code = pdfi_pdfmark_from_objarray(ctx, &o, 1, NULL, "OCProperties");
+                pdfi_countdown(o);
+                if (code < 0)
+                    /* Error message ? */
+                    ;
+            }
+        }
+    }
+
     if (ctx->main_stream) {
         if (ctx->main_stream->s) {
             sfclose(ctx->main_stream->s);
@@ -857,18 +933,16 @@ static int pdfi_process(pdf_context *ctx)
         }
         if (ctx->args.last_page != 0) {
             if (i > ctx->args.last_page - 1)
-                break;;
+                break;
         }
         if (ctx->args.pdfinfo)
             code = pdfi_output_page_info(ctx, i);
         else
             code = pdfi_page_render(ctx, i, true);
 
-        if (code < 0 && ctx->args.pdfstoponerror)
-            goto exit;
-        code = 0;
+        if (code < 0)
+            code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_GS_LIB_ERROR, "pdfi_process", NULL);
     }
- exit:
     pdfi_report_errors(ctx);
 
     return code;
@@ -898,9 +972,9 @@ int pdfi_prep_collection(pdf_context *ctx, uint64_t *TotalFiles, char ***names_a
     pdf_obj *EF = NULL, *F = NULL;
     char **working_array = NULL;
 
-    if (pdfi_dict_knownget_type(ctx, ctx->Root, "Names", PDF_DICT, &Names)) {
-        if(pdfi_dict_knownget_type(ctx, (pdf_dict *)Names, "EmbeddedFiles", PDF_DICT, &EmbeddedFiles)) {
-            if (pdfi_dict_knownget_type(ctx, (pdf_dict *)EmbeddedFiles, "Names", PDF_ARRAY, (pdf_obj **)&FileNames)) {
+    if (pdfi_dict_knownget_type(ctx, ctx->Root, "Names", PDF_DICT, &Names) > 0) {
+        if(pdfi_dict_knownget_type(ctx, (pdf_dict *)Names, "EmbeddedFiles", PDF_DICT, &EmbeddedFiles) > 0) {
+            if (pdfi_dict_knownget_type(ctx, (pdf_dict *)EmbeddedFiles, "Names", PDF_ARRAY, (pdf_obj **)&FileNames) > 0) {
                 int ix = 0, index = 0;
                 gp_file *scratch_file = NULL;
                 char scratch_name[gp_file_name_sizeof];
@@ -923,8 +997,8 @@ int pdfi_prep_collection(pdf_context *ctx, uint64_t *TotalFiles, char ***names_a
                         goto exit;
 
                     if (pdfi_type_of(File) == PDF_DICT) {
-                        if (pdfi_dict_knownget_type(ctx, (pdf_dict *)File, "EF", PDF_DICT, &EF)) {
-                            if (pdfi_dict_knownget_type(ctx, (pdf_dict *)EF, "F", PDF_STREAM, &F)) {
+                        if (pdfi_dict_knownget_type(ctx, (pdf_dict *)File, "EF", PDF_DICT, &EF) > 0) {
+                            if (pdfi_dict_knownget_type(ctx, (pdf_dict *)EF, "F", PDF_STREAM, &F) > 0) {
                                 pdf_dict *stream_dict = NULL;
                                 pdf_c_stream *s = NULL;
 
@@ -933,7 +1007,7 @@ int pdfi_prep_collection(pdf_context *ctx, uint64_t *TotalFiles, char ***names_a
                                  */
                                 code = pdfi_dict_from_obj(ctx, F, &stream_dict);
                                 if (code >= 0) {
-                                    if (!pdfi_dict_knownget_type(ctx, stream_dict, "Subtype", PDF_NAME, &Subtype)) {
+                                    if (pdfi_dict_knownget_type(ctx, stream_dict, "Subtype", PDF_NAME, &Subtype) <= 0) {
                                         /* No Subtype, (or not a name) we can't check the Mime type, so try to read the first 2Kb
                                          * and look for a %PDF- in that. If not present, assume its not a PDF
                                          */
@@ -1124,11 +1198,6 @@ int pdfi_process_pdf_file(pdf_context *ctx, char *filename)
         return code;
     }
 
-    /* Need to do this here so that ctx->writepdfmarks will be setup
-     * It is also called in pdfi_page_render()
-     * TODO: Should probably look into that..
-     */
-    pdfi_device_set_flags(ctx);
     /* Do any custom device configuration */
     pdfi_device_misc_config(ctx);
 
@@ -1156,10 +1225,12 @@ static int pdfi_init_file(pdf_context *ctx)
     code = pdfi_read_xref(ctx);
     if (code < 0) {
         if (ctx->is_hybrid) {
+            if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BADXREFSTREAM, "pdfi_init_file", NULL)) < 0) {
+                goto exit;
+            }
             /* If its a hybrid file, and we failed to read the XrefStm, try
              * again, but this time read the xref table instead.
              */
-            pdfi_set_error(ctx, 0, NULL, E_PDF_BADXREFSTREAM, "pdfi_init_file", NULL);
             pdfi_countdown(ctx->xref_table);
             ctx->xref_table = NULL;
             ctx->prefer_xrefstm = false;
@@ -1172,6 +1243,8 @@ static int pdfi_init_file(pdf_context *ctx)
         }
     }
 
+    pdfi_device_set_flags(ctx);
+
     if (ctx->Trailer) {
         /* See comment in pdfi_read_Root() (pdf_doc.c) for details */
         pdf_dict *d = ctx->Trailer;
@@ -1183,12 +1256,47 @@ static int pdfi_init_file(pdf_context *ctx)
             goto exit;
         if (code == 0) {
             if (pdfi_type_of(o) == PDF_DICT) {
+#if USE_PDF_PERMISSIONS
+	            double Permissions = 0;
+	            uint32_t P = 0;
+#endif
+
                 code = pdfi_initialise_Decryption(ctx);
                 if (code < 0)
                     goto exit;
+
+                /* This section is commetned out but deliberately retained. This code
+                 * prevents us from processing any PDF file where the 'print' permission
+                 * is not set. Additionally, if the 'print' permission is set, but bit 12
+                 * 'faitful digital copy' is not set, *and* we are writing to a high level
+                 * device, then we will throw an error..
+                 */
+#if USE_PDF_PERMISSIONS
+                code = pdfi_dict_knownget_number(ctx, (pdf_dict *)o, "P", &Permissions);
+                if (code < 0)
+                    goto exit;
+                if (Permissions > ((unsigned long)1 << 31)) {
+                    code = gs_note_error(gs_error_rangecheck);
+                    goto exit;
+                }
+                P = (uint32_t)Permissions;
+                if ((P & 0x04) == 0) {
+                    dmprintf(ctx->memory, "   ****The owner of this file has requested you do not print it.\n");
+                    code = gs_note_error(gs_error_invalidfileaccess);
+                    goto exit;
+                }
+                if (ctx->device_state.HighLevelDevice) {
+                    if ((P & 0x800) == 0) {
+                        dmprintf(ctx->memory, "   ****The owner of this file has requested you do not make a digital copy of it.\n");
+                        code = gs_note_error(gs_error_invalidfileaccess);
+                        goto exit;
+                    }
+                }
+#endif
             } else {
                 if (pdfi_type_of(o) != PDF_NULL)
-                    pdfi_set_error(ctx, code, NULL, E_PDF_BADENCRYPT, "pdfi_init_file", NULL);
+                    if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_typecheck), NULL, E_PDF_BADENCRYPT, "pdfi_init_file", NULL)) < 0)
+                        goto exit;
             }
         }
     }
@@ -1201,9 +1309,11 @@ read_root:
              * from a hybrid file, then try again, but this time use the xref table
              */
             if (code == gs_error_undefined && ctx->is_hybrid && ctx->prefer_xrefstm) {
-                pdfi_set_error(ctx, 0, NULL, E_PDF_BADXREFSTREAM, "pdfi_init_file", NULL);
                 pdfi_countdown(ctx->xref_table);
                 ctx->xref_table = NULL;
+                if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BADXREFSTREAM, "pdfi_init_file", NULL)) < 0)
+                    goto exit;
+
                 ctx->prefer_xrefstm = false;
                 code = pdfi_read_xref(ctx);
                 if (code < 0) {
@@ -1225,7 +1335,7 @@ read_root:
     if (ctx->Trailer) {
         code = pdfi_read_Info(ctx);
         if (code < 0 && code != gs_error_undefined) {
-            if (ctx->args.pdfstoponerror)
+            if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_GS_LIB_ERROR, "pdfi_init_file", NULL)) < 0)
                 goto exit;
             pdfi_clearstack(ctx);
         }
@@ -1255,13 +1365,13 @@ read_root:
 
     if (ctx->args.pdfinfo) {
         code = pdfi_output_metadata(ctx);
-        if (code < 0 && ctx->args.pdfstoponerror)
+        if (code < 0 && (code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_GS_LIB_ERROR, "pdfi_init_file", NULL)) < 0)
             goto exit;
     }
 
 exit:
     if (code < 0)
-        pdfi_set_error(ctx, code, NULL, 0, "pdfi_init_file", NULL);
+        pdfi_set_error(ctx, code, NULL, E_PDF_GS_LIB_ERROR, "pdfi_init_file", NULL);
     pdfi_countdown(o);
     return code;
 }
@@ -1269,10 +1379,10 @@ exit:
 int pdfi_set_input_stream(pdf_context *ctx, stream *stm)
 {
     byte *Buffer = NULL;
-    char *s = NULL;
+    char *s = NULL, *test = NULL;
     float version = 0.0;
     gs_offset_t Offset = 0;
-    int64_t bytes = 0, leftover = 0;
+    int64_t bytes = 0, leftover = 0, bytes_left = 0;
     bool found = false;
     int code;
 
@@ -1320,7 +1430,25 @@ int pdfi_set_input_stream(pdf_context *ctx, stream *stm)
     Buffer[Offset] = 0x00;
 
     /* First check for existence of header */
-    s = strstr((char *)Buffer, "%PDF");
+    test = (char *)Buffer;
+    bytes_left = bytes;
+    s = strstr((char *)test, "%PDF-");
+    if (s == NULL) {
+        if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_GARBAGE_B4HDR, "pdfi_set_input_stream", "")) < 0) {
+            goto error;
+        }
+    }
+    /* Garbage before the header can be anything, including binary and NULL (0x00) characters
+     * which can defeat using strstr, so if we fail to find a header, try moving on by the length
+     * of the C string + 1 and try again.
+     */
+    while (s == NULL) {
+        bytes_left -= (strlen(test) + 1);
+        if (bytes_left < 5)
+            break;
+        test += strlen(test) + 1;
+        s = strstr((char *)test, "%PDF-");
+    };
     if (s == NULL) {
         char extra_info[gp_file_name_sizeof];
 
@@ -1329,15 +1457,31 @@ int pdfi_set_input_stream(pdf_context *ctx, stream *stm)
         else
             gs_snprintf(extra_info, sizeof(extra_info), "%% File does not appear to be a PDF stream (no %%PDF in first 2Kb of stream)\n");
 
-        pdfi_set_error(ctx, 0, NULL, E_PDF_NOHEADER, "pdfi_set_input_stream", extra_info);
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_NOHEADER, "pdfi_set_input_stream", extra_info) ) < 0)
+            goto error;
     } else {
         /* Now extract header version (may be overridden later) */
         if (sscanf(s + 5, "%f", &version) != 1) {
             ctx->HeaderVersion = 0;
-            pdfi_set_error(ctx, 0, NULL, E_PDF_NOHEADERVERSION, "pdfi_set_input_stream", (char *)"%% Unable to read PDF version from header\n");
+            if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_NOHEADERVERSION, "pdfi_set_input_stream", (char *)"%% Unable to read PDF version from header\n")) < 0)
+                goto error;
         }
         else {
             ctx->HeaderVersion = version;
+            s += 5;
+            while (((*s >= '0' && *s <= '9') || *s == '.') && *s != 0x00)
+                s++;
+            while (*s != 0x00) {
+                if (*s == 0x09 || *s== 0x0c || *s == 0x20) {
+                    s++;
+                    continue;
+                }
+                if (*s != 0x0A && *s != 0x0D) {
+                    if ((code = pdfi_set_warning_stop(ctx, 0, NULL, W_PDF_VERSION_NO_EOL, "pdfi_set_input_stream", (char *)"%% PDF version not immediately followed with EOL\n")) < 0)
+                        goto error;
+                }
+                break;
+            }
         }
         if (ctx->args.pdfdebug)
             dmprintf1(ctx->memory, "%% Found header, PDF version is %f\n", ctx->HeaderVersion);
@@ -1347,7 +1491,7 @@ int pdfi_set_input_stream(pdf_context *ctx, stream *stm)
     pdfi_seek(ctx, ctx->main_stream, 0, SEEK_END);
 
     if (ctx->args.pdfdebug)
-        dmprintf(ctx->memory, "%% Searching for 'startxerf' keyword\n");
+        dmprintf(ctx->memory, "%% Searching for 'startxref' keyword\n");
 
     /* Initially read min(BUF_SIZE, file_length) bytes of data to the buffer */
     bytes = Offset;
@@ -1357,15 +1501,17 @@ int pdfi_set_input_stream(pdf_context *ctx, stream *stm)
         uint32_t read;
 
         if (pdfi_seek(ctx, ctx->main_stream, ctx->main_stream_length - Offset, SEEK_SET) != 0) {
-            emprintf1(ctx->memory, "File is smaller than %"PRIi64" bytes\n", (int64_t)Offset);
-            code = gs_error_ioerror;
+            char msg[128];
+            gs_snprintf(msg, 128, "%% File is smaller than %"PRIi64" bytes\n", (int64_t)Offset);
+            code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_ioerror), NULL, E_PDF_GS_LIB_ERROR, "pdfi_set_input_stream", msg);
             goto error;
         }
         read = pdfi_read_bytes(ctx, Buffer, 1, bytes, ctx->main_stream);
 
         if (read <= 0) {
-            emprintf1(ctx->memory, "Failed to read %"PRIi64" bytes from file\n", (int64_t)bytes);
-            code = gs_error_ioerror;
+            char msg[128];
+            gs_snprintf(msg, 128, "Failed to read %"PRIi64" bytes from file\n", (int64_t)bytes);
+            code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_ioerror), NULL, E_PDF_GS_LIB_ERROR, "pdfi_set_input_stream", msg);
             goto error;
         }
 
@@ -1403,7 +1549,8 @@ int pdfi_set_input_stream(pdf_context *ctx, stream *stm)
 
             /* Success! stop now */
             if(sscanf((char *)b, " %"PRIdOFFSET"", &ctx->startxref) != 1) {
-                dmprintf(ctx->memory, "Unable to read offset of xref from PDF file\n");
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_BADSTARTXREF, "pdfi_set_input_stream", "Unable to read offset of xref from PDF file\n")) < 0)
+                    goto error;
             }
             break;
         } else {
@@ -1435,8 +1582,10 @@ int pdfi_set_input_stream(pdf_context *ctx, stream *stm)
         Offset += bytes;
     } while(Offset < ctx->main_stream_length);
 
-    if (!found)
-        pdfi_set_error(ctx, 0, NULL, E_PDF_NOSTARTXREF, "pdfi_set_input_stream", NULL);
+    if (!found) {
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_undefined), NULL, E_PDF_NOSTARTXREF, "pdfi_set_input_stream", NULL)) < 0)
+            goto error;
+    }
 
     code = pdfi_init_file(ctx);
 
@@ -1524,7 +1673,7 @@ int pdfi_add_paths_to_search_paths(pdf_context *ctx, const char *ppath, int l, b
             for (i = 0; i < ctx->search_paths.num_resource_paths - ctx->search_paths.num_init_resource_paths; i++) {
                 pathstrings[i] = ctx->search_paths.resource_paths[i];
             }
-            /* NO NOT CHANGE "i" BETWEEN HERE....... */
+            /* DO NOT CHANGE "i" BETWEEN HERE....... */
             gs_free_object(ctx->memory, ctx->search_paths.resource_paths, "old array of paths");
             ctx->search_paths.resource_paths = pathstrings;
             ctx->search_paths.num_resource_paths += npaths;
@@ -1792,8 +1941,11 @@ pdf_context *pdfi_create_context(gs_memory_t *mem)
     /* Setup some flags that don't default to 'false' */
     ctx->args.showannots = true;
     ctx->args.preserveannots = true;
+    ctx->args.preservemarkedcontent = true;
+    ctx->args.preserveembeddedfiles = true;
+    ctx->args.preservedocview = true;
     /* NOTE: For testing certain annotations on cluster, might want to set this to false */
-    ctx->args.printed = true; /* TODO: Should be true if OutputFile is set, false otherwise */
+    ctx->args.printed = false; /* True if OutputFile is set, false otherwise see pdftop.c, pdf_impl_set_param() */
 
     /* Initially, prefer the XrefStm in a hybrid file */
     ctx->prefer_xrefstm = true;
@@ -2143,6 +2295,8 @@ int pdfi_free_context(pdf_context *ctx)
         pdfi_countdown(ctx->pdffontmap);
         ctx->pdffontmap = NULL;
     }
+    rc_decrement(ctx->devbbox, "pdfi_free_context");
+
     gs_free_object(ctx->memory, ctx, "pdfi_free_context");
 #if PDFI_LEAK_CHECK
     gs_memory_status(mem, &mstat);
