@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2022 Artifex Software, Inc.
+/* Copyright (C) 2019-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Checks for transparency and spots for the PDF interpreter */
@@ -72,6 +72,7 @@
  */
 typedef struct {
     bool transparent;
+    bool BM_Not_Normal;
     bool has_overprint; /* Does it have OP or op in an ExtGState? */
     pdf_dict *spot_dict;
     pdf_array *font_array;
@@ -286,7 +287,7 @@ static int pdfi_check_Shading_dict(pdf_context *ctx, pdf_dict *shading_dict,
 
         code = pdfi_dict_first(ctx, shading_dict, &Key, &Value, &index);
         if (code < 0 || !(pdfi_type_of(Value) == PDF_DICT || pdfi_type_of(Value) == PDF_STREAM))
-            goto error1;
+            goto error2;
 
         i = 1;
         do {
@@ -392,26 +393,41 @@ static int pdfi_check_XObject(pdf_context *ctx, pdf_dict *xobject, pdf_dict *pag
                 code = pdfi_dict_knownget_type(ctx, xobject, "Group", PDF_DICT, (pdf_obj **)&group_dict);
                 if (code > 0) {
                     tracker->transparent = true;
-                    if (tracker->spot_dict == NULL) {
+                    if (tracker->spot_dict != NULL) {
+                        /* Start a new loop detector group to avoid this being detected in the Resources check below */
+                        code = pdfi_loop_detector_mark(ctx); /* Mark the start of the XObject dictionary loop */
+                        if (code == 0) {
+                            code = pdfi_dict_knownget(ctx, group_dict, "CS", &CS);
+                            if (code > 0)
+                                /* We don't care if there's an error here, it'll be picked up if we use the ColorSpace later */
+                                (void)pdfi_check_ColorSpace_for_spots(ctx, CS, group_dict, page_dict, tracker->spot_dict);
+                            (void)pdfi_loop_detector_cleartomark(ctx); /* Clear to the mark for the XObject dictionary loop */
+                        }
+                        pdfi_countdown(group_dict);
+                        pdfi_countdown(CS);
+                    }
+                    else if (tracker->BM_Not_Normal)
+                    {
+                        /* We already know it has a non-normal Blend Mode. No point in keeping searching. */
                         pdfi_countdown(group_dict);
                         goto transparency_exit;
                     }
-
-                    /* Start a new loop detector group to avoid this being detected in the Resources check below */
-                    code = pdfi_loop_detector_mark(ctx); /* Mark the start of the XObject dictionary loop */
-                    if (code == 0) {
-                        code = pdfi_dict_knownget(ctx, group_dict, "CS", &CS);
-                        if (code > 0)
-                            /* We don't care if there's an error here, it'll be picked up if we use the ColorSpace later */
-                            (void)pdfi_check_ColorSpace_for_spots(ctx, CS, group_dict, page_dict, tracker->spot_dict);
-                        (void)pdfi_loop_detector_cleartomark(ctx); /* Clear to the mark for the XObject dictionary loop */
+                    else
+                    {
+                        pdfi_countdown(group_dict);
                     }
-                    pdfi_countdown(group_dict);
-                    pdfi_countdown(CS);
+                    /* We need to keep checking Resources in case there are non-Normal blend mode things still to be found. */
                 }
 
                 code = pdfi_dict_knownget_type(ctx, xobject, "Resources", PDF_DICT, (pdf_obj **)&resource_dict);
                 if (code > 0) {
+                    if (ctx->loop_detection && pdf_object_num((pdf_obj *)resource_dict) != 0) {
+                        code = pdfi_loop_detector_add_object(ctx, resource_dict->object_num);
+                        if (code < 0) {
+                            pdfi_countdown(resource_dict);
+                            goto transparency_exit;
+                        }
+                    }
                     code = pdfi_check_Resources(ctx, resource_dict, page_dict, tracker);
                     pdfi_countdown(resource_dict);
                     if (code < 0)
@@ -530,6 +546,27 @@ static int pdfi_check_ExtGState(pdf_context *ctx, pdf_dict *extgstate_dict, pdf_
         if (code == 0 && overprint)
             tracker->has_overprint = true;
 
+        /* Check Blend Mode *first* because we short-circuit transparency detection
+         * when we find transparency, and we want to be able to record the use of
+         * blend modes other than Compatible/Normal so that Patterns using these
+         * always use the clist and not tiles. The tiles implementation can't work
+         * if we change to a blend mode other than Normal or Compatible during
+         * the course of a Pattern.
+         */
+        code = pdfi_dict_knownget_type(ctx, extgstate_dict, "BM", PDF_NAME, &o);
+        if (code > 0) {
+            if (!pdfi_name_is((pdf_name *)o, "Normal")) {
+                if (!pdfi_name_is((pdf_name *)o, "Compatible")) {
+                    pdfi_countdown(o);
+                    tracker->transparent = true;
+                    tracker->BM_Not_Normal = true;
+                    return 0;
+                }
+            }
+        }
+        pdfi_countdown(o);
+        o = NULL;
+
         /* Check SMask */
         code = pdfi_dict_knownget(ctx, extgstate_dict, "SMask", &o);
         if (code > 0) {
@@ -566,19 +603,6 @@ static int pdfi_check_ExtGState(pdf_context *ctx, pdf_dict *extgstate_dict, pdf_
         pdfi_countdown(o);
         o = NULL;
 
-        code = pdfi_dict_knownget_type(ctx, extgstate_dict, "BM", PDF_NAME, &o);
-        if (code > 0) {
-            if (!pdfi_name_is((pdf_name *)o, "Normal")) {
-                if (!pdfi_name_is((pdf_name *)o, "Compatible")) {
-                    pdfi_countdown(o);
-                    tracker->transparent = true;
-                    return 0;
-                }
-            }
-        }
-        pdfi_countdown(o);
-        o = NULL;
-
         code = pdfi_dict_knownget_number(ctx, extgstate_dict, "CA", &f);
         if (code > 0) {
             if (f != 1.0) {
@@ -671,7 +695,7 @@ error1:
 
 /*
  * This routine checks a Pattern dictionary to see if it contains any spot
- * colour definitions, or transparency usage.
+ * colour definitions, or transparency usage (or blend modes other than Normal/Compatible).
  */
 static int pdfi_check_Pattern(pdf_context *ctx, pdf_dict *pattern, pdf_dict *page_dict,
                               pdfi_check_tracker_t *tracker)
@@ -715,10 +739,10 @@ transparency_exit:
  * This routine checks a Pattern dictionary for transparency.
  */
 int pdfi_check_Pattern_transparency(pdf_context *ctx, pdf_dict *pattern, pdf_dict *page_dict,
-                                    bool *transparent)
+                                    bool *transparent, bool *BM_Not_Normal)
 {
     int code;
-    pdfi_check_tracker_t tracker = {0, 0, NULL, NULL, 0, NULL};
+    pdfi_check_tracker_t tracker = {0, 0, 0, NULL, NULL, 0, NULL};
 
     /* NOTE: We use a "null" tracker that won't do any optimization to prevent
      * checking the same resource twice.
@@ -728,8 +752,10 @@ int pdfi_check_Pattern_transparency(pdf_context *ctx, pdf_dict *pattern, pdf_dic
      * pdfi_check_free_tracker() as in pdfi_check_page(), below.
      */
     code = pdfi_check_Pattern(ctx, pattern, page_dict, &tracker);
-    if (code == 0)
+    if (code == 0) {
         *transparent = tracker.transparent;
+        *BM_Not_Normal = tracker.BM_Not_Normal;
+    }
     else
         *transparent = false;
     return code;
@@ -898,7 +924,7 @@ static int pdfi_check_Font(pdf_context *ctx, pdf_dict *font, pdf_dict *page_dict
                 }
             }
 
-            if (known >= 0)
+            if (known > 0)
                 code = pdfi_dict_put(ctx, font_info_dict, "Embedded", PDF_TRUE_OBJ);
             else
                 code = pdfi_dict_put(ctx, font_info_dict, "Embedded", PDF_FALSE_OBJ);
@@ -913,7 +939,7 @@ static int pdfi_check_Font(pdf_context *ctx, pdf_dict *font, pdf_dict *page_dict
 
 
         code = pdfi_dict_knownget_type(ctx, font, "Subtype", PDF_NAME, &array_obj);
-        if (code >= 0) {
+        if (code > 0) {
             code = pdfi_dict_put(ctx, font_info_dict, "Subtype", array_obj);
             if (code < 0) {
                 pdfi_countdown(array_obj);
@@ -1017,7 +1043,7 @@ static int pdfi_check_Font(pdf_context *ctx, pdf_dict *font, pdf_dict *page_dict
 static int pdfi_check_Font_dict(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *page_dict,
                                 pdfi_check_tracker_t *tracker)
 {
-    int code;
+    int code = 0;
     uint64_t i, index;
     pdf_obj *Key = NULL, *Value = NULL;
 
@@ -1033,12 +1059,24 @@ static int pdfi_check_Font_dict(pdf_context *ctx, pdf_dict *font_dict, pdf_dict
             return code;
 
         code = pdfi_dict_first(ctx, font_dict, &Key, &Value, &index);
-        if (code < 0)
+        if (code < 0) {
+            (void)pdfi_loop_detector_cleartomark(ctx); /* Clear to the mark for the current resource loop */
             goto error1;
+        }
 
         i = 1;
         do {
-            code = pdfi_check_Font(ctx, (pdf_dict *)Value, page_dict, tracker);
+            if (pdfi_type_of(Value) == PDF_DICT)
+                code = pdfi_check_Font(ctx, (pdf_dict *)Value, page_dict, tracker);
+            else if (pdfi_type_of(Value) == PDF_FONT) {
+                pdf_dict *d = ((pdf_font *)Value)->PDF_font;
+
+                code = pdfi_check_Font(ctx, d, page_dict, tracker);
+            } else {
+                pdfi_set_warning(ctx, 0, NULL, W_PDF_FONTRESOURCE_TYPE, "pdfi_check_Font_dict", "");
+            }
+            if (code < 0)
+                break;
 
             pdfi_countdown(Key);
             Key = NULL;
@@ -1051,30 +1089,23 @@ static int pdfi_check_Font_dict(pdf_context *ctx, pdf_dict *font_dict, pdf_dict
             if (code < 0)
                 goto error1;
 
-            do {
-                if (i++ >= pdfi_dict_entries(font_dict)) {
-                    code = 0;
-                    goto transparency_exit;
-                }
+            if (i++ >= pdfi_dict_entries(font_dict)) {
+                code = 0;
+                break;
+            }
 
-                code = pdfi_dict_next(ctx, font_dict, &Key, &Value, &index);
-                if (code == 0 && pdfi_type_of(Value) == PDF_DICT)
-                    break;
-                pdfi_countdown(Key);
-                Key = NULL;
-                pdfi_countdown(Value);
-                Value = NULL;
-            } while(1);
+            code = pdfi_dict_next(ctx, font_dict, &Key, &Value, &index);
+            if (code < 0)
+                break;
         }while (1);
-    }
-    return 0;
 
-transparency_exit:
+        (void)pdfi_loop_detector_cleartomark(ctx); /* Clear to the mark for the current resource loop */
+    }
+
     pdfi_countdown(Key);
     pdfi_countdown(Value);
 
 error1:
-    (void)pdfi_loop_detector_cleartomark(ctx); /* Clear to the mark for the current resource loop */
     return code;
 }
 
@@ -1096,34 +1127,50 @@ static int pdfi_check_Resources(pdf_context *ctx, pdf_dict *Resources_dict,
      */
     if (tracker->spot_dict != NULL) {
         code = pdfi_dict_knownget_type(ctx, Resources_dict, "ColorSpace", PDF_DICT, &d);
-        if (code > 0)
-            (void)pdfi_check_ColorSpace_dict(ctx, (pdf_dict *)d, page_dict, tracker);
+        if (code < 0) {
+            if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_SPOT_CHK_BADTYPE, "pdfi_check_Resources", "")) < 0)
+                return code;
+        }
+        (void)pdfi_check_ColorSpace_dict(ctx, (pdf_dict *)d, page_dict, tracker);
 
         pdfi_countdown(d);
         d = NULL;
 
         code = pdfi_dict_knownget_type(ctx, Resources_dict, "Shading", PDF_DICT, &d);
-        if (code > 0)
-            (void)pdfi_check_Shading_dict(ctx, (pdf_dict *)d, page_dict, tracker);
+        if (code < 0) {
+            if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_SPOT_CHK_BADTYPE, "pdfi_check_Resources", "")) < 0)
+                return code;
+        }
+        (void)pdfi_check_Shading_dict(ctx, (pdf_dict *)d, page_dict, tracker);
         pdfi_countdown(d);
         d = NULL;
     }
 
     code = pdfi_dict_knownget_type(ctx, Resources_dict, "XObject", PDF_DICT, &d);
-    if (code > 0)
-        (void)pdfi_check_XObject_dict(ctx, (pdf_dict *)d, page_dict, tracker);
+    if (code < 0) {
+        if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_TRANS_CHK_BADTYPE, "pdfi_check_Resources", "")) < 0)
+            return code;
+    }
+    (void)pdfi_check_XObject_dict(ctx, (pdf_dict *)d, page_dict, tracker);
     pdfi_countdown(d);
     d = NULL;
 
     code = pdfi_dict_knownget_type(ctx, Resources_dict, "Pattern", PDF_DICT, &d);
-    if (code > 0)
-        (void)pdfi_check_Pattern_dict(ctx, (pdf_dict *)d, page_dict, tracker);
+    if (code < 0) {
+        if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_TRANS_CHK_BADTYPE, "pdfi_check_Resources", "")) < 0) {
+            return code;
+        }
+    }
+    (void)pdfi_check_Pattern_dict(ctx, (pdf_dict *)d, page_dict, tracker);
     pdfi_countdown(d);
     d = NULL;
 
     code = pdfi_dict_knownget_type(ctx, Resources_dict, "Font", PDF_DICT, &d);
-    if (code > 0)
-        (void)pdfi_check_Font_dict(ctx, (pdf_dict *)d, page_dict, tracker);
+    if (code < 0) {
+        if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_TRANS_CHK_BADTYPE, "pdfi_check_Resources", "")) < 0)
+            return code;
+    }
+    (void)pdfi_check_Font_dict(ctx, (pdf_dict *)d, page_dict, tracker);
     /* From this point onwards, if we detect transparency (or have already detected it) we
      * can exit, we have already counted up any spot colours.
      */
@@ -1131,8 +1178,11 @@ static int pdfi_check_Resources(pdf_context *ctx, pdf_dict *Resources_dict,
     d = NULL;
 
     code = pdfi_dict_knownget_type(ctx, Resources_dict, "ExtGState", PDF_DICT, &d);
-    if (code > 0)
-        (void)pdfi_check_ExtGState_dict(ctx, (pdf_dict *)d, page_dict, tracker);
+    if (code < 0) {
+        if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_TRANS_CHK_BADTYPE, "pdfi_check_Resources", "")) < 0)
+            return code;
+    }
+    (void)pdfi_check_ExtGState_dict(ctx, (pdf_dict *)d, page_dict, tracker);
     pdfi_countdown(d);
     d = NULL;
 
@@ -1190,8 +1240,9 @@ static int pdfi_check_annot_for_transparency(pdf_context *ctx, pdf_dict *annot,
 
     code = pdfi_dict_get_type(ctx, annot, "Subtype", PDF_NAME, (pdf_obj **)&n);
     if (code < 0) {
-        if (ctx->args.pdfstoponerror)
+        if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BAD_TYPE, "pdfi_check_annot_for_transparency", "")) < 0) {
             return code;
+        }
     } else {
         /* Check #2, Highlight annotations are always preformed with transparency */
         if (pdfi_name_is((const pdf_name *)n, "Highlight")) {
@@ -1260,8 +1311,9 @@ static int pdfi_check_Annots_for_transparency(pdf_context *ctx, pdf_array *annot
         code = pdfi_array_get_type(ctx, annots_array, (uint64_t)i, PDF_DICT, (pdf_obj **)&annot);
         if (code >= 0) {
             code = pdfi_check_annot_for_transparency(ctx, annot, page_dict, tracker);
-            if (code < 0 && ctx->args.pdfstoponerror)
+            if (code < 0 && (code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_INVALID_TRANS_XOBJECT, "pdfi_check_Annots_for_transparency", "")) < 0) {
                 goto exit;
+            }
 
             /* If we've found transparency, and don't need to continue checkign for spot colours
              * just exit as fast as possible.
@@ -1272,9 +1324,9 @@ static int pdfi_check_Annots_for_transparency(pdf_context *ctx, pdf_array *annot
             pdfi_countdown(annot);
             annot = NULL;
         }
-        if (code < 0 && ctx->args.pdfstoponerror)
+        else if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_INVALID_TRANS_XOBJECT, "pdfi_check_Annots_for_transparency", "")) < 0) {
             goto exit;
-        code = 0;
+        }
     }
 exit:
     pdfi_countdown(annot);
@@ -1320,8 +1372,12 @@ static int pdfi_check_page_inner(pdf_context *ctx, pdf_dict *page_dict,
             code = pdfi_dict_knownget(ctx, Group, "CS", &CS);
             if (code > 0)
                 code = pdfi_check_ColorSpace_for_spots(ctx, CS, Group, page_dict, tracker->spot_dict);
-            if (code < 0 && ctx->args.pdfstoponerror)
-                goto exit;
+
+            if (code < 0) {
+                if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_GS_LIB_ERROR, "pdfi_check_page_inner", "")) < 0) {
+                    goto exit;
+                }
+            }
         }
     }
 
@@ -1329,8 +1385,11 @@ static int pdfi_check_page_inner(pdf_context *ctx, pdf_dict *page_dict,
     code = pdfi_dict_knownget_type(ctx, page_dict, "Resources", PDF_DICT, (pdf_obj **)&Resources);
     if (code > 0)
         code = pdfi_check_Resources(ctx, Resources, page_dict, tracker);
-    if ((code < 0 && ctx->args.pdfstoponerror) || (code == gs_error_pdf_stackoverflow))
+
+    if (code == gs_error_pdf_stackoverflow || (code < 0 &&
+       (code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_GS_LIB_ERROR, "pdfi_check_page_inner", "")) < 0)) {
         goto exit;
+    }
 
     /* If we are drawing Annotations, check to see if the page uses any Annots */
     if (ctx->args.showannots) {
@@ -1338,8 +1397,11 @@ static int pdfi_check_page_inner(pdf_context *ctx, pdf_dict *page_dict,
         if (code > 0)
             code = pdfi_check_Annots_for_transparency(ctx, Annots, page_dict,
                                                       tracker);
-        if (code < 0 && ctx->args.pdfstoponerror)
-            goto exit;
+        if (code < 0) {
+            if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_GS_LIB_ERROR, "pdfi_check_page_inner", "")) < 0) {
+               goto exit;
+            }
+        }
     }
 
     code = 0;
@@ -1364,12 +1426,6 @@ int pdfi_check_page(pdf_context *ctx, pdf_dict *page_dict, pdf_array **fonts_arr
     ctx->page.num_spots = 0;
     ctx->page.has_transparency = false;
 
-    /* Need to do this here so that pdfi_check_init_tracker will be setup
-     * for spot colours, if the device is spot colour capable.
-     * It is also called in pdfi_page_render() and pdfi_process_pdf_file()
-     * TODO: Should probably look into that..
-     */
-    pdfi_device_set_flags(ctx);
     code = pdfi_check_init_tracker(ctx, &tracker, fonts_array, spots_array);
     if (code < 0)
         goto exit;
