@@ -1,4 +1,4 @@
-/* Copyright (C) 2020-2022 Artifex Software, Inc.
+/* Copyright (C) 2020-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Functions to deal with PDF structure, such as retrieving
@@ -85,7 +85,10 @@ int pdfi_read_Root(pdf_context *ctx)
     if (code < 0) {
         bool known = false;
 
-        pdfi_set_error(ctx, 0, NULL, E_PDF_MISSINGTYPE, "pdfi_read_Root", NULL);
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MISSINGTYPE, "pdfi_read_Root", NULL)) < 0) {
+            pdfi_countdown(o1);
+            return code;
+        }
 
         /* Missing the *required* /Type key! See if it has /Pages at least, if it does carry on */
         code = pdfi_dict_known(ctx, (pdf_dict *)o1, "Pages", &known);
@@ -96,14 +99,33 @@ int pdfi_read_Root(pdf_context *ctx)
     }
     else {
         if (pdfi_name_strcmp((pdf_name *)o, "Catalog") != 0){
-            pdfi_countdown(o);
-            pdfi_countdown(o1);
-            /* If we repaired the file, we may already have spotted a potential Root dictionary
-             * so if the one we found here isn't valid, try the one we found when scanning
+            pdf_obj *pages = NULL;
+
+            /* Bug #706038 has a Root dictionary with /Type /Calalog which (of course) Acrobat
+             * happily opens :-( So if we find a /Type and it's not /Catalog, try and see if
+             * we have a /Pages key (the only other required entry). If we do assume that the
+             * Type is wrong and use this dictionary, otherwise fall back to seeing if we
+             * have a repaired Root. See above for handling a missing /Type.....
              */
-            if (ctx->Root == NULL)
-                return_error(gs_error_syntaxerror);
-            return 0;
+            code = pdfi_dict_get_type(ctx, (pdf_dict *)o1, "Pages", PDF_DICT, &pages);
+            if (code < 0) {
+                pdfi_countdown(o);
+                pdfi_countdown(o1);
+                /* If we repaired the file, we may already have spotted a potential Root dictionary
+                 * so if the one we found here isn't valid, try the one we found when scanning
+                 */
+                if (ctx->Root == NULL) {
+                    pdfi_set_error(ctx, 0, NULL, E_PDF_NO_ROOT, "pdfi_read_Root", NULL);
+                    return_error(gs_error_syntaxerror);
+                }
+                return 0;
+            }
+            pdfi_countdown(pages);
+            if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_typecheck), NULL, E_PDF_BAD_ROOT_TYPE, "pdfi_read_Root", NULL)) < 0) {
+                pdfi_countdown(o);
+                pdfi_countdown(o1);
+                return code;
+            }
         }
         pdfi_countdown(o);
     }
@@ -142,8 +164,23 @@ static int Info_check_array(pdf_context *ctx, pdf_array *a)
                         goto error;
                 }
                 code = Info_check_dict(ctx, (pdf_dict *)array_obj);
-                if (code < 0)
+                if (code < 0) {
+                    if (array_obj->object_num != 0) {
+                        pdf_indirect_ref *i_o;
+
+                        code = pdfi_object_alloc(ctx, PDF_INDIRECT, 0, (pdf_obj **)&i_o);
+                        if (code < 0)
+                            goto error;
+                        i_o->ref_generation_num = array_obj->generation_num;
+                        i_o->ref_object_num = array_obj->object_num;
+                        i_o->indirect_num = array_obj->object_num;
+                        i_o->indirect_gen = array_obj->generation_num;
+                        code = pdfi_array_put(ctx, a, i, (pdf_obj *)i_o);
+                        if (code < 0)
+                            return code;
+                    }
                     goto error;
+                }
                 break;
             case PDF_ARRAY:
                 if (array_obj->object_num != 0) {
@@ -152,8 +189,23 @@ static int Info_check_array(pdf_context *ctx, pdf_array *a)
                         goto error;
                 }
                 code = Info_check_array(ctx, (pdf_array *)array_obj);
-                if (code < 0)
+                if (code < 0) {
+                    if (array_obj->object_num != 0) {
+                        pdf_indirect_ref *i_o;
+
+                        code = pdfi_object_alloc(ctx, PDF_INDIRECT, 0, (pdf_obj **)&i_o);
+                        if (code < 0)
+                            goto error;
+                        i_o->ref_generation_num = array_obj->generation_num;
+                        i_o->ref_object_num = array_obj->object_num;
+                        i_o->indirect_num = array_obj->object_num;
+                        i_o->indirect_gen = array_obj->generation_num;
+                        code = pdfi_array_put(ctx, a, i, (pdf_obj *)i_o);
+                        if (code < 0)
+                            return code;
+                    }
                     goto error;
+                }
                 break;
             default:
                 break;
@@ -329,7 +381,6 @@ int pdfi_read_Info(pdf_context *ctx)
 
     (void)pdfi_loop_detector_cleartomark(ctx);
 
-    pdfi_device_set_flags(ctx);
     pdfi_pdfmark_write_docinfo(ctx, Info);
 
     /* We don't pdfi_countdown(Info) now, because we've transferred our
@@ -453,7 +504,8 @@ int pdfi_read_Pages(pdf_context *ctx)
          * each node and add any leaf nodes.
          */
         for (i=0;i < a->size; i++) {
-            code = pdfi_array_get(ctx, a, i, &p);
+            /* We must not allow the entry in the array to be replaced, in case it's a circular reference */
+            code = pdfi_array_get_no_store_R(ctx, a, i, &p);
             if (code < 0)
                 continue;
             if (pdfi_type_of(p) != PDF_DICT) {
@@ -461,6 +513,24 @@ int pdfi_read_Pages(pdf_context *ctx)
                 p = NULL;
                 continue;
             }
+            /* Explicit check that the root node Kids array entry is not a self-reference
+             * back to the root node. We only check one level of the Kids array. so we don't
+             * need a full loop detection setup here.
+             */
+            if (p->object_num != 0 && p->object_num == o1->object_num) {
+                pdfi_countdown(p);
+                p = NULL;
+                pdfi_countdown(a);
+                pdfi_countdown(o1);
+                ctx->num_pages = 0;
+                return_error(gs_error_circular_reference);
+            }
+            /* Now we've checked for circular reference we can replace the entry in the
+             * array, and add the object to the cache.
+             * These are optimisations, so we don't especially care whether they succeed
+             */
+            (void)pdfi_array_put(ctx, a, i, p);
+            (void)replace_cache_entry(ctx, p);
             code = pdfi_dict_knownget_type(ctx, (pdf_dict *)p, "Type", PDF_NAME, (pdf_obj **)&p1);
             if (code <= 0) {
                 pdfi_countdown(p);
@@ -503,8 +573,13 @@ int pdfi_read_Pages(pdf_context *ctx)
      */
     if (pagecount != ctx->num_pages) {
         ctx->num_pages = pagecount;
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_BADPAGECOUNT, "pdfi_read_Pages", NULL)) < 0)
+            return code;
         code = pdfi_dict_put_int(ctx, (pdf_dict *)o1, "Count", ctx->num_pages);
-        pdfi_set_error(ctx, 0, NULL, E_PDF_BADPAGECOUNT, "pdfi_read_Pages", NULL);
+        if (code < 0) {
+            pdfi_countdown(o1);
+            return code;
+        }
     }
 
     /* We don't pdfi_countdown(o1) now, because we've transferred our
@@ -590,9 +665,27 @@ static int pdfi_get_child(pdf_context *ctx, pdf_array *Kids, int i, pdf_dict **p
             if (code < 0)
                 goto errorExit;
         }
+        /* It makes no sense for the Page dictionary to be a stream, but safedocs/DialectDictIsStream.pdf
+         * has this (stream length is 0, not that it matters) and Acrobat happily opens it....
+         */
         if (pdfi_type_of(child) != PDF_DICT) {
-            code = gs_note_error(gs_error_typecheck);
-            goto errorExit;
+            pdf_dict *d1 = NULL;
+
+            if (pdfi_type_of(child) != PDF_STREAM) {
+                code = gs_note_error(gs_error_typecheck);
+                goto errorExit;
+            }
+            if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_typecheck), NULL, E_PDF_DICT_IS_STREAM, "pdfi_get_child", NULL)) < 0)
+                goto errorExit;
+
+            code = pdfi_get_stream_dict(ctx, (pdf_stream *)child, &d1);
+            if (code < 0)
+                goto errorExit;
+            pdfi_countdown(child);
+            child = d1;
+            code = replace_cache_entry(ctx, (pdf_obj *)d1);
+            if (code < 0)
+                goto errorExit;
         }
         /* If its an intermediate node, store it in the page_table, if its a leaf node
          * then don't store it. Instead we create a special dictionary of our own which
@@ -616,7 +709,8 @@ static int pdfi_get_child(pdf_context *ctx, pdf_array *Kids, int i, pdf_dict **p
         } else {
             /* Bizarrely, one of the QL FTS files (FTS_07_0704.pdf) has a page diciotnary with a /Type of /Template */
             if (!pdfi_name_is(Type, "Page"))
-                pdfi_set_error(ctx, 0, NULL, E_PDF_BADPAGETYPE, "pdfi_get_child", NULL);
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_typecheck), NULL, E_PDF_BADPAGETYPE, "pdfi_get_child", NULL)) < 0)
+                    goto errorExit;
             /* Make a 'PageRef' entry (just stores an indirect reference to the actual page)
              * and store that in the Kids array for future reference. But pass on the
              * dereferenced Page dictionary, in case this is the target page.
@@ -815,7 +909,7 @@ int pdfi_get_page_dict(pdf_context *ctx, pdf_dict *d, uint64_t page_num, uint64_
                     if ((*page_offset) == page_num) {
                         pdf_dict *page_dict = NULL;
 
-                        code = pdfi_dict_get(ctx, child, "PageRef", (pdf_obj **)&page_dict);
+                        code = pdfi_dict_get_no_store_R(ctx, child, "PageRef", (pdf_obj **)&page_dict);
                         if (code < 0)
                             goto exit;
                         code = pdfi_merge_dicts(ctx, page_dict, inheritable);
@@ -827,8 +921,10 @@ int pdfi_get_page_dict(pdf_context *ctx, pdf_dict *d, uint64_t page_num, uint64_
                         *page_offset += 1;
                     }
                 } else {
-                    if (!pdfi_name_is(Type, "Page"))
-                        pdfi_set_error(ctx, 0, NULL, E_PDF_BADPAGETYPE, "pdfi_get_page_dict", NULL);
+                    if (!pdfi_name_is(Type, "Page")) {
+                        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_typecheck), NULL, E_PDF_BADPAGETYPE, "pdfi_get_page_dict", NULL)) < 0)
+                            goto exit;
+                    }
                     if ((*page_offset) == page_num) {
                         code = pdfi_merge_dicts(ctx, child, inheritable);
                         *target = child;
@@ -906,12 +1002,13 @@ int pdfi_find_resource(pdf_context *ctx, unsigned char *Type, pdf_name *name,
     pdf_dict *Parent = NULL;
     pdf_name *n = NULL;
     int code;
+    bool known = false;
 
     *o = NULL;
 
     /* Check the provided dict, stream_dict can be NULL if we are trying to find a Default* ColorSpace */
     if (dict != NULL) {
-        bool deref_parent = true;
+        bool deref_parent = true, dict_is_XObject = false;
 
         code = pdfi_resource_knownget_typedict(ctx, Type, dict, &typedict);
         if (code < 0)
@@ -929,33 +1026,63 @@ int pdfi_find_resource(pdf_context *ctx, unsigned char *Type, pdf_name *name,
         if (pdfi_dict_knownget_type(ctx, dict, "Type", PDF_NAME, (pdf_obj **)&n) > 0) {
             if (pdfi_name_is(n, "Page"))
                 deref_parent = false;
+            if (pdfi_name_is(n, "XObject"))
+                dict_is_XObject = true;
             pdfi_countdown(n);
         }
 
         if (deref_parent) {
-            code = pdfi_dict_knownget_type(ctx, dict, "Parent", PDF_DICT, (pdf_obj **)&Parent);
-            if (code < 0)
-                goto exit;
-            if (code > 0) {
+            code = pdfi_dict_known(ctx, dict, "Parent", &known);
+            if (code >= 0 && known == true) {
+                code = pdfi_dict_get_no_store_R(ctx, dict, "Parent", (pdf_obj **)&Parent);
+
+                if (code >= 0) {
+                    if (pdfi_type_of(Parent) != PDF_DICT) {
+                        if (pdfi_type_of(Parent) == PDF_INDIRECT) {
+                            pdf_indirect_ref *o = (pdf_indirect_ref *)Parent;
+
+                            Parent = NULL;
+                            code = pdfi_dereference(ctx, o->ref_object_num, o->ref_generation_num, (pdf_obj **)&Parent);
+                            pdfi_countdown(o);
+                            if (code >= 0 && pdfi_type_of(Parent) != PDF_DICT) {
+                                pdfi_countdown(Parent);
+                                Parent = NULL;
+                            }
+                        } else {
+                            pdfi_countdown(Parent);
+                            Parent = NULL;
+                        }
+                    }
+                } else
+                    Parent = NULL;
+            }
+
+            if (Parent != NULL) {
                 if (ctx->page.CurrentPageDict != NULL && Parent->object_num != ctx->page.CurrentPageDict->object_num) {
-                    if (pdfi_loop_detector_check_object(ctx, Parent->object_num) == true)
-                        return_error(gs_error_circular_reference);
+                    if (pdfi_loop_detector_check_object(ctx, Parent->object_num) == true) {
+                        code = gs_note_error(gs_error_circular_reference);
+                        goto exit;
+                    }
 
                     code = pdfi_loop_detector_mark(ctx);
                     if (code < 0)
-                        return code;
+                        goto exit;
 
                     code = pdfi_loop_detector_add_object(ctx, dict->object_num);
                     if (code < 0) {
                         (void)pdfi_loop_detector_cleartomark(ctx);
-                        return code;
+                        goto exit;
                     }
                     code = pdfi_find_resource(ctx, Type, name, Parent, page_dict, o);
                     (void)pdfi_loop_detector_cleartomark(ctx);
-                    if (code != gs_error_undefined)
+                    if (code != gs_error_undefined) {
+                        if (dict_is_XObject)
+                            pdfi_set_warning(ctx, 0, NULL, W_PDF_INHERITED_STREAM_RESOURCE, "pdfi_find_resource", (char *)"Couldn't find named resource in supplied dictionary, matching name located in Page Resource");
                         goto exit;
+                    }
                 }
             }
+            code = 0;
         }
         pdfi_countdown(typedict);
         typedict = NULL;
@@ -978,7 +1105,8 @@ int pdfi_find_resource(pdf_context *ctx, unsigned char *Type, pdf_name *name,
 
         if (code > 0) {
             code = pdfi_dict_get_no_store_R_key(ctx, typedict, name, o);
-            goto exit;
+            if (code != gs_error_undefined)
+                goto exit;
         }
     }
 
@@ -992,10 +1120,14 @@ int pdfi_find_resource(pdf_context *ctx, unsigned char *Type, pdf_name *name,
 
         if (code > 0) {
             code = pdfi_dict_get_no_store_R_key(ctx, typedict, name, o);
-            goto exit;
+            if (code != gs_error_undefined)
+                goto exit;
         }
     }
 
+    pdfi_countdown(typedict);
+    typedict = NULL;
+
     if (ctx->current_stream != NULL) {
         pdf_dict *stream_dict = NULL;
         pdf_stream *stream = ctx->current_stream;
@@ -1009,8 +1141,10 @@ int pdfi_find_resource(pdf_context *ctx, unsigned char *Type, pdf_name *name,
                 goto exit;
             if (code > 0) {
                 code = pdfi_dict_get_no_store_R_key(ctx, typedict, name, o);
-                pdfi_set_error(ctx, 0, NULL, E_PDF_INHERITED_STREAM_RESOURCE, "pdfi_find_resource", (char *)"Couldn't find named resource in suppled dictionary, or Parents, or Pages, matching name located in earlier stream Resource");
-                goto exit;
+                if (code == 0) {
+                    pdfi_set_error(ctx, 0, NULL, E_PDF_INHERITED_STREAM_RESOURCE, "pdfi_find_resource", (char *)"Couldn't find named resource in supplied dictionary, or Parents, or Pages, matching name located in earlier stream Resource");
+                    goto exit;
+                }
             }
             pdfi_countdown(typedict);
             typedict = NULL;
@@ -1019,7 +1153,7 @@ int pdfi_find_resource(pdf_context *ctx, unsigned char *Type, pdf_name *name,
     }
 
     /* If we got all the way down there, we didn't find it */
-    dmprintf(ctx->memory, "Couldn't find named resource\n");
+    pdfi_set_warning(ctx, 0, NULL, W_PDF_MISSING_NAMED_RESOURCE, "pdfi_find_resource", NULL);
     code = gs_error_undefined;
 
 exit:
@@ -1036,6 +1170,7 @@ static int pdfi_doc_mark_the_outline(pdf_context *ctx, pdf_dict *outline)
     uint64_t dictsize;
     uint64_t index;
     pdf_name *Key = NULL;
+    double num = 0;
 
     /* Basically we only do /Count, /Title, /A, /C, /F
      * The /First, /Last, /Next, /Parent get written magically by pdfwrite
@@ -1049,6 +1184,50 @@ static int pdfi_doc_mark_the_outline(pdf_context *ctx, pdf_dict *outline)
     code = pdfi_dict_copy(ctx, tempdict, outline);
     if (code < 0) goto exit;
 
+    /* Due to some craziness on the part of Adobe, the /Count in an Outline entry
+     * in a PDF file, and the /Count value in an /OUT pdfmark mean different things(!)
+     * In a PDF file it is the number of outline entries beneath the current entry
+     * (all child descsndants) whereas in a pdfmark it is the number of entries
+     * in just the next level. So we cannot use the /Count from the PDF file, we
+     * need to go to the /First entry of the next level, and then count all
+     * the entries at that level by following each /Next.
+     */
+    code = pdfi_dict_knownget_number(ctx, outline, "Count", &num);
+    if (code < 0)
+        goto exit;
+
+    /* We can't rely on Count being present to indicate that the Outline has descendants
+     * see bug #707228. Instead, look for the presence of a /First key. If there is no count
+     * key, or it is 0, then assume the entry is closed.
+     */
+    {
+        pdf_dict *current = NULL, *next = NULL;
+        int count = 0, code1;
+
+        code1 = pdfi_dict_knownget_type(ctx, outline, "First", PDF_DICT, (pdf_obj **)&current);
+        if (code1 > 0) {
+            if (code <= 0) {
+                if ((code = pdfi_set_warning_stop(ctx, code, NULL, W_PDF_OUTLINECHILD_NO_COUNT, "pdfi_doc_mark_the_outline", NULL)) < 0)
+                    goto exit;
+            }
+            count++;
+            do {
+                code1 = pdfi_dict_knownget_type(ctx, current, "Next", PDF_DICT, (pdf_obj **)&next);
+                if (code1 > 0) {
+                    pdfi_countdown(current);
+                    current = next;
+                    next = NULL;
+                    count++;
+                } else
+                    break;
+            } while (1);
+            pdfi_countdown(current);
+        }
+        if (num <= 0)
+            count *= -1;
+        pdfi_dict_put_int(ctx, tempdict, "Count", count);
+    }
+
     /* Go through the dict, removing some keys and doing special handling for others.
      */
     code = pdfi_dict_key_first(ctx, outline, (pdf_obj **)&Key, &index);
@@ -1276,6 +1455,7 @@ static int pdfi_doc_Info(pdf_context *ctx)
         if (pdfi_name_is(Key, "Author") || pdfi_name_is(Key, "Creator") ||
             pdfi_name_is(Key, "Title") || pdfi_name_is(Key, "Subject") ||
             pdfi_name_is(Key, "Keywords")) {
+
             code = pdfi_dict_put_obj(ctx, tempdict, (pdf_obj *)Key, Value, true);
             if (code < 0)
                 goto exit;
@@ -1330,10 +1510,12 @@ static int pdfi_doc_PageLabels(pdf_context *ctx)
             goto exit;
     }
 
-    /* This will send the PageLabels object as a 'pdfpagelabels' setdeviceparams */
-    code = pdfi_pdfmark_object(ctx, (pdf_obj *)PageLabels, "pdfpagelabels");
-    if (code < 0)
-        goto exit;
+    if (ctx->device_state.WantsPageLabels) {
+        /* This will send the PageLabels object as a 'pdfpagelabels' setdeviceparams */
+        code = pdfi_pdfmark_object(ctx, (pdf_obj *)PageLabels, "pdfpagelabels");
+        if (code < 0)
+            goto exit;
+    }
 
  exit:
     pdfi_countdown(PageLabels);
@@ -1487,6 +1669,13 @@ static int pdfi_doc_EmbeddedFiles(pdf_context *ctx)
     pdf_array *Names_array = NULL;
     pdf_array *Kids = NULL;
 
+    code = pdfi_dict_knownget_type(ctx, ctx->Root, "Collection", PDF_DICT, (pdf_obj **)&Names);
+    if (code < 0) goto exit;
+    if (code > 0) {
+        code = 0;
+        goto exit;
+    }
+
     code = pdfi_dict_knownget_type(ctx, ctx->Root, "Names", PDF_DICT, (pdf_obj **)&Names);
     if (code <= 0) goto exit;
 
@@ -1561,6 +1750,112 @@ static int pdfi_doc_AcroForm(pdf_context *ctx)
 }
 
 
+static int pdfi_doc_view(pdf_context *ctx)
+{
+    int code = 0;
+    pdf_dict *tempdict = NULL;
+    pdf_name *Mode = NULL, *Layout = NULL;
+    pdf_obj *ActionDest = NULL;
+
+    code = pdfi_dict_knownget_type(ctx, ctx->Root, "PageMode", PDF_NAME, (pdf_obj **)&Mode);
+    if (code < 0)
+        return code;
+
+    if (code != 0) {
+        code = pdfi_dict_alloc(ctx, 1, &tempdict);
+        if (code < 0)
+            goto exit;
+
+        pdfi_countup(tempdict);
+
+        code = pdfi_dict_put(ctx, tempdict, "PageMode", (pdf_obj *)Mode);
+        if (code < 0)
+            goto exit;
+
+        code = pdfi_pdfmark_from_dict(ctx, tempdict, NULL, "DOCVIEW");
+        if (code < 0)
+            goto exit;
+        pdfi_countdown(tempdict);
+        tempdict = NULL;
+    }
+
+    code = pdfi_dict_knownget_type(ctx, ctx->Root, "PageLayout", PDF_NAME, (pdf_obj **)&Layout);
+    if (code < 0)
+        goto exit;
+
+    if (code != 0) {
+        code = pdfi_dict_alloc(ctx, 1, &tempdict);
+        if (code < 0)
+            goto exit;
+
+        pdfi_countup(tempdict);
+
+        code = pdfi_dict_put(ctx, tempdict, "PageLayout", (pdf_obj *)Layout);
+        if (code < 0)
+            goto exit;
+
+        code = pdfi_pdfmark_from_dict(ctx, tempdict, NULL, "DOCVIEW");
+        if (code < 0)
+            goto exit;
+        pdfi_countdown(tempdict);
+        tempdict = NULL;
+    }
+
+    code = pdfi_dict_knownget(ctx, ctx->Root, "OpenAction", &ActionDest);
+    if (code < 0)
+        goto exit;
+
+    if (code != 0) {
+        if (pdfi_type_of(ActionDest) == PDF_DICT) {
+            /* Dictionary means this is an action */
+            code = pdfi_dict_alloc(ctx, 1, &tempdict);
+            if (code < 0)
+                goto exit;
+
+            pdfi_countup(tempdict);
+
+            code = pdfi_dict_put(ctx, tempdict, "A", (pdf_obj *)ActionDest);
+            if (code < 0)
+                goto exit;
+
+            code = pdfi_pdfmark_modA(ctx, tempdict);
+            if (code < 0) goto exit;
+
+            code = pdfi_pdfmark_from_dict(ctx, tempdict, NULL, "DOCVIEW");
+        } else {
+            if (pdfi_type_of(ActionDest) == PDF_ARRAY) {
+                /* Array means this is a destination */
+                code = pdfi_dict_alloc(ctx, 1, &tempdict);
+                if (code < 0)
+                    goto exit;
+
+                pdfi_countup(tempdict);
+
+                code = pdfi_dict_put(ctx, tempdict, "Dest", (pdf_obj *)ActionDest);
+                if (code < 0)
+                    goto exit;
+                code = pdfi_pdfmark_modDest(ctx, tempdict);
+                if (code < 0)
+                    goto exit;
+                code = pdfi_pdfmark_from_dict(ctx, tempdict, NULL, "DOCVIEW");
+                if (code < 0)
+                    goto exit;
+            } else {
+                code = gs_note_error(gs_error_typecheck);
+                goto exit;
+            }
+        }
+    }
+
+exit:
+    pdfi_countdown(ActionDest);
+    pdfi_countdown(Layout);
+    pdfi_countdown(Mode);
+    pdfi_countdown(tempdict);
+    return code;
+}
+
+
 /* See pdf_main.ps/process_trailer_attrs()
  * Some of this stuff is about pdfmarks, and some of it is just handling
  * random things in the trailer.
@@ -1570,33 +1865,47 @@ int pdfi_doc_trailer(pdf_context *ctx)
     int code = 0;
 
     /* Can't do this stuff with no Trailer */
-    if (!ctx->Trailer)
-        goto exit;
+    if (!ctx->Trailer) {
+        if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_undefined), NULL, W_PDF_BAD_TRAILER, "pdfi_doc_trailer", NULL)) < 0)
+            goto exit;
+    }
 
     if (ctx->device_state.writepdfmarks) {
         /* Handle Outlines */
         code = pdfi_doc_Outlines(ctx);
         if (code < 0) {
-            pdfi_set_warning(ctx, code, NULL, W_PDF_BAD_TRAILER, "pdfi_doc_trailer", NULL);
-            if (ctx->args.pdfstoponerror)
+            if ((code = pdfi_set_warning_stop(ctx, code, NULL, W_PDF_BAD_OUTLINES, "pdfi_doc_trailer", NULL)) < 0)
                 goto exit;
         }
 
+        /* Handle Docview pdfmark stuff */
+        if (ctx->args.preservedocview) {
+            code = pdfi_doc_view(ctx);
+            if (code < 0) {
+                if ((code = pdfi_set_warning_stop(ctx, code, NULL, W_PDF_BAD_VIEW, "pdfi_doc_view", NULL)) < 0)
+                    goto exit;
+            }
+        }
+
         /* Handle Info */
         code = pdfi_doc_Info(ctx);
         if (code < 0) {
-            pdfi_set_warning(ctx, code, NULL, W_PDF_BAD_TRAILER, "pdfi_doc_trailer", NULL);
-            if (ctx->args.pdfstoponerror)
+            /* pdfwrite will set a Fatal error when processing the DOCINFO if it has been
+             * told to create a PDF/A. the PDFA compatibility is 2, and the file info
+             * cannot be handled. In that case, abort immediately.
+             */
+            if ((code = pdfi_set_warning_stop(ctx, code, NULL, W_PDF_BAD_INFO, "pdfi_doc_trailer", NULL)) < 0)
                 goto exit;
         }
 
         /* Handle EmbeddedFiles */
         /* TODO: add a configuration option to embed or omit */
-        code = pdfi_doc_EmbeddedFiles(ctx);
-        if (code < 0) {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_TRAILER, "pdfi_doc_trailer", NULL);
-            if (ctx->args.pdfstoponerror)
-                goto exit;
+        if (ctx->args.preserveembeddedfiles) {
+            code = pdfi_doc_EmbeddedFiles(ctx);
+            if (code < 0) {
+                if ((code = pdfi_set_warning_stop(ctx, code, NULL, W_PDF_BAD_EMBEDDEDFILES, "pdfi_doc_trailer", NULL)) < 0)
+                    goto exit;
+            }
         }
     }
 
@@ -1606,24 +1915,21 @@ int pdfi_doc_trailer(pdf_context *ctx)
     /* Handle AcroForm -- this is some bookkeeping once per doc, not rendering them yet */
     code = pdfi_doc_AcroForm(ctx);
     if (code < 0) {
-        pdfi_set_warning(ctx, code, NULL, W_PDF_BAD_TRAILER, "pdfi_doc_trailer", NULL);
-        if (ctx->args.pdfstoponerror)
+        if ((code = pdfi_set_warning_stop(ctx, code, NULL, W_PDF_BAD_ACROFORM, "pdfi_doc_trailer", NULL)) < 0)
             goto exit;
     }
 
     /* Handle OutputIntent ICC Profile */
     code = pdfi_doc_OutputIntents(ctx);
     if (code < 0) {
-        pdfi_set_warning(ctx, code, NULL, W_PDF_BAD_TRAILER, "pdfi_doc_trailer", NULL);
-        if (ctx->args.pdfstoponerror)
+        if ((code = pdfi_set_warning_stop(ctx, code, NULL, W_PDF_BAD_OUTPUTINTENTS, "pdfi_doc_trailer", NULL)) < 0)
             goto exit;
     }
 
     /* Handle PageLabels */
     code = pdfi_doc_PageLabels(ctx);
     if (code < 0) {
-        pdfi_set_warning(ctx, code, NULL, W_PDF_BAD_TRAILER, "pdfi_doc_trailer", NULL);
-        if (ctx->args.pdfstoponerror)
+        if ((code = pdfi_set_warning_stop(ctx, code, NULL, W_PDF_BAD_PAGELABELS, "pdfi_doc_trailer", NULL)) < 0)
             goto exit;
     }
 
