@@ -1,4 +1,4 @@
-/* Copyright (C) 2018-2022 Artifex Software, Inc.
+/* Copyright (C) 2018-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 #include "ghostpdf.h"
@@ -22,6 +22,7 @@
 #include "pdf_array.h"
 #include "pdf_misc.h"
 #include "pdf_sec.h"
+#include "pdf_loop_detect.h"
 #include "stream.h"
 #include "strimpl.h"
 #include "strmio.h"
@@ -352,7 +353,7 @@ pdfi_JBIG2Decode_filter(pdf_context *ctx, pdf_dict *dict, pdf_dict *decode,
     int code;
     pdf_stream *Globals = NULL;
     byte *buf = NULL;
-    int64_t buflen;
+    int64_t buflen = 0;
     void *globalctx;
 
     s_jbig2decode_set_global_data((stream_state*)&state, NULL, NULL);
@@ -775,121 +776,137 @@ static int pdfi_apply_filter(pdf_context *ctx, pdf_dict *dict, pdf_name *n, pdf_
 {
     int code;
 
+    code = pdfi_loop_detector_mark(ctx);
+    if (code < 0)
+        return code;
+
+    if (dict->object_num != 0) {
+        code = pdfi_loop_detector_add_object(ctx, dict->object_num);
+        if (code < 0)
+            goto cleanupExit;
+    } else {
+        if (dict->indirect_num != 0) {
+            code = pdfi_loop_detector_add_object(ctx, dict->indirect_num);
+            if (code < 0)
+                goto cleanupExit;
+        }
+    }
+
     if (ctx->args.pdfdebug)
     {
-        char str[100];
+        char *str;
+        str = (char *)gs_alloc_bytes(ctx->memory, n->length + 1, "temp string for debug");
+        if (str == NULL)
+            return_error(gs_error_VMerror);
         memcpy(str, (const char *)n->data, n->length);
         str[n->length] = '\0';
         dmprintf1(ctx->memory, "FILTER NAME:%s\n", str);
+        gs_free_object(ctx->memory, str, "temp string for debug");
     }
 
     if (pdfi_name_is(n, "RunLengthDecode")) {
         code = pdfi_RunLength_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "CCITTFaxDecode")) {
         code = pdfi_CCITTFax_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "ASCIIHexDecode")) {
         code = pdfi_simple_filter(ctx, &s_AXD_template, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "ASCII85Decode")) {
         code = pdfi_ASCII85_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "SubFileDecode")) {
         code = pdfi_simple_filter(ctx, &s_SFD_template, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "FlateDecode")) {
         code = pdfi_Flate_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "JBIG2Decode")) {
         code = pdfi_JBIG2Decode_filter(ctx, dict, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "LZWDecode")) {
         code = pdfi_LZW_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "DCTDecode")) {
         code = pdfi_DCT_filter(ctx, dict, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "JPXDecode")) {
         code = pdfi_JPX_filter(ctx, dict, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
-
     if (pdfi_name_is(n, "AHx")) {
         if (!inline_image) {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL);
-            if (ctx->args.pdfstoponwarning)
-                return_error(gs_error_syntaxerror);
+            if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL)) < 0)
+                goto cleanupExit;
         }
         code = pdfi_simple_filter(ctx, &s_AXD_template, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "A85")) {
         if (!inline_image) {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL);
-            if (ctx->args.pdfstoponwarning)
-                return_error(gs_error_syntaxerror);
+            if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL)) < 0)
+                goto cleanupExit;
         }
         code = pdfi_ASCII85_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "LZW")) {
         if (!inline_image) {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL);
-            if (ctx->args.pdfstoponwarning)
-                return_error(gs_error_syntaxerror);
+            if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL)) < 0)
+                goto cleanupExit;
         }
         code = pdfi_LZW_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "CCF")) {
         if (!inline_image) {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL);
-            if (ctx->args.pdfstoponwarning)
-                return_error(gs_error_syntaxerror);
+            if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL)) < 0)
+                goto cleanupExit;
         }
         code = pdfi_CCITTFax_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "DCT")) {
         if (!inline_image) {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL);
-            if (ctx->args.pdfstoponwarning)
-                return_error(gs_error_syntaxerror);
+            if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL)) < 0)
+                goto cleanupExit;
         }
         code = pdfi_DCT_filter(ctx, dict, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "Fl")) {
         if (!inline_image) {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL);
-            if (ctx->args.pdfstoponwarning)
-                return_error(gs_error_syntaxerror);
+            if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL)) < 0)
+                goto cleanupExit;
         }
         code = pdfi_Flate_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
     if (pdfi_name_is(n, "RL")) {
         if (!inline_image) {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL);
-            if (ctx->args.pdfstoponwarning)
-                return_error(gs_error_syntaxerror);
+            if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_BAD_INLINEFILTER, "pdfi_apply_filter", NULL)) < 0)
+                goto cleanupExit;
         }
         code = pdfi_RunLength_filter(ctx, decode, source, new_stream);
-        return code;
+        goto cleanupExit;
     }
 
     pdfi_set_error(ctx, 0, NULL, E_PDF_UNKNOWNFILTER, "pdfi_apply_filter", NULL);
-    return_error(gs_error_undefined);
+    code = gs_error_undefined;
+
+cleanupExit:
+    pdfi_loop_detector_cleartomark(ctx);
+    return code;
 }
 
 int pdfi_filter_no_decryption(pdf_context *ctx, pdf_stream *stream_obj,
@@ -915,9 +932,14 @@ int pdfi_filter_no_decryption(pdf_context *ctx, pdf_stream *stream_obj,
     if (code < 0)
         goto exit;
 
-    code = pdfi_dict_knownget(ctx, stream_dict, "Filter", &Filter);
-    if (code == 0 && inline_image)
+    /* ISO 32000-2:2020 (PDF 2.0) - abbreviated names take precendence. */
+    if (inline_image) {
         code = pdfi_dict_knownget(ctx, stream_dict, "F", &Filter);
+        if (code == 0)
+            code = pdfi_dict_knownget(ctx, stream_dict, "Filter", &Filter);
+    } else
+        code = pdfi_dict_knownget(ctx, stream_dict, "Filter", &Filter);
+
     if (code < 0)
         goto exit;
     if (code == 0) {
@@ -931,9 +953,16 @@ int pdfi_filter_no_decryption(pdf_context *ctx, pdf_stream *stream_obj,
         code = gs_note_error(gs_error_typecheck);
         goto exit;
     case PDF_NAME:
-        code = pdfi_dict_knownget(ctx, stream_dict, "DecodeParms", &decode);
-        if (code == 0 && inline_image)
+        /* ISO 32000-2:2020 (PDF 2.0) - abbreviated names take precendence. */
+        if (inline_image) {
             code = pdfi_dict_knownget(ctx, stream_dict, "DP", &decode);
+            if (code == 0)
+                code = pdfi_dict_knownget(ctx, stream_dict, "DecodeParms", &decode);
+        } else {
+            code = pdfi_dict_knownget(ctx, stream_dict, "DecodeParms", &decode);
+            if (code == 0)
+                code = pdfi_dict_knownget(ctx, stream_dict, "DP", &decode);
+        }
         if (code < 0)
             goto exit;
 
@@ -948,9 +977,16 @@ int pdfi_filter_no_decryption(pdf_context *ctx, pdf_stream *stream_obj,
     {
         pdf_array *filter_array = (pdf_array *)Filter;
 
-        code = pdfi_dict_knownget_type(ctx, stream_dict, "DecodeParms", PDF_ARRAY, (pdf_obj **)&DecodeParams);
-        if (code == 0 && inline_image)
-            code = pdfi_dict_knownget_type(ctx, stream_dict, "DP", PDF_ARRAY, (pdf_obj **)&DecodeParams);
+        /* ISO 32000-2:2020 (PDF 2.0) - abbreviated names take precendence. */
+        if (inline_image) {
+            code = pdfi_dict_knownget(ctx, stream_dict, "DP", (pdf_obj **)&DecodeParams);
+            if (code == 0)
+                code = pdfi_dict_knownget(ctx, stream_dict, "DecodeParms", (pdf_obj **)&DecodeParams);
+        } else {
+            code = pdfi_dict_knownget(ctx, stream_dict, "DecodeParms", (pdf_obj **)&DecodeParams);
+            if (code == 0)
+                code = pdfi_dict_knownget(ctx, stream_dict, "DP", &decode);
+        }
         if (code < 0)
             goto exit;
 
@@ -958,7 +994,8 @@ int pdfi_filter_no_decryption(pdf_context *ctx, pdf_stream *stream_obj,
             if (pdfi_array_size(DecodeParams) == 0 || pdfi_array_size(DecodeParams) != pdfi_array_size(filter_array)) {
                 pdfi_countdown(DecodeParams);
                 DecodeParams = NULL;
-                pdfi_set_warning(ctx, 0, NULL, W_PDF_STREAM_BAD_DECODEPARMS, "pdfi_filter_no_decryption", NULL);
+                if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_rangecheck), NULL, W_PDF_STREAM_BAD_DECODEPARMS, "pdfi_filter_no_decryption", NULL)) < 0)
+                    goto exit;
             } else {
                 if (pdfi_array_size(DecodeParams) != pdfi_array_size(filter_array)) {
                     code = gs_note_error(gs_error_typecheck);
@@ -1011,7 +1048,8 @@ int pdfi_filter_no_decryption(pdf_context *ctx, pdf_stream *stream_obj,
             if (decode && decode != PDF_NULL_OBJ && pdfi_type_of(decode) != PDF_DICT) {
                 pdfi_countdown(decode);
                 decode = NULL;
-                pdfi_set_warning(ctx, 0, NULL, W_PDF_STREAM_BAD_DECODEPARMS, "pdfi_filter_no_decryption", NULL);
+                if ((code = pdfi_set_warning_stop(ctx, gs_note_error(gs_error_typecheck), NULL, W_PDF_STREAM_BAD_DECODEPARMS, "pdfi_filter_no_decryption", NULL)) < 0)
+                    goto error;
             }
             if (decode && decode == PDF_NULL_OBJ) {
                 pdfi_countdown(decode);
@@ -1083,6 +1121,7 @@ int pdfi_filter(pdf_context *ctx, pdf_stream *stream_obj, pdf_c_stream *source,
             pdf_obj *FS = NULL, *o = NULL;
             pdf_dict *dict = NULL;
             stream *gstream = NULL;
+            char CFileName[gp_file_name_sizeof];
 
             code = pdfi_dict_get(ctx, stream_dict, "F", &FileSpec);
             if (code < 0)
@@ -1105,12 +1144,20 @@ int pdfi_filter(pdf_context *ctx, pdf_stream *stream_obj, pdf_c_stream *source,
                 code = gs_note_error(gs_error_typecheck);
                 goto error;
             }
+
+            if (((pdf_string *)FileSpec)->length + 1 > gp_file_name_sizeof) {
+                code = gs_note_error(gs_error_ioerror);
+                goto error;
+            }
+            memcpy(CFileName, ((pdf_string *)FileSpec)->data, ((pdf_string *)FileSpec)->length);
+            CFileName[((pdf_string *)FileSpec)->length] = 0x00;
+
             /* We should now have a string with the filename (or URL). We need
              * to open the file and create a stream, if that succeeds.
              */
-            gstream = sfopen((const char *)((pdf_string *)FileSpec)->data, "r", ctx->memory);
+            gstream = sfopen((const char *)CFileName, "r", ctx->memory);
             if (gstream == NULL) {
-                emprintf1(ctx->memory, "Failed to open file %s\n", (const char *)((pdf_string *)FileSpec)->data);
+                emprintf1(ctx->memory, "Failed to open file %s\n", CFileName);
                 code = gs_note_error(gs_error_ioerror);
                 goto error;
             }
@@ -1210,7 +1257,7 @@ int pdfi_filter(pdf_context *ctx, pdf_stream *stream_obj, pdf_c_stream *source,
 
         SubFile_stream->original = source->s;
 
-        switch(ctx->encryption.StrF) {
+        switch(ctx->encryption.StmF) {
             case CRYPT_IDENTITY:
                 /* Can't happen, handled above */
                 break;
@@ -1358,102 +1405,24 @@ int pdfi_open_memory_stream_from_stream(pdf_context *ctx, unsigned int size, byt
  * This function returns < 0 for an error, and the length of the uncompressed data on success.
  */
 int pdfi_open_memory_stream_from_filtered_stream(pdf_context *ctx, pdf_stream *stream_obj,
-                                                 unsigned int size, byte **Buffer, pdf_c_stream *source,
-                                                 pdf_c_stream **new_pdf_stream, bool retain_ownership)
+                                                 byte **Buffer, pdf_c_stream **new_pdf_stream, bool retain_ownership)
 {
     int code;
-    pdf_dict *dict = NULL;
-    int decompressed_length = 0;
-    byte *decompressed_Buffer = NULL;
-    pdf_c_stream *compressed_stream = NULL, *decompressed_stream = NULL;
-    bool known = false;
+    int64_t bufferlen = 0;
 
-    code = pdfi_open_memory_stream_from_stream(ctx, (unsigned int)size, Buffer, source, new_pdf_stream, retain_ownership);
+    code = pdfi_stream_to_buffer(ctx, stream_obj, Buffer, &bufferlen);
     if (code < 0) {
-        pdfi_close_memory_stream(ctx, *Buffer, *new_pdf_stream);
         *Buffer = NULL;
         *new_pdf_stream = NULL;
         return code;
     }
-
-    if (stream_obj == NULL)
-        return size;
-
-    code = pdfi_dict_from_obj(ctx, (pdf_obj *)stream_obj, &dict);
-    if (code < 0)
-        return code;
-
-    pdfi_dict_known(ctx, dict, "F", &known);
-    if (!known)
-        pdfi_dict_known(ctx, dict, "Filter", &known);
-
-    if (!known)
-        return size;
-
-    compressed_stream = *new_pdf_stream;
-    /* This is again complicated by requiring a seekable stream, and the fact that,
-     * unlike fonts, there is no Length2 key to tell us how large the uncompressed
-     * stream is.
-     */
-    code = pdfi_filter(ctx, stream_obj, compressed_stream, &decompressed_stream, false);
+    code = pdfi_open_memory_stream_from_memory(ctx, (unsigned int)bufferlen, *Buffer, new_pdf_stream, retain_ownership);
     if (code < 0) {
-        pdfi_close_memory_stream(ctx, *Buffer, *new_pdf_stream);
+        gs_free_object(ctx->memory, *Buffer, "pdfi_open_memory_stream_from_filtered_stream");
         *Buffer = NULL;
         *new_pdf_stream = NULL;
-        return code;
     }
-    do {
-        byte b[512];
-        code = pdfi_read_bytes(ctx, (byte *)&b, 1, 512, decompressed_stream);
-        if (code <= 0)
-            break;
-        decompressed_length+=code;
-        if (code < 512)
-            break;
-    } while (true);
-    pdfi_close_file(ctx, decompressed_stream);
-
-    decompressed_Buffer = gs_alloc_bytes(ctx->memory, decompressed_length, "pdfi_open_memory_stream_from_filtered_stream (decompression buffer)");
-    if (decompressed_Buffer != NULL) {
-        code = srewind(compressed_stream->s);
-        if (code >= 0) {
-            code = pdfi_filter(ctx, stream_obj, compressed_stream,
-                               &decompressed_stream, false);
-            if (code >= 0) {
-                code = pdfi_read_bytes(ctx, decompressed_Buffer, 1, decompressed_length, decompressed_stream);
-                pdfi_close_file(ctx, decompressed_stream);
-                code = pdfi_close_memory_stream(ctx, *Buffer, *new_pdf_stream);
-                if (code >= 0) {
-                    *Buffer = decompressed_Buffer;
-                    code = pdfi_open_memory_stream_from_memory(ctx, (unsigned int)decompressed_length,
-                                                               *Buffer, new_pdf_stream, retain_ownership);
-                } else {
-                    *Buffer = NULL;
-                    *new_pdf_stream = NULL;
-                }
-            }
-        } else {
-            pdfi_close_memory_stream(ctx, *Buffer, *new_pdf_stream);
-            gs_free_object(ctx->memory, decompressed_Buffer, "pdfi_open_memory_stream_from_filtered_stream");
-            gs_free_object(ctx->memory, Buffer, "pdfi_open_memory_stream_from_filtered_stream");
-            *Buffer = NULL;
-            *new_pdf_stream = NULL;
-            return code;
-        }
-    } else {
-        pdfi_close_memory_stream(ctx, *Buffer, *new_pdf_stream);
-        gs_free_object(ctx->memory, Buffer, "pdfi_open_memory_stream_from_filtered_stream");
-        *Buffer = NULL;
-        *new_pdf_stream = NULL;
-        return_error(gs_error_VMerror);
-    }
-    if (code < 0) {
-        gs_free_object(ctx->memory, Buffer, "pdfi_build_function_4");
-        *Buffer = NULL;
-        *new_pdf_stream = NULL;
-        return code;
-    }
-    return decompressed_length;
+    return (int)bufferlen;
 }
 
 int pdfi_open_memory_stream_from_memory(pdf_context *ctx, unsigned int size, byte *Buffer, pdf_c_stream **new_pdf_stream, bool retain_ownership)
@@ -1516,12 +1485,18 @@ void pdfi_close_file(pdf_context *ctx, pdf_c_stream *s)
 
 int pdfi_seek(pdf_context *ctx, pdf_c_stream *s, gs_offset_t offset, uint32_t origin)
 {
+    int code = 0;
+
     if (origin == SEEK_CUR && s->unread_size != 0)
         offset -= s->unread_size;
 
     s->unread_size = 0;;
 
-    return (sfseek(s->s, offset, origin));
+    code = sfseek(s->s, offset, origin);
+    if (s->eof && code >= 0)
+        s->eof = 0;
+
+    return code;
 }
 
 /* We use 'stell' sometimes to save the position of the underlying file
@@ -1647,9 +1622,10 @@ pdfi_stream_to_buffer(pdf_context *ctx, pdf_stream *stream_obj, byte **buf, int6
 {
     byte *Buffer = NULL;
     int code = 0;
-    int64_t buflen = 0;
+    uint read = 0, buflen = 0;
+    int64_t ToRead = *bufferlen;
     gs_offset_t savedoffset;
-    pdf_c_stream *stream;
+    pdf_c_stream *stream = NULL, *SubFileStream = NULL;
     bool filtered;
     pdf_dict *stream_dict = NULL;
 
@@ -1672,20 +1648,29 @@ pdfi_stream_to_buffer(pdf_context *ctx, pdf_stream *stream_obj, byte **buf, int6
             goto exit;
     }
 
-    if (filtered || ctx->encryption.is_encrypted) {
-        code = pdfi_filter(ctx, stream_obj, ctx->main_stream, &stream, false);
-        if (code < 0) {
-            goto exit;
+retry:
+    if (ToRead == 0) {
+        if (filtered || ctx->encryption.is_encrypted) {
+            code = pdfi_apply_SubFileDecode_filter(ctx, 0, "endstream", ctx->main_stream, &SubFileStream, false);
+            if (code < 0)
+                goto exit;
+            code = pdfi_filter(ctx, stream_obj, SubFileStream, &stream, false);
+            if (code < 0) {
+                pdfi_close_file(ctx, SubFileStream);
+                goto exit;
+            }
+            while (seofp(stream->s) != true && serrorp(stream->s) != true) {
+                s_process_read_buf(stream->s);
+                buflen += sbufavailable(stream->s);
+                (void)sbufskip(stream->s, sbufavailable(stream->s));
+            }
+            pdfi_close_file(ctx, stream);
+            pdfi_close_file(ctx, SubFileStream);
+        } else {
+            buflen = pdfi_stream_length(ctx, stream_obj);
         }
-        while (seofp(stream->s) != true && serrorp(stream->s) != true) {
-            (void)sbufskip(stream->s, sbufavailable(stream->s));
-            s_process_read_buf(stream->s);
-            buflen += sbufavailable(stream->s);
-        }
-        pdfi_close_file(ctx, stream);
-    } else {
-        buflen = pdfi_stream_length(ctx, stream_obj);
-    }
+    } else
+        buflen = *bufferlen;
 
     /* Alloc buffer */
     Buffer = gs_alloc_bytes(ctx->memory, buflen, "pdfi_stream_to_buffer (Buffer)");
@@ -1693,36 +1678,109 @@ pdfi_stream_to_buffer(pdf_context *ctx, pdf_stream *stream_obj, byte **buf, int6
         code = gs_note_error(gs_error_VMerror);
         goto exit;
     }
+
     code = pdfi_seek(ctx, ctx->main_stream, pdfi_stream_offset(ctx, stream_obj), SEEK_SET);
-    if (code < 0)
+    if (code < 0) {
+        buflen = 0;
         goto exit;
+    }
     if (filtered || ctx->encryption.is_encrypted) {
-        code = pdfi_filter(ctx, stream_obj, ctx->main_stream, &stream, false);
+        if (ToRead && stream_obj->length_valid)
+            code = pdfi_apply_SubFileDecode_filter(ctx, stream_obj->Length, NULL, ctx->main_stream, &SubFileStream, false);
+        else
+            code = pdfi_apply_SubFileDecode_filter(ctx, 0, "endstream", ctx->main_stream, &SubFileStream, false);
         if (code < 0)
             goto exit;
-        sfread(Buffer, 1, buflen, stream->s);
+
+        code = pdfi_filter(ctx, stream_obj, SubFileStream, &stream, false);
+        if (code < 0) {
+            /* Because we opened the SubFileDecode separately to the filter chain, we need to close it separately too */
+            pdfi_close_file(ctx, SubFileStream);
+            goto exit;
+        }
+
+        code = sgets(stream->s, Buffer, buflen, (unsigned int *)&read);
+        if (read < buflen) {
+            memset(Buffer + read, 0x00, buflen - read);
+        }
+
         pdfi_close_file(ctx, stream);
+        /* Because we opened the SubFileDecode separately to the filter chain, we need to close it separately too */
+        pdfi_close_file(ctx, SubFileStream);
+        if (code == ERRC || code == EOFC) {
+            code = 0;
+            /* Error reading the expected number of bytes. If we already calculated the number of
+             * bytes in the loop above, then ignore the error and carry on. If, however, we were
+             * told how many bytes to expect, and failed to read that many, go back and do this
+             * the slow way to determine how many bytes are *really* available.
+             */
+            if(ToRead != 0) {
+                buflen = ToRead = 0;
+                code = pdfi_seek(ctx, ctx->main_stream, pdfi_stream_offset(ctx, stream_obj), SEEK_SET);
+                if (code < 0)
+                    goto exit;
+                gs_free_object(ctx->memory, Buffer, "pdfi_stream_to_buffer (Buffer)");
+                goto retry;
+            }
+        }
     } else {
-        sfread(Buffer, 1, buflen, ctx->main_stream->s);
+        if (ToRead && stream_obj->length_valid)
+            code = pdfi_apply_SubFileDecode_filter(ctx, stream_obj->Length, NULL, ctx->main_stream, &SubFileStream, false);
+        else
+            code = pdfi_apply_SubFileDecode_filter(ctx, ToRead, "endstream", ctx->main_stream, &SubFileStream, false);
+        if (code < 0)
+            goto exit;
+
+        code = sgets(SubFileStream->s, Buffer, buflen, (unsigned int *)&read);
+        if (read < buflen) {
+            memset(Buffer + read, 0x00, buflen - read);
+        }
+
+        pdfi_close_file(ctx, SubFileStream);
+        if (code == ERRC || code == EOFC) {
+            code = 0;
+            /* Error reading the expected number of bytes. If we already calculated the number of
+             * bytes in the loop above, then ignore the error and carry on. If, however, we were
+             * told how many bytes to expect, and failed to read that many, go back and do this
+             * the slow way to determine how many bytes are *really* available.
+             */
+            buflen = ToRead = 0;
+            /* Setting filtered to true is a lie, but it forces the code to go through the slow path and check the *real* number of bytes
+             * in the stream. This will be slow, but it should only happen when we get a file which is invalid.
+             */
+            filtered = 1;
+            code = pdfi_seek(ctx, ctx->main_stream, pdfi_stream_offset(ctx, stream_obj), SEEK_SET);
+            if (code < 0)
+                goto exit;
+            gs_free_object(ctx->memory, Buffer, "pdfi_stream_to_buffer (Buffer)");
+            goto retry;
+        }
     }
 
  exit:
     pdfi_seek(ctx, ctx->main_stream, savedoffset, SEEK_SET);
     if (Buffer && code < 0)
         gs_free_object(ctx->memory, Buffer, "pdfi_stream_to_buffer (Buffer)");
-    *buf = Buffer;
+    else
+        *buf = Buffer;
     *bufferlen = buflen;
     return code;
 }
 
-int pdfi_open_resource_file(pdf_context *ctx, const char *fname, const int fnamelen, stream **s)
+static int pdfi_open_resource_file_inner(pdf_context *ctx, const char *fname, const int fnamelen, stream **s)
 {
     int code = 0;
-    if (fname == NULL || fnamelen == 0)
+    if (fname == NULL || fnamelen == 0 || fnamelen >= gp_file_name_sizeof)
         *s = NULL;
     else if (gp_file_name_is_absolute(fname, fnamelen) || fname[0] == '%') {
+        char CFileName[gp_file_name_sizeof];
+
+        if (fnamelen + 1 > gp_file_name_sizeof)
+            return_error(gs_error_ioerror);
+        memcpy(CFileName, fname, fnamelen);
+        CFileName[fnamelen] = 0x00;
         /* If it's an absolute path or an explicit PS style device, just try to open it */
-        *s = sfopen(fname, "r", ctx->memory);
+        *s = sfopen(CFileName, "r", ctx->memory);
     }
     else {
         char fnametotry[gp_file_name_sizeof];
@@ -1782,17 +1840,38 @@ retry:
     return 0;
 }
 
-int pdfi_open_font_file(pdf_context *ctx, const char *fname, const int fnamelen, stream **s)
+int pdfi_open_resource_file(pdf_context *ctx, const char *fname, const int fnamelen, stream **s)
+{
+    return pdfi_open_resource_file_inner(ctx, fname, fnamelen, s);
+}
+
+bool pdfi_resource_file_exists(pdf_context *ctx, const char *fname, const int fnamelen)
+{
+    stream *s = NULL;
+    int code = pdfi_open_resource_file_inner(ctx, fname, fnamelen, &s);
+    if (s)
+        sfclose(s);
+
+    return (code >= 0);
+}
+
+static int pdfi_open_font_file_inner(pdf_context *ctx, const char *fname, const int fnamelen, stream **s)
 {
     int code = 0;
     const char *fontdirstr = "Font/";
     const int fontdirstrlen = strlen(fontdirstr);
 
-    if (fname == NULL || fnamelen == 0)
+    if (fname == NULL || fnamelen == 0 || fnamelen >= (gp_file_name_sizeof - fontdirstrlen))
         *s = NULL;
     else if (gp_file_name_is_absolute(fname, fnamelen) || fname[0] == '%') {
+        char CFileName[gp_file_name_sizeof];
+
+        if (fnamelen + 1 > gp_file_name_sizeof)
+            return_error(gs_error_ioerror);
+        memcpy(CFileName, fname, fnamelen);
+        CFileName[fnamelen] = 0x00;
         /* If it's an absolute path or an explicit PS style device, just try to open it */
-        *s = sfopen(fname, "r", ctx->memory);
+        *s = sfopen(CFileName, "r", ctx->memory);
     }
     else {
         char fnametotry[gp_file_name_sizeof];
@@ -1846,7 +1925,22 @@ int pdfi_open_font_file(pdf_context *ctx, const char *fname, const int fnamelen,
         }
     }
     if (*s == NULL)
-        return pdfi_open_resource_file(ctx, fname, fnamelen, s);
+        return pdfi_open_resource_file_inner(ctx, fname, fnamelen, s);
 
     return 0;
 }
+
+int pdfi_open_font_file(pdf_context *ctx, const char *fname, const int fnamelen, stream **s)
+{
+    return pdfi_open_font_file_inner(ctx, fname, fnamelen, s);
+}
+
+bool pdfi_font_file_exists(pdf_context *ctx, const char *fname, const int fnamelen)
+{
+    stream *s = NULL;
+    int code = pdfi_open_font_file_inner(ctx, fname, fnamelen, &s);
+    if (s)
+        sfclose(s);
+
+    return (code >= 0);
+}
