@@ -546,6 +546,7 @@ static int pdfi_type1_add_to_native_map(pdf_context *ctx, stream *f, char *fname
     char *typestr;
     bool pin_eol = false; /* initialised just to placate coverity */
     int type = -1;
+    int lines = 0;
     buf.data = (byte *)pname;
     buf.size = pname_size;
 
@@ -553,7 +554,10 @@ static int pdfi_type1_add_to_native_map(pdf_context *ctx, stream *f, char *fname
        /FontType and /FontName keys start in column 0 of their lines
      */
     while ((code = sreadline(f, NULL, NULL, NULL, &buf, NULL, &count, &pin_eol, NULL)) >= 0) {
-        if (buf.size > 9 && memcmp(buf.data, "/FontName", 9) == 0) {
+        lines++;
+        if (lines > 100 || (buf.size >= 17 && memcmp(buf.data, "currentfile eexec", 17) == 0))
+            break;
+        else if (buf.size > 9 && memcmp(buf.data, "/FontName", 9) == 0) {
             namestr = (char *)buf.data + 9;
             while (pdfi_end_ps_token(*namestr))
                 namestr++;
@@ -594,8 +598,6 @@ static int pdfi_type1_add_to_native_map(pdf_context *ctx, stream *f, char *fname
                 }
             }
         }
-        else if (buf.size >= 17 && memcmp(buf.data, "currentfile eexec", 17) == 0)
-            break;
         count = 0;
     }
     if (type == 1 && namestr != NULL) {
@@ -912,9 +914,10 @@ static int pdfi_add_font_to_native_map(pdf_context *ctx, const char *fp, char *w
               code = gs_error_undefined;
           break;
         case type1_font:
-        default:
           code = pdfi_type1_add_to_native_map(ctx, sf, (char *)fp, working, gp_file_name_sizeof);
           break;
+        default:
+          break;
     }
     sfclose(sf);
     /* We ignore most errors, on the basis it probably means it wasn't a valid font file */
@@ -964,13 +967,11 @@ done:
 static int pdfi_generate_native_fontmap(pdf_context *ctx)
 {
     file_enum *fe;
-    int i, j;
+    int i;
     char *patrn= NULL;
     char *result = NULL;
     char *working = NULL;
     int code = 0, l;
-    gs_param_string *respaths[2];
-    int nrespaths[2];
 
     if (ctx->pdfnativefontmap != NULL) /* Only run this once */
         return 0;
@@ -995,33 +996,26 @@ static int pdfi_generate_native_fontmap(pdf_context *ctx)
         return_error(gs_error_VMerror);
     }
 
-    respaths[0] = ctx->search_paths.font_paths;
-    nrespaths[0] = ctx->search_paths.num_font_paths;
-    respaths[1] = ctx->search_paths.resource_paths;
-    nrespaths[1] = ctx->search_paths.num_resource_paths;
+    for (i = 0; i < ctx->search_paths.num_font_paths; i++) {
 
-    for (j = 0; j < sizeof(respaths) / sizeof(respaths[0]); j++) {
-        for (i = 0; i < nrespaths[j]; i++) {
+        memcpy(patrn, ctx->search_paths.font_paths[i].data, ctx->search_paths.font_paths[i].size);
+        memcpy(patrn + ctx->search_paths.font_paths[i].size, "/*", 2);
+        patrn[ctx->search_paths.font_paths[i].size + 2] = '\0';
 
-            memcpy(patrn, respaths[j][i].data, respaths[j][i].size);
-            memcpy(patrn + respaths[j][i].size, "/*", 2);
-            patrn[respaths[j][i].size + 2] = '\0';
+        fe = gp_enumerate_files_init(ctx->memory, (const char *)patrn, strlen(patrn));
+        while ((l = gp_enumerate_files_next(ctx->memory, fe, result, gp_file_name_sizeof - 1)) != ~(uint) 0) {
+            result[l] = '\0';
 
-            fe = gp_enumerate_files_init(ctx->memory, (const char *)patrn, strlen(patrn));
-            while ((l = gp_enumerate_files_next(ctx->memory, fe, result, gp_file_name_sizeof - 1)) != ~(uint) 0) {
-                result[l] = '\0';
+            code = pdfi_add_font_to_native_map(ctx, result, working);
 
-                code = pdfi_add_font_to_native_map(ctx, result, working);
-
-                /* We ignore most errors, on the basis it probably means it wasn't a valid font file */
-                if (code == gs_error_VMerror)
-                    break;
-                code = 0;
-            }
-            /* We only need to explicitly destroy the enumerator if we exit before enumeration is complete */
-            if (code < 0)
-                gp_enumerate_files_close(ctx->memory, fe);
+            /* We ignore most errors, on the basis it probably means it wasn't a valid font file */
+            if (code == gs_error_VMerror)
+                break;
+            code = 0;
         }
+        /* We only need to explicitly destroy the enumerator if we exit before enumeration is complete */
+        if (code < 0)
+            gp_enumerate_files_close(ctx->memory, fe);
     }
 
 #ifdef DUMP_NATIVE_FONTMAP
