@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2022 Artifex Software, Inc.
+/* Copyright (C) 2019-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Include this first so that we don't get a macro redefnition of 'offsetof' */
@@ -34,35 +34,6 @@
 
 #include "gsutil.h"        /* For gs_next_ids() */
 
-extern const pdfi_cid_decoding_t *pdfi_cid_decoding_list[];
-extern const pdfi_cid_subst_nwp_table_t *pdfi_cid_substnwp_list[];
-
-static void pdfi_font0_cid_subst_tables(const char *reg, const int reglen, const char *ord,
-                const int ordlen, pdfi_cid_decoding_t **decoding, pdfi_cid_subst_nwp_table_t **substnwp)
-{
-    int i;
-    *decoding = NULL;
-    *substnwp = NULL;
-    /* This only makes sense for Adobe orderings */
-    if (reglen == 5 && !memcmp(reg, "Adobe", 5)) {
-        for (i = 0; pdfi_cid_decoding_list[i] != NULL; i++) {
-            if (strlen(pdfi_cid_decoding_list[i]->s_order) == ordlen &&
-                !memcmp(pdfi_cid_decoding_list[i]->s_order, ord, ordlen)) {
-                *decoding = (pdfi_cid_decoding_t *)pdfi_cid_decoding_list[i];
-                break;
-            }
-        }
-        /* For now, also only for Adobe orderings */
-        for (i = 0; pdfi_cid_substnwp_list[i] != NULL; i++) {
-            if (strlen(pdfi_cid_substnwp_list[i]->ordering) == ordlen &&
-                !memcmp(pdfi_cid_substnwp_list[i]->ordering, ord, ordlen)) {
-                *substnwp = (pdfi_cid_subst_nwp_table_t *)pdfi_cid_substnwp_list[i];
-                break;
-            }
-        }
-    }
-}
-
 static int
 pdfi_font0_glyph_name(gs_font *pfont, gs_glyph index, gs_const_string *pstr)
 {
@@ -198,6 +169,15 @@ int pdfi_read_type0_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream
     /* We're supposed to have a FontDescriptor, it can be missing, and we have to carry on */
     (void)pdfi_dict_get(ctx, font_dict, "FontDescriptor", (pdf_obj **)&fontdesc);
 
+    if (fontdesc != NULL) {
+        pdf_obj *Name = NULL;
+
+        code = pdfi_dict_get_type(ctx, (pdf_dict *) fontdesc, "FontName", PDF_NAME, (pdf_obj**)&Name);
+        if (code < 0)
+            pdfi_set_warning(ctx, 0, NULL, W_PDF_FDESC_BAD_FONTNAME, "pdfi_load_font", "");
+        pdfi_countdown(Name);
+    }
+
     code = pdfi_dict_get(ctx, font_dict, "Encoding", &cmap);
     if (code < 0) goto error;
 
@@ -326,11 +306,10 @@ int pdfi_read_type0_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream
             olen = pcmap->csi_ord.size;
         }
         if (rlen > 0 && olen > 0)
-            pdfi_font0_cid_subst_tables(r, rlen, o, olen, &dec, &substnwp);
-        else {
-            dec = NULL;
-            substnwp = NULL;
-        }
+            pdfi_cidfont_cid_subst_tables(r, rlen, o, olen, &dec, &substnwp);
+
+        ((pdf_cidfont_t *)descpfont)->decoding = dec;
+        ((pdf_cidfont_t *)descpfont)->substnwp = substnwp;
     }
     /* reference is now owned by the descendent font created above */
     pdfi_countdown(decfontdict);
@@ -345,6 +324,10 @@ int pdfi_read_type0_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream
         code = gs_note_error(gs_error_VMerror);
         goto error;
     }
+
+    /* This is the only font type these entries not relevant for */
+    pdft0->copyright = pdft0->notice = pdft0->fullname = pdft0->familyname = NULL;
+
     code = pdfi_array_alloc(ctx, 1, &arr);
     if (code < 0) {
         gs_free_object(ctx->memory, pdft0, "pdfi_read_type0_font(pdft0)");
@@ -513,6 +496,8 @@ int pdfi_read_type0_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream
     return 0;
 
 error:
+    (void)pdfi_set_error_var(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_type0_font", "Error reading embedded Type0 font object %u\n", font_dict->object_num);
+
     pdfi_countdown(arr);
     pdfi_countdown(pcmap);
     pdfi_countdown(tounicode);
@@ -539,6 +524,10 @@ pdfi_free_font_type0(pdf_obj *font)
     pdfi_countdown(pdft0->DescendantFonts);
     pdfi_countdown(pdft0->ToUnicode);
     pdfi_countdown(pdft0->filename);
+    pdfi_countdown(pdft0->copyright);
+    pdfi_countdown(pdft0->notice);
+    pdfi_countdown(pdft0->fullname);
+    pdfi_countdown(pdft0->familyname);
 
     gs_free_object(OBJ_MEMORY(pdft0), pfont0->data.Encoding, "pdfi_free_font_type0(data.Encoding)");
     /* We shouldn't need to free the fonts in the FDepVector, that should happen
