@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2022 Artifex Software, Inc.
+/* Copyright (C) 2019-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* code for type 1 font handling */
@@ -471,8 +471,8 @@ pdfi_alloc_t1_font(pdf_context *ctx, pdf_font_type1 **font, uint32_t obj_num)
     pfont->id = gs_next_ids(ctx->memory, 1);
     uid_set_UniqueID(&pfont->UID, pfont->id);
 
-    pfont->encoding_index = 1;          /****** WRONG ******/
-    pfont->nearest_encoding_index = 1;          /****** WRONG ******/
+    pfont->encoding_index = ENCODING_INDEX_UNKNOWN;
+    pfont->nearest_encoding_index = ENCODING_INDEX_UNKNOWN;
 
     pfont->client_data = (void *)t1font;
 
@@ -494,7 +494,10 @@ pdfi_t1_font_set_procs(pdf_context *ctx, pdf_font_type1 *font)
     pfont->procs.decode_glyph = pdfi_decode_glyph;
     pfont->procs.define_font = gs_no_define_font;
     pfont->procs.make_font = gs_no_make_font;
-    pfont->procs.font_info = gs_default_font_info;
+
+    font->default_font_info = gs_default_font_info;
+    pfont->procs.font_info = pdfi_default_font_info;
+
     pfont->procs.glyph_info = pdfi_t1_glyph_info;
     pfont->procs.glyph_outline = pdfi_t1_glyph_outline;
     pfont->procs.same_font = gs_default_same_font;
@@ -508,6 +511,17 @@ pdfi_t1_font_set_procs(pdf_context *ctx, pdf_font_type1 *font)
     pfont->data.interpret = gs_type1_interpret;
 }
 
+static inline void
+pdfi_type1_font_priv_defaults(ps_font_interp_private *pfpriv)
+{
+    pfpriv->gsu.gst1.data.lenIV = 4;
+    pfpriv->gsu.gst1.data.ExpansionFactor = 0.06;
+    pfpriv->gsu.gst1.data.BlueShift = 7;
+    pfpriv->gsu.gst1.data.BlueFuzz = 1;
+    pfpriv->gsu.gst1.data.BlueScale = 0.039625;
+    uid_set_invalid(&pfpriv->gsu.gst1.UID);
+}
+
 int
 pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dict, pdf_dict *page_dict, byte *fbuf, int64_t fbuflen, pdf_font **ppdffont)
 {
@@ -521,6 +535,7 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
     pdf_obj *tounicode = NULL;
     ps_font_interp_private fpriv = { 0 };
     bool key_known;
+    bool force_symbolic = false;
 
     if (font_dict != NULL)
         (void)pdfi_dict_knownget_type(ctx, font_dict, "FontDescriptor", PDF_DICT, &fontdesc);
@@ -539,7 +554,7 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
     }
 
     if (code >= 0) {
-        fpriv.gsu.gst1.data.lenIV = 4;
+        pdfi_type1_font_priv_defaults(&fpriv);
         code = pdfi_read_ps_font(ctx, font_dict, fbuf, fbuflen, &fpriv);
         gs_free_object(ctx->memory, fbuf, "pdfi_read_type1_font");
 
@@ -565,7 +580,13 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
             if (fpriv.gsu.gst1.UID.id != 0)
                 memcpy(&pfont1->UID, &fpriv.gsu.gst1.UID, sizeof(pfont1->UID));
             fpriv.gsu.gst1.UID.xvalues = NULL; /* In case of error */
-            pfont1->WMode = fpriv.gsu.gst1.WMode;
+            if (fpriv.gsu.gst1.WMode != 0) {
+                if (fpriv.gsu.gst1.WMode != 1)
+                    pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_WMODE, "pdfi_read_type1_font", NULL);
+                pfont1->WMode = 1;
+            }
+            else
+                pfont1->WMode = 0;
             pfont1->PaintType = fpriv.gsu.gst1.PaintType;
             pfont1->StrokeWidth = fpriv.gsu.gst1.StrokeWidth;
 
@@ -590,7 +611,6 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
                 pdfi_countup(basefont);
             }
 
-            t1f->descflags = 0;
             if (t1f->FontDescriptor != NULL) {
                 code = pdfi_dict_get_int(ctx, t1f->FontDescriptor, "Flags", &t1f->descflags);
                 if (code >= 0) {
@@ -603,6 +623,7 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
             }
 
             if (pdfi_font_known_symbolic(basefont)) {
+                force_symbolic = true;
                 t1f->descflags |= 4;
             }
 
@@ -643,22 +664,22 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
             else
                 code = gs_error_undefined;
             if (code == 1) {
-                if ((pdfi_type_of(tmp) == PDF_NAME || pdfi_type_of(tmp) == PDF_DICT) && (t1f->descflags & 4) == 0) {
-                    code = pdfi_create_Encoding(ctx, tmp, NULL, (pdf_obj **) & t1f->Encoding);
-                    if (code >= 0)
-                        code = 1;
+                if (pdfi_type_of(tmp) == PDF_NAME && force_symbolic == true) {
+                    t1f->Encoding = fpriv.u.t1.Encoding;
+                    pdfi_countup(t1f->Encoding);
                 }
                 else if (pdfi_type_of(tmp) == PDF_DICT && (t1f->descflags & 4) != 0) {
-                    code = pdfi_create_Encoding(ctx, tmp, (pdf_obj *)fpriv.u.t1.Encoding, (pdf_obj **) & t1f->Encoding);
+                    code = pdfi_create_Encoding(ctx, (pdf_font *)t1f, tmp, (pdf_obj *)fpriv.u.t1.Encoding, (pdf_obj **) & t1f->Encoding);
+                    if (code >= 0)
+                        code = 1;
+                }
+                else {
+                    code = pdfi_create_Encoding(ctx, (pdf_font *)t1f, tmp, NULL, (pdf_obj **) & t1f->Encoding);
                     if (code >= 0)
                         code = 1;
                 }
-                else
-                    code = gs_error_undefined;
                 pdfi_countdown(tmp);
                 tmp = NULL;
-                if (code == 1) {
-                }
             }
             else {
                 pdfi_countdown(tmp);
@@ -687,6 +708,12 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
             t1f->Subrs = fpriv.u.t1.Subrs;
             fpriv.u.t1.Subrs = NULL;
 
+            t1f->copyright = fpriv.u.t1.copyright;
+            t1f->notice = fpriv.u.t1.notice;
+            t1f->fullname = fpriv.u.t1.fullname;
+            t1f->familyname = fpriv.u.t1.familyname;
+            fpriv.u.t1.copyright = fpriv.u.t1.notice = fpriv.u.t1.fullname = fpriv.u.t1.familyname = NULL;
+
             code = pdfi_font_generate_pseudo_XUID(ctx, font_dict, t1f->pfont);
             if (code < 0) {
                 goto error;
@@ -708,11 +735,15 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
                      code = pdfi_dict_known(ctx, t1f->FontDescriptor, "FontFile2", &key_known);
                      if (code < 0 || key_known == false) {
                          code = pdfi_dict_known(ctx, t1f->FontDescriptor, "FontFile3", &key_known);
+                         if (code < 0) {
+                             key_known = false;
+                         }
                      }
                  }
             }
             t1f->pfont->is_resource = (key_known == false);
 
+            pdfi_font_set_orig_fonttype(ctx, (pdf_font *)t1f);
             code = gs_definefont(ctx->font_dir, (gs_font *) t1f->pfont);
             if (code < 0) {
                 goto error;
@@ -743,11 +774,35 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
     pdfi_countdown(fpriv.u.t1.blendfontbbox);
     pdfi_countdown(fpriv.u.t1.blendaxistypes);
     pdfi_countdown(fpriv.u.t1.Subrs);
+    pdfi_countdown(fpriv.u.t1.copyright);
+    pdfi_countdown(fpriv.u.t1.notice);
+    pdfi_countdown(fpriv.u.t1.fullname);
+    pdfi_countdown(fpriv.u.t1.familyname);
     if (fpriv.gsu.gst1.UID.xvalues != NULL) {
         gs_free_object(ctx->memory, fpriv.gsu.gst1.UID.xvalues, "pdfi_read_type1_font(xuid)");
     }
 
     if (code < 0) {
+        tmp = NULL;
+        if (font_dict != NULL) {
+            if (pdfi_dict_get(ctx, font_dict, ".Path", &tmp) >= 0)
+            {
+                char fname[gp_file_name_sizeof + 1];
+                pdf_string *fobj = (pdf_string *)tmp;
+
+                memcpy(fname, fobj->data, fobj->length > gp_file_name_sizeof ? gp_file_name_sizeof : fobj->length);
+                fname[fobj->length > gp_file_name_sizeof ? gp_file_name_sizeof : fobj->length] = '\0';
+
+                (void)pdfi_set_error_var(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_type1_font", "Error reading Type 1 font file %s\n", fname);
+            }
+            else {
+                (void)pdfi_set_error_var(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_type1_font", "Error reading embedded Type 1 font object %u\n", font_dict->object_num);
+            }
+        }
+        else {
+            pdfi_set_error(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_truetype_font", "Error reading font\n");
+        }
+        pdfi_countdown(tmp);
         pdfi_countdown(t1f);
     }
     return code;
@@ -762,6 +817,7 @@ pdfi_copy_type1_font(pdf_context *ctx, pdf_font *spdffont, pdf_dict *font_dict,
     gs_font_type1 *dpfont1;
     gs_id t_id;
     pdf_obj *tmp;
+    bool force_symbolic = false;
 
     if (font_dict == NULL)
         return_error(gs_error_invalidfont);
@@ -813,6 +869,10 @@ pdfi_copy_type1_font(pdf_context *ctx, pdf_font *spdffont, pdf_dict *font_dict,
     pdfi_countup(font->blendfontbbox);
     pdfi_countup(font->blendaxistypes);
     pdfi_countup(font->Subrs);
+    pdfi_countup(font->copyright);
+    pdfi_countup(font->notice);
+    pdfi_countup(font->fullname);
+    pdfi_countup(font->familyname);
 
     if (font->BaseFont != NULL && ((pdf_name *)font->BaseFont)->length <= gs_font_name_max - 1) {
         memcpy(dpfont1->key_name.chars, ((pdf_name *)font->BaseFont)->data, ((pdf_name *)font->BaseFont)->length);
@@ -843,24 +903,27 @@ pdfi_copy_type1_font(pdf_context *ctx, pdf_font *spdffont, pdf_dict *font_dict,
     }
 
     if (pdfi_font_known_symbolic(font->BaseFont)) {
+        force_symbolic = true;
         font->descflags |= 4;
     }
 
     tmp = NULL;
     code = pdfi_dict_knownget(ctx, font_dict, "Encoding", &tmp);
     if (code == 1) {
-        if ((pdfi_type_of(tmp) == PDF_NAME || pdfi_type_of(tmp) == PDF_DICT) && (font->descflags & 4) == 0) {
-            code = pdfi_create_Encoding(ctx, tmp, NULL, (pdf_obj **) & font->Encoding);
-            if (code >= 0)
-                code = 1;
+        if (pdfi_type_of(tmp) == PDF_NAME && force_symbolic == true) {
+            font->Encoding = spdffont->Encoding;
+            pdfi_countup(font->Encoding);
         }
         else if (pdfi_type_of(tmp) == PDF_DICT && (font->descflags & 4) != 0) {
-            code = pdfi_create_Encoding(ctx, tmp, (pdf_obj *)spdffont->Encoding, (pdf_obj **) &font->Encoding);
+            code = pdfi_create_Encoding(ctx, (pdf_font *)font, tmp, (pdf_obj *)spdffont->Encoding, (pdf_obj **) &font->Encoding);
+            if (code >= 0)
+                code = 1;
+        }
+        else {
+            code = pdfi_create_Encoding(ctx, (pdf_font *)font, tmp, NULL, (pdf_obj **) & font->Encoding);
             if (code >= 0)
                 code = 1;
         }
-        else
-            code = gs_error_undefined;
         pdfi_countdown(tmp);
         tmp = NULL;
     }
@@ -875,13 +938,10 @@ pdfi_copy_type1_font(pdf_context *ctx, pdf_font *spdffont, pdf_dict *font_dict,
         pdfi_countup(font->Encoding);
     }
 
-    /* Since various aspects of the font may differ (widths, encoding, etc)
-       we cannot reliably use the UniqueID/XUID for copied fonts.
-     */
-    if (uid_is_XUID(&font->pfont->UID))
-        uid_free(&font->pfont->UID, font->pfont->memory, "pdfi_read_type1_font");
-    uid_set_invalid(&font->pfont->UID);
-
+    code = uid_copy(&font->pfont->UID, font->pfont->memory, "pdfi_copy_type1_font");
+    if (code < 0) {
+        uid_set_invalid(&font->pfont->UID);
+    }
     code = pdfi_font_generate_pseudo_XUID(ctx, font_dict, font->pfont);
     if (code < 0) {
         goto error;
@@ -906,6 +966,7 @@ pdfi_copy_type1_font(pdf_context *ctx, pdf_font *spdffont, pdf_dict *font_dict,
     }
     font->ToUnicode = tmp;
 
+    pdfi_font_set_orig_fonttype(ctx, (pdf_font *)font);
     code = gs_definefont(ctx->font_dir, (gs_font *) font->pfont);
     if (code < 0) {
         goto error;
@@ -949,6 +1010,10 @@ pdfi_free_font_type1(pdf_obj *font)
     pdfi_countdown(t1f->blendaxistypes);
     pdfi_countdown(t1f->Subrs);
     pdfi_countdown(t1f->filename);
+    pdfi_countdown(t1f->copyright);
+    pdfi_countdown(t1f->notice);
+    pdfi_countdown(t1f->fullname);
+    pdfi_countdown(t1f->familyname);
 
     gs_free_object(OBJ_MEMORY(font), t1f->Widths, "Free Type 1 fontWidths");
     gs_free_object(OBJ_MEMORY(font), t1f, "Free Type 1 font");
