@@ -1,4 +1,4 @@
-/* Copyright (C) 2020-2022 Artifex Software, Inc.
+/* Copyright (C) 2020-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* common code for Postscript-type font handling */
@@ -117,51 +117,6 @@ pdf_ps_end_number_object(int c)
     return (c != '.' && c != 'e' && c != '-' && (c < '0' || c > '9'));
 }
 
-static inline bool
-ishex(char c)
-{
-    if (c < 0x30)
-        return false;
-
-    if (c > 0x39) {
-        if (c > 'F') {
-            if (c < 'a')
-                return false;
-            if (c > 'f')
-                return false;
-            return true;
-        }
-        else {
-            if (c < 'A')
-                return false;
-            return true;
-        }
-    }
-    else
-        return true;
-}
-
-static inline int
-decodehex(char c)
-{
-    int retval = 0;
-
-    if (ishex(c)) {
-        if (c > 0x39) {
-            if (c > 'F') {
-                retval = c - 0x57;
-            }
-            else {
-                retval = c - 0x37;
-            }
-        }
-        else {
-            retval = c - 0x30;
-        }
-    }
-    return retval;
-}
-
 int
 pdfi_pscript_interpret(pdf_ps_ctx_t *cs, byte *pdfpsbuf, int64_t buflen)
 {
@@ -214,9 +169,9 @@ pdfi_pscript_interpret(pdf_ps_ctx_t *cs, byte *pdfpsbuf, int64_t buflen)
             case '<':          /* hex string */
                 {
                     byte *s = pdfpsbuf;
-                    byte *s2 = s;
-                    int len, i;
-                    byte hbuf[2];
+                    stream_cursor_read pr;
+                    stream_cursor_write pw;
+                    int odd_digit = -1;
 
                     if (pdfpsbuf < buflim && *pdfpsbuf == '<') { /* Dict opening "<<" - we don't care */
                         pdfpsbuf++;
@@ -224,16 +179,15 @@ pdfi_pscript_interpret(pdf_ps_ctx_t *cs, byte *pdfpsbuf, int64_t buflen)
                     }
                     while (pdfpsbuf < buflim && *pdfpsbuf != '>')
                         pdfpsbuf++;
-                    len = pdfpsbuf - s;
-                    while (len % 2)
-                        len--;
-                    for (i = 0; i < len; i += 2) {
-                        hbuf[0] = s[i];
-                        hbuf[1] = s[i + 1];
-                        *s2++ = (decodehex(hbuf[0]) << 4) | decodehex(hbuf[1]);
+
+                    pr.ptr = s - 1;
+                    pr.limit = pdfpsbuf - 1;
+                    pw.ptr = s - 1;
+                    pw.limit = pdfpsbuf - 1;
+                    code = s_hex_process(&pr, &pw, &odd_digit, hex_ignore_garbage);
+                    if (code != ERRC && pw.ptr - (s - 1) > 0) {
+                        code = pdf_ps_stack_push_string(cs, s, pw.ptr - (s - 1));
                     }
-                    pdfpsbuf++; /* move past the trailing '>' */
-                    code = pdf_ps_stack_push_string(cs, s, len >> 1);
                 }
                 break;
             case '>': /* For hex strings, this should be handled above */
@@ -414,7 +368,7 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
 
           case 4:
               if (pdf_ps_name_cmp(&s->cur[-1], "XUID")) {
-                  if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_ARRAY)) {
+                  if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_ARRAY) && uid_is_valid(&priv->gsu.gst1.UID) == false) {
                       int i, size = s->cur[0].size;
                       long *xvals = (long *)gs_alloc_bytes(mem, size *sizeof(long), "ps_font_def_func(xuid vals)");
 
@@ -466,7 +420,13 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
               }
               else if (pdf_ps_name_cmp(&s->cur[-1], "WMode")) {
                   if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_INTEGER)) {
-                      priv->gsu.gst1.WMode = s->cur[0].val.i;
+                      if (s->cur[0].val.i != 0) {
+                          if (s->cur[0].val.i != 1)
+                            pdfi_set_warning(s->pdfi_ctx, 0, NULL, W_PDF_BAD_WMODE, "ps_font_def_func", NULL);
+                        priv->gsu.gst1.WMode = 1;
+                      }
+                      else
+                        priv->gsu.gst1.WMode = 0;
                   }
               }
               else if (pdf_ps_name_cmp(&s->cur[-1], "lenIV")) {
@@ -476,6 +436,23 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
               }
               break;
 
+          case 6:
+              if (pdf_ps_name_cmp(&s->cur[-1], "Notice") && priv->u.t1.notice == NULL) {
+                  if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_STRING)) {
+                      pdf_string *subr_str;
+
+                      code = pdfi_object_alloc(s->pdfi_ctx, PDF_STRING, (unsigned int)s->cur[0].size, (pdf_obj **)&subr_str);
+                      if (code < 0) {
+                          return code;
+                      }
+                      pdfi_countup(subr_str);
+                      memcpy(subr_str->data, s->cur[0].val.name, s->cur[0].size);
+
+                      pdfi_countdown(priv->u.t1.notice);
+                      priv->u.t1.notice = subr_str;
+                  }
+              }
+              break;
           case 8:
               if (pdf_ps_name_cmp(&s->cur[-1], "FontName")) {
                   int fnlen = 0;
@@ -489,7 +466,7 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                       fnlen = s->cur[0].size > gs_font_name_max ? gs_font_name_max : s->cur[0].size;
                       pname = (char *)s->cur[0].val.string;
                   }
-                  if (pname) {
+                  if (pname && priv->gsu.gst1.key_name.chars[0] == '\0') {
                       memcpy(priv->gsu.gst1.key_name.chars, pname, fnlen);
                       priv->gsu.gst1.key_name.chars[fnlen] = '\0';
                       priv->gsu.gst1.key_name.size = fnlen;
@@ -583,7 +560,7 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                       if (code >= 0) {
                           pdfi_countup(pname);
 
-                          code = pdfi_create_Encoding(s->pdfi_ctx, (pdf_obj *) pname, NULL, (pdf_obj **) &new_enc);
+                          code = pdfi_create_Encoding(s->pdfi_ctx, NULL, (pdf_obj *) pname, NULL, (pdf_obj **) &new_enc);
                           if (code >= 0) {
                               pdfi_countdown(priv->u.t1.Encoding);
                               priv->u.t1.Encoding = new_enc;
@@ -622,7 +599,7 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                       }
                   }
               }
-              else if (pdf_ps_name_cmp(&s->cur[-1], "UniqueID")) {
+              else if (pdf_ps_name_cmp(&s->cur[-1], "UniqueID") && uid_is_valid(&priv->gsu.gst1.UID) == false) {
                   /* Ignore UniqueID if we already have a XUID */
                   if (priv->gsu.gst1.UID.id >= 0) {
                       if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_INTEGER)) {
@@ -630,6 +607,21 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                       }
                   }
               }
+              else if (pdf_ps_name_cmp(&s->cur[-1], "FullName")) {
+                  if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_STRING) && priv->u.t1.fullname == NULL) {
+                      pdf_string *subr_str;
+
+                      code = pdfi_object_alloc(s->pdfi_ctx, PDF_STRING, (unsigned int)s->cur[0].size, (pdf_obj **)&subr_str);
+                      if (code < 0) {
+                          return code;
+                      }
+                      pdfi_countup(subr_str);
+                      memcpy(subr_str->data, s->cur[0].val.name, s->cur[0].size);
+
+                      pdfi_countdown(priv->u.t1.fullname);
+                      priv->u.t1.fullname = subr_str;
+                  }
+              }
               break;
 
           case 9:
@@ -665,7 +657,7 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                               priv->gsu.gst1.data.StemSnapV.values[i] = s->cur[0].val.arr[i].val.f;
                           }
                       }
-                      priv->gsu.gst1.data.StemSnapH.count = size;
+                      priv->gsu.gst1.data.StemSnapV.count = size;
                   }
               }
               else if (pdf_ps_name_cmp(&s->cur[-1], "BlueScale")) {
@@ -676,11 +668,27 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                       priv->gsu.gst1.data.BlueScale = (float)s->cur[0].val.f;
                   }
               }
+              else if (pdf_ps_name_cmp(&s->cur[-1], "Copyright")) {
+                  if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_STRING)) {
+                      pdf_string *subr_str;
+
+                      code = pdfi_object_alloc(s->pdfi_ctx, PDF_STRING, (unsigned int)s->cur[0].size, (pdf_obj **)&subr_str);
+                      if (code < 0) {
+                          return code;
+                      }
+                      pdfi_countup(subr_str);
+                      memcpy(subr_str->data, s->cur[0].val.name, s->cur[0].size);
+
+                      pdfi_countdown(priv->u.t1.copyright);
+                      priv->u.t1.copyright = subr_str;
+                  }
+              }
               break;
 
           case 10:
               if (pdf_ps_name_cmp(&s->cur[-1], "FontMatrix")) {
-                  if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_ARRAY) && s->cur[0].size >= 6) {
+                  if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_ARRAY) && s->cur[0].size >= 6
+                    && (priv->gsu.gst1.FontMatrix.xx * priv->gsu.gst1.FontMatrix.yy - priv->gsu.gst1.FontMatrix.yx * priv->gsu.gst1.FontMatrix.xy == 0)) {
                       int i;
                       double fmat[6] = { 0.001, 0, 0, 0.001, 0, 0 };
                       for (i = 0; i < 6; i++) {
@@ -722,6 +730,21 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                       priv->gsu.gst1.data.BlueValues.count = size;
                   }
               }
+              else if (pdf_ps_name_cmp(&s->cur[-1], "FamilyName")) {
+                  if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_STRING) && priv->u.t1.familyname == NULL) {
+                      pdf_string *subr_str;
+
+                      code = pdfi_object_alloc(s->pdfi_ctx, PDF_STRING, (unsigned int)s->cur[0].size, (pdf_obj **)&subr_str);
+                      if (code < 0) {
+                          return code;
+                      }
+                      pdfi_countup(subr_str);
+                      memcpy(subr_str->data, s->cur[0].val.name, s->cur[0].size);
+
+                      pdfi_countdown(priv->u.t1.familyname);
+                      priv->u.t1.familyname = subr_str;
+                  }
+              }
               break;
 
           case 11:
@@ -737,8 +760,9 @@ ps_font_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
           case 12:
               if (pdf_ps_name_cmp(&s->cur[-1], "WeightVector")) {
                   if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_ARRAY)) {
-                      int i;
-                      for (i = 0; i < s->cur[0].size; i++) {
+                      int i, size = s->cur[0].size > 16 ? 16 : s->cur[0].size;
+
+                      for (i = 0; i < size; i++) {
                           if (pdf_ps_obj_has_type(&s->cur[0].val.arr[i], PDF_PS_OBJ_INTEGER)) {
                               priv->gsu.gst1.data.WeightVector.values[i] = (float)s->cur[0].val.arr[i].val.i;
                           }
@@ -1153,6 +1177,9 @@ pdf_ps_RD_oper_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                 int inx = s->cur[-1].val.i;
 
                 size = s->cur[0].val.i;
+                if (size < 0)
+                    return_error(gs_error_invalidfont);
+
                 buf++;
                 if (buf + size < bufend) {
                     pdf_string *subr_str;
@@ -1177,6 +1204,9 @@ pdf_ps_RD_oper_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
                 pdf_obj *key = NULL;
 
                 size = s->cur[0].val.i;
+                if (size < 0)
+                    return_error(gs_error_invalidfont);
+
                 buf++;
                 code = pdfi_name_alloc(s->pdfi_ctx, (byte *) s->cur[-1].val.name, s->cur[-1].size, &key);
                 if (code < 0) {
@@ -1236,6 +1266,12 @@ pdf_ps_put_oper_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
     return code;
 }
 
+static int
+pdf_ps_closefile_oper_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *bufend)
+{
+    return bufend - buf;
+}
+
 static pdf_ps_oper_list_t ps_font_oper_list[] = {
     {PDF_PS_OPER_NAME_AND_LEN("RD"), pdf_ps_RD_oper_func},
     {PDF_PS_OPER_NAME_AND_LEN("-|"), pdf_ps_RD_oper_func},
@@ -1260,6 +1296,7 @@ static pdf_ps_oper_list_t ps_font_oper_list[] = {
     {PDF_PS_OPER_NAME_AND_LEN("for"), pdf_ps_pop4_oper_func},
     {PDF_PS_OPER_NAME_AND_LEN("put"), pdf_ps_put_oper_func},
     {PDF_PS_OPER_NAME_AND_LEN("StandardEncoding"), pdf_ps_standardencoding_oper_func},
+    {PDF_PS_OPER_NAME_AND_LEN("closefile"), pdf_ps_closefile_oper_func},
     {NULL, 0, NULL}
 };
 
