@@ -1,4 +1,4 @@
-/* Copyright (C) 2018-2022 Artifex Software, Inc.
+/* Copyright (C) 2018-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* The PDF interpreter written in C */
@@ -68,40 +68,6 @@ static bool isdelimiter(char c)
         return false;
 }
 
-static bool ishex(char c)
-{
-    if (c < 0x30)
-        return false;
-
-    if (c > 0x39) {
-        if (c > 'F') {
-            if (c < 'a')
-                return false;
-            if (c > 'f')
-                return false;
-            return true;
-        } else {
-            if (c < 'A')
-                return false;
-            return true;
-        }
-    } else
-        return true;
-}
-
-/* You must ensure the character is a hex character before calling this, no error trapping here */
-static int fromhex(char c)
-{
-    if (c > 0x39) {
-        if (c > 'F') {
-            return c - 0x57;
-        } else {
-            return c - 0x37;
-        }
-    } else
-        return c - 0x30;
-}
-
 /* The 'read' functions all return the newly created object on the context's stack
  * which means these objects are created with a reference count of 0, and only when
  * pushed onto the stack does the reference count become 1, indicating the stack is
@@ -215,14 +181,12 @@ restart:
         } else if (c == '.') {
             goto error;
         } else if (c == 'e' || c == 'E') {
-            pdfi_set_warning(ctx, 0, NULL, W_PDF_NUM_EXPONENT, "pdfi_read_num", NULL);
+            pdfi_log_info(ctx, "pdfi_read_bare_int", (char *)"Invalid number format: scientific notation\n");
             goto error;
         } else if (c == '-') {
             /* Any - sign not at the start of the string indicates a malformed number. */
             if (index != 0 || negative) {
-                pdfi_set_error(ctx, 0, NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", NULL);
-                if (ctx->args.pdfstoponerror)
-                    return_error(gs_error_syntaxerror);
+                pdfi_log_info(ctx, "pdfi_read_bare_int", (char *)"Invalid number format: sign not at the start\n");
                 goto error;
             }
             negative = 1;
@@ -232,22 +196,19 @@ restart:
                  * of our negation handling for floats. */
                 continue;
             } else {
-                pdfi_set_error(ctx, 0, NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", NULL);
-                if (ctx->args.pdfstoponerror)
-                    return_error(gs_error_syntaxerror);
+                pdfi_log_info(ctx, "pdfi_read_bare_int", (char *)"Invalid number format: sign not at the start\n");
                 goto error;
             }
         } else {
             if (index > 0) {
-                pdfi_set_error(ctx, 0, NULL, E_PDF_MISSINGWHITESPACE, "pdfi_read_num", (char *)"Ignoring missing white space while parsing number");
-                if (ctx->args.pdfstoponerror)
-                    return_error(gs_error_syntaxerror);
+                pdfi_log_info(ctx, "pdfi_read_bare_int", (char *)"Invalid number format: Ignoring missing white space while parsing number\n");
+                goto error;
             }
             pdfi_unread_byte(ctx, s, (byte)c);
             goto error;
         }
         if (++index > 255)
-            return_error(gs_error_syntaxerror);
+            goto error;
     } while(1);
 
     *parsed_int = negative ? -int_val : int_val;
@@ -269,8 +230,9 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
     bool has_exponent = false;
     unsigned short exponent_index = 0;
     pdf_num *num;
-    int code = 0, malformed = false, doubleneg = false, recovered = false, negative = false;
-    int int_val = 0;
+    int code = 0, malformed = false, doubleneg = false, recovered = false, negative = false, overflowed = false;
+    unsigned int int_val = 0;
+    int tenth_max_int = max_int / 10, tenth_max_uint = max_uint / 10;
 
     pdfi_skip_white(ctx, s);
 
@@ -295,12 +257,21 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
         Buffer[index] = (byte)c;
 
         if (c >= '0' && c <= '9') {
-            if  (!(malformed && recovered))
-                int_val = int_val*10 + c - '0';
+            if  (!(malformed && recovered) && !overflowed && !real) {
+                if ((negative && int_val <= tenth_max_int) || (!negative && int_val <= tenth_max_uint))
+                    int_val = int_val*10 + c - '0';
+                else {
+                    if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_NUMBEROVERFLOW, "pdfi_read_num", NULL)) < 0) {
+                        return code;
+                    }
+                    overflowed = true;
+                }
+            }
         } else if (c == '.') {
             if (has_decimal_point == true) {
-                if (ctx->args.pdfstoponerror)
-                    return_error(gs_error_syntaxerror);
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", NULL)) < 0) {
+                    return code;
+                }
                 malformed = true;
             } else {
                 has_decimal_point = true;
@@ -311,8 +282,9 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
              * but gs seems to accept it, so we should also?
              */
             if (has_exponent == true) {
-                if (ctx->args.pdfstoponerror)
-                    return_error(gs_error_syntaxerror);
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", NULL)) < 0) {
+                    return code;
+                }
                 malformed = true;
             } else {
                 pdfi_set_warning(ctx, 0, NULL, W_PDF_NUM_EXPONENT, "pdfi_read_num", NULL);
@@ -324,9 +296,9 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
             /* Any - sign not at the start of the string, or just after an exponent
              * indicates a malformed number. */
             if (!(index == 0 || (has_exponent && index == exponent_index+1))) {
-                pdfi_set_error(ctx, 0, NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", NULL);
-                if (ctx->args.pdfstoponerror)
-                    return_error(gs_error_syntaxerror);
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", NULL)) < 0) {
+                    return code;
+                }
                 if (Buffer[index - 1] != '-') {
                     /* We are parsing a number line 123-56. We should continue parsing, but
                      * ignore anything from the second -. */
@@ -345,9 +317,9 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
                  * of our negation handling for floats. */
                 index--;
             } else {
-                pdfi_set_error(ctx, 0, NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", NULL);
-                if (ctx->args.pdfstoponerror)
-                    return_error(gs_error_syntaxerror);
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", NULL)) < 0) {
+                    return code;
+                }
                 if (Buffer[index - 1] != '-') {
                     /* We are parsing a number line 123-56. We should continue parsing, but
                      * ignore anything from the second -. */
@@ -357,9 +329,9 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
                 }
             }
         } else {
-            pdfi_set_error(ctx, 0, NULL, E_PDF_MISSINGWHITESPACE, "pdfi_read_num", (char *)"Ignoring missing white space while parsing number");
-            if (ctx->args.pdfstoponerror)
-                return_error(gs_error_syntaxerror);
+            if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MISSINGWHITESPACE, "pdfi_read_num", NULL)) < 0) {
+                return code;
+            }
             pdfi_unread_byte(ctx, s, (byte)c);
             Buffer[index] = 0x00;
             break;
@@ -376,7 +348,9 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
         return code;
 
     if ((malformed && !recovered) || (!real && doubleneg)) {
-        pdfi_set_error_var(ctx, 0, NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", "Treating malformed number %s as 0", Buffer);
+        if ((code = pdfi_set_warning_var(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", "Treating malformed number %s as 0", Buffer)) < 0) {
+            goto exit;
+        }
         num->value.i = 0;
     } else if (has_exponent) {
         float f, exp;
@@ -386,19 +360,25 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
             p = strstr((const char *)Buffer, "E");
 
         if (p == NULL) {
-            pdfi_set_error_var(ctx, 0, NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", "Treating malformed float %s as 0", Buffer);
+            if ((code = pdfi_set_warning_var(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", "Treating malformed float %s as 0", Buffer)) < 0) {
+                goto exit;
+            }
             num->value.d = 0;
         } else {
             p++;
 
             if (sscanf((char *)p, "%g", &exp) != 1 || exp > 38) {
-                pdfi_set_error_var(ctx, 0, NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", "Treating malformed float %s as 0", Buffer);
+                if ((code = pdfi_set_warning_var(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", "Treating malformed float %s as 0", Buffer)) < 0) {
+                    goto exit;
+                }
                 num->value.d = 0;
             } else {
                 if (sscanf((char *)Buffer, "%g", &f) == 1) {
                     num->value.d = f;
                 } else {
-                    pdfi_set_error_var(ctx, 0, NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", "Treating malformed float %s as 0", Buffer);
+                    if ((code = pdfi_set_warning_var(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MALFORMEDNUMBER, "pdfi_read_num", "Treating malformed float %s as 0", Buffer)) < 0) {
+                        goto exit;
+                    }
                     num->value.d = 0;
                 }
             }
@@ -407,7 +387,7 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
         num->value.d = acrobat_compatible_atof((char *)Buffer);
     } else {
         /* The doubleneg case is taken care of above. */
-        num->value.i = negative ? -int_val : int_val;
+        num->value.i = negative ? (int64_t)int_val * -1 : (int64_t)int_val;
     }
     if (ctx->args.pdfdebug) {
         if (real)
@@ -420,6 +400,7 @@ static int pdfi_read_num(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect_nu
 
     code = pdfi_push(ctx, (pdf_obj *)num);
 
+exit:
     if (code < 0)
         pdfi_free_object((pdf_obj *)num);
 
@@ -635,8 +616,10 @@ static int pdfi_read_string(pdf_context *ctx, pdf_c_stream *s, uint32_t indirect
         c = pdfi_read_byte(ctx, s);
 
         if (c < 0) {
-            if (nesting > 0)
-                pdfi_set_error(ctx, 0, NULL, E_PDF_UNESCAPEDSTRING, "pdfi_read_string", NULL);
+            if (nesting > 0 && (code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_UNESCAPEDSTRING, "pdfi_read_string", NULL) < 0)) {
+                gs_free_object(ctx->memory, Buffer, "pdfi_read_string error");
+                return code;
+            }
             Buffer[index] = 0x00;
             break;
         }
@@ -816,7 +799,7 @@ int pdfi_skip_comment(pdf_context *ctx, pdf_c_stream *s)
             break;
 
         if (ctx->args.pdfdebug)
-            dmprintf1 (ctx->memory, " %c", (char)c);
+            dmprintf1 (ctx->memory, "%c", (char)c);
 
     } while (c != 0x0a && c != 0x0d);
 
@@ -836,7 +819,7 @@ typedef int (*bsearch_comparator)(const void *, const void *);
 int pdfi_read_bare_keyword(pdf_context *ctx, pdf_c_stream *s)
 {
     byte Buffer[256];
-    int index = 0;
+    int code, index = 0;
     int c;
     void *t;
 
@@ -856,8 +839,9 @@ int pdfi_read_bare_keyword(pdf_context *ctx, pdf_c_stream *s)
     } while (index < 255);
 
     if (index >= 255 || index == 0) {
-        if (ctx->args.pdfstoponerror)
-            return_error(gs_error_syntaxerror);
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_NOERROR, "pdfi_read_bare_keyword", "")) < 0) {
+            return code;
+        }
         return TOKEN_INVALID_KEY;
     }
 
@@ -921,8 +905,9 @@ static int pdfi_read_keyword(pdf_context *ctx, pdf_c_stream *s, uint32_t indirec
     } while (index < 255);
 
     if (index >= 255 || index == 0) {
-        if (ctx->args.pdfstoponerror)
-            return_error(gs_error_syntaxerror);
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, 0, "pdfi_read_keyword", NULL) < 0)) {
+            return code;
+        }
         key = (index >= 255 ? TOKEN_TOO_LONG : TOKEN_INVALID_KEY);
         index = 0;
         Buffer[0] = 0;
@@ -978,8 +963,8 @@ static int pdfi_read_keyword(pdf_context *ctx, pdf_c_stream *s, uint32_t indirec
                 if (code < 0)
                     return code;
                 /* fallthrough */
-            case TOKEN_TRUE:
-            case TOKEN_FALSE:
+            case TOKEN_PDF_TRUE:
+            case TOKEN_PDF_FALSE:
             case TOKEN_null:
             default:
                 /* This is the fast, common exit case. We just push the key
@@ -1001,9 +986,6 @@ static int pdfi_read_keyword(pdf_context *ctx, pdf_c_stream *s, uint32_t indirec
     keyword->indirect_num = indirect_num;
     keyword->indirect_gen = indirect_gen;
 
-    if (ctx->args.pdfdebug)
-        dmprintf1(ctx->memory, " %s\n", Buffer);
-
     code = pdfi_push(ctx, (pdf_obj *)keyword);
     if (code < 0)
         pdfi_free_object((pdf_obj *)keyword);
@@ -1063,13 +1045,17 @@ rescan:
                 c = pdfi_read_byte(ctx, s);
             }
             if (c == '<') {
-                if (ctx->object_nesting < MAX_NESTING_DEPTH)
-                    ctx->object_nesting++;
                 if (ctx->args.pdfdebug)
                     dmprintf (ctx->memory, " <<\n");
-                code = pdfi_mark_stack(ctx, PDF_DICT_MARK);
-                if (code < 0)
+                if (ctx->object_nesting < MAX_NESTING_DEPTH) {
+                    ctx->object_nesting++;
+                    code = pdfi_mark_stack(ctx, PDF_DICT_MARK);
+                    if (code < 0)
+                        return code;
+                }
+                else if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_limitcheck), NULL, E_PDF_NESTEDTOODEEP, "pdfi_read_token", NULL) < 0)) {
                     return code;
+                }
                 return 1;
             } else if (c == '>') {
                 pdfi_unread_byte(ctx, s, (byte)c);
@@ -1091,14 +1077,17 @@ rescan:
             if (c < 0)
                 return (gs_error_ioerror);
             if (c == '>') {
-                if (ctx->object_nesting > 0)
+                if (ctx->object_nesting > 0) {
                     ctx->object_nesting--;
-                if (ctx->object_nesting < MAX_NESTING_DEPTH) {
                     code = pdfi_dict_from_stack(ctx, indirect_num, indirect_gen, false);
                     if (code < 0)
                         return code;
-                } else
-                    pdfi_set_error(ctx, 0, NULL, E_PDF_NESTEDTOODEEP, "pdfi_read_token", NULL);
+                } else {
+                    if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_unmatchedmark), NULL, E_PDF_UNMATCHEDMARK, "pdfi_read_token", NULL) < 0)) {
+                        return code;
+                    }
+                    goto rescan;
+                }
                 return 1;
             } else {
                 pdfi_unread_byte(ctx, s, (byte)c);
@@ -1112,24 +1101,30 @@ rescan:
             return 1;
             break;
         case '[':
-            if (ctx->object_nesting < MAX_NESTING_DEPTH)
-                ctx->object_nesting++;
             if (ctx->args.pdfdebug)
                 dmprintf (ctx->memory, "[");
-            code = pdfi_mark_stack(ctx, PDF_ARRAY_MARK);
-            if (code < 0)
-                return code;
-            return 1;
-            break;
-        case ']':
-            if (ctx->object_nesting > 0)
-                ctx->object_nesting--;
             if (ctx->object_nesting < MAX_NESTING_DEPTH) {
-                code = pdfi_array_from_stack(ctx, indirect_num, indirect_gen);
+                ctx->object_nesting++;
+                code = pdfi_mark_stack(ctx, PDF_ARRAY_MARK);
                 if (code < 0)
                     return code;
             } else
-                pdfi_set_error(ctx, 0, NULL, E_PDF_NESTEDTOODEEP, "pdfi_read_token", NULL);
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_NESTEDTOODEEP, "pdfi_read_token", NULL)) < 0)
+                    return code;
+            return 1;
+            break;
+        case ']':
+            if (ctx->object_nesting > 0) {
+                ctx->object_nesting--;
+                code = pdfi_array_from_stack(ctx, indirect_num, indirect_gen);
+                if (code < 0)
+                    return code;
+            } else {
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_unmatchedmark), NULL, E_PDF_UNMATCHEDMARK, "pdfi_read_token", NULL) < 0)) {
+                    return code;
+                }
+                goto rescan;
+            }
             break;
         case '{':
             if (ctx->args.pdfdebug)
@@ -1149,8 +1144,9 @@ rescan:
             break;
         default:
             if (isdelimiter(c)) {
-                if (ctx->args.pdfstoponerror)
-                    return_error(gs_error_syntaxerror);
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, 0, "pdfi_read_token", NULL) < 0)) {
+                    return code;
+                }
                 goto rescan;
             }
             pdfi_unread_byte(ctx, s, (byte)c);
@@ -1393,6 +1389,7 @@ static int split_bogus_operator(pdf_context *ctx, pdf_c_stream *source, pdf_dict
         goto error_exit;
 
 match:
+    pdfi_set_warning(ctx, 0, NULL, W_PDF_MISSING_WHITE_OPS, "split_bogus_operator", NULL);
     /* If we get here, we have two PDF_KEYWORD objects. We push them on the stack
      * one at a time, and execute them.
      */
@@ -1404,8 +1401,13 @@ match:
     pdfi_push(ctx, (pdf_obj *)key2);
     code = pdfi_interpret_stream_operator(ctx, source, stream_dict, page_dict);
 
+    pdfi_countdown(key1);
+    pdfi_countdown(key2);
+    pdfi_clearstack(ctx);
+    return code;
+
 error_exit:
-    pdfi_set_error(ctx, 0, NULL, E_PDF_TOKENERROR, "split_bogus_operator", NULL);
+    pdfi_set_error(ctx, code, NULL, E_PDF_TOKENERROR, "split_bogus_operator", NULL);
     pdfi_countdown(key1);
     pdfi_countdown(key2);
     pdfi_clearstack(ctx);
@@ -1728,26 +1730,11 @@ static int pdfi_interpret_stream_operator(pdf_context *ctx, pdf_c_stream *source
         if (code > 0)
             code = 0;
         return code;
-    } else if (((pdf_keyword *)keyword)->length > 3) {
-        if (ctx->args.pdfdebug) {
-            char Buffer[1024];
-            int length;
-
-            if (((pdf_keyword *)keyword)->length > 1023)
-                length = 1023;
-            else
-                length = ((pdf_keyword *)keyword)->length;
-
-            memcpy(Buffer, ((pdf_keyword *)keyword)->data, length);
-            Buffer[length] = 0x00;
-            dmprintf1(ctx->memory, " %s\n", Buffer);
-        }
-
+    } else {
         /* This means we either have a corrupted or illegal operator. The most
          * usual corruption is two concatented operators (eg QBT instead of Q BT)
          * I plan to tackle this by trying to see if I can make two or more operators
-         * out of the mangled one. Note this will also be done below in the 'default'
-         * case where we don't recognise a keyword with 3 or fewer characters.
+         * out of the mangled one.
          */
         code = split_bogus_operator(ctx, source, stream_dict, page_dict);
         if (code < 0)
@@ -1940,8 +1927,8 @@ pdfi_interpret_inner_content(pdf_context *ctx, pdf_c_stream *content_stream, pdf
 
     cleanup_context_interpretation(ctx, &local_entry_save);
     local_restore_stream_state(ctx, &local_entry_save);
-    if (!ctx->args.pdfstoponerror)
-        code = 0;
+    if (code < 0)
+        code = pdfi_set_error_stop(ctx, code, NULL, 0, "pdfi_interpret_inner_content", NULL);
     return code;
 }
 
@@ -1957,6 +1944,9 @@ pdfi_interpret_inner_content_buffer(pdf_context *ctx, byte *content_data,
     pdf_c_stream *stream = NULL;
     pdf_stream *stream_obj = NULL;
 
+    if (content_length == 0)
+        return 0;
+
     code = pdfi_open_memory_stream_from_memory(ctx, content_length,
                                                content_data, &stream, true);
     if (code < 0)
@@ -1984,6 +1974,8 @@ pdfi_interpret_inner_content_c_string(pdf_context *ctx, char *content_string,
     bool decrypt_strings;
     int code;
 
+    if (length == 0)
+        return 0;
 
     /* Underlying buffer limit is uint32, so handle the extremely unlikely case that
      * our string is too big.
@@ -2033,10 +2025,11 @@ pdfi_interpret_content_stream(pdf_context *ctx, pdf_c_stream *content_stream,
                               pdf_stream *stream_obj, pdf_dict *page_dict)
 {
     int code;
-    pdf_c_stream *stream;
+    pdf_c_stream *stream = NULL, *SubFile_stream = NULL;
     pdf_keyword *keyword;
     pdf_stream *s = ctx->current_stream;
     pdf_obj_type type;
+    char EODString[] = "endstream";
 
     /* Check this stream, and all the streams currently being executed, to see
      * if the stream we've been given is already in train. If it is, then we
@@ -2071,9 +2064,21 @@ pdfi_interpret_content_stream(pdf_context *ctx, pdf_c_stream *content_stream,
         if (code < 0)
             return code;
 
-        code = pdfi_filter(ctx, stream_obj, ctx->main_stream, &stream, false);
+        if (stream_obj->length_valid) {
+            if (stream_obj->Length == 0)
+                return 0;
+            code = pdfi_apply_SubFileDecode_filter(ctx, stream_obj->Length, NULL, ctx->main_stream, &SubFile_stream, false);
+        }
+        else
+            code = pdfi_apply_SubFileDecode_filter(ctx, 0, EODString, ctx->main_stream, &SubFile_stream, false);
         if (code < 0)
             return code;
+
+        code = pdfi_filter(ctx, stream_obj, SubFile_stream, &stream, false);
+        if (code < 0) {
+            pdfi_close_file(ctx, SubFile_stream);
+            return code;
+        }
     }
 
     pdfi_set_stream_parent(ctx, stream_obj, ctx->current_stream);
@@ -2084,9 +2089,9 @@ pdfi_interpret_content_stream(pdf_context *ctx, pdf_c_stream *content_stream,
         if (code < 0) {
             if (code == gs_error_ioerror || code == gs_error_VMerror || ctx->args.pdfstoponerror) {
                 if (code == gs_error_ioerror) {
-                    pdfi_set_error(ctx, 0, NULL, E_PDF_BADSTREAM, "pdfi_interpret_content_stream", (char *)"**** Error reading a content stream.  The page may be incomplete");
+                    pdfi_set_error(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_interpret_content_stream", (char *)"**** Error reading a content stream.  The page may be incomplete");
                 } else if (code == gs_error_VMerror) {
-                    pdfi_set_error(ctx, 0, NULL, E_PDF_OUTOFMEMORY, "pdfi_interpret_content_stream", (char *)"**** Error ran out of memory reading a content stream.  The page may be incomplete");
+                    pdfi_set_error(ctx, code, NULL, E_PDF_OUTOFMEMORY, "pdfi_interpret_content_stream", (char *)"**** Error ran out of memory reading a content stream.  The page may be incomplete");
                 }
                 goto exit;
             }
@@ -2110,18 +2115,18 @@ repaired_keyword:
                     break;
                 case TOKEN_ENDOBJ:
                     pdfi_clearstack(ctx);
-                    pdfi_set_error(ctx, 0, NULL, E_PDF_MISSINGENDSTREAM, "pdfi_interpret_content_stream", NULL);
-                    if (ctx->args.pdfstoponerror)
-                        code = gs_note_error(gs_error_syntaxerror);
+                    code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MISSINGENDSTREAM, "pdfi_interpret_content_stream", NULL);
                     goto exit;
                     break;
                 case TOKEN_INVALID_KEY:
-                    pdfi_set_error(ctx, 0, NULL, E_PDF_KEYWORDTOOLONG, "pdfi_interpret_content_stream", NULL);
                     pdfi_clearstack(ctx);
+                    if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_limitcheck), NULL, E_PDF_KEYWORDTOOLONG, "pdfi_interpret_content_stream", NULL)) < 0)
+                        goto exit;
                     break;
                 case TOKEN_TOO_LONG:
-                    pdfi_set_error(ctx, 0, NULL, E_PDF_MISSINGENDSTREAM, "pdfi_interpret_content_stream", NULL);
                     pdfi_clearstack(ctx);
+                    if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_MISSINGENDSTREAM, "pdfi_interpret_content_stream", NULL)) < 0)
+                        goto exit;
                     break;
                 default:
                     goto execute;
@@ -2142,8 +2147,7 @@ execute:
                     goto repaired_keyword;
 
                 if (code < 0) {
-                    pdfi_set_error(ctx, code, NULL, E_PDF_TOKENERROR, "pdf_interpret_content_stream", NULL);
-                    if (ctx->args.pdfstoponerror) {
+                    if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_TOKENERROR, "pdf_interpret_content_stream", NULL)) < 0) {
                         pdfi_clearstack(ctx);
                         goto exit;
                     }
@@ -2158,5 +2162,7 @@ exit:
     ctx->current_stream = pdfi_stream_parent(ctx, stream_obj);
     pdfi_clear_stream_parent(ctx, stream_obj);
     pdfi_close_file(ctx, stream);
+    if (SubFile_stream != NULL)
+        pdfi_close_file(ctx, SubFile_stream);
     return code;
 }
