@@ -1,4 +1,4 @@
-/* Copyright (C) 2020-2022 Artifex Software, Inc.
+/* Copyright (C) 2020-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* pdfmark handling for the PDF interpreter */
@@ -29,6 +29,8 @@
 
 #include "gscoord.h"         /* For gs_currentmatrix */
 
+static int pdfi_get_named_dest(pdf_context *ctx, pdf_obj *Named, pdf_obj **Dest);
+
 static int pdfi_pdfmark_setparam_obj(pdf_context *ctx, pdf_obj *obj, gs_param_string *entry)
 {
     int code = 0;
@@ -395,21 +397,46 @@ static int pdfi_pdfmark_add_Page_View(pdf_context *ctx, pdf_dict *link_dict, pdf
         page_num = ((pdf_num *)page_dict)->value.i;
     } else {
         if (pdfi_type_of(page_dict) != PDF_DICT) {
-            code = gs_note_error(gs_error_typecheck);
-            goto exit;
+            if (pdfi_type_of(page_dict) != PDF_NULL) {
+                code = gs_note_error(gs_error_typecheck);
+                goto exit;
+            }
+            page_num = 0;
+        } else {
+            /* Find out which page number this is */
+            code = pdfi_page_get_number(ctx, page_dict, &page_num);
+            if (code < 0) goto exit;
         }
-
-        /* Find out which page number this is */
-        code = pdfi_page_get_number(ctx, page_dict, &page_num);
-        if (code < 0) goto exit;
     }
 
     page_num += ctx->Pdfmark_InitialPage;
 
-    /* Add /Page key to the link_dict
-     * Of course pdfwrite is numbering its pages starting at 1, because... of course :(
+    if (pdfi_type_of(page_dict) != PDF_NULL)
+        /* Add /Page key to the link_dict
+         * Of course pdfwrite is numbering its pages starting at 1, because... of course :(
+         */
+        code = pdfi_dict_put_int(ctx, link_dict, "Page", page_num+1);
+    else
+        code = pdfi_dict_put_int(ctx, link_dict, "Page", 0);
+
+    if (code < 0)
+        goto exit;
+
+    /* Get and check the destination type, again without storing the deref in the array.
+     * In this case a memory leak  can only happen if the destination
+     * is an invalid type, but lets take no chances.
      */
-    code = pdfi_dict_put_int(ctx, link_dict, "Page", page_num+1);
+    code = pdfi_array_get_no_store_R(ctx, dest_array, 1, &temp_obj);
+    if (code < 0) goto exit;
+
+    if (pdfi_type_of(temp_obj) != PDF_NAME) {
+        pdfi_countdown(temp_obj);
+        temp_obj = NULL;
+        code = gs_note_error(gs_error_typecheck);
+        goto exit;
+    }
+    pdfi_countdown(temp_obj);
+    temp_obj = NULL;
 
     /* Build an array for /View, out of the remainder of the Dest entry */
     array_size = pdfi_array_size(dest_array) - 1;
@@ -567,16 +594,36 @@ int pdfi_pdfmark_modDest(pdf_context *ctx, pdf_dict *link_dict)
             if (code < 0) goto exit;
             if (code == 0) {
                 /* TODO: Not found -- not sure if there is another case here or not */
+                code = gs_note_error(gs_error_undefined);
                 goto exit;
             }
 
             code = pdfi_dict_knownget_type(ctx, Dests, "Names", PDF_ARRAY, (pdf_obj **)&Names);
             if (code < 0) goto exit;
             if (code == 0) {
-                /* TODO: Not found -- not sure if there is another case here or not */
-                goto exit;
-            }
-            code = pdfi_pdfmark_handle_dest_names(ctx, link_dict, Dest, Names);
+                code = pdfi_get_named_dest(ctx, Dest, (pdf_obj **)&dest_array);
+                if (code < 0)
+                    goto exit;
+                if (pdfi_type_of(dest_array) == PDF_DICT) {
+                    pdf_obj *D = NULL;
+                    /* Dict is supposed to contain key "D" with Dest array */
+                    code = pdfi_dict_knownget_type(ctx, (pdf_dict *)dest_array, "D", PDF_ARRAY, &D);
+                    if (code <= 0) goto exit;
+
+                    pdfi_countdown(dest_array);
+                    dest_array = (pdf_array *)D;
+                }
+                if (pdfi_type_of(dest_array) == PDF_ARRAY) {
+                    code = pdfi_pdfmark_add_Page_View(ctx, link_dict, dest_array);
+                    if (code < 0)
+                        goto exit;
+                } else {
+                    if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_undefined), NULL, E_PDF_BAD_NAMED_DEST, "pdfi_pdfmark_modDest", NULL)) < 0)
+                        goto exit;
+                }
+                break;
+            } else
+                code = pdfi_pdfmark_handle_dest_names(ctx, link_dict, Dest, Names);
             if (code < 0) goto exit;
         } else {
             /* TODO: Ignore it -- flag a warning? */
@@ -617,7 +664,7 @@ static int pdfi_check_limits(pdf_context *ctx, pdf_dict *node, char *str, int le
         /* Limits are not valid, just ignore them. The calling code will then check
          * the Names array.
          */
-        pdfi_set_warning(ctx, 0, NULL, PDF_W_BAD_TREE_LIMITS, "pdfi_get_name_from_node", 0);
+        pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_TREE_LIMITS, "pdfi_check_limits", 0);
         goto error;
     }
 
@@ -631,7 +678,7 @@ static int pdfi_check_limits(pdf_context *ctx, pdf_dict *node, char *str, int le
             return code;
     } else {
         len2 = ((pdf_string *)Str)->length;
-        str2 = (char *)gs_alloc_bytes(ctx->memory, len2 + 1, "pdfi_get_named_dest");
+        str2 = (char *)gs_alloc_bytes(ctx->memory, len2 + 1, "pdfi_check_limits");
         if (str2 == NULL) {
             code = gs_note_error(gs_error_VMerror);
             goto error;
@@ -659,7 +706,7 @@ static int pdfi_check_limits(pdf_context *ctx, pdf_dict *node, char *str, int le
         code = gs_note_error(gs_error_undefined);
         goto error;
     }
-    gs_free_object(ctx->memory, str2, "pdfi_get_named_dest");
+    gs_free_object(ctx->memory, str2, "pdfi_check_limits");
     str2 = NULL;
 
     code = pdfi_array_get_type(ctx, Limits, 1, PDF_STRING, (pdf_obj **)&Str);
@@ -672,7 +719,7 @@ static int pdfi_check_limits(pdf_context *ctx, pdf_dict *node, char *str, int le
             return code;
     } else {
         len2 = ((pdf_string *)Str)->length;
-        str2 = (char *)gs_alloc_bytes(ctx->memory, len2 + 1, "pdfi_get_named_dest");
+        str2 = (char *)gs_alloc_bytes(ctx->memory, len2 + 1, "pdfi_check_limits");
         if (str2 == NULL) {
             code = gs_note_error(gs_error_VMerror);
             goto error;
@@ -701,20 +748,24 @@ static int pdfi_check_limits(pdf_context *ctx, pdf_dict *node, char *str, int le
         code = gs_note_error(gs_error_undefined);
 
 error:
-    gs_free_object(ctx->memory, str2, "pdfi_get_named_dest");
+    gs_free_object(ctx->memory, str2, "pdfi_check_limits");
     pdfi_countdown(Str);
     pdfi_countdown(Limits);
     return code;
 }
 
-static int pdfi_get_name_from_node(pdf_context *ctx, pdf_dict *node, char *str, pdf_obj **Name)
+static int pdfi_get_name_from_node(pdf_context *ctx, pdf_dict *node, char *str, int len, pdf_obj **Name, bool is_root)
 {
-    int i = 0, len = strlen(str), code = 0;
+    int i = 0, code = 0;
     pdf_string *StrKey = NULL;
     pdf_array *NamesArray = NULL;
     pdf_dict *Kid = NULL;
     bool known;
 
+    code = pdfi_loop_detector_mark(ctx);
+    if (code < 0)
+        return code;
+
     code = pdfi_dict_known(ctx, node, "Names", &known);
     if (code < 0)
         goto error;
@@ -725,10 +776,11 @@ static int pdfi_get_name_from_node(pdf_context *ctx, pdf_dict *node, char *str,
             goto error;
 
         if (!known) {
-            /* No Limits array (a required entry), so just assume that the
-             * string is in this node and check all the Names anyway
-             */
-            pdfi_set_warning(ctx, 0, NULL, PDF_W_NO_TREE_LIMITS, "pdfi_get_name_from_node", 0);
+            if (!is_root)
+                /* No Limits array (a required entry, except in the Root), so just assume that the
+                 * string is in this node and check all the Names anyway
+                 */
+                pdfi_set_warning(ctx, 0, NULL, W_PDF_NO_TREE_LIMITS, "pdfi_get_name_from_node", 0);
         } else {
             code = pdfi_check_limits(ctx, node, str, len);
             if (code < 0)
@@ -740,14 +792,14 @@ static int pdfi_get_name_from_node(pdf_context *ctx, pdf_dict *node, char *str,
             goto error;
 
         if (pdfi_array_size(NamesArray) & 1)
-            pdfi_set_warning(ctx, 0, NULL, PDF_W_NAMES_ARRAY_SIZE, "pdfi_get_name_from_node", 0);
+            pdfi_set_warning(ctx, 0, NULL, W_PDF_NAMES_ARRAY_SIZE, "pdfi_get_name_from_node", 0);
 
         for (i = 0;i < pdfi_array_size(NamesArray) / 2; i++) {
             code = pdfi_array_get_type(ctx, NamesArray, i * 2, PDF_STRING, (pdf_obj **)&StrKey);
             if (code < 0)
                 goto error;
 
-            if (StrKey->length == len && strncmp((const char *)StrKey->data, str, len) == 0) {
+            if (StrKey->length == len && memcmp((const char *)StrKey->data, str, len) == 0) {
                 code = pdfi_array_get(ctx, NamesArray, (i * 2) + 1, (pdf_obj **)Name);
                 goto error;
             }
@@ -768,7 +820,7 @@ static int pdfi_get_name_from_node(pdf_context *ctx, pdf_dict *node, char *str,
         if (code < 0)
             goto error;
 
-        code = pdfi_get_name_from_node(ctx, Kid, str, Name);
+        code = pdfi_get_name_from_node(ctx, Kid, str, len, Name, false);
         pdfi_countdown(Kid);
         Kid = NULL;
         if (code == 0)
@@ -785,6 +837,7 @@ error:
     pdfi_countdown(Kid);
     pdfi_countdown(StrKey);
     pdfi_countdown(NamesArray);
+    (void)pdfi_loop_detector_cleartomark(ctx);
     return code;
 }
 
@@ -794,6 +847,10 @@ static int pdfi_get_named_dest(pdf_context *ctx, pdf_obj *Named, pdf_obj **Dest)
     pdf_dict *Names = NULL, *Dests = NULL;
     char *str = NULL;
 
+    code = pdfi_loop_detector_mark(ctx);
+    if (code < 0)
+        return code;
+
     code = pdfi_dict_get_type(ctx, ctx->Root, "Names", PDF_DICT, (pdf_obj **)&Names);
     if (code < 0)
         goto error;
@@ -817,7 +874,7 @@ static int pdfi_get_named_dest(pdf_context *ctx, pdf_obj *Named, pdf_obj **Dest)
         str[len] = 0;
     }
 
-    code = pdfi_get_name_from_node(ctx, Dests, str, Dest);
+    code = pdfi_get_name_from_node(ctx, Dests, str, len, Dest, true);
 
 error:
     if (pdfi_type_of(Named) == PDF_NAME)
@@ -826,6 +883,7 @@ error:
         gs_free_object(ctx->memory, str, "pdfi_get_named_dest");
     pdfi_countdown(Names);
     pdfi_countdown(Dests);
+    pdfi_loop_detector_cleartomark(ctx);
     return code;
 }
 
@@ -881,15 +939,18 @@ int pdfi_pdfmark_modA(pdf_context *ctx, pdf_dict *dict)
                 goto exit;
             pdfi_countdown(D_array);
             D_array = NULL;
-            if (pdfi_type_of(Dest) != PDF_DICT) {
+            if (pdfi_type_of(Dest) != PDF_ARRAY) {
+                if (pdfi_type_of(Dest) != PDF_DICT) {
+                    pdfi_countdown(Dest);
+                    code = gs_note_error(gs_error_typecheck);
+                    goto exit;
+                }
+                code = pdfi_dict_knownget(ctx, (pdf_dict *)Dest, "D", (pdf_obj **)&D_array);
                 pdfi_countdown(Dest);
-                code = gs_note_error(gs_error_typecheck);
-                goto exit;
-            }
-            code = pdfi_dict_knownget(ctx, (pdf_dict *)Dest, "D", (pdf_obj **)&D_array);
-            pdfi_countdown(Dest);
-            if (code <= 0)
-                goto exit;
+                if (code <= 0)
+                    goto exit;
+            } else
+                D_array = (pdf_array *)Dest;
         }
         if (pdfi_type_of(D_array) != PDF_ARRAY) {
             code = gs_note_error(gs_error_typecheck);
@@ -1257,8 +1318,8 @@ void pdfi_pdfmark_write_boxes(pdf_context *ctx, pdf_dict *page_dict)
         "CropBox", "BleedBox", "TrimBox", "ArtBox"
     };
 
-    /* If the device doesn't support pdfmar, exit now */
-    if (!ctx->device_state.writepdfmarks)
+    /* If the device doesn't support pdfmark, or the device modifies the Page Size (making the other boxes unreliable) exit now */
+    if (!ctx->device_state.writepdfmarks || ctx->device_state.ModifiesPageSize)
         return;
 
     /* If we are using somethign other than the MediaBox, don't send these */
