@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2022 Artifex Software, Inc.
+/* Copyright (C) 2019-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Optional Content routines */
@@ -109,21 +109,21 @@ pdfi_oc_check_OCG_usage(pdf_context *ctx, pdf_dict *ocdict)
     }
 
     if (ctx->args.printed) {
-        code = pdfi_dict_knownget_type(ctx, ocdict, "Print", PDF_DICT, (pdf_obj **)&dict);
+        code = pdfi_dict_knownget_type(ctx, Usage, "Print", PDF_DICT, (pdf_obj **)&dict);
         if (code <= 0)
             goto cleanup;
         code = pdfi_dict_knownget_type(ctx, dict, "PrintState", PDF_NAME, &name);
         if (code <= 0)
             goto cleanup;
     } else {
-        code = pdfi_dict_knownget_type(ctx, ocdict, "View", PDF_DICT, (pdf_obj **)&dict);
+        code = pdfi_dict_knownget_type(ctx, Usage, "View", PDF_DICT, (pdf_obj **)&dict);
         if (code <= 0)
             goto cleanup;
         code = pdfi_dict_knownget_type(ctx, dict, "ViewState", PDF_NAME, &name);
         if (code <= 0)
             goto cleanup;
     }
-    if (pdfi_name_strcmp((pdf_name *)name, "OFF")) {
+    if (pdfi_name_strcmp((pdf_name *)name, "OFF") == 0) {
         is_visible = false;
     }
 
@@ -142,10 +142,35 @@ typedef enum {
     P_AnyOff
 } ocmd_p_type;
 
+static bool pdfi_oc_default_visibility(pdf_context *ctx)
+{
+    int code;
+    pdf_dict *D_dict = NULL;
+    pdf_name *B = NULL;
+
+    code = pdfi_dict_knownget_type(ctx, ctx->OCProperties, "D", PDF_DICT, (pdf_obj **)&D_dict);
+    if (code < 0 || D_dict == NULL)
+        return true;
+
+    code = pdfi_dict_knownget_type(ctx, D_dict, "BaseState", PDF_NAME, (pdf_obj **)&B);
+    (void)pdfi_countdown(D_dict);
+    D_dict = NULL;
+    if (code < 0 || B == NULL)
+        return true;
+
+    if (pdfi_name_is(B, "OFF")) {
+        (void)pdfi_countdown(B);
+        return false;
+    }
+
+    (void)pdfi_countdown(B);
+    return true;
+}
+
 static bool
 pdfi_oc_check_OCMD_array(pdf_context *ctx, pdf_array *array, ocmd_p_type type)
 {
-    bool is_visible;
+    bool is_visible, hit = false;
     uint64_t i;
     int code;
     pdf_obj *val = NULL;
@@ -174,6 +199,7 @@ pdfi_oc_check_OCMD_array(pdf_context *ctx, pdf_array *array, ocmd_p_type type)
             continue;
         }
 
+        hit = true;
         vis = pdfi_get_default_OCG_val(ctx, (pdf_dict *)val);
         switch (type) {
         case P_AnyOn:
@@ -209,7 +235,17 @@ pdfi_oc_check_OCMD_array(pdf_context *ctx, pdf_array *array, ocmd_p_type type)
         val = NULL;
     }
 
- cleanup:
+    /* If the array was empty, or contained only null or deleted entries, then it has no effect
+     * PDF Reference 1.7 p366, table 4.49, OCGs entry. I'm interpreting this to mean that we should use
+     * the OCProperties 'D' dictionary, set the visibility state to the BaseState. Since this is an OCMD
+     * I'm assuming that the ON and OFF arrays (which apply to Optional Content Groups) shouold not be
+     * consulted. We need to cater for the fact that BaseState is optional (but default is ON). D is
+     * specified as 'Required' but it seems best not to take any chances on that!
+     */
+    if (!hit)
+        is_visible = pdfi_oc_default_visibility(ctx);
+
+cleanup:
     pdfi_countdown(val);
     return is_visible;
 }
@@ -224,6 +260,8 @@ pdfi_oc_check_OCMD(pdf_context *ctx, pdf_dict *ocdict)
     pdf_obj *Pname = NULL;
     pdf_dict *OCGs_dict = NULL; /* alias, don't need to free */
     pdf_array *OCGs_array = NULL; /* alias, don't need to free */
+    pdf_dict *UsageDict = NULL, *StateDict = NULL;
+    pdf_obj *State = NULL;
     ocmd_p_type Ptype = P_AnyOn;
 
     /* TODO: We don't support this, so log a warning and ignore */
@@ -240,6 +278,7 @@ pdfi_oc_check_OCMD(pdf_context *ctx, pdf_dict *ocdict)
     } else if (pdfi_type_of(obj) == PDF_DICT) {
         OCGs_dict = (pdf_dict *)obj;
     } else {
+        is_visible = pdfi_oc_default_visibility(ctx);
         goto cleanup;
     }
 
@@ -273,7 +312,47 @@ pdfi_oc_check_OCMD(pdf_context *ctx, pdf_dict *ocdict)
         is_visible = pdfi_oc_check_OCMD_array(ctx, OCGs_array, Ptype);
     }
 
+    if (OCGs_dict) {
+        code = pdfi_dict_knownget_type(ctx, OCGs_dict, "Usage", PDF_DICT, (pdf_obj **)&UsageDict);
+        if (code < 0)
+            goto cleanup;
+
+        if (UsageDict != NULL) {
+            if (ctx->args.printed) {
+                code = pdfi_dict_knownget_type(ctx, UsageDict, "Print", PDF_DICT, (pdf_obj **)&StateDict);
+                if (code < 0)
+                    goto cleanup;
+                if (StateDict) {
+                    code = pdfi_dict_knownget_type(ctx, StateDict, "PrintState", PDF_NAME, &State);
+                    if (code < 0)
+                        goto cleanup;
+                }
+            } else {
+                code = pdfi_dict_knownget_type(ctx, UsageDict, "View", PDF_DICT, (pdf_obj **)&StateDict);
+                if (code < 0)
+                    goto cleanup;
+                if (StateDict) {
+                    code = pdfi_dict_knownget_type(ctx, StateDict, "ViewState", PDF_NAME, &State);
+                    if (code < 0)
+                        goto cleanup;
+                }
+            }
+            if (State) {
+                if (pdfi_name_is((const pdf_name *)State, "ON"))
+                    is_visible = true;
+                else
+                    if (pdfi_name_is((const pdf_name *)State, "OFF"))
+                        is_visible = false;
+                    else
+                        pdfi_set_error(ctx, 0, NULL, E_PDF_BAD_VALUE, "pdfi_oc_check_OCMD", "Usage Dictionary State is neither ON nor OFF");
+            }
+        }
+    }
+
  cleanup:
+    pdfi_countdown(State);
+    pdfi_countdown(StateDict);
+    pdfi_countdown(UsageDict);
     pdfi_countdown(VE);
     pdfi_countdown(obj);
     pdfi_countdown(Pname);
@@ -442,8 +521,10 @@ int pdfi_op_MP(pdf_context *ctx)
     if (pdfi_count_stack(ctx) < 1)
         return_error(gs_error_stackunderflow);
 
-    if (!ctx->device_state.writepdfmarks || !ctx->args.preservemarkedcontent)
+    if (!ctx->device_state.writepdfmarks || !ctx->args.preservemarkedcontent) {
+        pdfi_pop(ctx, 1);
         goto exit;
+    }
 
     o = ctx->stack_top[-1];
     pdfi_countup(o);
@@ -492,7 +573,7 @@ int pdfi_op_DP(pdf_context *ctx, pdf_dict *stream_dict, pdf_dict *page_dict)
 
     objarray[0] = ctx->stack_top[-2];
     pdfi_countup(objarray[0]);
-    o = ctx->stack_top[-2];
+    o = ctx->stack_top[-1];
     pdfi_countup(o);
     pdfi_pop(ctx, 2); /* pop args */
 
@@ -540,6 +621,10 @@ int pdfi_op_BMC(pdf_context *ctx)
         return_error(gs_error_stackunderflow);
 
     if (!ctx->device_state.writepdfmarks || !ctx->args.preservemarkedcontent) {
+        /* Need to increment the BMCLevel anyway, as the EMC will count it down.
+         * If we already have a BDC, then that effectively will turn it on.
+         */
+        ctx->BMClevel ++;
         pdfi_pop(ctx, 1);
         goto exit;
     }
@@ -570,7 +655,8 @@ int pdfi_op_BDC(pdf_context *ctx, pdf_dict *stream_dict, pdf_dict *page_dict)
     pdf_dict *oc_dict = NULL;
     int code = 0;
     bool ocg_is_visible;
-    pdf_obj **objarray = NULL, *o = NULL;;
+    pdf_obj **objarray = NULL, *o = NULL;
+    pdf_indirect_ref *dictref = NULL;
 
     /* This will also prevent us writing out an EMC if the BDC is in any way invalid */
     ctx->BDCWasOC = true;
@@ -591,11 +677,10 @@ int pdfi_op_BDC(pdf_context *ctx, pdf_dict *stream_dict, pdf_dict *page_dict)
     if (pdfi_type_of(tag) != PDF_NAME)
         goto exit;
 
-    if (!pdfi_name_is(tag, "OC")) {
+    if (!pdfi_name_is(tag, "OC"))
         ctx->BDCWasOC = false;
-        if (!ctx->device_state.writepdfmarks || !ctx->args.preservemarkedcontent)
-            goto exit;
 
+    if (ctx->device_state.writepdfmarks && ctx->args.preservemarkedcontent && (!ctx->BDCWasOC || ctx->device_state.WantsOptionalContent)) {
         objarray = (pdf_obj **)gs_alloc_bytes(ctx->memory, 2 * sizeof(pdf_obj *), "pdfi_op_BDC");
         if (objarray == NULL) {
             code = gs_note_error(gs_error_VMerror);
@@ -613,7 +698,52 @@ int pdfi_op_BDC(pdf_context *ctx, pdf_dict *stream_dict, pdf_dict *page_dict)
                     code = gs_note_error(gs_error_typecheck);
                     goto exit;
                 }
-                objarray[1] = (pdf_obj *)oc_dict;
+                /* If we are producing PDF/A we must not include any Metadata, as that
+                 * requires us to modify the XMP Metadata, which we don't know how to do.
+                 */
+                if (ctx->args.PDFA > 0) {
+                    uint64_t index = 0;
+                    pdf_name *Key = NULL;
+                    pdf_obj *Value = NULL;
+
+                    code = pdfi_dict_first(ctx, oc_dict, (pdf_obj **)&Key, &Value, &index);
+                    if (code < 0) {
+                        if (code == gs_error_undefined)
+                            code = 0;
+                        goto exit;
+                    }
+                    while (code >= 0) {
+                        if (pdfi_name_is(Key, "Metadata")) {
+                            pdfi_dict_delete_pair(ctx, oc_dict, Key);
+                        }
+                        pdfi_countdown(Key);
+                        Key = NULL;
+                        pdfi_countdown(Value);
+                        Value = NULL;
+
+                        code = pdfi_dict_next(ctx, oc_dict, (pdf_obj **)&Key, &Value, &index);
+                        if (code == gs_error_undefined) {
+                            code = 0;
+                            break;
+                        }
+                    }
+                }
+                if (pdfi_dict_entries(oc_dict) == 0)
+                    goto exit;
+
+                code = pdfi_pdfmark_dict(ctx, oc_dict);
+                if (code < 0)
+                    goto exit;
+
+                /* Create an indirect ref for the dict */
+                code = pdfi_object_alloc(ctx, PDF_INDIRECT, 0, (pdf_obj **)&dictref);
+                if (code < 0) goto exit;
+                pdfi_countup(dictref);
+                dictref->ref_object_num = oc_dict->object_num;
+                dictref->ref_generation_num = oc_dict->generation_num;
+                dictref->is_marking = true;
+
+                objarray[1] = (pdf_obj *)dictref;
                 break;
             case PDF_DICT:
                 objarray[1] = o;
@@ -627,30 +757,33 @@ int pdfi_op_BDC(pdf_context *ctx, pdf_dict *stream_dict, pdf_dict *page_dict)
         goto exit;
     }
 
-    /* Check if first arg is a name and handle it if so */
-    /* TODO: spec says it could also be an inline dict that we should be able to handle,
-     * but I am just matching what gs does for now, and it doesn't handle that case.
-     */
-    properties = (pdf_name *)o;
-    if (pdfi_type_of(properties) != PDF_NAME)
-        goto exit;
+    if (pdfi_name_is(tag, "OC")) {
+        /* Check if first arg is a name and handle it if so */
+        /* TODO: spec says it could also be an inline dict that we should be able to handle,
+         * but I am just matching what gs does for now, and it doesn't handle that case.
+         */
+        properties = (pdf_name *)o;
+        if (pdfi_type_of(properties) != PDF_NAME)
+            goto exit;
 
-    /* If it's a name, look it up in Properties */
-    code = pdfi_find_resource(ctx, (unsigned char *)"Properties", properties,
-                              (pdf_dict *)stream_dict, page_dict, (pdf_obj **)&oc_dict);
-    if (code != 0)
-        goto exit;
-    if (pdfi_type_of(oc_dict) != PDF_DICT)
-        goto exit;
+        /* If it's a name, look it up in Properties */
+        code = pdfi_find_resource(ctx, (unsigned char *)"Properties", properties,
+                                  (pdf_dict *)stream_dict, page_dict, (pdf_obj **)&oc_dict);
+        if (code != 0)
+            goto exit;
+        if (pdfi_type_of(oc_dict) != PDF_DICT)
+            goto exit;
 
-    /* Now we have an OC dict, see if it's visible */
-    ocg_is_visible = pdfi_oc_is_ocg_visible(ctx, oc_dict);
-    if (!ocg_is_visible)
-        code = pdfi_oc_levels_set(ctx, ctx->OFFlevels, ctx->BMClevel);
+        ocg_is_visible = pdfi_oc_is_ocg_visible(ctx, oc_dict);
+        if (!ocg_is_visible)
+            code = pdfi_oc_levels_set(ctx, ctx->OFFlevels, ctx->BMClevel);
 
- exit:
+    }
+
+exit:
     if (objarray != NULL)
         gs_free_object(ctx->memory, objarray, "free pdfi_op_BDC");
+    pdfi_countdown(dictref);
     pdfi_countdown(o);
     pdfi_countdown(tag);
     pdfi_countdown(oc_dict);
@@ -662,7 +795,7 @@ int pdfi_op_EMC(pdf_context *ctx)
 {
     int code, code1 = 0;
 
-    if (ctx->device_state.writepdfmarks && ctx->args.preservemarkedcontent && !ctx->BDCWasOC)
+    if (ctx->device_state.writepdfmarks && ctx->args.preservemarkedcontent && (!ctx->BDCWasOC || ctx->device_state.WantsOptionalContent))
         code1 = pdfi_pdfmark_from_objarray(ctx, NULL, 0, NULL, "EMC");
 
     code = pdfi_oc_levels_clear(ctx, ctx->OFFlevels, ctx->BMClevel);
