@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2022 Artifex Software, Inc.
+/* Copyright (C) 2019-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Transparency support */
@@ -100,11 +100,11 @@ static int pdfi_trans_set_mask(pdf_context *ctx, pdfi_int_gstate *igs, int color
     pdf_name *n = NULL;
     pdf_name *S = NULL;
     pdf_obj *CS = NULL;
-    double f;
     gs_matrix save_matrix, GroupMat, group_Matrix;
     gs_transparency_mask_subtype_t subtype = TRANSPARENCY_MASK_Luminosity;
     bool Processed, ProcessedKnown = 0;
     bool save_OverrideICC = gs_currentoverrideicc(ctx->pgs);
+    gs_gstate *saved_gs = NULL;
 
 #if DEBUG_TRANSPARENCY
     dbgmprintf(ctx->memory, "pdfi_trans_set_mask (.execmaskgroup) BEGIN\n");
@@ -142,9 +142,8 @@ static int pdfi_trans_set_mask(pdf_context *ctx, pdfi_int_gstate *igs, int color
         /* G is transparency group XObject (required) */
         code = pdfi_dict_knownget_type(ctx, SMask, "G", PDF_STREAM, (pdf_obj **)&G_stream);
         if (code <= 0) {
-            dmprintf(ctx->memory, "WARNING: Missing 'G' in SMask, ignoring.\n");
             pdfi_trans_end_smask_notify(ctx);
-            code = 0;
+            code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_SMASK_MISSING_G, "pdfi_trans_set_mask", "");
             goto exit;
         }
 
@@ -155,16 +154,16 @@ static int pdfi_trans_set_mask(pdf_context *ctx, pdfi_int_gstate *igs, int color
         /* S is a subtype name (required) */
         code = pdfi_dict_knownget_type(ctx, SMask, "S", PDF_NAME, (pdf_obj **)&S);
         if (code <= 0) {
-            dmprintf(ctx->memory, "WARNING: Missing 'S' in SMask (defaulting to Luminosity)\n");
             subtype = TRANSPARENCY_MASK_Luminosity;
+            pdfi_set_warning(ctx, 0, NULL, W_PDF_SMASK_MISSING_S, "pdfi_trans_set_mask", "");
         }
         else if (pdfi_name_is(S, "Luminosity")) {
             subtype = TRANSPARENCY_MASK_Luminosity;
         } else if (pdfi_name_is(S, "Alpha")) {
             subtype = TRANSPARENCY_MASK_Alpha;
         } else {
-            dmprintf(ctx->memory, "WARNING: Unknown subtype 'S' in SMask (defaulting to Luminosity)\n");
             subtype = TRANSPARENCY_MASK_Luminosity;
+            pdfi_set_warning(ctx, 0, NULL, W_PDF_SMASK_UNKNOWN_S, "pdfi_trans_set_mask", "");
         }
 
         /* TR is transfer function (Optional) */
@@ -179,16 +178,16 @@ static int pdfi_trans_set_mask(pdf_context *ctx, pdfi_int_gstate *igs, int color
                     if (gsfunc->params.m != 1 || gsfunc->params.n != 1) {
                         pdfi_free_function(ctx, gsfunc);
                         gsfunc = NULL;
-                        dmprintf(ctx->memory, "WARNING: Ignoring invalid TR (number of inpuits or outputs not 1) in SMask\n");
+                        pdfi_set_warning(ctx, 0, NULL, W_PDF_SMASK_INVALID_TR, "pdfi_trans_set_mask", "");
                     }
                     break;
                 case PDF_NAME:
                     if (!pdfi_name_is((pdf_name *)TR, "Identity")) {
-                        dmprintf(ctx->memory, "WARNING: Unknown TR in SMask\n");
+                        pdfi_set_warning(ctx, 0, NULL, W_PDF_SMASK_UNKNOWN_TR, "pdfi_trans_set_mask", "");
                     }
                     break;
                 default:
-                    dmprintf(ctx->memory, "WARNING: Ignoring invalid TR in SMask\n");
+                    pdfi_set_warning(ctx, 0, NULL, W_PDF_SMASK_UNKNOWN_TR_TYPE, "pdfi_trans_set_mask", "");
             }
         }
 
@@ -197,22 +196,6 @@ static int pdfi_trans_set_mask(pdf_context *ctx, pdfi_int_gstate *igs, int color
         if (code < 0)
             goto exit;
 
-        code = pdfi_dict_knownget_type(ctx, G_stream_dict, "Matte", PDF_ARRAY, (pdf_obj **)&a);
-        if (code > 0) {
-            int ix;
-
-            for (ix = 0; ix < pdfi_array_size(a); ix++) {
-                code = pdfi_array_get_number(ctx, a, (uint64_t)ix, &f);
-                if (code < 0)
-                    break;
-                params.Matte[ix] = f;
-            }
-            if (ix >= pdfi_array_size(a))
-                params.Matte_components = pdfi_array_size(a);
-            else
-                params.Matte_components = 0;
-        }
-
         code = pdfi_dict_knownget_type(ctx, G_stream_dict, "BBox", PDF_ARRAY, (pdf_obj **)&BBox);
         if (code < 0)
             goto exit;
@@ -267,10 +250,9 @@ static int pdfi_trans_set_mask(pdf_context *ctx, pdfi_int_gstate *igs, int color
                 params.ColorSpace = pcs;
                 if (code < 0)
                     goto exit;
-            } else {
-                /* Inherit current colorspace */
-                params.ColorSpace = ctx->pgs->color[colorindex].color_space;
             }
+            else
+                   params.ColorSpace = NULL;
         } else {
             /* GS and Adobe will ignore the whole mask in this case, so we do the same.
             */
@@ -280,39 +262,99 @@ static int pdfi_trans_set_mask(pdf_context *ctx, pdfi_int_gstate *igs, int color
 
         /* If there's a BC, put it in the params */
         if (BC) {
-            int i;
-            double num;
-            for (i=0; i<pdfi_array_size(BC); i++) {
-                if (i > GS_CLIENT_COLOR_MAX_COMPONENTS)
-                    break;
-                code = pdfi_array_get_number(ctx, BC, i, &num);
-                if (code < 0)
-                    break;
-                params.Background[i] = (float)num;
+            if (params.ColorSpace == NULL) {
+                if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_undefined), NULL, E_PDF_GROUP_BAD_BC_NO_CS, "pdfi_trans_set_mask", NULL)) < 0)
+                    goto exit;
+            } else {
+                int i, components = pdfi_array_size(BC);
+                double num;
+
+                if (components > GS_CLIENT_COLOR_MAX_COMPONENTS) {
+                    if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_limitcheck), NULL, E_PDF_GROUP_BAD_BC_TOO_BIG, "pdfi_trans_set_mask", NULL)) < 0)
+                        goto exit;
+                } else {
+                    if (gs_color_space_num_components(params.ColorSpace) != components) {
+                        pdfi_set_warning(ctx, 0, NULL, W_PDF_GROUP_BAD_BC, "pdfi_trans_set_mask", NULL);
+                        components = min(components, gs_color_space_num_components(params.ColorSpace));
+                    }
+
+                    for (i=0; i < components; i++) {
+                        code = pdfi_array_get_number(ctx, BC, i, &num);
+                        if (code < 0)
+                            break;
+                        params.Background[i] = (float)num;
+                    }
+                    params.Background_components = components;
+
+                    /* TODO: Not sure how to handle this...  recheck PS code (pdf_draw.ps/gssmask) */
+                    /* This should be "currentgray" for the color that we put in params.ColorSpace,
+                     * It looks super-convoluted to actually get this value.  Really?
+                     * (see zcurrentgray())
+                     * For now, use simple definition from PLRM2 and assume it is RGB or CMYK
+                     */
+                    pdfi_set_GrayBackground(&params);
+                }
             }
-            params.Background_components = pdfi_array_size(BC);
-
-            if (gs_color_space_num_components(params.ColorSpace) != params.Background_components)
-                pdfi_set_warning(ctx, 0, NULL, W_PDF_GROUP_BAD_BC, "pdfi_trans_set_mask", NULL);
-
-            /* TODO: Not sure how to handle this...  recheck PS code (pdf_draw.ps/gssmask) */
-            /* This should be "currentgray" for the color that we put in params.ColorSpace,
-             * It looks super-convoluted to actually get this value.  Really?
-             * (see zcurrentgray())
-             * For now, use simple definition from PLRM2 and assume it is RGB or CMYK
-             */
-            pdfi_set_GrayBackground(&params);
         }
 
-        code = gs_begin_transparency_mask(ctx->pgs, &params, &bbox, false);
-        if (code < 0)
-            goto exit;
+        /* When we call gs_begin_transparency_mask, the pdf14 compisitor will perform
+         * a horrid pass through the graphics state, replacing "default" colorspaces
+         * with "ps" ones. This is required, apparently, to give us proper rendering
+         * of SMasks. Bug 705993 shows a case where the current colorspace differs
+         * from the colorspace that will be used once the igs->GroupGState is swapped
+         * in. Simply proceeding to call gs_begin_transparency_mask here, and then
+         * setting the igs->GroupGState, leaves us with a colorspace where the
+         * "default" -> "ps" change has not taken place, and we get incorrect rendering.
+         *
+         * As the igs->GroupGState can only be applied AFTER the
+         * gs_begin_transparancy_mask call (that is, I couldn't make it work applying
+         * it beforehand!), this means we need to manually swap in the colorspaces
+         * and colors, and be careful to restore them afterwards.
+         *
+         * NOTE: Attempts to gsave/grestore around the gs_begin_transparency_mask/
+         * gs_end_transparency_mask calls fail, showing diffs. My suspicion is that
+         * this actually disables the softmask that we've just rendered!
+         */
+        if (code >= 0) {
+            /* Override the colorspace if specified */
+            saved_gs = gs_gstate_copy(ctx->pgs, ctx->memory);
+            if (saved_gs == NULL)
+                code = gs_note_error(gs_error_VMerror);
+            if (code >= 0)
+                code = pdfi_gs_setcolorspace(ctx, igs->GroupGState->color[0].color_space);
+            if (code >= 0)
+                code = gs_setcolor(ctx->pgs, igs->GroupGState->color[0].ccolor);
+            gs_swapcolors_quick(ctx->pgs);
+            if (code >= 0)
+                code = pdfi_gs_setcolorspace(ctx, igs->GroupGState->color[1].color_space);
+            if (code >= 0)
+                code = gs_setcolor(ctx->pgs, igs->GroupGState->color[1].ccolor);
+            gs_swapcolors_quick(ctx->pgs);
+        }
 
-        code = pdfi_form_execgroup(ctx, ctx->page.CurrentPageDict, G_stream,
-                                   igs->GroupGState, NULL, &group_Matrix);
-        code1 = gs_end_transparency_mask(ctx->pgs, colorindex);
-        if (code == 0)
-            code = code1;
+        if (code >= 0)
+            code = gs_begin_transparency_mask(ctx->pgs, &params, &bbox, false);
+
+        if (code >= 0) {
+            code = pdfi_form_execgroup(ctx, ctx->page.CurrentPageDict, G_stream,
+                                       igs->GroupGState, NULL, NULL, &group_Matrix);
+            code1 = gs_end_transparency_mask(ctx->pgs, colorindex);
+            if (code == 0)
+                code = code1;
+        }
+
+        if (saved_gs) {
+            code1 = pdfi_gs_setcolorspace(ctx, saved_gs->color[0].color_space);
+            if (code >= 0) code = code1;
+            code1 = gs_setcolor(ctx->pgs, saved_gs->color[0].ccolor);
+            if (code >= 0) code = code1;
+            gs_swapcolors_quick(ctx->pgs);
+            code1 = pdfi_gs_setcolorspace(ctx, saved_gs->color[1].color_space);
+            if (code >= 0) code = code1;
+            code1 = gs_setcolor(ctx->pgs, saved_gs->color[1].ccolor);
+            if (code >= 0) code = code1;
+            gs_swapcolors_quick(ctx->pgs);
+        }
 
         /* Put back the matrix (we couldn't just rely on gsave/grestore for whatever reason,
          * according to PS code anyway...
@@ -328,10 +370,11 @@ static int pdfi_trans_set_mask(pdf_context *ctx, pdfi_int_gstate *igs, int color
         }
     } else {
         /* take action on a non-/Mask entry. What does this mean ? What do we need to do */
-        dmprintf(ctx->memory, "Warning: Type is not /Mask, entry ignored in pdfi_set_trans_mask\n");
+        pdfi_set_warning(ctx, 0, NULL, W_PDF_SMASK_UNKNOWN_TYPE, "pdfi_trans_set_mask", "");
     }
 
  exit:
+    gs_gstate_free(saved_gs);
     gs_setoverrideicc(ctx->pgs, save_OverrideICC);
     if (gsfunc)
         pdfi_free_function(ctx, gsfunc);
@@ -717,7 +760,7 @@ int pdfi_trans_setup(pdf_context *ctx, pdfi_trans_state_t *state, gs_rect *bbox,
     bool current_overprint;
     bool okOPcs = false;
     bool ChangeBM = false;
-    gs_blend_mode_t mode;
+    gs_blend_mode_t  mode = gs_currentblendmode(ctx->pgs);  /* quite warning */
     bool need_group = false;
 
     memset(state, 0, sizeof(*state));
@@ -782,7 +825,12 @@ int pdfi_trans_setup(pdf_context *ctx, pdfi_trans_state_t *state, gs_rect *bbox,
         code = gs_setfillconstantalpha(ctx->pgs, 1.0);
         code = gs_setstrokeconstantalpha(ctx->pgs, 1.0);
     }
-    if (ChangeBM) {
+
+    /* If we are in a fill stroke situation, do not change the blend mode.
+       We can have situations where the fill and the stroke have different
+       blending needs (one may need compatible overprint and the other may
+       need the normal mode) This is all handled in the fill stroke logic */
+    if (ChangeBM && caller != TRANSPARENCY_Caller_FillStroke) {
         state->saveBM = mode;
         state->ChangeBM = true;
         code = gs_setblendmode(ctx->pgs, BLEND_MODE_CompatibleOverprint);
