@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -44,16 +44,6 @@
 #include "store.h"
 #include "scanchar.h"
 
-/* Procedure for handling DSC comments if desired. */
-/* Set at initialization if a DSC handling module is included. */
-int (*gs_scan_dsc_proc) (const byte *, uint) = NULL;
-
-/* Procedure for handling all comments if desired. */
-/* Set at initialization if a comment handling module is included. */
-/* If both gs_scan_comment_proc and gs_scan_dsc_proc are set, */
-/* scan_comment_proc is called only for non-DSC comments. */
-int (*gs_scan_comment_proc) (const byte *, uint) = NULL;
-
 /*
  * Level 2 includes some changes in the scanner:
  *      - \ is always recognized in strings, regardless of the data source;
@@ -127,10 +117,11 @@ dynamic_grow(da_ptr pda, byte * next, uint max_size)
     pda->next = next;
     if (old_size >= max_size)
         return_error(gs_error_limitcheck);
-    while ((code = dynamic_resize(pda, new_size)) < 0 &&
-           new_size > old_size
-        ) {                     /* Try trimming down the requested new size. */
+    while ((code = dynamic_resize(pda, new_size)) < 0) {
+        /* Try trimming down the requested new size. */
         new_size -= (new_size - old_size + 1) >> 1;
+        if (new_size <= old_size)
+                break;
     }
     return code;
 }
@@ -145,7 +136,13 @@ dynamic_save(da_ptr pda)
 
         if (len > sizeof(pda->buf))
             len = sizeof(pda->buf);
-        memcpy(pda->buf, pda->base, len);
+        /* This can happen if we get a /<CR> at the end of a buffer, and the file is
+         * not at EOF. In this case 'len' will be zero so we don't actually copy any
+         * bytes. So this is safe on current C run-time libraries, but it's probably
+         * best to avoid it. Coverity ID C382008
+         */
+        if (pda->base != NULL)
+            memcpy(pda->buf, pda->base, len);
         pda->next = pda->buf + len;
         pda->base = pda->buf;
     }
@@ -355,10 +352,6 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
             dmputs(imemory, "\n");
         }
 #endif
-        if (gs_scan_dsc_proc != NULL) {
-            code = gs_scan_dsc_proc(base, len);
-            return (code < 0 ? code : 0);
-        }
         if (pstate->s_options & SCAN_PROCESS_DSC_COMMENTS) {
             code = scan_DSC_Comment;
             goto comment;
@@ -374,10 +367,6 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
         }
     }
 #endif
-    if (gs_scan_comment_proc != NULL) {
-        code = gs_scan_comment_proc(base, len);
-        return (code < 0 ? code : 0);
-    }
     if (pstate->s_options & SCAN_PROCESS_COMMENTS) {
         code = scan_Comment;
         goto comment;
@@ -445,7 +434,7 @@ gs_scan_string_token_options(i_ctx_t *i_ctx_p, ref * pstr, ref * pref,
  * as well as for scan_Refill.
  */
 int
-gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
+gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate) /* lgtm [cpp/use-of-goto] */
 {
     stream *const s = pstate->s_file.value.pfile;
     ref *myref = pref;
@@ -934,7 +923,24 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
         case '-':
             sign = -1;
             if(i_ctx_p->scanner_options & SCAN_PDF_INV_NUM) {
+                const byte *osptr = sptr;
                 do {
+                    /* This is slightly unpleasant: we have to bounds check the buffer,
+                       rather than just incrementing the point until we find a non '-' character.
+                       But we cannot differentiate between multiple '-' characters that
+                       straddle a buffer boundary, or a token that is only one or more '-' characters.
+                       Handling this relies on the fact that the Postscript-based PDF interpreter
+                       always uses the "token" operator to tokenize a stream, thus we can assume
+                       here that the current buffer contains the entire token. So if we reach
+                       the end of the buffer without hitting a character taht is not a '-', we'll reset
+                       the buffer pointer, and retry, treating it as a name object.
+                     */
+                    if (sptr + 1 > endptr) {
+                        sptr = osptr;
+                        sstate.s_ss.s_name.s_name_type = 0;
+                        sstate.s_ss.s_name.s_try_number = true;
+                        goto do_name;
+                    }
                     if (*(sptr + 1) == '-') {
                         sptr++;
                     } else
