@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -144,10 +144,18 @@ zchar42_set_cache(i_ctx_t *i_ctx_p, gs_font_base *pbfont, ref *cnref,
        which stretch subglyphs. Uniting with FontBBox helps.
        In same time, FontBBox with no glyph bbox
        doesn't work for 34_all.PS page 4. */
-    bbox.p.x = min(sbw_bbox[4], pbfont->FontBBox.p.y);
-    bbox.p.y = min(sbw_bbox[5], pbfont->FontBBox.p.y);
-    bbox.q.x = max(sbw_bbox[6], pbfont->FontBBox.q.x);
-    bbox.q.y = max(sbw_bbox[7], pbfont->FontBBox.q.y);
+    /* Previously we used to expand the bbox to the maximum/minimum
+     * of the glyph and font bounding boxes combined, as noted above.
+     * However this causes incorrect output (bug #703697) for vertical
+     * writing fonts with pdfwrite. Since we no longer use this code
+     * for rendering, and pdfwrite doesn't use the metrics for clipping,
+     * but does for positioning, we've removed this code.
+     */
+
+    bbox.p.x = sbw_bbox[4];
+    bbox.p.y = sbw_bbox[5];
+    bbox.q.x = sbw_bbox[6];
+    bbox.q.y = sbw_bbox[7];
     return zchar_set_cache(i_ctx_p, pbfont, cnref,
                            NULL,
                            w, &bbox,
