@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -86,12 +86,20 @@ zcopydevice2(i_ctx_t *i_ctx_p)
     int code;
     psi_device_ref *psdev;
 
+    check_op(2);
     check_read_type(op[-1], t_device);
     check_type(*op, t_boolean);
     if (op[-1].value.pdevice == NULL)
         /* This can happen if we invalidated devices on the stack by calling nulldevice after they were pushed */
         return_error(gs_error_undefined);
 
+    if (gs_is_path_control_active((const gs_memory_t *)i_ctx_p->memory.current)) {
+        const gx_device *dev = (const gx_device *)op[-1].value.pdevice->device;
+
+        if (gs_check_device_permission((gs_memory_t *)i_ctx_p->memory.current, dev->dname, strlen(dev->dname)) == 0)
+            return_error(gs_error_invalidaccess);
+    }
+
     code = gs_copydevice2(&new_dev, op[-1].value.pdevice->device, op->value.boolval,
                           imemory);
     if (code < 0)
@@ -173,6 +181,7 @@ zdevicename(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     const char *dname;
 
+    check_op(1);
     check_read_type(*op, t_device);
     if (op->value.pdevice == NULL)
         /* This can happen if we invalidated devices on the stack by calling nulldevice after they were pushed */
@@ -225,6 +234,7 @@ zgetbitsrect(i_ctx_t *i_ctx_p)
     int num_rows;
     int code;
 
+    check_op(7);
     check_read_type(op[-7], t_device);
     if (op[-7].value.pdevice == NULL)
         /* This can happen if we invalidated devices on the stack by calling nulldevice after they were pushed */
@@ -303,6 +313,7 @@ zgetdevice(i_ctx_t *i_ctx_p)
     const gx_device *dev;
     psi_device_ref *psdev;
 
+    check_op(1);
     check_type(*op, t_integer);
     if (op->value.intval != (int)(op->value.intval))
         return_error(gs_error_rangecheck);	/* won't fit in an int */
@@ -357,6 +368,7 @@ zget_device_params(i_ctx_t *i_ctx_p, bool is_hardware)
     int code;
     ref *pmark;
 
+    check_op(2);
     check_read_type(op[-1], t_device);
 
     if(!r_has_type(op, t_null)) {
@@ -386,6 +398,8 @@ zget_device_params(i_ctx_t *i_ctx_p, bool is_hardware)
         return code;
     }
     pmark = ref_stack_index(&o_stack, list.count * 2);
+    if (pmark == NULL)
+        return_error(gs_error_stackunderflow);
     make_mark(pmark);
     return 0;
 }
@@ -415,6 +429,7 @@ zmakewordimagedevice(i_ctx_t *i_ctx_p)
     int code;
     psi_device_ref *psdev;
 
+    check_op(5);
     check_int_leu(op[-3], max_uint >> 1);	/* width */
     check_int_leu(op[-2], max_uint >> 1);	/* height */
     check_type(*op, t_boolean);
@@ -483,6 +498,7 @@ zoutputpage(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     int code;
 
+    check_op(2);
     check_type(op[-1], t_integer);
     check_type(*op, t_boolean);
     if (gs_debug[':']) {
@@ -528,9 +544,13 @@ zputdeviceparams(i_ctx_t *i_ctx_p)
     if (count == 0)
         return_error(gs_error_unmatchedmark);
     prequire_all = ref_stack_index(&o_stack, count);
+    if (prequire_all == NULL)
+        return_error(gs_error_stackunderflow);
     ppolicy = ref_stack_index(&o_stack, count + 1);
+    if (ppolicy == NULL)
+        return_error(gs_error_stackunderflow);
     pdev = ref_stack_index(&o_stack, count + 2);
-    if (pdev == 0)
+    if (pdev == NULL)
         return_error(gs_error_stackunderflow);
     check_type_only(*prequire_all, t_boolean);
     check_write_type_only(*pdev, t_device);
@@ -546,14 +566,20 @@ zputdeviceparams(i_ctx_t *i_ctx_p)
     old_height = dev->height;
     code = gs_putdeviceparams(dev, (gs_param_list *) & list);
     /* Check for names that were undefined or caused errors. */
-    for (dest = count - 2, i = 0; i < count >> 1; i++)
+    for (dest = count - 2, i = 0; i < count >> 1; i++) {
+        ref *o;
         if (list.results[i] < 0) {
-            *ref_stack_index(&o_stack, dest) =
-                *ref_stack_index(&o_stack, count - (i << 1) - 2);
-            gs_errorname(i_ctx_p, list.results[i],
-                         ref_stack_index(&o_stack, dest - 1));
+            o = ref_stack_index(&o_stack, dest);
+            if (o == NULL)
+                continue;
+            *o = *ref_stack_index(&o_stack, count - (i << 1) - 2);
+            o = ref_stack_index(&o_stack, dest - 1);
+            if (o == NULL)
+                continue;
+            gs_errorname(i_ctx_p, list.results[i], o);
             dest -= 2;
         }
+    }
     iparam_list_release(&list);
     if (code < 0) {		/* There were errors reported. */
         ref_stack_pop(&o_stack, dest + 1);
@@ -612,6 +638,7 @@ zsetdevice(i_ctx_t *i_ctx_p)
 
     if (code < 0)
         return code;
+    check_op(1);
     check_write_type(*op, t_device);
 
     if (op->value.pdevice == 0)
