@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -32,6 +32,7 @@ zdict(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
+    check_op(1);
     check_type(*op, t_integer);
     if (op->value.intval < 0)
         return_error(gs_error_rangecheck);
@@ -44,6 +45,7 @@ zmaxlength(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
+    check_op(1);
     check_type(*op, t_dictionary);
     check_dict_read(*op);
     make_int(op, dict_maxlength(op));
@@ -56,6 +58,7 @@ zbegin(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
+    check_op(1);
     check_type(*op, t_dictionary);
     check_dict_read(*op);
     if ( dsp == dstop ) {
@@ -106,6 +109,7 @@ zop_def(i_ctx_t *i_ctx_p)
     os_ptr op1 = op - 1;
     ref *pvslot;
 
+    check_op(2);
     /* The following combines a check_op(2) with a type check. */
     switch (r_type(op1)) {
         case t_name: {
@@ -168,6 +172,7 @@ zload(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     ref *pvalue;
 
+    check_op(1);
     switch (r_type(op)) {
         case t_name:
             /* Use the fast lookup. */
@@ -187,6 +192,9 @@ zload(i_ctx_t *i_ctx_p)
                 for (i = 0; i < size; i++) {
                     ref *dp = ref_stack_index(&d_stack, i);
 
+                    if (dp == NULL)
+                        return_error(gs_error_stackunderflow);
+
                     check_dict_read(*dp);
                     if (dict_find(dp, op, &pvalue) > 0) {
                         ref_assign(op, pvalue);
@@ -211,6 +219,7 @@ zundef(i_ctx_t *i_ctx_p)
     os_ptr op1 = op - 1;
     int code;
 
+    check_op(2);
     check_type(*op1, t_dictionary);
     check_dict_write(*op1);
     code = idict_undef(op1, op);
@@ -229,6 +238,7 @@ zknown(i_ctx_t *i_ctx_p)
     ref *pvalue;
     int code;
 
+    check_op(2);
     check_type(*op1, t_dictionary);
     check_dict_read(*op1);
     code = dict_find(op1, op, &pvalue);
@@ -287,6 +297,7 @@ zcopy_dict(i_ctx_t *i_ctx_p)
     os_ptr op1 = op - 1;
     int code;
 
+    check_op(2);
     check_type(*op1, t_dictionary);
     check_dict_read(*op1);
     check_dict_write(*op);
@@ -341,6 +352,7 @@ zdictstack(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     uint count = ref_stack_count(&d_stack);
 
+    check_op(1);
     if (!level2_enabled)
         count--;		/* see dstack.h */
     if (!r_is_array(op))
@@ -423,6 +435,7 @@ zforcecopynew(i_ctx_t *i_ctx_p)
     os_ptr op1 = op - 1;
     int code;
 
+    check_op(2);
     check_type(*op1, t_dictionary);
     check_dict_read(*op1);
     check_type(*op, t_dictionary);
@@ -449,6 +462,7 @@ zforceundef(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
+    check_op(2);
     check_type(op[-1], t_dictionary);
     /* Don't check_dict_write */
     idict_undef(op - 1, op);	/* ignore undefined error */
@@ -465,6 +479,7 @@ zknownget(i_ctx_t *i_ctx_p)
     register os_ptr op1 = op - 1;
     ref *pvalue;
 
+    check_op(2);
     check_type(*op1, t_dictionary);
     check_dict_read(*op1);
     if (dict_find(op1, op, &pvalue) <= 0) {
@@ -485,6 +500,7 @@ zknownundef(i_ctx_t *i_ctx_p)
     os_ptr op1 = op - 1;
     int code;
 
+    check_op(2);
     check_type(*op1, t_dictionary);
     check_dict_write(*op1);
     code = idict_undef(op1, op);
@@ -502,6 +518,7 @@ zsetmaxlength(i_ctx_t *i_ctx_p)
     uint new_size;
     int code;
 
+    check_op(2);
     check_type(*op1, t_dictionary);
     check_dict_write(*op1);
     check_type(*op, t_integer);
