@@ -7,8 +7,9 @@ use Data::Dumper;
 
 my $verbose=0;
 
-# bmpcmp usage: [gs] [pcl] [xps] [mupdf] [mujstest] [bmpcmp] [arm] [lowres] [highres] [32] [pdfwrite] [ps2write] [xpswrite] [nopdfwrite] [relaxtimeout] [extended] [smoke] [cull] [avx2] [$user] | abort
-
+# bmpcmp usage: [-L] [gs] [pcl] [xps] [mupdf] [mujstest] [murun] [mudrawpy] [runtests] [extract] [extractmu] [extractgs] [bmpcmp] [arm] [lowres] [highres] [32] [pdfwrite] [ps2write] [xpswrite] [nopdfwrite] [relaxtimeout] [extended] [smoke] [cull] [avx2] [$user] | abort
+#
+# -L passes -L to rsync, which expands softlinks.
 
 
 my %products=('abort' =>1,
@@ -22,8 +23,18 @@ my %products=('abort' =>1,
               'gpdf'=>1,
               'gpdl'=>1,
               'mupdf'=>1,
-              'mujstest'=>1);
+              'mujstest'=>1,
+              'murun'=>1,
+              'mudrawpy'=>1,
+              'runtests'=>1,
+              'extract'=>1,
+              'extractmu'=>1,
+              'muwasm'=>1,
+              'mucsharp'=>1,
+              'pymupdf'=>1,
+              'extractgs'=>1);
 
+my $rsync_L = "";
 my $user;
 my $product="";
 my $filters="";
@@ -70,7 +81,7 @@ while ($t1=shift) {
     $cull="cull";
   } elsif ($t1 eq "avx2") {
     $avx2="avx2";
-  } elsif ($t1 eq "nr" || $t1 eq "nonredundnat") {
+  } elsif ($t1 eq "nr" || $t1 eq "nonredundant") {
     $nr="nonredundant";
   } elsif ($t1 eq "pdfwrite" || $t1 eq "ps2write" || $t1 eq "xpswrite") {
     $pdfwrite="pdfwrite";
@@ -78,8 +89,12 @@ while ($t1=shift) {
     $nopdfwrite="nopdfwrite";
   } elsif ($t1 eq "timeout" || $t1 eq "relaxtimeout") {
     $relaxTimeout="relaxTimeout";
+  } elsif ($t1 eq "-L") {
+    $rsync_L="L";
   } elsif ($t1=~m/^-/ || $t1=~m/^\d/) {
     $command.=$t1.' ';
+  } elsif ($t1 =~ m/ifilter=.*/) {
+    $filters.=$t1.' ';
   } elsif ($t1 =~ m/filter=.*/) {
     $filters.=$t1.' ';
   } elsif ($t1 =~ m/extras=.*/) {
@@ -133,8 +148,17 @@ my $directory=`pwd`;
 chomp $directory;
 
 $directory =~ s|.+/||;
-if ($directory ne 'gs' && $directory ne 'ghostpdl' && $directory ne 'mupdf' && $directory ne 'ghostpdl.git' && $directory ne 'mupdf.git') {
+if ($directory ne 'gs' &&
+    $directory ne 'ghostpdl' &&
+    $directory ne 'mupdf' &&
+    $directory ne 'ghostpdl.git' &&
+    $directory ne 'mupdf.git' &&
+    $directory ne 'extract' &&
+    $directory ne 'extract.git') {
   $directory="";
+  if (-f "include/extract.h") {
+    $directory='extract';
+  }
   if (-d "base" && -d "Resource") {
     $directory='gs';
   }
@@ -149,13 +173,15 @@ if ($directory ne 'gs' && $directory ne 'ghostpdl' && $directory ne 'mupdf' && $
 #$directory="gs" if ($directory eq "" && $product eq "bmpcmp");
 $directory="gs" if ($directory eq "" && $product && $product eq "abort");
 
-die "can't figure out if this is a ghostpdl, gs, or mupdf source directory" if ($directory eq "");
+die "can't figure out if this is a ghostpdl, gs, mupdf or extract source directory" if ($directory eq "");
 
 if (!$product) {
   if ($directory eq 'mupdf') {
     $product='mupdf';
+  } elsif ($directory eq 'extract') {
+    $product='extract';
   } else {
-    $product='gs pcl xps gpdl'
+    $product='gs pcl xps gpdl gpdf'
   }
 }
 
@@ -193,23 +219,54 @@ if ($msys) {
   $hostpath="regression:$dir/$user/$directory";
 }
 
-my $cmd="rsync -avxcz ".
-" --max-size=30000000".
+my $cmd="rsync -axcz" . $rsync_L;
+
+if ($product eq "extractgs") {
+  $cmd .= "L"; # expand links.
+}
+
+if ($verbose) {
+  $cmd .= "i";
+} else {
+  $cmd .= "v";
+}
+
+$cmd .= " --max-size=30000000".
 " --delete --delete-excluded".
 " --exclude .svn --exclude .git".
 " --exclude _darcs --exclude .bzr --exclude .hg".
 " --exclude .deps --exclude .libs --exclude autom4te.cache".
 " --exclude bin --exclude obj --exclude debugobj --exclude pgobj".
 " --exclude bin64 --exclude obj64 --exclude debugobj64 --exclude pgobj64".
+" --exclude luratechbin --exclude luratechobj --exclude luratechbin64 --exclude luratechobj64".
 " --exclude membin --exclude memobj --exclude membin64 --exclude memobj64".
 " --exclude profbin --exclude profobj --exclude profbin64 --exclude profobj64".
-" --exclude sobin --exclude soobj --exclude debugbin".
+" --exclude sanbin --exclude sanobj --exclude sanbin64 --exclude sanobj64".
+" --exclude sobin --exclude soobj --exclude debugbin --exclude sodebugbin --exclude sodebugobj".
 " --exclude ufst --exclude ufst-obj --exclude ufst-debugobj".
+" --exclude '*-bin' --exclude '*-obj'".
 " --exclude config.log --exclude .png".
 " --exclude .ppm --exclude .pkm --exclude .pgm --exclude .pbm".
 " --exclude .tif --exclude .bmp".
 " --exclude debug --exclude release --exclude generated --exclude sanitize".  # we cannot just exclude build, since tiff/build/Makefile.in, etc. is needed
 " --exclude tiff-config".
+" --include='shared-inl.h' --exclude 'shared-*'".
+" --exclude user.make".
+
+# Exclude files generated by scripts/mupdfwrap.py:
+" --exclude mupdfwrap_ref".
+" --exclude platform/c++".
+" --exclude platform/python".
+" --exclude include/html".
+" --exclude include/latex".
+
+" --exclude /build/".
+
+# Excludes for extract library.
+" --exclude /src/build/".
+" --exclude extract/src/build/".
+" --exclude extract/test/generated/".
+
 # " --exclude Makefile". We can't just exclude Makefile, since the MuPDF Makefile is not a derived file.
 " -e \"$ssh\" ".
 " .".
