@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -177,6 +177,7 @@ void xps_free_part(xps_context_t *ctx, xps_part_t *part);
 
 typedef struct xps_document_s xps_document_t;
 typedef struct xps_page_s xps_page_t;
+typedef struct xps_page_range_s xps_page_range_t;
 
 struct xps_document_s
 {
@@ -192,6 +193,15 @@ struct xps_page_s
     xps_page_t *next;
 };
 
+struct xps_page_range_s
+{
+    int first;
+    int last;
+    int reverse;
+    int current;
+    char *page_list;
+};
+
 int xps_parse_metadata(xps_context_t *ctx, xps_part_t *part);
 void xps_free_fixed_pages(xps_context_t *ctx);
 void xps_free_fixed_documents(xps_context_t *ctx);
@@ -220,6 +230,7 @@ struct xps_image_s
     byte *alpha; /* isolated alpha plane */
     byte *profile;
     int profilesize;
+    bool invert_decode;
 };
 
 int xps_decode_jpeg(xps_context_t *ctx, byte *rbuf, int rlen, xps_image_t *image);
@@ -258,6 +269,9 @@ struct xps_font_s
     byte *gsubrs;
     byte *subrs;
     byte *charstrings;
+    char **names;
+    int max_name_index;
+    int next_name_index;
 };
 
 struct xps_glyph_metrics_s
@@ -410,6 +424,8 @@ struct xps_context_s
     xps_page_t *first_page; /* first page of document */
     xps_page_t *last_page; /* last page of document */
 
+    xps_page_range_t *page_range; /* interpreter-based page range handling */
+
     char *base_uri; /* base uri for parsing XML and resolving relative paths */
     char *part_uri; /* part uri for parsing metadata relations */
 
@@ -433,6 +449,12 @@ struct xps_context_s
      * 1=nonzero, 0=evenodd
      */
     int fill_rule;
+
+    /* Enable us to detect that we are currently sending high level
+       pattern to a device.  If during this process we encounter
+       a glyph that has a pattern, make sure to set the color
+       space appropriately prior to setting the text. */
+    bool in_high_level_pattern;
 };
 
 int xps_process_file(xps_context_t *ctx, const char *filename);
