@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
  */
 
 
@@ -91,6 +91,13 @@ xps_deobfuscate_font_resource(xps_context_t *ctx, xps_part_t *part)
     char *p;
     int i;
 
+    /* Ensure the part has at least 32 bytes we can write */
+    if (part->size < 32)
+    {
+        gs_warn("obfuscated font part is too small");
+        return;
+    }
+
     p = strrchr(part->name, '/');
     if (!p)
         p = part->name;
@@ -284,7 +291,7 @@ xps_flush_text_buffer(xps_context_t *ctx, xps_font_t *font,
             if (code != 0)
                 return gs_throw1(-1, "cannot gs_text_process() (%d)", code);
 
-            gs_text_release(Tr_textenum, "gslt font render");
+            gs_text_release(ctx->pgs, Tr_textenum, "gslt font render");
 
             gs_stroke(ctx->pgs);
         }
@@ -300,7 +307,7 @@ xps_flush_text_buffer(xps_context_t *ctx, xps_font_t *font,
     if (code != 0)
         return gs_throw1(-1, "cannot gs_text_process() (%d)", code);
 
-    gs_text_release(textenum, "gslt font render");
+    gs_text_release(ctx->pgs, textenum, "gslt font render");
 
     buf->count = 0;
 
@@ -776,8 +783,12 @@ xps_parse_glyphs(xps_context_t *ctx,
 
         if (sim_bold)
         {
-            /* widening strokes by 1% of em size */
-            gs_setlinewidth(ctx->pgs, font_size * 0.02);
+            if (!ctx->preserve_tr_mode)
+                /* widening strokes by 1% of em size */
+                gs_setlinewidth(ctx->pgs, font_size * 0.02);
+            else
+                /* Undo CTM scaling */
+                gs_setlinewidth(ctx->pgs, font_size * 0.02 * fabs(ctx->pgs->ctm.xx) / (ctx->pgs->device->HWResolution[0] / 72.0));
             gs_settextrenderingmode(ctx->pgs, 2);
         }
 
@@ -809,6 +820,15 @@ xps_parse_glyphs(xps_context_t *ctx,
 
     if (fill_tag)
     {
+        if (ctx->in_high_level_pattern)
+        {
+            float value[2];  /* alpha and gray */
+
+            value[0] = gs_getfillconstantalpha(ctx->pgs);
+            value[1] = 0;
+            xps_set_color(ctx, ctx->gray, value);
+        }
+
         ctx->fill_rule = 1; /* always use non-zero winding rule for char paths */
         code = xps_parse_glyphs_imp(ctx, font, font_size,
                 atof(origin_x_att), atof(origin_y_att),
