@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -619,7 +619,7 @@ xps_reorder_pages(xps_context_t *ctx)
     char *page_list = ctx->page_range->page_list;
     xps_page_t **page_ptr_array, *page = ctx->first_page;
     int count = 0, k;
-    int code;
+    int code = 0;
     int start;
     int end;
     xps_page_t* first_page = NULL;
